@@ -1,5 +1,6 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "map.h"
 #include "player.h"
 #include "vehicle.h"
 #include "engine.h"
@@ -68,10 +69,11 @@ static void AiCase1(Player *p)
 static void AiStateVehLoop(Player *p)
 {
 	Vehicle *v;
+	uint index;
 
-	v = p->ai.cur_veh == NULL ? _vehicles : p->ai.cur_veh+1;
+	index = (p->ai.cur_veh == NULL) ? 0 : p->ai.cur_veh->index + 1;
 
-	for (;v != endof(_vehicles); v++) {
+	FOR_ALL_VEHICLES_FROM(v, index) {
 		if (v->type == 0 || v->owner != _current_player)
 			continue;
 
@@ -113,25 +115,19 @@ static void AiStateVehLoop(Player *p)
 	p->ai.state_counter = 0;
 }
 
-// XXX
-static const byte _rail_locos_count[3] = {
-	27, 3, 5
-};
-extern const byte _rail_engines_start[3];
-
 static int AiChooseTrainToBuild(byte railtype, int32 money, byte flag)
 {
 	int best_veh_index = -1;
 	byte best_veh_score = 0;
 	int32 r;
+	int i;
 
-	int i = _rail_engines_start[railtype];
-	int end = i + _rail_locos_count[railtype];
-	Engine *e = &_engines[i];
-	do {
-		assert(!(_rail_vehicle_info[i].flags & RVI_WAGON));
+	for (i = 0; i < NUM_TRAIN_ENGINES; i++) {
+		const RailVehicleInfo *rvi = RailVehInfo(i);
+		Engine *e = DEREF_ENGINE(i);
 
-		if (!HASBIT(e->player_avail, _current_player) || e->reliability < 0x8A3D)
+		if (e->railtype != railtype || rvi->flags & RVI_WAGON
+		    || !HASBIT(e->player_avail, _current_player) || e->reliability < 0x8A3D)
 			continue;
 
 		r = DoCommandByTile(0, i, 0, 0, CMD_BUILD_RAIL_VEHICLE);
@@ -142,7 +138,7 @@ static int AiChooseTrainToBuild(byte railtype, int32 money, byte flag)
 			best_veh_score = _cmd_build_rail_veh_score;
 			best_veh_index = i;
 		}
-	} while (++e, ++i != end);
+	}
 
 	return best_veh_index;
 }
@@ -254,7 +250,7 @@ static int AiChooseShipToReplaceWith(Player *p, Vehicle *v)
 
 static void AiHandleGotoDepot(Player *p, int cmd)
 {
-	if ((p->ai.cur_veh->next_order & OT_MASK) != OT_GOTO_DEPOT)
+	if (p->ai.cur_veh->current_order.type != OT_GOTO_DEPOT)
 		DoCommandByTile(0, p->ai.cur_veh->index, 0, DC_EXEC, cmd);
 
 	if (++p->ai.state_counter <= 1387) {
@@ -262,21 +258,20 @@ static void AiHandleGotoDepot(Player *p, int cmd)
 		return;
 	}
 
-	if ((p->ai.cur_veh->next_order&OT_MASK) == OT_GOTO_DEPOT) {
-		p->ai.cur_veh->next_order = OT_DUMMY;
+	if (p->ai.cur_veh->current_order.type == OT_GOTO_DEPOT) {
+		p->ai.cur_veh->current_order.type = OT_DUMMY;
+		p->ai.cur_veh->current_order.flags = 0;
 		InvalidateWindow(WC_VEHICLE_VIEW, p->ai.cur_veh->index);
 	}
 }
 
 static void AiRestoreVehicleOrders(Vehicle *v, BackuppedOrders *bak)
 {
-	uint16 *os = bak->order, ord;
-	int ind = 0;
-	while ((ord = *os++) != 0) {
-		if (DoCommandByTile(0, v->index + (ind << 16), ord, DC_EXEC, CMD_INSERT_ORDER) == CMD_ERROR)
+	int i;
+
+	for (i = 0; bak->order[i].type != OT_NOTHING; i++)
+		if (!DoCommandP(0, v->index + (i << 16), PackOrder(&bak->order[i]), NULL, CMD_INSERT_ORDER | CMD_NO_TEST_IF_IN_NETWORK))
 			break;
-		ind++;
-	}
 }
 
 static void AiHandleReplaceTrain(Player *p)
@@ -300,7 +295,7 @@ static void AiHandleReplaceTrain(Player *p)
 		if (DoCommandByTile(0, v->index, 2, DC_EXEC, CMD_SELL_RAIL_WAGON) != CMD_ERROR &&
 			 DoCommandByTile(tile, veh, 0, DC_EXEC, CMD_BUILD_RAIL_VEHICLE) != CMD_ERROR) {
 			veh = _new_train_id;
-			AiRestoreVehicleOrders(&_vehicles[veh], orderbak);
+			AiRestoreVehicleOrders(GetVehicle(veh), orderbak);
 			DoCommandByTile(0, veh, 0, DC_EXEC, CMD_START_STOP_TRAIN);
 
 			DoCommandByTile(0, veh, _ai_service_interval, DC_EXEC, CMD_CHANGE_TRAIN_SERVICE_INT);
@@ -328,7 +323,7 @@ static void AiHandleReplaceRoadVeh(Player *p)
 		if (DoCommandByTile(0, v->index, 0, DC_EXEC, CMD_SELL_ROAD_VEH) != CMD_ERROR &&
 			 DoCommandByTile(tile, veh, 0, DC_EXEC, CMD_BUILD_ROAD_VEH) != CMD_ERROR) {
 			veh = _new_roadveh_id;
-			AiRestoreVehicleOrders(&_vehicles[veh], orderbak);
+			AiRestoreVehicleOrders(GetVehicle(veh), orderbak);
 			DoCommandByTile(0, veh, 0, DC_EXEC, CMD_START_STOP_ROADVEH);
 
 			DoCommandByTile(0, veh, _ai_service_interval, DC_EXEC, CMD_CHANGE_TRAIN_SERVICE_INT);
@@ -356,7 +351,7 @@ static void AiHandleReplaceAircraft(Player *p)
 		if (DoCommandByTile(0, v->index, 0, DC_EXEC, CMD_SELL_AIRCRAFT) != CMD_ERROR &&
 			 DoCommandByTile(tile, veh, 0, DC_EXEC, CMD_BUILD_AIRCRAFT) != CMD_ERROR) {
 			veh = _new_aircraft_id;
-			AiRestoreVehicleOrders(&_vehicles[veh], orderbak);
+			AiRestoreVehicleOrders(GetVehicle(veh), orderbak);
 			DoCommandByTile(0, veh, 0, DC_EXEC, CMD_START_STOP_AIRCRAFT);
 
 			DoCommandByTile(0, veh, _ai_service_interval, DC_EXEC, CMD_CHANGE_TRAIN_SERVICE_INT);
@@ -415,19 +410,21 @@ typedef struct FoundRoute {
 	void *to;
 } FoundRoute;
 
-static Town *AiFindRandomTown() {
-	Town *t = DEREF_TOWN(RandomRange(_total_towns));
+static Town *AiFindRandomTown(void)
+{
+	Town *t = GetTown(RandomRange(_total_towns));
 	return (t->xy != 0) ? t : NULL;
 }
 
-static Industry *AiFindRandomIndustry() {
-	Industry *i = DEREF_INDUSTRY(RandomRange(_total_industries));
+static Industry *AiFindRandomIndustry(void)
+{
+	Industry *i = GetIndustry(RandomRange(_total_industries));
 	return (i->xy != 0) ? i : NULL;
 }
 
 static void AiFindSubsidyIndustryRoute(FoundRoute *fr)
 {
-	int i;
+	uint i;
 	byte cargo;
 	Subsidy *s;
 	Industry *from, *to_ind;
@@ -450,16 +447,16 @@ static void AiFindSubsidyIndustryRoute(FoundRoute *fr)
 		return;
 	fr->cargo = cargo;
 
-	fr->from = from = DEREF_INDUSTRY(s->from);
+	fr->from = from = GetIndustry(s->from);
 
 	if (cargo == CT_GOODS || cargo == CT_FOOD) {
-		to_tow = DEREF_TOWN(s->to);
+		to_tow = GetTown(s->to);
 		if (to_tow->population < (uint32)(cargo == CT_FOOD ? 200 : 900))
 			return; // error
 		fr->to = to_tow;
 		to_xy = to_tow->xy;
 	} else {
-		to_ind = DEREF_INDUSTRY(s->to);
+		to_ind = GetIndustry(s->to);
 		fr->to = to_ind;
 		to_xy = to_ind->xy;
 	}
@@ -488,8 +485,8 @@ static void AiFindSubsidyPassengerRoute(FoundRoute *fr)
 		return;
 	fr->cargo = s->cargo_type;
 
-	fr->from = from = DEREF_TOWN(s->from);
-	fr->to = to = DEREF_TOWN(s->to);
+	fr->from = from = GetTown(s->from);
+	fr->to = to = GetTown(s->to);
 
 	// They must be big enough
 	if (from->population < 400 || to->population < 400)
@@ -624,17 +621,17 @@ static bool AiCheckIfRouteIsGood(Player *p, FoundRoute *fr, byte bitmask)
 
 static byte AiGetDirectionBetweenTiles(TileIndex a, TileIndex b)
 {
-	byte i = (GET_TILE_X(a) < GET_TILE_X(b)) ? 1 : 0;
-	if (GET_TILE_Y(a) >= GET_TILE_Y(b))	i ^= 3;
+	byte i = (TileX(a) < TileX(b)) ? 1 : 0;
+	if (TileY(a) >= TileY(b)) i ^= 3;
 	return i;
 }
 
 static TileIndex AiGetPctTileBetween(TileIndex a, TileIndex b, byte pct)
 {
 	return TILE_XY(
-			GET_TILE_X(a) + ((GET_TILE_X(b) - GET_TILE_X(a)) * pct >> 8),
-			GET_TILE_Y(a) + ((GET_TILE_Y(b) - GET_TILE_Y(a)) * pct >> 8)
-		);
+		TileX(a) + ((TileX(b) - TileX(a)) * pct >> 8),
+		TileY(a) + ((TileY(b) - TileY(a)) * pct >> 8)
+	);
 }
 
 static void AiWantLongIndustryRoute(Player *p)
@@ -1419,7 +1416,7 @@ static void AiWantOilRigAircraftRoute(Player *p)
 		t = AiFindRandomTown();
 		if (t != NULL) {
 			// Find a random oil rig industry
-			in = DEREF_INDUSTRY(RandomRange(lengthof(_industries)));
+			in = GetIndustry(RandomRange(_total_industries));
 			if (in != NULL && in->type == IT_OIL_RIG) {
 				if (GetTileDist(t->xy, in->xy) < 60)
 					break;
@@ -1534,19 +1531,27 @@ static bool AiCheckTrackResources(TileIndex tile, const AiDefaultBlockData *p, b
 	uint values[NUM_CARGO];
 	int w,h;
 	uint tile2;
+	int rad;
 
 	for(;p->mode != 4;p++) if (p->mode == 1) {
-		tile2 = TILE_ADD(tile, p->tileoffs);
+		tile2 = TILE_ADD(tile, ToTileIndexDiff(p->tileoffs));
 
 		w = ((p->attr>>1) & 7);
 		h = ((p->attr>>4) & 7);
 		if (p->attr&1) intswap(w, h);
 
+
+		if (_patches.modified_catchment) {
+			rad = CA_TRAIN;
+		} else {
+			rad = 4;
+		}
+
 		if (cargo & 0x80) {
-			GetProductionAroundTiles(values, tile2, w, h);
+			GetProductionAroundTiles(values, tile2, w, h, rad);
 			return values[cargo & 0x7F] != 0;
 		} else {
-			GetAcceptanceAroundTiles(values, tile2, w, h);
+			GetAcceptanceAroundTiles(values, tile2, w, h, rad);
 			if (!(values[cargo] & ~7))
 				return false;
 			if (cargo != CT_MAIL)
@@ -1568,7 +1573,7 @@ static int32 AiDoBuildDefaultRailTrack(TileIndex tile, const AiDefaultBlockData
 
 	for(;;) {
 		// This will seldomly overflow for valid reasons. Mask it to be on the safe side.
-		uint c = TILE_MASK(tile + p->tileoffs);
+		uint c = TILE_MASK(tile + ToTileIndexDiff(p->tileoffs));
 
 		_cleared_town = NULL;
 
@@ -1593,7 +1598,7 @@ clear_town_stuff:;
 			}
 		} else if (p->mode == 2) {
 			// Rail
-			if (IS_TILETYPE(c, MP_RAILWAY))
+			if (IsTileType(c, MP_RAILWAY))
 				return CMD_ERROR;
 
 			j = p->attr;
@@ -1612,7 +1617,7 @@ clear_town_stuff:;
 			/* signals too? */
 			if (j&3) {
 				// Can't build signals on a road.
-				if (IS_TILETYPE(c, MP_STREET))
+				if (IsTileType(c, MP_STREET))
 					return CMD_ERROR;
 
 				if (flag & DC_EXEC) {
@@ -1711,7 +1716,7 @@ static void AiDoTerraformLand(TileIndex tile, int dir, int unk, int mode)
 	unk &= (int)r;
 
 	do {
-		tile = TILE_MASK(tile + _tileoffs_by_dir[dir]);
+		tile = TILE_MASK(tile + TileOffsByDir(dir));
 
 		r >>= 2;
 		if (r&2) {
@@ -1817,7 +1822,7 @@ static TileIndex AiGetEdgeOfDefaultRailBlock(byte rule, TileIndex tile, byte cmd
 
 	while (p->mode != 3 || !((--cmd) & 0x80)) p++;
 
-	return tile + p->tileoffs - _tileoffs_by_dir[*dir = p->attr];
+	return tile + ToTileIndexDiff(p->tileoffs) - TileOffsByDir(*dir = p->attr);
 }
 
 typedef struct AiRailPathFindData {
@@ -1850,7 +1855,7 @@ static bool AiDoFollowTrack(Player *p)
 	arpfd.tile2 = p->ai.cur_tile_a;
 	arpfd.flag = false;
 	arpfd.count = 0;
-	FollowTrack(p->ai.cur_tile_a + _tileoffs_by_dir[p->ai.cur_dir_a], 0x2000 | TRANSPORT_RAIL, p->ai.cur_dir_a^2,
+	FollowTrack(p->ai.cur_tile_a + TileOffsByDir(p->ai.cur_dir_a), 0x2000 | TRANSPORT_RAIL, p->ai.cur_dir_a^2,
 		(TPFEnumProc*)AiEnumFollowTrack, NULL, &arpfd);
 	return arpfd.count > 8;
 }
@@ -1942,7 +1947,7 @@ static bool AiCheckRailPathBetter(AiRailFinder *arf, const byte *p)
 	return better;
 }
 
-static void FORCEINLINE AiCheckBuildRailBridgeHere(AiRailFinder *arf, TileIndex tile, const byte *p)
+static inline void AiCheckBuildRailBridgeHere(AiRailFinder *arf, TileIndex tile, const byte *p)
 {
 	TileIndex tile_new;
 	bool flag;
@@ -1956,8 +1961,8 @@ static void FORCEINLINE AiCheckBuildRailBridgeHere(AiRailFinder *arf, TileIndex
 		// Allow bridges directly over bottom tiles
 		flag = arf->ti.z == 0;
 		for(;;) {
-			if (tile_new < -_tileoffs_by_dir[dir2]) return; // Wraping around map, no bridge possible!
-			tile_new = TILE_MASK(tile_new + _tileoffs_by_dir[dir2]);
+			if (tile_new < -TileOffsByDir(dir2)) return; // Wraping around map, no bridge possible!
+			tile_new = TILE_MASK(tile_new + TileOffsByDir(dir2));
 			FindLandscapeHeightByTile(&arf->ti, tile_new);
 			if (arf->ti.tileh != 0 || arf->ti.type == MP_CLEAR || arf->ti.type == MP_TREES) {
 				if (!flag) return;
@@ -1982,7 +1987,7 @@ static void FORCEINLINE AiCheckBuildRailBridgeHere(AiRailFinder *arf, TileIndex
 	}
 }
 
-static void FORCEINLINE AiCheckBuildRailTunnelHere(AiRailFinder *arf, TileIndex tile, const byte *p)
+static inline void AiCheckBuildRailTunnelHere(AiRailFinder *arf, TileIndex tile, const byte *p)
 {
 	FindLandscapeHeightByTile(&arf->ti, tile);
 
@@ -2003,7 +2008,7 @@ static void AiBuildRailRecursive(AiRailFinder *arf, TileIndex tile, int dir)
 {
 	const byte *p;
 
-	tile = TILE_MASK(tile + _tileoffs_by_dir[dir]);
+	tile = TILE_MASK(tile + TileOffsByDir(dir));
 
 	// Reached destination?
 	if (tile == arf->final_tile) {
@@ -2121,7 +2126,7 @@ static void AiBuildRailConstruct(Player *p)
 		return;
 	}
 
-	p->ai.cur_tile_a += _tileoffs_by_dir[p->ai.cur_dir_a];
+	p->ai.cur_tile_a += TileOffsByDir(p->ai.cur_dir_a);
 
 	if (arf.best_ptr[0]&0x80) {
 		int i;
@@ -2173,12 +2178,12 @@ static bool AiRemoveTileAndGoForward(Player *p)
 	int offs;
 	TileIndex tilenew;
 
-	if (IS_TILETYPE(tile, MP_TUNNELBRIDGE)) {
+	if (IsTileType(tile, MP_TUNNELBRIDGE)) {
 		if (!(_map5[tile] & 0x80)) {
 			// Clear the tunnel and continue at the other side of it.
 			if (DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR) == CMD_ERROR)
 				return false;
-			p->ai.cur_tile_a = TILE_MASK(_build_tunnel_endtile - _tileoffs_by_dir[p->ai.cur_dir_a]);
+			p->ai.cur_tile_a = TILE_MASK(_build_tunnel_endtile - TileOffsByDir(p->ai.cur_dir_a));
 			return true;
 		}
 
@@ -2190,7 +2195,7 @@ static bool AiRemoveTileAndGoForward(Player *p)
 				return false;
 
 			// Find other side of bridge.
-			offs = _tileoffs_by_dir[p->ai.cur_dir_a];
+			offs = TileOffsByDir(p->ai.cur_dir_a);
 			do {
 				tile = TILE_MASK(tile - offs);
 			} while (_map5[tile] & 0x40);
@@ -2213,7 +2218,7 @@ static bool AiRemoveTileAndGoForward(Player *p)
 	bit = FindFirstBit(b);
 
 	// Then remove and signals if there are any.
-	if (IS_TILETYPE(tile, MP_RAILWAY) &&
+	if (IsTileType(tile, MP_RAILWAY) &&
 			(_map5[tile]&0xC0) == 0x40) {
 		DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_REMOVE_SIGNALS);
 	}
@@ -2228,7 +2233,7 @@ static bool AiRemoveTileAndGoForward(Player *p)
 	p->ai.cur_dir_a = ptr[1] ^ 2;
 
 	// And then also switch tile.
-	p->ai.cur_tile_a = TILE_MASK(p->ai.cur_tile_a - _tileoffs_by_dir[p->ai.cur_dir_a]);
+	p->ai.cur_tile_a = TILE_MASK(p->ai.cur_tile_a - TileOffsByDir(p->ai.cur_dir_a));
 
 	return true;
 }
@@ -2327,7 +2332,7 @@ static void AiStateBuildRail(Player *p)
 	p->ai.cur_tile_a = tile;
 	p->ai.start_dir_a = dir;
 	p->ai.cur_dir_a = dir;
-	DoCommandByTile(TILE_MASK(tile + _tileoffs_by_dir[dir]), 0, (dir&1)?1:0, DC_EXEC, CMD_REMOVE_SINGLE_RAIL);
+	DoCommandByTile(TILE_MASK(tile + TileOffsByDir(dir)), 0, (dir&1)?1:0, DC_EXEC, CMD_REMOVE_SINGLE_RAIL);
 
 	assert(TILE_MASK(tile) != 0xFF00);
 
@@ -2338,7 +2343,7 @@ static void AiStateBuildRail(Player *p)
 	p->ai.cur_tile_b = tile;
 	p->ai.start_dir_b = dir;
 	p->ai.cur_dir_b = dir;
-	DoCommandByTile(TILE_MASK(tile + _tileoffs_by_dir[dir]), 0, (dir&1)?1:0, DC_EXEC, CMD_REMOVE_SINGLE_RAIL);
+	DoCommandByTile(TILE_MASK(tile + TileOffsByDir(dir)), 0, (dir&1)?1:0, DC_EXEC, CMD_REMOVE_SINGLE_RAIL);
 
 	assert(TILE_MASK(tile) != 0xFF00);
 
@@ -2352,7 +2357,7 @@ static int AiGetStationIdByDef(TileIndex tile, int id)
 {
 	const AiDefaultBlockData *p = _default_rail_track_data[id]->data;
 	while (p->mode != 1) p++;
-	return _map2[TILE_ADD(tile,p->tileoffs)];
+	return _map2[TILE_ADD(tile, ToTileIndexDiff(p->tileoffs))];
 }
 
 static void AiStateBuildRailVeh(Player *p)
@@ -2368,7 +2373,7 @@ static void AiStateBuildRailVeh(Player *p)
 	ptr = _default_rail_track_data[p->ai.src.cur_building_rule]->data;
 	while (ptr->mode != 0) {	ptr++; }
 
-	tile = TILE_ADD(p->ai.src.use_tile, ptr->tileoffs);
+	tile = TILE_ADD(p->ai.src.use_tile, ToTileIndexDiff(ptr->tileoffs));
 
 	cargo = p->ai.cargo_type;
 	for(i=0;;) {
@@ -2407,7 +2412,7 @@ handle_nocash:
 	loco_id = _new_train_id;
 
 	// Sell a vehicle if the train is double headed.
-	v = &_vehicles[loco_id];
+	v = GetVehicle(loco_id);
 	if (v->next != NULL) {
 		i = p->ai.wagon_list[p->ai.num_wagons*2-2];
 		p->ai.wagon_list[p->ai.num_wagons*2-2] = INVALID_VEHICLE;
@@ -2421,15 +2426,20 @@ handle_nocash:
 
 	for(i=0; p->ai.order_list_blocks[i] != 0xFF; i++) {
 		AiBuildRec *aib = (&p->ai.src) + p->ai.order_list_blocks[i];
-		uint flags = (AiGetStationIdByDef(aib->use_tile, aib->cur_building_rule) << 8) + OT_GOTO_STATION;
 		bool is_pass = (p->ai.cargo_type == CT_PASSENGERS ||
 							p->ai.cargo_type == CT_MAIL ||
 							(_opt.landscape==LT_NORMAL && p->ai.cargo_type == CT_VALUABLES));
+		Order order;
 
-		if (!is_pass && i == 1) flags |= OF_UNLOAD;
-		if (p->ai.num_want_fullload != 0 && (is_pass || i == 0)) flags |= OF_FULL_LOAD;
+		order.type = OT_GOTO_STATION;
+		order.flags = 0;
+		order.station = AiGetStationIdByDef(aib->use_tile, aib->cur_building_rule);
 
-		DoCommandByTile(0, loco_id + (i << 16),	flags, DC_EXEC, CMD_INSERT_ORDER);
+		if (!is_pass && i == 1) order.flags |= OF_UNLOAD;
+		if (p->ai.num_want_fullload != 0 && (is_pass || i == 0))
+			order.flags |= OF_FULL_LOAD;
+
+		DoCommandByTile(0, loco_id + (i << 16),	PackOrder(&order), DC_EXEC, CMD_INSERT_ORDER);
 	}
 
 	DoCommandByTile(0, loco_id, 0, DC_EXEC, CMD_START_STOP_TRAIN);
@@ -2459,7 +2469,7 @@ static void AiStateDeleteRailBlocks(Player *p)
 		if (aib->cur_building_rule != 255) {
 			b = _default_rail_track_data[aib->cur_building_rule]->data;
 			while (b->mode != 4) {
-				DoCommandByTile(TILE_ADD(aib->use_tile, b->tileoffs), 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
+				DoCommandByTile(TILE_ADD(aib->use_tile, ToTileIndexDiff(b->tileoffs)), 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
 				b++;
 			}
 		}
@@ -2471,16 +2481,24 @@ static void AiStateDeleteRailBlocks(Player *p)
 static bool AiCheckRoadResources(TileIndex tile, const AiDefaultBlockData *p, byte cargo)
 {
 	uint values[NUM_CARGO];
+	int rad;
+
+	if (_patches.modified_catchment) {
+		rad = CA_TRUCK;		//Same as CA_BUS at the moment?
+	} else {			//change that at some point?
+		rad = 4;
+	}
+
 	for(;;p++) {
 		if (p->mode == 4) {
 			return true;
 		} else if (p->mode == 1) {
-			uint tile2 = TILE_ADD(tile, p->tileoffs);
+			uint tile2 = TILE_ADD(tile, ToTileIndexDiff(p->tileoffs));
 			if (cargo & 0x80) {
-				GetProductionAroundTiles(values, tile2, 1, 1);
+				GetProductionAroundTiles(values, tile2, 1, 1, rad);
 				return values[cargo & 0x7F] != 0;
 			} else {
-				GetAcceptanceAroundTiles(values, tile2, 1, 1);
+				GetAcceptanceAroundTiles(values, tile2, 1, 1, rad);
 				return (values[cargo]&~7) != 0;
 			}
 		}
@@ -2518,12 +2536,12 @@ static int32 AiDoBuildDefaultRoadBlock(TileIndex tile, const AiDefaultBlockData
 	int roadflag = 0;
 
 	for(;p->mode != 4;p++) {
-		uint c = TILE_MASK(tile+ p->tileoffs);
+		uint c = TILE_MASK(tile + ToTileIndexDiff(p->tileoffs));
 
 		_cleared_town = NULL;
 
 		if (p->mode == 2) {
-			if (IS_TILETYPE(c, MP_STREET) &&
+			if (IsTileType(c, MP_STREET) &&
 					(_map5[c]&0xF0)==0 &&
 					(_map5[c]&p->attr)!=0) {
 				roadflag |= 2;
@@ -2571,7 +2589,7 @@ clear_town_stuff:;
 			if (GetTileSlope(c, NULL) != 0)
 				return CMD_ERROR;
 
-			if (!(IS_TILETYPE(c, MP_STREET) && (_map5[c]&0xF0)==0)) {
+			if (!(IsTileType(c, MP_STREET) && (_map5[c] & 0xF0) == 0)) {
 				r = DoCommandByTile(c, 0, 0, flag | DC_AUTO | DC_NO_WATER | DC_AI_BUILDING, CMD_LANDSCAPE_CLEAR);
 				if (r == CMD_ERROR) return CMD_ERROR;
 			}
@@ -2753,8 +2771,8 @@ static bool AiEnumFollowRoad(uint tile, AiRoadEnum *a, int track, uint length, b
 	uint tile2;
 
 	if (dist <= a->best_dist) {
-		tile2 = TILE_MASK(tile + _tileoffs_by_dir[_dir_by_track[track]]);
-		if (IS_TILETYPE(tile2, MP_STREET) &&
+		tile2 = TILE_MASK(tile + TileOffsByDir(_dir_by_track[track]));
+		if (IsTileType(tile2, MP_STREET) &&
 				(_map5[tile2]&0xF0) == 0) {
 			a->best_dist = dist;
 			a->best_tile = tile;
@@ -2772,7 +2790,7 @@ static const uint16 _ai_road_table_and[4] = {
 	0x2A00,
 };
 
-bool AiCheckRoadFinished(Player *p)
+static bool AiCheckRoadFinished(Player *p)
 {
 	AiRoadEnum are;
 	uint tile;
@@ -2781,7 +2799,7 @@ bool AiCheckRoadFinished(Player *p)
 	int i;
 
 	are.dest = p->ai.cur_tile_b;
-	tile = TILE_MASK(p->ai.cur_tile_a + _tileoffs_by_dir[dir]);
+	tile = TILE_MASK(p->ai.cur_tile_a + TileOffsByDir(dir));
 
 	bits = GetTileTrackStatus(tile, TRANSPORT_ROAD) & _ai_road_table_and[dir];
 	if (bits == 0) {
@@ -2819,7 +2837,7 @@ static bool AiBuildRoadHelper(uint tile, int flags, int type)
 	return DoCommandByTile(tile, _road_bits[type], 0, flags, CMD_BUILD_ROAD) != CMD_ERROR;
 }
 
-static void FORCEINLINE AiCheckBuildRoadBridgeHere(AiRoadFinder *arf, TileIndex tile, const byte *p)
+static inline void AiCheckBuildRoadBridgeHere(AiRoadFinder *arf, TileIndex tile, const byte *p)
 {
 	TileIndex tile_new;
 	bool flag;
@@ -2832,8 +2850,8 @@ static void FORCEINLINE AiCheckBuildRoadBridgeHere(AiRoadFinder *arf, TileIndex
 		// Allow bridges directly over bottom tiles
 		flag = arf->ti.z == 0;
 		for(;;) {
-			if (tile_new < -_tileoffs_by_dir[dir2]) return; // Wraping around map, no bridge possible!
-			tile_new = TILE_MASK(tile_new + _tileoffs_by_dir[dir2]);
+			if (tile_new < -TileOffsByDir(dir2)) return; // Wraping around map, no bridge possible!
+			tile_new = TILE_MASK(tile_new + TileOffsByDir(dir2));
 			FindLandscapeHeightByTile(&arf->ti, tile_new);
 			if (arf->ti.tileh != 0 || arf->ti.type == MP_CLEAR || arf->ti.type == MP_TREES) {
 				// Allow a bridge if either we have a tile that's water, rail or street,
@@ -2859,7 +2877,7 @@ static void FORCEINLINE AiCheckBuildRoadBridgeHere(AiRoadFinder *arf, TileIndex
 	}
 }
 
-static void FORCEINLINE AiCheckBuildRoadTunnelHere(AiRoadFinder *arf, TileIndex tile, const byte *p)
+static inline void AiCheckBuildRoadTunnelHere(AiRoadFinder *arf, TileIndex tile, const byte *p)
 {
 	FindLandscapeHeightByTile(&arf->ti, tile);
 
@@ -2881,7 +2899,7 @@ static void AiBuildRoadRecursive(AiRoadFinder *arf, TileIndex tile, int dir)
 {
 	const byte *p;
 
-	tile = TILE_MASK(tile + _tileoffs_by_dir[dir]);
+	tile = TILE_MASK(tile + TileOffsByDir(dir));
 
 	// Reached destination?
 	if (tile == arf->final_tile) {
@@ -2983,14 +3001,14 @@ do_some_terraform:
 		if (++p->ai.state_counter == 21) {
 			p->ai.state_mode = 1;
 
-			p->ai.cur_tile_a = TILE_MASK(p->ai.cur_tile_a + _tileoffs_by_dir[p->ai.cur_dir_a]);
+			p->ai.cur_tile_a = TILE_MASK(p->ai.cur_tile_a + TileOffsByDir(p->ai.cur_dir_a));
 			p->ai.cur_dir_a ^= 2;
 			p->ai.state_counter = 0;
 		}
 		return;
 	}
 
-	tile = TILE_MASK(p->ai.cur_tile_a + _tileoffs_by_dir[p->ai.cur_dir_a]);
+	tile = TILE_MASK(p->ai.cur_tile_a + TileOffsByDir(p->ai.cur_dir_a));
 
 	if (arf.best_ptr[0]&0x80) {
 		int i;
@@ -3067,7 +3085,7 @@ static TileIndex AiGetRoadBlockEdge(byte rule, TileIndex tile, int *dir)
 	const AiDefaultBlockData *p = _road_default_block_data[rule]->data;
 	while (p->mode != 1) p++;
 	*dir = p->attr;
-	return TILE_ADD(tile, p->tileoffs);
+	return TILE_ADD(tile, ToTileIndexDiff(p->tileoffs));
 }
 
 
@@ -3140,7 +3158,7 @@ static int AiGetStationIdFromRoadBlock(TileIndex tile, int id)
 {
 	const AiDefaultBlockData *p = _road_default_block_data[id]->data;
 	while (p->mode != 1) p++;
-	return _map2[TILE_ADD(tile, p->tileoffs)];
+	return _map2[TILE_ADD(tile, ToTileIndexDiff(p->tileoffs))];
 }
 
 static void AiStateBuildRoadVehicles(Player *p)
@@ -3152,7 +3170,7 @@ static void AiStateBuildRoadVehicles(Player *p)
 
 	ptr = _road_default_block_data[p->ai.src.cur_building_rule]->data;
 	for(;ptr->mode != 0;ptr++) {}
-	tile = TILE_ADD(p->ai.src.use_tile, ptr->tileoffs);
+	tile = TILE_ADD(p->ai.src.use_tile, ToTileIndexDiff(ptr->tileoffs));
 
 	veh = AiChooseRoadVehToBuild(p->ai.cargo_type, p->player_money);
 	if (veh == -1) {
@@ -3168,15 +3186,20 @@ static void AiStateBuildRoadVehicles(Player *p)
 
 	for(i=0; p->ai.order_list_blocks[i] != 0xFF; i++) {
 		AiBuildRec *aib = (&p->ai.src) + p->ai.order_list_blocks[i];
-		uint flags  = (AiGetStationIdFromRoadBlock(aib->use_tile, aib->cur_building_rule) << 8) + OT_GOTO_STATION;
 		bool is_pass = (p->ai.cargo_type == CT_PASSENGERS ||
 							p->ai.cargo_type == CT_MAIL ||
 							(_opt.landscape==LT_NORMAL && p->ai.cargo_type == CT_VALUABLES));
+		Order order;
 
-		if (!is_pass && i == 1) flags |= OF_UNLOAD;
-		if (p->ai.num_want_fullload != 0 && (is_pass || i == 0)) flags |= OF_FULL_LOAD;
+		order.type = OT_GOTO_STATION;
+		order.flags = 0;
+		order.station = AiGetStationIdFromRoadBlock(aib->use_tile, aib->cur_building_rule);
 
-		DoCommandByTile(0, loco_id + (i << 16),	flags, DC_EXEC, CMD_INSERT_ORDER);
+		if (!is_pass && i == 1) order.flags |= OF_UNLOAD;
+		if (p->ai.num_want_fullload != 0 && (is_pass || i == 0))
+			order.flags |= OF_FULL_LOAD;
+
+		DoCommandByTile(0, loco_id + (i << 16),	PackOrder(&order), DC_EXEC, CMD_INSERT_ORDER);
 	}
 
 	DoCommandByTile(0, loco_id, 0, DC_EXEC, CMD_START_STOP_ROADVEH);
@@ -3204,7 +3227,7 @@ static void AiStateDeleteRoadBlocks(Player *p)
 			b = _road_default_block_data[aib->cur_building_rule]->data;
 			while (b->mode != 4) {
 				if (b->mode <= 1) {
-					DoCommandByTile(TILE_ADD(aib->use_tile, b->tileoffs), 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
+					DoCommandByTile(TILE_ADD(aib->use_tile, ToTileIndexDiff(b->tileoffs)), 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
 				}
 				b++;
 			}
@@ -3316,7 +3339,7 @@ static int32 AiDoBuildDefaultAirportBlock(TileIndex tile, const AiDefaultBlockDa
 	for(;p->mode == 0;p++) {
 		if (!HASBIT(_avail_aircraft, p->attr))
 			return CMD_ERROR;
-		r = DoCommandByTile(TILE_MASK(tile + p->tileoffs), p->attr,0,flag | DC_AUTO | DC_NO_WATER,CMD_BUILD_AIRPORT);
+		r = DoCommandByTile(TILE_MASK(tile + ToTileIndexDiff(p->tileoffs)), p->attr,0,flag | DC_AUTO | DC_NO_WATER,CMD_BUILD_AIRPORT);
 		if (r == CMD_ERROR)
 			return CMD_ERROR;
 		total_cost += r;
@@ -3330,16 +3353,23 @@ static bool AiCheckAirportResources(TileIndex tile, const AiDefaultBlockData *p,
 	uint values[NUM_CARGO];
 	int w,h;
 	uint tile2;
+	int rad;
+
+	if (_patches.modified_catchment) {
+		rad = CA_AIR_LARGE;		//I Have NFI what airport the
+	} else {				//AI is going to build here
+		rad = 4;
+	}
 
 	for(;p->mode==0;p++) {
-		tile2 = TILE_ADD(tile, p->tileoffs);
+		tile2 = TILE_ADD(tile, ToTileIndexDiff(p->tileoffs));
 		w = _airport_size_x[p->attr];
 		h = _airport_size_y[p->attr];
 		if (cargo & 0x80) {
-			GetProductionAroundTiles(values, tile2, w, h);
+			GetProductionAroundTiles(values, tile2, w, h, rad);
 			return values[cargo & 0x7F] != 0;
 		} else {
-			GetAcceptanceAroundTiles(values, tile2, w, h);
+			GetAcceptanceAroundTiles(values, tile2, w, h, rad);
 			return values[cargo] >= 8;
 		}
 	}
@@ -3395,9 +3425,9 @@ static void AiStateBuildDefaultAirportBlocks(Player *p)
 			rule = AiFindBestDefaultAirportBlock(aib->use_tile, aib->cargo, p->ai.build_kind, &cost);
 
 #if 0
-			if (!IS_TILETYPE(aib->use_tile, MP_STREET) &&
-					!IS_TILETYPE(aib->use_tile, MP_RAILWAY) &&
-					!IS_TILETYPE(aib->use_tile, MP_STATION)
+			if (!IsTileType(aib->use_tile, MP_STREET) &&
+					!IsTileType(aib->use_tile, MP_RAILWAY) &&
+					!IsTileType(aib->use_tile, MP_STATION)
 					) {
 
 				_map_type_and_height[aib->use_tile] = 0xa1;
@@ -3449,7 +3479,7 @@ static int AiGetStationIdFromAircraftBlock(TileIndex tile, int id)
 {
 	const AiDefaultBlockData *p = _airport_default_block_data[id];
 	while (p->mode != 1) p++;
-	return _map2[TILE_ADD(tile, p->tileoffs)];
+	return _map2[TILE_ADD(tile, ToTileIndexDiff(p->tileoffs))];
 }
 
 static void AiStateBuildAircraftVehicles(Player *p)
@@ -3464,7 +3494,7 @@ static void AiStateBuildAircraftVehicles(Player *p)
 	ptr = _airport_default_block_data[p->ai.src.cur_building_rule];
 	for(;ptr->mode!=0;ptr++) {}
 
-	tile = TILE_ADD(p->ai.src.use_tile, ptr->tileoffs);
+	tile = TILE_ADD(p->ai.src.use_tile, ToTileIndexDiff(ptr->tileoffs));
 
 	veh = AiChooseAircraftToBuild(p->player_money, p->ai.build_kind!=0 ? 1 : 0);
 	if (veh == -1) {
@@ -3478,13 +3508,18 @@ static void AiStateBuildAircraftVehicles(Player *p)
 
 	for(i=0; p->ai.order_list_blocks[i] != 0xFF; i++) {
 		AiBuildRec *aib = (&p->ai.src) + p->ai.order_list_blocks[i];
-		uint flags = (AiGetStationIdFromAircraftBlock(aib->use_tile, aib->cur_building_rule) << 8) + OT_GOTO_STATION;
 		bool is_pass = (p->ai.cargo_type == CT_PASSENGERS || p->ai.cargo_type == CT_MAIL);
+		Order order;
 
-		if (!is_pass && i == 1) flags |= OF_UNLOAD;
-		if (p->ai.num_want_fullload != 0 && (is_pass || i == 0)) flags |= OF_FULL_LOAD;
+		order.type = OT_GOTO_STATION;
+		order.flags = 0;
+		order.station = AiGetStationIdFromAircraftBlock(aib->use_tile, aib->cur_building_rule);
 
-		DoCommandByTile(0, loco_id + (i << 16), flags, DC_EXEC, CMD_INSERT_ORDER);
+		if (!is_pass && i == 1) order.flags |= OF_UNLOAD;
+		if (p->ai.num_want_fullload != 0 && (is_pass || i == 0))
+			order.flags |= OF_FULL_LOAD;
+
+		DoCommandByTile(0, loco_id + (i << 16), PackOrder(&order), DC_EXEC, CMD_INSERT_ORDER);
 	}
 
 	DoCommandByTile(0, loco_id, 0, DC_EXEC, CMD_START_STOP_AIRCRAFT);
@@ -3525,7 +3560,7 @@ static void AiStateSellVeh(Player *p)
 		if (v->type == VEH_Train) {
 
 			if (!IsTrainDepotTile(v->tile) || v->u.rail.track != 0x80 || !(v->vehstatus&VS_STOPPED)) {
-				if ((v->next_order & OT_MASK) != OT_GOTO_DEPOT)
+				if (v->current_order.type != OT_GOTO_DEPOT)
 					DoCommandByTile(0, v->index, 0, DC_EXEC, CMD_TRAIN_GOTO_DEPOT);
 				goto going_to_depot;
 			}
@@ -3535,7 +3570,7 @@ static void AiStateSellVeh(Player *p)
 
 		} else if (v->type == VEH_Road) {
 			if (!IsRoadDepotTile(v->tile) || v->u.road.state != 254 || !(v->vehstatus&VS_STOPPED)) {
-				if ((v->next_order & OT_MASK) != OT_GOTO_DEPOT)
+				if (v->current_order.type != OT_GOTO_DEPOT)
 					DoCommandByTile(0, v->index, 0, DC_EXEC, CMD_SEND_ROADVEH_TO_DEPOT);
 				goto going_to_depot;
 			}
@@ -3543,7 +3578,7 @@ static void AiStateSellVeh(Player *p)
 			DoCommandByTile(0, v->index, 0, DC_EXEC, CMD_SELL_ROAD_VEH);
 		} else if (v->type == VEH_Aircraft) {
 			if (!IsAircraftHangarTile(v->tile) && !(v->vehstatus&VS_STOPPED)) {
-				if ((v->next_order & OT_MASK) != OT_GOTO_DEPOT)
+				if (v->current_order.type != OT_GOTO_DEPOT)
 					DoCommandByTile(0, v->index, 0, DC_EXEC, CMD_SEND_AIRCRAFT_TO_HANGAR);
 				goto going_to_depot;
 			}
@@ -3560,8 +3595,9 @@ going_to_depot:;
 	if (++p->ai.state_counter <= 832)
 		return;
 
-	if ((v->next_order&OT_MASK) == OT_GOTO_DEPOT) {
-		v->next_order = OT_DUMMY;
+	if (v->current_order.type == OT_GOTO_DEPOT) {
+		v->current_order.type = OT_DUMMY;
+		v->current_order.flags = 0;
 		InvalidateWindow(WC_VEHICLE_VIEW, v->index);
 	}
 return_to_loop:;
@@ -3572,7 +3608,7 @@ static void AiStateRemoveStation(Player *p)
 {
 	// Remove stations that aren't in use by any vehicle
 	byte in_use[256], *used;
-	uint16 *ord;
+	const Order *ord;
 	Station *st;
 	uint tile;
 
@@ -3581,9 +3617,9 @@ static void AiStateRemoveStation(Player *p)
 
 	// Get a list of all stations that are in use by a vehicle
 	memset(in_use, 0, sizeof(in_use));
-	for(ord=_order_array; ord != _ptr_to_next_order; ord++) {
-		if ((*ord & OT_MASK) == OT_GOTO_STATION)
-			in_use[*ord >> 8] = 1;
+	FOR_ALL_ORDERS(ord) {
+		if (ord->type == OT_GOTO_STATION)
+			in_use[ord->station] = 1;
 	}
 
 	// Go through all stations and delete those that aren't in use
@@ -3606,7 +3642,7 @@ static void AiRemovePlayerRailOrRoad(Player *p, uint tile)
 {
 	byte m5;
 
-	if (IS_TILETYPE(tile, MP_RAILWAY)) {
+	if (IsTileType(tile, MP_RAILWAY)) {
 		if (_map_owner[tile] != _current_player)
 			return;
 		m5 = _map5[tile];
@@ -3660,12 +3696,12 @@ pos_3:
 			static const byte _depot_bits[] = {0x19,0x16,0x25,0x2A};
 
 			m5 &= 3;
-			if (GetRailTrackStatus(tile + _tileoffs_by_dir[m5]) & _depot_bits[m5])
+			if (GetRailTrackStatus(tile + TileOffsByDir(m5)) & _depot_bits[m5])
 				return;
 
 			DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
 		}
-	} else if (IS_TILETYPE(tile, MP_STREET)) {
+	} else if (IsTileType(tile, MP_STREET)) {
 		if (_map_owner[tile] != _current_player)
 			return;
 
@@ -3676,19 +3712,19 @@ pos_3:
 			int dir;
 
 			// Check if there are any stations around.
-			if (IS_TILETYPE(tile + TILE_XY(-1,0), MP_STATION) &&
+			if (IsTileType(tile + TILE_XY(-1,0), MP_STATION) &&
 					_map_owner[tile + TILE_XY(-1,0)] == _current_player)
 						return;
 
-			if (IS_TILETYPE(tile + TILE_XY(1,0), MP_STATION) &&
+			if (IsTileType(tile + TILE_XY(1,0), MP_STATION) &&
 					_map_owner[tile + TILE_XY(1,0)] == _current_player)
 						return;
 
-			if (IS_TILETYPE(tile + TILE_XY(0,-1), MP_STATION) &&
+			if (IsTileType(tile + TILE_XY(0,-1), MP_STATION) &&
 					_map_owner[tile + TILE_XY(0,-1)] == _current_player)
 						return;
 
-			if (IS_TILETYPE(tile + TILE_XY(0,1), MP_STATION) &&
+			if (IsTileType(tile + TILE_XY(0,1), MP_STATION) &&
 					_map_owner[tile + TILE_XY(0,1)] == _current_player)
 						return;
 
@@ -3696,13 +3732,13 @@ pos_3:
 
 			DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
 			DoCommandByTile(
-				TILE_MASK(tile + _tileoffs_by_dir[dir]),
+				TILE_MASK(tile + TileOffsByDir(dir)),
 				8 >> (dir ^ 2),
 				0,
 				DC_EXEC,
 				CMD_REMOVE_ROAD);
 		}
-	} else if (IS_TILETYPE(tile, MP_TUNNELBRIDGE)) {
+	} else if (IsTileType(tile, MP_TUNNELBRIDGE)) {
 		byte b;
 
 		if (_map_owner[tile] != _current_player || (_map5[tile] & 0xC6) != 0x80)
