@@ -1,5 +1,7 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
+#include "map.h"
 #include "viewport.h"
 #include "command.h"
 
@@ -28,7 +30,7 @@ static int TerraformAllowTileProcess(TerraformerState *ts, TileIndex tile)
 	TileIndex *t;
 	int count;
 
-	if ((GET_TILE_X(tile) == TILE_X_MAX) || (GET_TILE_Y(tile) == TILE_Y_MAX))
+	if (TileX(tile) == MapMaxX() || TileY(tile) == MapMaxY())
 		return -1;
 
 	t = ts->tile_table;
@@ -50,7 +52,7 @@ static int TerraformGetHeightOfTile(TerraformerState *ts, TileIndex tile)
 			return mod->height;
 	}
 
-	return _map_type_and_height[tile] & 0xF;
+	return TileHeight(tile);
 }
 
 static void TerraformAddDirtyTile(TerraformerState *ts, TileIndex tile)
@@ -84,19 +86,23 @@ static int TerraformProc(TerraformerState *ts, uint tile, int mode)
 	int r;
 	int32 ret;
 
-	assert(tile < TILES_X * TILES_Y);
+	assert(tile < MapSize());
 
 	if ((r=TerraformAllowTileProcess(ts, tile)) <= 0)
 		return r;
 
-	if ((_map_type_and_height[tile] >> 4) == MP_RAILWAY) {
+	if (IsTileType(tile, MP_RAILWAY)) {
 		static const byte _railway_modes[4] = {8, 0x10, 4, 0x20};
 		static const byte _railway_dangslopes[4] = {0xd, 0xe, 7, 0xb};
+		static const byte _railway_dangslopes2[4] = {0x2, 0x1, 0x8, 0x4};
 
 		// Nothing could be built at the steep slope - this avoids a bug
 		// when you have a single diagonal track in one corner on a
-		// basement and then you raise the other corner.
-		if ((GetTileSlope(tile, NULL)&0xF) == _railway_dangslopes[mode]) {
+		// basement and then you raise/lower the other corner.
+		int tileh = GetTileSlope(tile, NULL) & 0xF;
+		if (tileh == _railway_dangslopes[mode] ||
+				tileh == _railway_dangslopes2[mode]) {
+			_terraform_err_tile = tile;
 			_error_message = STR_1008_MUST_REMOVE_RAILROAD_TRACK;
 			return -1;
 		}
@@ -129,7 +135,7 @@ static bool TerraformTileHeight(TerraformerState *ts, uint tile, int height)
 	TerraformerHeightMod *mod;
 	int count;
 
-	assert(tile < TILES_X * TILES_Y);
+	assert(tile < MapSize());
 
 	if (height < 0) {
 		_error_message = STR_1003_ALREADY_AT_SEA_LEVEL;
@@ -180,12 +186,17 @@ static bool TerraformTileHeight(TerraformerState *ts, uint tile, int height)
 
 	{
 		int direction = ts->direction, r;
-		const TileIndexDiff *ttm;
+		const TileIndexDiffC *ttm;
 
-		static const TileIndexDiff _terraform_tilepos[5] = {TILE_XY(1,0), TILE_XY(-2,0), TILE_XY(1,1), TILE_XY(0,-2), 0 };
+		static const TileIndexDiffC _terraform_tilepos[] = {
+			{ 1,  0},
+			{-2,  0},
+			{ 1,  1},
+			{ 0, -2}
+		};
 
-		for(ttm = _terraform_tilepos; *ttm != 0; ttm++) {
-			tile += *ttm;
+		for(ttm = _terraform_tilepos; ttm != endof(_terraform_tilepos); ttm++) {
+			tile += ToTileIndexDiff(*ttm);
 
 			r = TerraformGetHeightOfTile(ts, tile);
 			if (r != height && r-direction != height && r+direction != height) {
@@ -228,25 +239,25 @@ int32 CmdTerraformLand(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 	if (p1 & 1) {
 		if (!TerraformTileHeight(&ts, tile+TILE_XY(1,0),
-				(_map_type_and_height[tile+TILE_XY(1,0)]&0xF) + direction))
+				TileHeight(tile + TILE_XY(1, 0)) + direction))
 					return CMD_ERROR;
 	}
 
 	if (p1 & 2) {
 		if (!TerraformTileHeight(&ts, tile+TILE_XY(1,1),
-				(_map_type_and_height[tile+TILE_XY(1,1)]&0xF) + direction))
+				TileHeight(tile + TILE_XY(1, 1)) + direction))
 					return CMD_ERROR;
 	}
 
 	if (p1 & 4) {
 		if (!TerraformTileHeight(&ts, tile+TILE_XY(0,1),
-				(_map_type_and_height[tile+TILE_XY(0,1)]&0xF) + direction))
+				TileHeight(tile + TILE_XY(0, 1)) + direction))
 					return CMD_ERROR;
 	}
 
 	if (p1 & 8) {
 		if (!TerraformTileHeight(&ts, tile+TILE_XY(0,0),
-				(_map_type_and_height[tile+TILE_XY(0,0)]&0xF) + direction))
+				TileHeight(tile + TILE_XY(0, 0)) + direction))
 					return CMD_ERROR;
 	}
 
@@ -292,9 +303,7 @@ int32 CmdTerraformLand(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			for(count = ts.modheight_count; count != 0; count--, mod++) {
 				til = mod->tile;
 
-				// Change tile height
-				_map_type_and_height[til] = (_map_type_and_height[til]&~0x0F)|mod->height;
-
+				SetTileHeight(til, mod->height);
 				TerraformAddDirtyTileAround(&ts, til);
 			}
 		}
@@ -324,14 +333,16 @@ int32 CmdLevelLand(int ex, int ey, uint32 flags, uint32 p1, uint32 p2)
 	uint tile;
 	int32 ret, cost, money;
 
+	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
+
 	// remember level height
-	h = _map_type_and_height[p1]&0xF;
+	h = TileHeight(p1);
 
 	ex >>= 4; ey >>= 4;
 
 	// make sure sx,sy are smaller than ex,ey
-	sx = GET_TILE_X(p1);
-	sy = GET_TILE_Y(p1);
+	sx = TileX(p1);
+	sy = TileY(p1);
 	if (ex < sx) intswap(ex, sx);
 	if (ey < sy) intswap(ey, sy);
 	tile = TILE_XY(sx,sy);
@@ -343,7 +354,7 @@ int32 CmdLevelLand(int ex, int ey, uint32 flags, uint32 p1, uint32 p2)
 	cost = 0;
 
 	BEGIN_TILE_LOOP(tile2, size_x, size_y, tile)
-		curh = _map_type_and_height[tile2]&0xF;
+		curh = TileHeight(tile2);
 		while (curh != h) {
 			ret = DoCommandByTile(tile2, 8, (curh > h)?0:1, flags & ~DC_EXEC, CMD_TERRAFORM_LAND);
 			if (ret == CMD_ERROR) break;
@@ -382,7 +393,7 @@ int32 CmdPurchaseLandArea(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (!EnsureNoVehicle(tile))
 		return CMD_ERROR;
 
-	if (IS_TILETYPE(tile, MP_UNMOVABLE) &&
+	if (IsTileType(tile, MP_UNMOVABLE) &&
 			_map5[tile] == 3 &&
 			_map_owner[tile] == _current_player)
 		return_cmd_error(STR_5807_YOU_ALREADY_OWN_IT);
@@ -402,7 +413,8 @@ int32 CmdPurchaseLandArea(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 }
 
 
-int32 ClearTile_Clear(uint tile, byte flags) {
+static int32 ClearTile_Clear(uint tile, byte flags)
+{
 	static const int32 * _clear_price_table[] = {
 			NULL,
 			&_price.clear_1, &_price.clear_1,&_price.clear_1,
@@ -512,14 +524,17 @@ static void DrawTile_Clear(TileInfo *ti)
 	DrawClearLandFence(ti, _map3_hi[ti->tile] >> 2);
 }
 
-uint GetSlopeZ_Clear(TileInfo *ti) { return GetPartialZ(ti->x&0xF, ti->y&0xF, ti->tileh) + ti->z; }
+static uint GetSlopeZ_Clear(TileInfo *ti)
+{
+	return GetPartialZ(ti->x & 0xF, ti->y & 0xF, ti->tileh) + ti->z;
+}
 
-uint GetSlopeTileh_Clear(TileInfo *ti)
+static uint GetSlopeTileh_Clear(TileInfo *ti)
 {
 	return ti->tileh;
 }
 
-static void GetAcceptedCargo_Clear(uint tile, AcceptedCargo *ac)
+static void GetAcceptedCargo_Clear(uint tile, AcceptedCargo ac)
 {
 	/* unused */
 }
@@ -536,16 +551,16 @@ void TileLoopClearHelper(uint tile)
 	uint dirty = -1;
 
 	img_1 = 0;
-	if (IS_TILETYPE(tile, MP_CLEAR)) {
+	if (IsTileType(tile, MP_CLEAR)) {
 		img_1 = img_by_map5[(_map5[tile] & 0x1C) >> 2];
-	} else if (IS_TILETYPE(tile, MP_TREES) && (_map2[tile] & 0x30) == 0x20) {
+	} else if (IsTileType(tile, MP_TREES) && (_map2[tile] & 0x30) == 0x20) {
 		img_1 = 1;
 	}
 
 	img_2 = 0;
-	if (IS_TILETYPE(TILE_ADDXY(tile, 1, 0), MP_CLEAR)) {
+	if (IsTileType(TILE_ADDXY(tile, 1, 0), MP_CLEAR)) {
 		img_2 = img_by_map5[(_map5[TILE_ADDXY(tile, 1, 0)] & 0x1C) >> 2];
-	} else if (IS_TILETYPE(TILE_ADDXY(tile, 1, 0), MP_TREES) && (_map2[TILE_ADDXY(tile, 1, 0)] & 0x30) == 0x20) {
+	} else if (IsTileType(TILE_ADDXY(tile, 1, 0), MP_TREES) && (_map2[TILE_ADDXY(tile, 1, 0)] & 0x30) == 0x20) {
 		img_2 = 1;
 	}
 
@@ -562,9 +577,9 @@ void TileLoopClearHelper(uint tile)
 	}
 
 	img_2 = 0;
-	if (IS_TILETYPE(TILE_ADDXY(tile, 0, 1), MP_CLEAR)) {
+	if (IsTileType(TILE_ADDXY(tile, 0, 1), MP_CLEAR)) {
 		img_2 = img_by_map5[(_map5[TILE_ADDXY(tile, 0, 1)] & 0x1C) >> 2];
-	} else if (IS_TILETYPE(TILE_ADDXY(tile, 0, 1), MP_TREES) && (_map2[TILE_ADDXY(tile, 0, 1)] & 0x30) == 0x20) {
+	} else if (IsTileType(TILE_ADDXY(tile, 0, 1), MP_TREES) && (_map2[TILE_ADDXY(tile, 0, 1)] & 0x30) == 0x20) {
 		img_2 = 1;
 	}
 
@@ -580,7 +595,7 @@ void TileLoopClearHelper(uint tile)
 		}
 	}
 
-	if (dirty != -1)
+	if (dirty != (uint) -1)
 		MarkTileDirtyByTile(dirty);
 }
 
@@ -718,7 +733,7 @@ static void TileLoop_Clear(uint tile)
 	MarkTileDirtyByTile(tile);
 }
 
-void GenerateClearTile()
+void GenerateClearTile(void)
 {
 	int i,j;
 	uint tile,tile_new;
@@ -728,7 +743,7 @@ void GenerateClearTile()
 	i = (Random() & 0x3FF) | 0x400;
 	do {
 		tile = TILE_MASK(Random());
-		if (IS_TILETYPE(tile, MP_CLEAR))
+		if (IsTileType(tile, MP_CLEAR))
 			_map5[tile] = (byte)((_map5[tile] & ~(3<<2)) | (1<<2));
 	} while (--i);
 
@@ -737,14 +752,14 @@ void GenerateClearTile()
 	do {
 		r = Random();
 		tile = TILE_MASK(r);
-		if (IS_TILETYPE(tile, MP_CLEAR)) {
+		if (IsTileType(tile, MP_CLEAR)) {
 			j = ((r >> 16) & 0xF) + 5;
 			for(;;) {
 				_map5[tile] = (byte)((_map5[tile] & ~(3<<2)) | (2<<2));
 				do {
 					if (--j == 0) goto get_out;
-					tile_new = tile + _tileoffs_by_dir[Random() & 3];
-				} while (!IS_TILETYPE(tile_new, MP_CLEAR));
+					tile_new = tile + TileOffsByDir(Random() & 3);
+				} while (!IsTileType(tile_new, MP_CLEAR));
 				tile = tile_new;
 			}
 get_out:;
@@ -757,7 +772,7 @@ static void ClickTile_Clear(uint tile)
 	/* not used */
 }
 
-uint32 GetTileTrackStatus_Clear(uint tile, TransportType mode)
+static uint32 GetTileTrackStatus_Clear(uint tile, TransportType mode)
 {
 	return 0;
 }
@@ -790,7 +805,8 @@ static void ChangeTileOwner_Clear(uint tile, byte old_player, byte new_player)
 	return;
 }
 
-void InitializeClearLand() {
+void InitializeClearLand(void)
+{
 	_opt.snow_line = _patches.snow_line_height * 8;
 }
 
