@@ -1,5 +1,7 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
+#include "map.h"
 #include "news.h"
 #include "player.h"
 #include "station.h"
@@ -12,6 +14,34 @@
 #include "industry.h"
 #include "town.h"
 #include "network.h"
+#include "sound.h"
+#include "engine.h"
+#include "network_data.h"
+
+// get a mask of the allowed currencies depending on the year
+uint GetMaskOfAllowedCurrencies(void)
+{
+	int i;
+	uint mask = 0;
+	for (i = 0; i != lengthof(_currency_specs); i++) {
+		uint16 to_euro = _currency_specs[i].to_euro;
+		if (i == 23) mask |= (1 << 23); // always allow custom currency
+		if (to_euro != CF_NOEURO && to_euro != CF_ISEURO && _cur_year >= (to_euro-MAX_YEAR_BEGIN_REAL)) continue;
+		if (_cur_year < (2000-MAX_YEAR_BEGIN_REAL) && (to_euro == CF_ISEURO)) continue;
+		mask |= (1 << i);
+	}
+	return mask;
+}
+
+void CheckSwitchToEuro(void)
+{
+	if (_currency_specs[_opt.currency].to_euro != CF_NOEURO &&
+			_currency_specs[_opt.currency].to_euro != CF_ISEURO &&
+			_cur_year >= (_currency_specs[_opt.currency].to_euro-MAX_YEAR_BEGIN_REAL)) {
+		_opt.currency = 2; // this is the index of euro above.
+		AddNewsItem(STR_EURO_INTRODUCE, NEWS_FLAGS(NM_NORMAL,0,NT_ECONOMY,0), 0, 0);
+	}
+}
 
 void UpdatePlayerHouse(Player *p, uint score)
 {
@@ -75,10 +105,9 @@ int64 CalculateCompanyValue(Player *p) {
 		}
 	}
 
-	if (p->player_money > 0)
-		value += p->money64; // add real money value
+	value += p->money64 - p->current_loan; // add real money value
 
-	return value;
+	return max(value, 1);
 }
 
 // if update is set to true, the economy is updated with this score
@@ -99,7 +128,7 @@ int UpdateCompanyRatingAndValue(Player *p, bool update)
 		FOR_ALL_VEHICLES(v) {
 			if (v->owner != owner)
 				continue;
-			if ((v->type == VEH_Train && v->subtype == 0) ||
+			if ((v->type == VEH_Train && v->subtype == TS_Front_Engine) ||
 					v->type == VEH_Road ||
 					(v->type == VEH_Aircraft && v->subtype<=2) ||
 					v->type == VEH_Ship) {
@@ -136,7 +165,7 @@ int UpdateCompanyRatingAndValue(Player *p, bool update)
 		PlayerEconomyEntry *pee;
 		int numec;
 		int32 min_income;
-		uint32 max_income;
+		int32 max_income;
 
 		numec = min(p->num_valid_stat_ent, 12);
 		if (numec != 0) {
@@ -240,7 +269,7 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 
 		for(s=_subsidies; s != endof(_subsidies); s++) {
 			if (s->cargo_type != 0xff && s->age >= 12) {
-				Station *st = DEREF_STATION(s->to);
+				Station *st = GetStation(s->to);
 				if (st->owner == old_player)
 					s->cargo_type = 0xff;
 			}
@@ -281,7 +310,7 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 		// Determine Ids for the new vehicles
 		FOR_ALL_VEHICLES(v) {
 			if (v->owner == new_player) {
-				if (v->type == VEH_Train && v->subtype == 0)
+				if (v->type == VEH_Train && v->subtype == TS_Front_Engine)
 					num_train++;
 				else if (v->type == VEH_Road)
 					num_road++;
@@ -301,7 +330,7 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 					DeleteVehicle(v);
 				} else {
 					v->owner = new_player;
-					if (v->type == VEH_Train && v->subtype == 0)
+					if (v->type == VEH_Train && v->subtype == TS_Front_Engine)
 						v->unitnumber = ++num_train;
 					else if (v->type == VEH_Road)
 						v->unitnumber = ++num_road;
@@ -319,7 +348,7 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 		uint tile = 0;
 		do {
 			ChangeTileOwner(tile, old_player, new_player);
-		} while (++tile != TILES_X * TILES_Y);
+		} while (++tile != MapSize());
 	}
 
 	// Change color of existing windows
@@ -331,6 +360,24 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 		}
 	}
 
+	{
+		Player *p;
+		uint i;
+
+		/* Check for shares */
+		FOR_ALL_PLAYERS(p) {
+			for(i = 0; i < 4; i++) {
+				/* 'Sell' the share if this player has any */
+				if (p->share_owners[i] == _current_player)
+					p->share_owners[i] = 0xFF;
+			}
+		}
+		p = DEREF_PLAYER(_current_player);
+		/* Sell all the shares that people have on this company */
+		for(i = 0; i < 4; i++)
+			p->share_owners[i] = 0xFF;
+	}
+
 	_current_player = old;
 
 	MarkWholeScreenDirty();
@@ -343,6 +390,7 @@ static void PlayersCheckBankrupt(Player *p)
 	int owner;
 	int64 val;
 
+	// If the player has money again, it does not go bankrupt
 	if (p->player_money >= 0) {
 		p->quarters_of_bankrupcy = 0;
 		return;
@@ -352,43 +400,74 @@ static void PlayersCheckBankrupt(Player *p)
 
 	owner = p->index;
 
-	if (p->quarters_of_bankrupcy == 2) {
-year_2:
-		AddNewsItem( (StringID)(owner + 16),
-			NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
+	switch (p->quarters_of_bankrupcy) {
+		case 2:
+			AddNewsItem( (StringID)(owner + 16),
+				NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
+			break;
+		case 3: {
+			/* XXX - In multiplayer, should we ask other players if it wants to take
+		          over when it is a human company? -- TrueLight */
+			if (IS_HUMAN_PLAYER(owner)) {
+				AddNewsItem( (StringID)(owner + 16),
+					NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
+				break;
+			}
 
-	} else if (p->quarters_of_bankrupcy == 3) {
-		if (IS_HUMAN_PLAYER(owner))
-			goto year_2;
-
-		val = CalculateCompanyValue(p);
-		if (val == 0) goto year_4;
-
-		p->bankrupt_value = val;
-		p->bankrupt_asked = 1 << owner;
-		p->bankrupt_timeout = 0;
-	} else if (p->quarters_of_bankrupcy == 4) {
-year_4:
-		DeletePlayerWindows(owner);
-
-		if (IS_HUMAN_PLAYER(owner)) {
-// what does this code do??
-			InitNewsItemStructs();
-			DeleteWindowById(WC_NEWS_WINDOW, 0);
+			// Check if the company has any value.. if not, declare it bankrupt
+			//  right now
+			val = CalculateCompanyValue(p);
+			if (val > 0) {
+				p->bankrupt_value = val;
+				p->bankrupt_asked = 1 << owner; // Don't ask the owner
+				p->bankrupt_timeout = 0;
+				break;
+			}
+			// Else, falltrue to case 4...
 		}
+		case 4: {
+			// Close everything the owner has open
+			DeletePlayerWindows(owner);
 
-// Show bankrupt news
-		SET_DPARAM16(0, p->name_1);
-		SET_DPARAM32(1, p->name_2);
-		AddNewsItem( (StringID)(owner + 16*3), NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
+//		Show bankrupt news
+			SetDParam(0, p->name_1);
+			SetDParam(1, p->name_2);
+			AddNewsItem( (StringID)(owner + 16*3), NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
 
-		if (IS_HUMAN_PLAYER(owner)) {
-			p->bankrupt_asked = 255;
-			p->bankrupt_timeout = 0x456;
-		} else {
-			p->money64 = p->player_money = 100000000;
-			ChangeOwnershipOfPlayerItems(owner, 0xFF); // 255 is no owner
-			p->is_active = false;
+			// If the player is human, and it is no network play, leave the player playing
+			if (IS_HUMAN_PLAYER(owner) && !_networking) {
+				p->bankrupt_asked = 255;
+				p->bankrupt_timeout = 0x456;
+			} else {
+#ifdef ENABLE_NETWORK
+				if (IS_HUMAN_PLAYER(owner) && _network_server) {
+					// If we are the server, make sure it is clear that his player is no
+					//  longer with us!
+					NetworkClientInfo *ci;
+					NetworkClientState *cs;
+					/* Find all clients that were in control of this company */
+					FOR_ALL_CLIENTS(cs) {
+						ci = DEREF_CLIENT_INFO(cs);
+						if ((ci->client_playas-1) == owner) {
+							ci->client_playas = OWNER_SPECTATOR;
+							// Send the new info to all the clients
+							NetworkUpdateClientInfo(_network_own_client_index);
+						}
+					}
+				}
+				// Make sure the player no longer controls the company
+				if (IS_HUMAN_PLAYER(owner) && owner == _local_player) {
+					// Switch the player to spectator..
+					_local_player = OWNER_SPECTATOR;
+				}
+#endif /* ENABLE_NETWORK */
+
+				// Convert everything the player owns to NO_OWNER
+				p->money64 = p->player_money = 100000000;
+				ChangeOwnershipOfPlayerItems(owner, 0xFF); // 255 is no owner
+				// Register the player as not-active
+				p->is_active = false;
+			}
 		}
 	}
 }
@@ -403,8 +482,8 @@ void DrawNewsBankrupcy(Window *w)
 	DrawPlayerFace(p->face, p->player_color, 2, 23);
 	GfxFillRect(3, 23, 3+91, 23+118, 0x4323);
 
-	SET_DPARAM16(0, p->president_name_1);
-	SET_DPARAM32(1, p->president_name_2);
+	SetDParam(0, p->president_name_1);
+	SetDParam(1, p->president_name_2);
 
 	DrawStringMultiCenter(49, 148, STR_7058_PRESIDENT, 94);
 
@@ -412,8 +491,8 @@ void DrawNewsBankrupcy(Window *w)
 	case 1:
 		DrawStringCentered(w->width>>1, 1, STR_7056_TRANSPORT_COMPANY_IN_TROUBLE, 0);
 
-		SET_DPARAM16(0, p->name_1);
-		SET_DPARAM32(1, p->name_2);
+		SetDParam(0, p->name_1);
+		SetDParam(1, p->name_2);
 
 		DrawStringMultiCenter(
 			((w->width - 101) >> 1) + 98,
@@ -427,10 +506,10 @@ void DrawNewsBankrupcy(Window *w)
 
 		DrawStringCentered(w->width>>1, 1, STR_7059_TRANSPORT_COMPANY_MERGER, 0);
 		COPY_IN_DPARAM(0,WP(w,news_d).ni->params, 2);
-		SET_DPARAM16(2, p->name_1);
-		SET_DPARAM32(3, p->name_2);
+		SetDParam(2, p->name_1);
+		SetDParam(3, p->name_2);
 		price = WP(w,news_d).ni->params[2];
-		SET_DPARAM32(4, price);
+		SetDParam(4, price);
 		DrawStringMultiCenter(
 			((w->width - 101) >> 1) + 98,
 			90,
@@ -451,8 +530,8 @@ void DrawNewsBankrupcy(Window *w)
 
 	case 4:
 		DrawStringCentered(w->width>>1, 1, STR_705E_NEW_TRANSPORT_COMPANY_LAUNCHED, 0);
-		SET_DPARAM16(0, p->name_1);
-		SET_DPARAM32(1, p->name_2);
+		SetDParam(0, p->name_1);
+		SetDParam(1, p->name_2);
 		COPY_IN_DPARAM(2,WP(w,news_d).ni->params, 2);
 		DrawStringMultiCenter(
 			((w->width - 101) >> 1) + 98,
@@ -472,29 +551,29 @@ StringID GetNewsStringBankrupcy(NewsItem *ni)
 
 	switch(ni->string_id >> 4) {
 	case 1:
-		SET_DPARAM16(0, STR_7056_TRANSPORT_COMPANY_IN_TROUBLE);
-		SET_DPARAM16(1, STR_7057_WILL_BE_SOLD_OFF_OR_DECLARED);
-		SET_DPARAM16(2, p->name_1);
-		SET_DPARAM32(3, p->name_2);
+		SetDParam(0, STR_7056_TRANSPORT_COMPANY_IN_TROUBLE);
+		SetDParam(1, STR_7057_WILL_BE_SOLD_OFF_OR_DECLARED);
+		SetDParam(2, p->name_1);
+		SetDParam(3, p->name_2);
 		return STR_02B6;
 	case 2:
-		SET_DPARAM16(0, STR_7059_TRANSPORT_COMPANY_MERGER);
-		SET_DPARAM16(1, STR_705A_HAS_BEEN_SOLD_TO_FOR);
+		SetDParam(0, STR_7059_TRANSPORT_COMPANY_MERGER);
+		SetDParam(1, STR_705A_HAS_BEEN_SOLD_TO_FOR);
 		COPY_IN_DPARAM(2,ni->params, 2);
-		SET_DPARAM16(4, p->name_1);
-		SET_DPARAM32(5, p->name_2);
+		SetDParam(4, p->name_1);
+		SetDParam(5, p->name_2);
 		COPY_IN_DPARAM(6,ni->params + 2, 1);
 		return STR_02B6;
 	case 3:
-		SET_DPARAM16(0, STR_705C_BANKRUPT);
-		SET_DPARAM16(1, STR_705D_HAS_BEEN_CLOSED_DOWN_BY);
+		SetDParam(0, STR_705C_BANKRUPT);
+		SetDParam(1, STR_705D_HAS_BEEN_CLOSED_DOWN_BY);
 		COPY_IN_DPARAM(2,ni->params, 2);
 		return STR_02B6;
 	case 4:
-		SET_DPARAM16(0, STR_705E_NEW_TRANSPORT_COMPANY_LAUNCHED);
-		SET_DPARAM16(1, STR_705F_STARTS_CONSTRUCTION_NEAR);
-		SET_DPARAM16(2, p->name_1);
-		SET_DPARAM32(3, p->name_2);
+		SetDParam(0, STR_705E_NEW_TRANSPORT_COMPANY_LAUNCHED);
+		SetDParam(1, STR_705F_STARTS_CONSTRUCTION_NEAR);
+		SetDParam(2, p->name_1);
+		SetDParam(3, p->name_2);
 		COPY_IN_DPARAM(4,ni->params, 2);
 		return STR_02B6;
 	default:
@@ -505,7 +584,7 @@ StringID GetNewsStringBankrupcy(NewsItem *ni)
 	return 0;
 }
 
-static void PlayersGenStatistics()
+static void PlayersGenStatistics(void)
 {
 	Station *st;
 	Player *p;
@@ -554,7 +633,7 @@ static void AddSingleInflation(int32 *value, uint16 *frac, int32 amt)
 	*value += (int32)(tmp >> 16) + (low >> 16);
 }
 
-static void AddInflation()
+static void AddInflation(void)
 {
 	int i;
 	int32 inf = _economy.infl_amount * 54;
@@ -578,11 +657,12 @@ static void AddInflation()
 	}
 
 	InvalidateWindowClasses(WC_BUILD_VEHICLE);
+	InvalidateWindowClasses(WC_REPLACE_VEHICLE);
 	InvalidateWindowClasses(WC_VEHICLE_DETAILS);
 	InvalidateWindow(WC_PAYMENT_RATES, 0);
 }
 
-static void PlayersPayInterest()
+static void PlayersPayInterest(void)
 {
 	Player *p;
 	int interest = _economy.interest_rate * 54;
@@ -601,7 +681,7 @@ static void PlayersPayInterest()
 	}
 }
 
-static void HandleEconomyFluctuations()
+static void HandleEconomyFluctuations(void)
 {
 	if (_opt.diff.economy == 0)
 		return;
@@ -677,7 +757,7 @@ static const int32 _price_base[NUM_PRICES] = {
 	1000000, // build_industry
 };
 
-void StartupEconomy()
+void StartupEconomy(void)
 {
 	int i;
 
@@ -713,50 +793,50 @@ Pair SetupSubsidyDecodeParam(Subsidy *s, bool mode)
 	Pair tp;
 
 	/* if mode is false, convert into plural */
-	SET_DPARAM16(0, _cargoc.names_s[s->cargo_type] + (mode?0:32));
+	SetDParam(0, _cargoc.names_s[s->cargo_type] + (mode?0:32));
 
 	if (s->age < 12) {
 		if (!(s->cargo_type == CT_PASSENGERS || s->cargo_type == CT_MAIL)) {
-			SET_DPARAM16(1, STR_2029);
-			i = DEREF_INDUSTRY(s->from);
+			SetDParam(1, STR_2029);
+			i = GetIndustry(s->from);
 			tile = i->xy;
-			SET_DPARAM16(2, i->town->townnametype);
-			SET_DPARAM32(3, i->town->townnameparts);
+			SetDParam(2, i->town->townnametype);
+			SetDParam(3, i->town->townnameparts);
 
-			SET_DPARAM16(4, i->type + STR_4802_COAL_MINE);
+			SetDParam(4, i->type + STR_4802_COAL_MINE);
 
 			if (s->cargo_type != CT_GOODS && s->cargo_type != CT_FOOD) {
-				SET_DPARAM16(5, STR_2029);
-				i = DEREF_INDUSTRY(s->to);
+				SetDParam(5, STR_2029);
+				i = GetIndustry(s->to);
 				tile2 = i->xy;
-				SET_DPARAM16(8, i->type + STR_4802_COAL_MINE);
-				SET_DPARAM16(6, i->town->townnametype);
-				SET_DPARAM32(7, i->town->townnameparts);
+				SetDParam(8, i->type + STR_4802_COAL_MINE);
+				SetDParam(6, i->town->townnametype);
+				SetDParam(7, i->town->townnameparts);
 			} else {
-				t = DEREF_TOWN(s->to);
+				t = GetTown(s->to);
 				tile2 = t->xy;
-				SET_DPARAM16(5, t->townnametype);
-				SET_DPARAM32(6, t->townnameparts);
+				SetDParam(5, t->townnametype);
+				SetDParam(6, t->townnameparts);
 			}
 		} else {
-			t = DEREF_TOWN(s->from);
+			t = GetTown(s->from);
 			tile = t->xy;
-			SET_DPARAM16(1, t->townnametype);
-			SET_DPARAM32(2, t->townnameparts);
+			SetDParam(1, t->townnametype);
+			SetDParam(2, t->townnameparts);
 
-			t = DEREF_TOWN(s->to);
+			t = GetTown(s->to);
 			tile2 = t->xy;
-			SET_DPARAM16(3, t->townnametype);
-			SET_DPARAM32(4, t->townnameparts);
+			SetDParam(3, t->townnametype);
+			SetDParam(4, t->townnameparts);
 		}
 	} else {
-		st = DEREF_STATION(s->from);
+		st = GetStation(s->from);
 		tile = st->xy;
-		SET_DPARAM16(1, st->index);
+		SetDParam(1, st->index);
 
-		st = DEREF_STATION(s->to);
+		st = GetStation(s->to);
 		tile2 = st->xy;
-		SET_DPARAM16(2, st->index);
+		SetDParam(2, st->index);
 	}
 
 	tp.a = tile;
@@ -765,7 +845,7 @@ Pair SetupSubsidyDecodeParam(Subsidy *s, bool mode)
 	return tp;
 }
 
-void DeleteSubsidyWithIndustry(byte index)
+void DeleteSubsidyWithIndustry(uint16 index)
 {
 	Subsidy *s;
 
@@ -778,7 +858,7 @@ void DeleteSubsidyWithIndustry(byte index)
 	}
 }
 
-void DeleteSubsidyWithStation(byte index)
+void DeleteSubsidyWithStation(uint16 index)
 {
 	Subsidy *s;
 	bool dirty = false;
@@ -808,11 +888,11 @@ static void FindSubsidyPassengerRoute(FoundRoute *fr)
 
 	fr->distance = (uint)-1;
 
-	fr->from = from = DEREF_TOWN(InteractiveRandomRange(_total_towns));
+	fr->from = from = GetTown(RandomRange(_total_towns));
 	if (from->xy == 0 || from->population < 400)
 		return;
 
-	fr->to = to = DEREF_TOWN(InteractiveRandomRange(_total_towns));
+	fr->to = to = GetTown(RandomRange(_total_towns));
 	if (from==to || to->xy == 0 || to->population < 400 || to->pct_pass_transported > 42)
 		return;
 
@@ -827,12 +907,12 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 
 	fr->distance = (uint)-1;
 
-	fr->from = i = DEREF_INDUSTRY(InteractiveRandomRange(_total_industries));
+	fr->from = i = GetIndustry(RandomRange(_total_industries));
 	if (i->xy == 0)
 		return;
 
 	// Randomize cargo type
-	if (InteractiveRandom()&1 && i->produced_cargo[1] != 0xFF) {
+	if (Random()&1 && i->produced_cargo[1] != 0xFF) {
 		cargo = i->produced_cargo[1];
 		trans = i->pct_transported[1];
 		total = i->total_production[1];
@@ -852,7 +932,7 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 
 	if (cargo == CT_GOODS || cargo == CT_FOOD) {
 		// The destination is a town
-		Town *t = DEREF_TOWN(InteractiveRandomRange(_total_towns));
+		Town *t = GetTown(RandomRange(_total_towns));
 
 		// Only want big towns
 		if (t->xy == 0 || t->population < 900)
@@ -861,7 +941,7 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 		fr->to = t;
 	} else {
 		// The destination is an industry
-		Industry *i2 = DEREF_INDUSTRY(InteractiveRandomRange(_total_industries));
+		Industry *i2 = GetIndustry(RandomRange(_total_industries));
 
 		// The industry must accept the cargo
 		if (i == i2 || i2->xy == 0 ||
@@ -909,7 +989,7 @@ void RemoteSubsidyAdd(Subsidy *s_new)
 	InvalidateWindow(WC_SUBSIDIES_LIST, 0);
 }
 
-static void SubsidyMonthlyHandler()
+static void SubsidyMonthlyHandler(void)
 {
 	Subsidy *s;
 	Pair pair;
@@ -928,7 +1008,7 @@ static void SubsidyMonthlyHandler()
 			s->cargo_type = 0xFF;
 			modified = true;
 		} else if (s->age == 2*12-1) {
-			st = DEREF_STATION(s->to);
+			st = GetStation(s->to);
 			if (st->owner == _local_player) {
 				pair = SetupSubsidyDecodeParam(s, 1);
 				AddNewsItem(STR_202F_SUBSIDY_WITHDRAWN_SERVICE, NEWS_FLAGS(NM_NORMAL, NF_TILE, NT_SUBSIDIES, 0), pair.a, pair.b);
@@ -940,10 +1020,8 @@ static void SubsidyMonthlyHandler()
 		}
 	}
 
-	if ((_networking) && (!_networking_server)) return;
-
 	// 25% chance to go on
-	if (ICHANCE16(1,4)) {
+	if (CHANCE16(1,4)) {
 		// Find a free slot
 		s = _subsidies;
 		while (s->cargo_type != 0xFF) {
@@ -963,13 +1041,12 @@ static void SubsidyMonthlyHandler()
 			FindSubsidyCargoRoute(&fr);
 			if (fr.distance <= 70) {
 				s->cargo_type = fr.cargo;
-				s->from = (Industry*)fr.from - _industries;
-				s->to = (fr.cargo == CT_GOODS || fr.cargo == CT_FOOD) ? ((Town*)fr.to)->index : (Industry*)fr.to - _industries;
+				s->from = ((Industry*)fr.from)->index;
+				s->to = (fr.cargo == CT_GOODS || fr.cargo == CT_FOOD) ? ((Town*)fr.to)->index : ((Industry*)fr.to)->index;
 	add_subsidy:
 				if (!CheckSubsidyDuplicate(s)) {
 					s->age = 0;
 					pair = SetupSubsidyDecodeParam(s, 0);
-					if (_networking_server) NetworkSendEvent(NET_EVENT_SUBSIDY,sizeof(Subsidy),s);
 					AddNewsItem(STR_2030_SERVICE_SUBSIDY_OFFERED, NEWS_FLAGS(NM_NORMAL, NF_TILE, NT_SUBSIDIES, 0), pair.a, pair.b);
 					modified = true;
 					break;
@@ -985,12 +1062,14 @@ no_add:;
 static const byte _subsidies_desc[] = {
 	SLE_VAR(Subsidy,cargo_type,		SLE_UINT8),
 	SLE_VAR(Subsidy,age,					SLE_UINT8),
-	SLE_VAR(Subsidy,from,					SLE_UINT8),
-	SLE_VAR(Subsidy,to,						SLE_UINT8),
+	SLE_CONDVAR(Subsidy,from,			SLE_FILE_U8 | SLE_VAR_U16, 0, 4),
+	SLE_CONDVAR(Subsidy,from,			SLE_UINT16, 5, 255),
+	SLE_CONDVAR(Subsidy,to,				SLE_FILE_U8 | SLE_VAR_U16, 0, 4),
+	SLE_CONDVAR(Subsidy,to,				SLE_UINT16, 5, 255),
 	SLE_END()
 };
 
-static void Save_SUBS()
+static void Save_SUBS(void)
 {
 	int i;
 	Subsidy *s;
@@ -1004,7 +1083,7 @@ static void Save_SUBS()
 	}
 }
 
-static void Load_SUBS()
+static void Load_SUBS(void)
 {
 	int index;
 	while ((index = SlIterateArray()) != -1)
@@ -1048,7 +1127,7 @@ static void DeliverGoodsToIndustry(TileIndex xy, byte cargo_type, int num_pieces
 	 * the cargo */
 	best = NULL;
 	u = _patches.station_spread + 8;
-	for(ind = _industries; ind != endof(_industries); ind++) {
+	FOR_ALL_INDUSTRIES(ind) {
 		if (ind->xy != 0 && (cargo_type == ind->accepts_cargo[0] || cargo_type
 				 == ind->accepts_cargo[1] || cargo_type == ind->accepts_cargo[2]) &&
 				 ind->produced_cargo[0] != 0xFF &&
@@ -1088,18 +1167,18 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 
 			/* Check distance from source */
 			if (cargo_type == CT_PASSENGERS || cargo_type == CT_MAIL) {
-				xy = DEREF_TOWN(s->from)->xy;
+				xy = GetTown(s->from)->xy;
 			} else {
-				xy = (DEREF_INDUSTRY(s->from))->xy;
+				xy = (GetIndustry(s->from))->xy;
 			}
 			if (GetTileDist1D(xy, from->xy) > 9)
 				continue;
 
 			/* Check distance from dest */
 			if (cargo_type == CT_PASSENGERS || cargo_type == CT_MAIL || cargo_type == CT_GOODS || cargo_type == CT_FOOD) {
-				xy = DEREF_TOWN(s->to)->xy;
+				xy = GetTown(s->to)->xy;
 			} else {
-				xy = (DEREF_INDUSTRY(s->to))->xy;
+				xy = (GetIndustry(s->to))->xy;
 			}
 
 			if (GetTileDist1D(xy, to->xy) > 9)
@@ -1115,8 +1194,8 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 			INJECT_DPARAM(2);
 
 			p = DEREF_PLAYER(_current_player);
-			SET_DPARAM16(0, p->name_1);
-			SET_DPARAM32(1, p->name_2);
+			SetDParam(0, p->name_1);
+			SetDParam(1, p->name_2);
 			AddNewsItem(
 				STR_2031_SERVICE_SUBSIDY_AWARDED + _opt.diff.subsidy_multiplier,
 				NEWS_FLAGS(NM_NORMAL, NF_TILE, NT_SUBSIDIES, 0),
@@ -1145,8 +1224,8 @@ static int32 DeliverGoods(int num_pieces, byte cargo_type, byte source, byte des
 	}
 
 	// Get station pointers.
-	s_from = DEREF_STATION(source);
-	s_to = DEREF_STATION(dest);
+	s_from = GetStation(source);
+	s_to = GetStation(dest);
 
 	// Check if a subsidy applies.
 	subsidised = CheckSubsidised(s_from, s_to, cargo_type);
@@ -1197,6 +1276,59 @@ static int32 DeliverGoods(int num_pieces, byte cargo_type, byte source, byte des
 	return profit;
 }
 
+/*
+ * Returns true if Vehicle v should wait loading because other vehicle is
+ * already loading the same cargo type
+ * v = vehicle to load, u = GetFirstInChain(v)
+ */
+static bool LoadWait(const Vehicle *v, const Vehicle *u) {
+	const Vehicle *w;
+	const Vehicle *x;
+	bool has_any_cargo = false;
+
+	if (!(u->current_order.flags & OF_FULL_LOAD)) return false;
+
+	for (w = u; w != NULL; w = w->next) {
+		if (w->cargo_count != 0) {
+			if (v->cargo_type == w->cargo_type &&
+					u->last_station_visited == w->cargo_source)
+				return false;
+			has_any_cargo = true;
+		}
+	}
+
+	FOR_ALL_VEHICLES(x) {
+		if ((x->type != VEH_Train || x->subtype == TS_Front_Engine) && // for all locs
+				u->last_station_visited == x->last_station_visited && // at the same station
+				!(x->vehstatus & VS_STOPPED) && // not stopped
+				x->current_order.type == OT_LOADING && // loading
+				u != x) { // not itself
+			bool other_has_any_cargo = false;
+			bool has_space_for_same_type = false;
+			bool other_has_same_type = false;
+
+			for (w = x; w != NULL; w = w->next) {
+				if (w->cargo_count < w->cargo_cap && v->cargo_type == w->cargo_type)
+					has_space_for_same_type = true;
+
+				if (w->cargo_count != 0) {
+					if (v->cargo_type == w->cargo_type &&
+							u->last_station_visited == w->cargo_source)
+						other_has_same_type = true;
+					other_has_any_cargo = true;
+				}
+			}
+
+			if (has_space_for_same_type) {
+				if (other_has_same_type) return true;
+				if (other_has_any_cargo && !has_any_cargo) return true;
+			}
+		}
+	}
+
+	return false;
+}
+
 int LoadUnloadVehicle(Vehicle *v)
 {
 	int profit = 0;
@@ -1208,17 +1340,19 @@ int LoadUnloadVehicle(Vehicle *v)
 	GoodsEntry *ge;
 	int t;
 	uint count, cap;
+	byte old_player;
+	bool completely_empty = true;
 
-	assert((v->next_order&0x1F) == OT_LOADING);
+	assert(v->current_order.type == OT_LOADING);
 
 	v->cur_speed = 0;
+	old_player = _current_player;
 	_current_player = v->owner;
 
-	st = DEREF_STATION(last_visited = v->last_station_visited);
+	st = GetStation(last_visited = v->last_station_visited);
 
-	for(;;) {
-		if (v->cargo_cap == 0)
-			goto next_vehicle;
+	for (; v != NULL; v = v->next) {
+		if (v->cargo_cap == 0) continue;
 
 		ge = &st->goods[v->cargo_type];
 
@@ -1230,7 +1364,7 @@ int LoadUnloadVehicle(Vehicle *v)
 				profit += DeliverGoods(v->cargo_count, v->cargo_type, v->cargo_source, last_visited, v->cargo_days);
 				result |= 1;
 				v->cargo_count = 0;
-			} else if (u->next_order & OF_UNLOAD) {
+			} else if (u->current_order.flags & OF_UNLOAD) {
 				/* unload goods and let it wait at the station */
 				st->time_since_unload = 0;
 
@@ -1250,11 +1384,13 @@ int LoadUnloadVehicle(Vehicle *v)
 				result |= 2;
 				v->cargo_count = 0;
 			}
+
+			if (v->cargo_count != 0)
+				completely_empty = false;
 		}
 
 		/* don't pick up goods that we unloaded */
-		if (u->next_order & OF_UNLOAD)
-			goto next_vehicle;
+		if (u->current_order.flags & OF_UNLOAD) continue;
 
 		/* update stats */
 		ge->days_since_pickup = 0;
@@ -1269,6 +1405,22 @@ int LoadUnloadVehicle(Vehicle *v)
 		//  has capacity for it, load it on the vehicle.
 		if ((count=ge->waiting_acceptance & 0xFFF) != 0 &&
 				(cap = v->cargo_cap - v->cargo_count) != 0) {
+			if (v->cargo_count == 0)
+				TriggerVehicle(v, VEHICLE_TRIGGER_NEW_CARGO);
+
+			/* Skip loading this vehicle if another train/vehicle is already handling
+			 * the same cargo type at this station */
+			if (_patches.improved_load && LoadWait(v,u)) continue;
+
+			/* TODO: Regarding this, when we do gradual loading, we
+			 * should first unload all vehicles and then start
+			 * loading them. Since this will cause
+			 * VEHICLE_TRIGGER_EMPTY to be called at the time when
+			 * the whole vehicle chain is really totally empty, the
+			 * @completely_empty assignment can then be safely
+			 * removed; that's how TTDPatch behaves too. --pasky */
+			completely_empty = false;
+
 			if (cap > count) cap = count;
 			v->cargo_count += cap;
 			ge->waiting_acceptance -= cap;
@@ -1281,10 +1433,6 @@ int LoadUnloadVehicle(Vehicle *v)
 			result |= 2;
 			st->last_vehicle = v->index;
 		}
-
-next_vehicle:;
-		if (!(v = v->next))
-			break;
 	}
 
 	v = u;
@@ -1301,6 +1449,10 @@ next_vehicle:;
 
 	v->load_unload_time_rem = unloading_time;
 
+	if (completely_empty) {
+		TriggerVehicle(v, VEHICLE_TRIGGER_EMPTY);
+	}
+
 	if (result != 0) {
 		InvalidateWindow(WC_VEHICLE_DETAILS, v->index);
 
@@ -1312,16 +1464,17 @@ next_vehicle:;
 			SubtractMoneyFromPlayer(-profit);
 
 			if (_current_player == _local_player)
-				SndPlayVehicleFx(0x12, v);
+				SndPlayVehicleFx(SND_14_CASHTILL, v);
 
 			ShowCostOrIncomeAnimation(v->x_pos, v->y_pos, v->z_pos, -profit);
 		}
 	}
 
+	_current_player = old_player;
 	return result;
 }
 
-void PlayersMonthlyLoop()
+void PlayersMonthlyLoop(void)
 {
 	PlayersGenStatistics();
 	if (_patches.inflation)
@@ -1339,9 +1492,9 @@ static void DoAcquireCompany(Player *p)
 	int i,pi;
 	int64 value;
 
-	SET_DPARAM16(0, p->name_1);
-	SET_DPARAM32(1, p->name_2);
-	SET_DPARAM32(2, p->bankrupt_value);
+	SetDParam(0, p->name_1);
+	SetDParam(1, p->name_2);
+	SetDParam(2, p->bankrupt_value);
 	AddNewsItem( (StringID)(_current_player + 16*2), NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
 
 	// original code does this a little bit differently
@@ -1366,8 +1519,11 @@ static void DoAcquireCompany(Player *p)
 	p->is_active = false;
 
 	DeletePlayerWindows(pi);
+	RebuildVehicleLists();	//Updates the open windows to add the newly acquired vehicles to the lists
 }
 
+extern int GetAmountOwnedBy(Player *p, byte owner);
+
 int32 CmdBuyShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	Player *p;
@@ -1378,6 +1534,24 @@ int32 CmdBuyShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	SET_EXPENSES_TYPE(EXPENSES_OTHER);
 	p = DEREF_PLAYER(p1);
 
+
+	if (_cur_year - p->inaugurated_year < 6) {
+		_error_message = STR_7080_PROTECTED;
+		return CMD_ERROR;
+	}
+
+	/* Check if buying shares is allowed (protection against modified clients */
+	if (!_patches.allow_shares)
+		return CMD_ERROR;
+
+	/* Those lines are here for network-protection (clients can be slow) */
+	if (GetAmountOwnedBy(p, OWNER_SPECTATOR) == 0)
+		return 0;
+
+	/* We can not buy out a real player in networking */
+	if (GetAmountOwnedBy(p, OWNER_SPECTATOR) == 1 && !p->is_ai)
+		return 0;
+
 	cost = CalculateCompanyValue(p) >> 2;
 	if (flags & DC_EXEC) {
 		b = p->share_owners;
@@ -1405,6 +1579,14 @@ int32 CmdSellShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	SET_EXPENSES_TYPE(EXPENSES_OTHER);
 	p = DEREF_PLAYER(p1);
 
+	/* Check if selling shares is allowed (protection against modified clients */
+	if (!_patches.allow_shares)
+		return CMD_ERROR;
+
+	/* Those lines are here for network-protection (clients can be slow) */
+	if (GetAmountOwnedBy(p, _current_player) == 0)
+		return 0;
+
 	/* adjust it a little to make it less profitable to sell and buy */
 	cost = CalculateCompanyValue(p) >> 2;
 	cost = -(cost - (cost >> 7));
@@ -1430,14 +1612,14 @@ int32 CmdBuyCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 }
 
 // Prices
-static void SaveLoad_PRIC()
+static void SaveLoad_PRIC(void)
 {
 	SlArray(&_price, NUM_PRICES, SLE_INT32);
 	SlArray(&_price_frac, NUM_PRICES, SLE_UINT16);
 }
 
 // Cargo payment rates
-static void SaveLoad_CAPR()
+static void SaveLoad_CAPR(void)
 {
 	SlArray(&_cargo_payment_rates, NUM_CARGO, SLE_INT32);
 	SlArray(&_cargo_payment_rates_frac, NUM_CARGO, SLE_UINT16);
@@ -1454,7 +1636,7 @@ static const byte _economy_desc[] = {
 };
 
 // Economy variables
-static void SaveLoad_ECMY()
+static void SaveLoad_ECMY(void)
 {
 	SlObject(&_economy, &_economy_desc);
 }
