@@ -1,5 +1,7 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
+#include "map.h"
 #include "viewport.h"
 #include "command.h"
 #include "industry.h"
@@ -8,6 +10,7 @@
 #include "news.h"
 #include "saveload.h"
 #include "economy.h"
+#include "sound.h"
 
 byte _industry_sound_ctr;
 TileIndex _industry_sound_tile;
@@ -46,7 +49,7 @@ typedef struct DrawIndustrySpec4Struct {
 
 
 typedef struct IndustryTileTable {
-	TileIndexDiff ti;
+	TileIndexDiffC ti;
 	byte map5;
 } IndustryTileTable;
 
@@ -227,7 +230,7 @@ static void IndustryDrawTileProc3(TileInfo *ti)
 		AddChildSpriteScreen(0x128B, 5,
 			_industry_anim_offs_2[_map3_lo[ti->tile]]);
 	}
-	AddChildSpriteScreen(0x30B928A, 3, 67);
+	AddChildSpriteScreen(4746, 3, 67);
 }
 
 static void IndustryDrawTileProc4(TileInfo *ti)
@@ -279,7 +282,7 @@ static void DrawTile_Industry(TileInfo *ti)
 	uint32 image, ormod;
 
 	/* Pointer to industry */
-	ind = DEREF_INDUSTRY(_map2[ti->tile]);
+	ind = GetIndustry(_map2[ti->tile]);
 	ormod = (ind->color_map+0x307)<<16;
 
 	/* Retrieve pointer to the draw industry tile struct */
@@ -306,7 +309,7 @@ static void DrawTile_Industry(TileInfo *ti)
 		if (image&0x8000 && (image & 0xFFFF0000) == 0)
 			image |= ormod;
 
-		if (!(_display_opt & DO_TRANS_BUILDINGS))
+		if (_display_opt & DO_TRANS_BUILDINGS)
 			image = (image & 0x3FFF) | 0x3224000;
 
 		AddSortableSpriteToDraw(image,
@@ -317,7 +320,7 @@ static void DrawTile_Industry(TileInfo *ti)
 			dits->dz,
 			z);
 
-		if (!(_display_opt & DO_TRANS_BUILDINGS))
+		if (_display_opt & DO_TRANS_BUILDINGS)
 			return;
 	}
 
@@ -338,45 +341,36 @@ static uint GetSlopeTileh_Industry(TileInfo *ti) {
 	return 0;
 }
 
-static void GetAcceptedCargo_Industry(uint tile, AcceptedCargo *ac)
+static void GetAcceptedCargo_Industry(uint tile, AcceptedCargo ac)
 {
 	int m5 = _map5[tile];
 	int a;
 
 	a = _industry_map5_accepts_1[m5];
-	if (a >= 0) {
-		ac->type_1 = a;
-		ac->amount_1 = (a == 0) ? 1 : 8;
-	}
+	if (a >= 0) ac[a] = (a == 0) ? 1 : 8;
 
 	a = _industry_map5_accepts_2[m5];
-	if (a >= 0) {
-		ac->type_2 = a;
-		ac->amount_2 = 8;
-	}
+	if (a >= 0) ac[a] = 8;
 
 	a = _industry_map5_accepts_3[m5];
-	if (a >= 0) {
-		ac->type_3 = a;
-		ac->amount_3 = 8;
-	}
+	if (a >= 0) ac[a] = 8;
 }
 
 static void GetTileDesc_Industry(uint tile, TileDesc *td)
 {
-	Industry *i = DEREF_INDUSTRY(_map2[tile]);
+	Industry *i = GetIndustry(_map2[tile]);
 
 	td->owner = i->owner;
 	td->str = STR_4802_COAL_MINE + i->type;
 	if ((_map_owner[tile] & 0x80) == 0) {
-		SET_DPARAMX16(td->dparam, 0, td->str);
+		SetDParamX(td->dparam, 0, td->str);
 		td->str = STR_2058_UNDER_CONSTRUCTION;
 	}
 }
 
 static int32 ClearTile_Industry(uint tile, byte flags)
 {
-	Industry *i = DEREF_INDUSTRY(_map2[tile]);
+	Industry *i = GetIndustry(_map2[tile]);
 
 	/*	* water can destroy industries
 			* in editor you can bulldoze industries
@@ -386,7 +380,7 @@ static int32 ClearTile_Industry(uint tile, byte flags)
 	if ((_current_player != OWNER_WATER && _game_mode != GM_EDITOR &&
 			!_cheats.magic_bulldozer.value) ||
 			(_current_player == OWNER_WATER && i->type == IT_OIL_RIG) ) {
- 		SET_DPARAM16(0, STR_4802_COAL_MINE + i->type);
+ 		SetDParam(0, STR_4802_COAL_MINE + i->type);
 		return_cmd_error(STR_4800_IN_THE_WAY);
 	}
 
@@ -401,9 +395,11 @@ static int32 ClearTile_Industry(uint tile, byte flags)
 int32 CmdDestroyIndustry(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	uint tile = TILE_FROM_XY(x,y);
-	Industry *i = DEREF_INDUSTRY((uint16)p1);
+	Industry *i = GetIndustry((uint16)p1);
   Town *t = ClosestTownFromTile(tile, (uint)-1); // find closest town to penaltize (ALWAYS penaltize)
 
+	SET_EXPENSES_TYPE(EXPENSES_OTHER);
+
 	// check if you're allowed to remove the industry. Minimum amount
 	// of ratings to remove the industry depends on difficulty setting
 	if (!CheckforTownRating(tile, flags, t, INDUSTRY_REMOVE))
@@ -412,7 +408,7 @@ int32 CmdDestroyIndustry(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (flags & DC_EXEC) {
 		DeleteIndustry(i);
 		CreateEffectVehicleAbove(x + 8,y + 8, 2, EV_DEMOLISH);
-		ChangeTownRating(t, -1500, -1000);	// penalty is 1500
+		ChangeTownRating(t, RATING_INDUSTRY_DOWN_STEP, RATING_INDUSTRY_MINIMUM);
 	}
 
 	return (_price.build_industry >> 5) * _industry_type_costs[i->type]*2;
@@ -427,14 +423,14 @@ static const byte _industry_min_cargo[] = {
 	5, 5, 5, 5, 5,
 };
 
-void TransportIndustryGoods(uint tile)
+static void TransportIndustryGoods(uint tile)
 {
 	Industry *i;
 	int type;
 	uint cw, am;
 	byte m5;
 
-	i = DEREF_INDUSTRY(_map2[tile]);
+	i = GetIndustry(_map2[tile]);
 
 	type = i->type;
 	cw = min(i->cargo_waiting[0], 255);
@@ -480,8 +476,8 @@ static void AnimateTile_Industry(uint tile)
 			m = _map3_lo[tile] + 1;
 
 			switch(m & 7) {
-			case 2:	SndPlayTileFx(45, tile); break;
-			case 6: SndPlayTileFx(41, tile); break;
+			case 2:	SndPlayTileFx(SND_2D_RIP_2, tile); break;
+			case 6: SndPlayTileFx(SND_29_RIP, tile); break;
 			}
 
 			if (m >= 96) {
@@ -499,7 +495,7 @@ static void AnimateTile_Industry(uint tile)
 			m = _map3_lo[tile];
 
 			if (_industry_anim_offs[m] == 0xFF) {
-				SndPlayTileFx(48, tile);
+				SndPlayTileFx(SND_30_CARTOON_SOUND, tile);
 			}
 
 			if (++m >= 70) {
@@ -545,11 +541,11 @@ static void AnimateTile_Industry(uint tile)
 			m = _map3_lo[tile] + 1;
 
 			if (m == 1) {
-				SndPlayTileFx(44, tile);
+				SndPlayTileFx(SND_2C_MACHINERY, tile);
 			} else if (m == 23) {
-				SndPlayTileFx(43, tile);
+				SndPlayTileFx(SND_2B_COMEDY_HIT, tile);
 			} else if (m == 28) {
-				SndPlayTileFx(42, tile);
+				SndPlayTileFx(SND_2A_EXTRACT_AND_POP, tile);
 			}
 
 			if (m >= 50 && (m=0,++_map3_hi[tile] >= 8)) {
@@ -602,7 +598,7 @@ static void AnimateTile_Industry(uint tile)
 				if (state < 0x20 || state >= 0x180) {
 					if (!	(_map_owner[tile] & 0x40)) {
 						_map_owner[tile] |= 0x40;
-						SndPlayTileFx(9, tile);
+						SndPlayTileFx(SND_0B_MINING_MACHINERY, tile);
 					}
 					if (state & 7)
 						return;
@@ -632,7 +628,7 @@ static void AnimateTile_Industry(uint tile)
 static void MakeIndustryTileBiggerCase8(uint tile)
 {
 	TileInfo ti;
-	FindLandscapeHeight(&ti, GET_TILE_X(tile)*16, GET_TILE_Y(tile)*16);
+	FindLandscapeHeight(&ti, TileX(tile) * 16, TileY(tile) * 16);
 	CreateEffectVehicle(ti.x + 15, ti.y + 14, ti.z + 59 + (ti.tileh != 0 ? 8 : 0), EV_INDUSTRYSMOKE);
 }
 
@@ -691,13 +687,13 @@ static void TileLoopIndustryCase161(uint tile)
 		49, 59, 60, 65,
 	};
 
-	SndPlayTileFx(46, tile);
+	SndPlayTileFx(SND_2E_EXTRACT_AND_POP, tile);
 
 	dir = Random() & 3;
 
 	v = CreateEffectVehicleAbove(
-		GET_TILE_X(tile)*16 + _tileloop_ind_case_161[dir + 0],
-		GET_TILE_Y(tile)*16 + _tileloop_ind_case_161[dir + 4],
+		TileX(tile) * 16 + _tileloop_ind_case_161[dir + 0],
+		TileY(tile) * 16 + _tileloop_ind_case_161[dir + 4],
 		_tileloop_ind_case_161[dir + 8],
 		EV_INDUSTRY_SMOKE
 	);
@@ -777,18 +773,18 @@ static void TileLoop_Industry(uint tile)
 
 	case 10:
 		if (CHANCE16(1,3)) {
-			SndPlayTileFx(10, tile);
+			SndPlayTileFx(SND_0C_ELECTRIC_SPARK, tile);
 			AddAnimatedTile(tile);
 		}
 		break;
 
 	case 49: {
-		CreateEffectVehicleAbove(GET_TILE_X(tile)*16 + 6, GET_TILE_Y(tile)*16 + 6, 43, EV_SMOKE_3);
+		CreateEffectVehicleAbove(TileX(tile) * 16 + 6, TileY(tile) * 16 + 6, 43, EV_SMOKE_3);
 	} break;
 
 
 	case 143: {
-			Industry *i = DEREF_INDUSTRY(_map2[tile]);
+			Industry *i = GetIndustry(_map2[tile]);
 			if (i->was_cargo_delivered) {
 				i->was_cargo_delivered = false;
 				if ((_map3_hi[tile]|_map3_lo[tile]) != 0)
@@ -827,7 +823,7 @@ static uint32 GetTileTrackStatus_Industry(uint tile, TransportType mode)
 
 static void GetProducedCargo_Industry(uint tile, byte *b)
 {
-	Industry *i = DEREF_INDUSTRY(_map2[tile]);
+	Industry *i = GetIndustry(_map2[tile]);
 	b[0] = i->produced_cargo[0];
 	b[1] = i->produced_cargo[1];
 }
@@ -839,52 +835,54 @@ static void ChangeTileOwner_Industry(uint tile, byte old_player, byte new_player
 
 void DeleteIndustry(Industry *i)
 {
-	int index = i - _industries;
-
-	BEGIN_TILE_LOOP(tile_cur,	i->width, i->height, i->xy);
-		if (IS_TILETYPE(tile_cur, MP_INDUSTRY)) {
-			if (_map2[tile_cur] == (byte)index) {
+	BEGIN_TILE_LOOP(tile_cur, i->width, i->height, i->xy);
+		if (IsTileType(tile_cur, MP_INDUSTRY)) {
+			if (_map2[tile_cur] == i->index) {
 				DoClearSquare(tile_cur);
 			}
-		} else if (IS_TILETYPE(tile_cur, MP_STATION) && _map5[tile_cur] == 0x4B) {
+		} else if (IsTileType(tile_cur, MP_STATION) && _map5[tile_cur] == 0x4B) {
 			DeleteOilRig(tile_cur);
 		}
-	END_TILE_LOOP(tile_cur,	i->width, i->height, i->xy);
+	END_TILE_LOOP(tile_cur, i->width, i->height, i->xy);
 
 	i->xy = 0;
 	_industry_sort_dirty = true;
-	DeleteSubsidyWithIndustry(index);
-	DeleteWindowById(WC_INDUSTRY_VIEW, index);
+	DeleteSubsidyWithIndustry(i->index);
+	DeleteWindowById(WC_INDUSTRY_VIEW, i->index);
 	InvalidateWindow(WC_INDUSTRY_DIRECTORY, 0);
 }
 
 static const byte _plantfarmfield_type[] = {1, 1, 1, 1, 1, 3, 3, 4, 4, 4, 5, 5, 5, 6, 6, 6};
 
-static bool IsBadFarmFieldTile(uint tile)
+static bool IsBadFarmFieldTile(TileIndex tile)
 {
-	if (IS_TILETYPE(tile,MP_CLEAR)) {
-		byte m5 = _map5[tile] & 0x1C;
-		if (m5 == 0xC || m5 == 0x10)
+	switch (TileType(tile)) {
+		case MP_CLEAR: {
+			byte m5 = _map5[tile] & 0x1C;
+			return m5 == 0xC || m5 == 0x10;
+		}
+
+		case MP_TREES:
+			return false;
+
+		default:
 			return true;
-		return false;
-	} else if (IS_TILETYPE(tile,MP_TREES)) {
-		return false;
-	} else {
-		return true;
 	}
 }
 
-static bool IsBadFarmFieldTile2(uint tile)
+static bool IsBadFarmFieldTile2(TileIndex tile)
 {
-	if (IS_TILETYPE(tile,MP_CLEAR)) {
-		byte m5 = _map5[tile] & 0x1C;
-		if (m5 == 0x10)
+	switch (TileType(tile)) {
+		case MP_CLEAR: {
+			byte m5 = _map5[tile] & 0x1C;
+			return m5 == 0x10;
+		}
+
+		case MP_TREES:
+			return false;
+
+		default:
 			return true;
-		return false;
-	} else if (IS_TILETYPE(tile,MP_TREES)) {
-		return false;
-	} else {
-		return true;
 	}
 }
 
@@ -895,7 +893,7 @@ static void SetupFarmFieldFence(uint tile, int size, byte type, int direction)
 	do {
 		tile = TILE_MASK(tile);
 
-		if (IS_TILETYPE(tile, MP_CLEAR) || IS_TILETYPE(tile, MP_TREES)) {
+		if (IsTileType(tile, MP_CLEAR) || IsTileType(tile, MP_TREES)) {
 
 			or = type;
 			if (or == 1 && (uint16)Random() <= 9362) or = 2;
@@ -976,7 +974,9 @@ static void MaybePlantFarmField(Industry *i)
 {
 	uint tile;
 	if (CHANCE16(1,8)) {
-		tile = TileAddWrap(i->xy, ((i->width>>1) + Random() % 31 - 16), ((i->height>>1) + Random() % 31 - 16));
+		int x = (i->width>>1) + Random() % 31 - 16;
+		int y = (i->height>>1) + Random() % 31 - 16;
+		tile = TileAddWrap(i->xy, x, y);
 		if (tile != TILE_WRAPPED)
 			PlantFarmField(tile);
 	}
@@ -984,7 +984,12 @@ static void MaybePlantFarmField(Industry *i)
 
 static void ChopLumberMillTrees(Industry *i)
 {
-	static const TileIndexDiff _chop_dir[4] = { TILE_XY(0,1), TILE_XY(1,0), TILE_XY(0,-1), TILE_XY(-1,0) };
+	static const TileIndexDiffC _chop_dir[] = {
+		{ 0,  1},
+		{ 1,  0},
+		{ 0, -1},
+		{-1,  0}
+	};
 
 	uint tile = i->xy;
 	int dir, a, j;
@@ -998,21 +1003,24 @@ static void ChopLumberMillTrees(Industry *i)
 			j = a;
 			do {
 				tile = TILE_MASK(tile);
-				if (IS_TILETYPE(tile, MP_TREES)) {
+				if (IsTileType(tile, MP_TREES)) {
+					uint old_player = _current_player;
 					/* found a tree */
 
 					_current_player = OWNER_NONE;
 					_industry_sound_ctr = 1;
 					_industry_sound_tile = tile;
-					SndPlayTileFx(56, tile);
+					SndPlayTileFx(SND_38_CHAINSAW, tile);
 
 					DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
 					SetMapExtraBits(tile, 0);
 
 					i->cargo_waiting[0] = min(0xffff, i->cargo_waiting[0] + 45);
+
+					_current_player = old_player;
 					return;
 				}
-				tile += _chop_dir[dir];
+				tile += ToTileIndexDiff(_chop_dir[dir]);
 			} while (--j);
 		}
 		tile -= TILE_XY(1,1);
@@ -1022,19 +1030,19 @@ static void ChopLumberMillTrees(Industry *i)
 static const byte _industry_sounds[37][2] = {
 	{0},
 	{0},
-	{1,38},
+	{1, SND_28_SAWMILL},
 	{0},
 	{0},
 	{0},
-	{1,1},
-	{1,1},
+	{1, SND_03_FACTORY_WHISTLE},
+	{1, SND_03_FACTORY_WHISTLE},
 	{0},
-	{3,34},
+	{3, SND_24_SHEEP},
 	{0},
 	{0},
 	{0},
 	{0},
-	{1,38},
+	{1, SND_28_SAWMILL},
 	{0},
 	{0},
 	{0},
@@ -1043,7 +1051,7 @@ static const byte _industry_sounds[37][2] = {
 	{0},
 	{0},
 	{0},
-	{1,1},
+	{1, SND_03_FACTORY_WHISTLE},
 	{0},
 	{0},
 	{0},
@@ -1052,7 +1060,7 @@ static const byte _industry_sounds[37][2] = {
 	{0},
 	{0},
 	{0},
-	{1,51},
+	{1, SND_33_PLASTIC_MINE},
 	{0},
 	{0},
 	{0},
@@ -1088,7 +1096,7 @@ static void ProduceIndustryGoods(Industry *i)
 	}
 }
 
-void OnTick_Industry()
+void OnTick_Industry(void)
 {
 	Industry *i;
 
@@ -1096,17 +1104,17 @@ void OnTick_Industry()
 		_industry_sound_ctr++;
 
 		if (_industry_sound_ctr == 75) {
-			SndPlayTileFx(55, _industry_sound_tile);
+			SndPlayTileFx(SND_37_BALLOON_SQUEAK, _industry_sound_tile);
 		} else if (_industry_sound_ctr == 160) {
 			_industry_sound_ctr = 0;
-			SndPlayTileFx(54, _industry_sound_tile);
+			SndPlayTileFx(SND_36_CARTOON_CRASH, _industry_sound_tile);
 		}
 	}
 
 	if (_game_mode == GM_EDITOR)
 		return;
 
-	for(i=_industries; i != endof(_industries); i++) {
+	FOR_ALL_INDUSTRIES(i) {
 		if (i->xy != 0)
 			ProduceIndustryGoods(i);
 	}
@@ -1141,8 +1149,8 @@ static bool CheckNewIndustry_Oilwell(uint tile, int type)
 	if (type != IT_OIL_RIG && _game_mode == GM_EDITOR)
 		return true;
 
-	x = GET_TILE_X(tile);
-	y = GET_TILE_Y(tile);
+	x = TileX(tile);
+	y = TileY(tile);
 
 	if ( x < 15 || y < 15 || x > 238 || y > 238)
 		return true;
@@ -1213,10 +1221,10 @@ static CheckNewIndustryProc * const _check_new_industry_procs[] = {
 
 static bool CheckSuitableIndustryPos(uint tile)
 {
-	int x = GET_TILE_X(tile);
-	int y = GET_TILE_Y(tile);
+	uint x = TileX(tile);
+	uint y = TileY(tile);
 
-	if ( x < 2 || y < 2 || x > TILE_X_MAX - 3 || y > TILE_Y_MAX - 3) {
+	if ( x < 2 || y < 2 || x > MapMaxX() - 3 || y > MapMaxY() - 3) {
 		_error_message = STR_0239_SITE_UNSUITABLE;
 		return false;
 	}
@@ -1234,7 +1242,7 @@ static Town *CheckMultipleIndustryInTown(uint tile, int type)
 	if (_patches.multiple_industry_per_town)
 		return t;
 
-	for(i=_industries; i != endof(_industries); i++) {
+	FOR_ALL_INDUSTRIES(i) {
 		if (i->xy != 0 &&
 				i->type == (byte)type &&
 				i->town == t) {
@@ -1279,7 +1287,7 @@ static bool CheckIfIndustryTilesAreFree(uint tile, const IndustryTileTable *it,
 	_error_message = STR_0239_SITE_UNSUITABLE;
 
 	do {
-		cur_tile = tile + it->ti;
+		cur_tile = tile + ToTileIndexDiff(it->ti);
 		if (!IsValidTile(cur_tile)) {
 			if (it->map5 == 0xff)
 				continue;
@@ -1352,7 +1360,7 @@ do_clear:
 				}
 			}
 		}
-	} while ( (++it)->ti != -0x8000);
+	} while ((++it)->ti.x != -0x80);
 
 	return true;
 }
@@ -1367,12 +1375,15 @@ static bool CheckIfTooCloseToIndustry(uint tile, int type)
 	if (_patches.same_industry_close && (spec->accepts_cargo[0] == 0xFF) )
 		return true;
 
-	for(i=_industries; i != endof(_industries); i++) {
+	FOR_ALL_INDUSTRIES(i) {
 		// check if an industry that accepts the same goods is nearby
 		if (i->xy != 0 &&
 				(GetTileDist1D(tile, i->xy) <= 14) &&
 				spec->accepts_cargo[0] != 0xFF &&
-				spec->accepts_cargo[0] == i->accepts_cargo[0]) {
+				spec->accepts_cargo[0] == i->accepts_cargo[0] &&
+				!(_game_mode == GM_EDITOR &&
+					_patches.same_industry_close &&
+					_patches.multiple_industry_per_town)) {
 				_error_message = STR_INDUSTRY_TOO_CLOSE;
 				return false;
 				}
@@ -1388,14 +1399,15 @@ static bool CheckIfTooCloseToIndustry(uint tile, int type)
 	return true;
 }
 
-static Industry *AllocateIndustry()
+static Industry *AllocateIndustry(void)
 {
 	Industry *i;
 
-	for(i=_industries; i != endof(_industries); i++) {
+	FOR_ALL_INDUSTRIES(i) {
 		if (i->xy == 0) {
-			int index = i - _industries;
-		    if (index > _total_industries) _total_industries++;
+			if (i->index > _total_industries)
+				_total_industries = i->index;
+
 			return i;
 		}
 	}
@@ -1455,24 +1467,24 @@ static void DoCreateNewIndustry(Industry *i, uint tile, int type, const Industry
 	i->prod_level = 0x10;
 
 	do {
-		cur_tile = tile + it->ti;
+		cur_tile = tile + ToTileIndexDiff(it->ti);
 
 		if (it->map5 != 0xFF) {
 			byte size;
 
-			size = GET_TILE_X((TileIndex)it->ti);
+			size = it->ti.x;
 			if (size > i->width) i->width = size;
-			size = GET_TILE_Y((TileIndex)it->ti);
+			size = it->ti.y;
 			if (size > i->height)i->height = size;
 
 			DoCommandByTile(cur_tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
 
-			_map_type_and_height[cur_tile] = (_map_type_and_height[cur_tile]&~0xF0) | (MP_INDUSTRY<<4);
+			SetTileType(cur_tile, MP_INDUSTRY);
 			_map5[cur_tile] = it->map5;
-			_map2[cur_tile] = i - _industries;
+			_map2[cur_tile] = i->index;
 			_map_owner[cur_tile] = _generating_world ? 0x1E : 0; /* maturity */
 		}
-	} while ( (++it)->ti != -0x8000);
+	} while ((++it)->ti.x != -0x80);
 
 	i->width++;
 	i->height++;
@@ -1480,7 +1492,9 @@ static void DoCreateNewIndustry(Industry *i, uint tile, int type, const Industry
 	if (i->type == IT_FARM || i->type == IT_FARM_2) {
 		tile = i->xy + TILE_XY((i->width >> 1), (i->height >> 1));
 		for(j=0; j!=50; j++) {
-			uint new_tile = TileAddWrap(tile, Random() % 31 - 16, Random() % 31 - 16);
+			int x = Random() % 31 - 16;
+			int y = Random() % 31 - 16;
+			uint new_tile = TileAddWrap(tile, x, y);
 			if (new_tile != TILE_WRAPPED)
 				PlantFarmField(new_tile);
 		}
@@ -1584,6 +1598,8 @@ static void PlaceInitialIndustry(byte type, int amount)
 
 	if (_opt.diff.number_industries != 0)
 	{
+		byte old_player = _current_player;
+		_current_player = OWNER_NONE;
 		assert(num > 0);
 
 		do {
@@ -1593,10 +1609,12 @@ static void PlaceInitialIndustry(byte type, int amount)
 					break;
 			} while (--i != 0);
 		} while (--num);
+
+		_current_player = old_player;
 	}
 }
 
-void GenerateIndustries()
+void GenerateIndustries(void)
 {
 	const byte *b;
 
@@ -1643,10 +1661,10 @@ static void ExtChangeIndustryProduction(Industry *i)
 
 			mag = abs(percent);
 			if (mag >= 10) {
-				SET_DPARAM16(3, mag);
-				SET_DPARAM16(0,_cargoc.names_s[i->produced_cargo[j]]);
-				SET_DPARAM16(1, i->town->index);
-				SET_DPARAM16(2, i->type + STR_4802_COAL_MINE);
+				SetDParam(3, mag);
+				SetDParam(0,_cargoc.names_s[i->produced_cargo[j]]);
+				SetDParam(1, i->town->index);
+				SetDParam(2, i->type + STR_4802_COAL_MINE);
 				AddNewsItem(percent>=0 ? STR_INDUSTRY_PROD_GOUP : STR_INDUSTRY_PROD_GODOWN,
 						NEWS_FLAGS(NM_THIN, NF_VIEWPORT|NF_TILE, NT_ECONOMY, 0),
 						i->xy + TILE_XY(1,1), 0);
@@ -1656,8 +1674,8 @@ static void ExtChangeIndustryProduction(Industry *i)
 
 	if (closeit) {
 		i->prod_level = 0;
-		SET_DPARAM16(1, i->type + STR_4802_COAL_MINE);
-		SET_DPARAM16(0, i->town->index);
+		SetDParam(1, i->type + STR_4802_COAL_MINE);
+		SetDParam(0, i->town->index);
 		AddNewsItem(_industry_close_strings[i->type], NEWS_FLAGS(NM_THIN, NF_VIEWPORT|NF_TILE, NT_ECONOMY, 0), i->xy + TILE_XY(1,1), 0);
 	}
 }
@@ -1699,7 +1717,7 @@ static void UpdateIndustryStatistics(Industry *i)
 
 
 	if ( i->produced_cargo[0] != 0xFF || i->produced_cargo[1] != 0xFF )
-		InvalidateWindow(WC_INDUSTRY_VIEW, i - _industries);
+		InvalidateWindow(WC_INDUSTRY_VIEW, i->index);
 
 	if (i->prod_level == 0)
 		DeleteIndustry(i);
@@ -1728,7 +1746,6 @@ static void MaybeNewIndustry(uint32 r)
 		return;
 
 	j = 2000;
-	_current_player = OWNER_NONE;
 	for(;;) {
 		i = CreateNewIndustry(TILE_MASK(Random()), type);
 		if (i != NULL)
@@ -1737,8 +1754,8 @@ static void MaybeNewIndustry(uint32 r)
 			return;
 	}
 
-	SET_DPARAM16(0, type + STR_4802_COAL_MINE);
-	SET_DPARAM16(1, i->town->index);
+	SetDParam(0, type + STR_4802_COAL_MINE);
+	SetDParam(1, i->town->index);
 	AddNewsItem(	STR_482D_NEW_UNDER_CONSTRUCTION + (type == IT_FOREST), NEWS_FLAGS(NM_THIN,NF_VIEWPORT|NF_TILE,NT_ECONOMY,0), i->xy, 0);
 }
 
@@ -1790,23 +1807,25 @@ close_industry:
 			i->prod_level = 0;
 			str = _industry_close_strings[type];
 add_news:
-			SET_DPARAM16(1, type + STR_4802_COAL_MINE);
-			SET_DPARAM16(0, i->town->index);
+			SetDParam(1, type + STR_4802_COAL_MINE);
+			SetDParam(0, i->town->index);
 			AddNewsItem(str, NEWS_FLAGS(NM_THIN, NF_VIEWPORT|NF_TILE, NT_ECONOMY, 0), i->xy + TILE_XY(1,1), 0);
 		}
 	}
 }
 
-void IndustryMonthlyLoop()
+void IndustryMonthlyLoop(void)
 {
 	Industry *i;
+	byte old_player = _current_player;
+	_current_player = OWNER_NONE;
 
-	for(i=_industries; i != endof(_industries); i++) {
+	FOR_ALL_INDUSTRIES(i) {
 		if (i->xy != 0)
 			UpdateIndustryStatistics(i);
 	}
 
-	i = DEREF_INDUSTRY(RandomRange(lengthof(_industries)));
+	i = GetIndustry(RandomRange(_industries_size));
 
 	if (i->xy == 0) {
 		uint32 r;
@@ -1815,15 +1834,26 @@ void IndustryMonthlyLoop()
 	} else if (!_patches.smooth_economy) {
 		MaybeCloseIndustry(i);
 	}
+
+	_current_player = old_player;
+
 	// production-change
 	_industry_sort_dirty = true;
 	InvalidateWindow(WC_INDUSTRY_DIRECTORY, 0);
 }
 
 
-void InitializeIndustries()
+void InitializeIndustries(void)
 {
-	memset(_industries, 0, sizeof(_industries));
+	Industry *i;
+	int j;
+
+	memset(_industries, 0, sizeof(_industries[0]) * _industries_size);
+
+	j = 0;
+	FOR_ALL_INDUSTRIES(i)
+		i->index = j++;
+
 	_total_industries = 0;
 	_industry_sort_dirty = true;
 }
@@ -1875,27 +1905,28 @@ static const byte _industry_desc[] = {
 	SLE_END()
 };
 
-static void Save_INDY()
+static void Save_INDY(void)
 {
 	Industry *ind;
-	int i;
+
 	// Write the vehicles
-	for(ind=_industries,i=0; i!=lengthof(_industries); ind++,i++) {
+	FOR_ALL_INDUSTRIES(ind) {
 		if (ind->xy != 0) {
-			SlSetArrayIndex(i);
+			SlSetArrayIndex(ind->index);
 			SlObject(ind, _industry_desc);
 		}
 	}
 }
 
-static void Load_INDY()
+static void Load_INDY(void)
 {
 	int index;
 	_total_industries = 0;
 	while ((index = SlIterateArray()) != -1) {
-		SlObject(DEREF_INDUSTRY(index), _industry_desc);
-		if (index + 1 > _total_industries)
-		    _total_industries = index + 1;
+		Industry *i = GetIndustry(index);
+
+		SlObject(i, _industry_desc);
+		if (index > _total_industries) _total_industries = index;
 	}
 }
 
