@@ -1,20 +1,13 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "map.h"
 #include <stdarg.h>
 #include "gfx.h"
 #include "viewport.h"
 #include "command.h"
 #include "vehicle.h"
 
-byte _map_type_and_height[TILES_X * TILES_Y];
-byte _map5[TILES_X * TILES_Y];
-byte _map3_lo[TILES_X * TILES_Y];
-byte _map3_hi[TILES_X * TILES_Y];
-byte _map_owner[TILES_X * TILES_Y];
-byte _map2[TILES_X * TILES_Y];
-byte _map_extra_bits[TILES_X * TILES_Y/4];
-
-extern const TileTypeProcs 
+extern const TileTypeProcs
 	_tile_type_clear_procs,
 	_tile_type_rail_procs,
 	_tile_type_road_procs,
@@ -52,22 +45,22 @@ uint GetTileSlope(uint tile, int *h)
 	uint a,b,c,d,min;
 	int r;
 
-	if (GET_TILE_X(tile) == TILE_X_MAX || GET_TILE_Y(tile) == TILE_Y_MAX) {
+	if (TileX(tile) == MapMaxX() || TileY(tile) == MapMaxY()) {
 		if (h)
 			*h = 0;
 		return 0;
 	}
 
-	assert(tile < TILES_X * TILES_Y && GET_TILE_X(tile) != TILE_X_MAX && GET_TILE_Y(tile) != TILE_Y_MAX);
+	assert(tile < MapSize() && TileX(tile) != MapMaxX() && TileY(tile) != MapMaxY());
 
-	min = a = _map_type_and_height[tile] & 0xF;
-	b = _map_type_and_height[tile+TILE_XY(1,0)] & 0xF;
+	min = a = TileHeight(tile);
+	b = TileHeight(tile + TILE_XY(1,0));
 	if (min >= b) min = b;
-	c = _map_type_and_height[tile+TILE_XY(0,1)] & 0xF;
+	c = TileHeight(tile + TILE_XY(0,1));
 	if (min >= c) min = c;
-	d = _map_type_and_height[tile+TILE_XY(1,1)] & 0xF;
+	d = TileHeight(tile + TILE_XY(1,1));
 	if (min >= d) min = d;
-	
+
 	r = 0;
 	if ((a-=min)!=0) { r += (--a << 4) + 8; }
 	if ((c-=min)!=0) { r += (--c << 4) + 4; }
@@ -89,10 +82,9 @@ int GetTileZ(uint tile)
 
 void FindLandscapeHeightByTile(TileInfo *ti, uint tile)
 {
-	if (GET_TILE_X(tile) == TILE_X_MAX ||
-			GET_TILE_Y(tile) == TILE_Y_MAX) {
+	if (TileX(tile) == MapMaxX() || TileY(tile) == MapMaxY()) {
 		ti->tileh = 0;
-		ti->type = MP_STRANGE;
+		ti->type = MP_VOID;
 		ti->tile = 0;
 		ti->map5 = 0;
 		ti->z = 0;
@@ -101,7 +93,7 @@ void FindLandscapeHeightByTile(TileInfo *ti, uint tile)
 
 	ti->tile = tile;
 	ti->map5 = _map5[tile];
-	ti->type = GET_TILETYPE(tile);
+	ti->type = TileType(tile);
 	ti->tileh = GetTileSlope(tile, &ti->z);
 //	ti->z = min * 8;
 }
@@ -114,9 +106,9 @@ void FindLandscapeHeight(TileInfo *ti, uint x, uint y)
 	ti->x = x;
 	ti->y = y;
 
-	if (x >= TILE_X_MAX*16-1 || y >= TILE_Y_MAX*16-1) {
+	if (x >= MapMaxX() * 16 - 1 || y >= MapMaxY() * 16 - 1) {
 		ti->tileh = 0;
-		ti->type = MP_STRANGE;
+		ti->type = MP_VOID;
 		ti->tile = 0;
 		ti->map5 = 0;
 		ti->z = 0;
@@ -136,7 +128,7 @@ uint GetPartialZ(int x, int y, int corners)
 		if (x - y >= 0)
 			z = (x - y) >> 1;
 		break;
-	
+
 	case 2:
 		y^=0xF;
 		if ( (x - y) >= 0)
@@ -213,7 +205,7 @@ uint GetPartialZ(int x, int y, int corners)
 	case 29:
 		z = 1 + (((x^0xF)+(y^0xF))>>1);
 		break;
-		
+
 	case 30:
 		z = 1 + (((x^0xF)+(y^0xF))>>1);
 		break;
@@ -237,26 +229,55 @@ uint GetSlopeZ(int x,  int y)
 
 	assert(z < 256);
 */
-
 	return _tile_type_procs[ti.type]->get_slope_z_proc(&ti);
 }
 
+// direction=true:  check for foundation in east and south corner
+// direction=false: check for foundation in west and south corner
+static bool hasFoundation(TileInfo *ti, bool direction)
+{
+	bool south, other; // southern corner and east/west corner
+	uint slope = _tile_type_procs[ti->type]->get_slope_tileh_proc(ti);
+	uint tileh = ti->tileh;
+
+	if(slope==0 && slope!=tileh) tileh=15;
+	south = (tileh & 2) != (slope & 2);
+
+	if(direction)
+		other = (tileh & 4) != (slope & 4);
+	else
+		other = (tileh & 1) != (slope & 1);
+	return south || other;
+
+}
+
 void DrawFoundation(TileInfo *ti, uint f)
 {
+	uint32 sprite_base = SPR_SLOPES_BASE-14;
+
+	TileInfo ti2;
+	FindLandscapeHeight(&ti2, ti->x, ti->y-1);
+	if(hasFoundation( &ti2, true )) sprite_base += 22;		// foundation in NW direction
+	FindLandscapeHeight(&ti2, ti->x-1, ti->y);
+	if(hasFoundation( &ti2, false )) sprite_base += 22*2;	// foundation in NE direction
+
 	if (f < 15) {
-		// leveled foundation	
-		AddSortableSpriteToDraw(f + 0x3DE - 1, ti->x, ti->y, 16, 16, 7, ti->z);
+		// leveled foundation
+		if( sprite_base < SPR_SLOPES_BASE ) sprite_base = 990; // use original slope sprites
+
+		AddSortableSpriteToDraw(f-1 + sprite_base, ti->x, ti->y, 16, 16, 7, ti->z);
 		ti->z += 8;
 		ti->tileh = 0;
 		OffsetGroundSprite(31, 1);
 	} else {
 		// inclined foundation
-		
+		sprite_base += 14;
+
 		AddSortableSpriteToDraw(
-			HASBIT( (1<<1) | (1<<2) | (1<<4) | (1<<8), ti->tileh) ? (SPR_OPENTTD_BASE+17) + (f - 15) : 	ti->tileh + 0x3DE - 1,
+			HASBIT( (1<<1) | (1<<2) | (1<<4) | (1<<8), ti->tileh) ? sprite_base + (f - 15) : 	ti->tileh + 0x3DE - 1,
 			ti->x, ti->y, 1, 1, 1, ti->z
 		);
-		
+
 		ti->tileh = _inclined_tileh[f - 15];
 		OffsetGroundSprite(31, 9);
 	}
@@ -272,30 +293,30 @@ void DoClearSquare(uint tile)
 	);
 }
 
-uint32 GetTileTrackStatus(uint tile, int mode)
+uint32 GetTileTrackStatus(uint tile, TransportType mode)
 {
-	return _tile_type_procs[GET_TILETYPE(tile)]->get_tile_track_status_proc(tile, mode);
+	return _tile_type_procs[TileType(tile)]->get_tile_track_status_proc(tile, mode);
 }
 
 void ChangeTileOwner(uint tile, byte old_player, byte new_player)
 {
-	_tile_type_procs[GET_TILETYPE(tile)]->change_tile_owner_proc(tile, old_player, new_player);
+	_tile_type_procs[TileType(tile)]->change_tile_owner_proc(tile, old_player, new_player);
 }
 
-void GetAcceptedCargo(uint tile, AcceptedCargo *ac)
+void GetAcceptedCargo(uint tile, AcceptedCargo ac)
 {
 	memset(ac, 0, sizeof(AcceptedCargo));
-	_tile_type_procs[GET_TILETYPE(tile)]->get_accepted_cargo_proc(tile, ac);
+	_tile_type_procs[TileType(tile)]->get_accepted_cargo_proc(tile, ac);
 }
 
 void AnimateTile(uint tile)
 {
-	_tile_type_procs[GET_TILETYPE(tile)]->animate_tile_proc(tile);
+	_tile_type_procs[TileType(tile)]->animate_tile_proc(tile);
 }
 
 void ClickTile(uint tile)
 {
-	_tile_type_procs[GET_TILETYPE(tile)]->click_tile_proc(tile);
+	_tile_type_procs[TileType(tile)]->click_tile_proc(tile);
 }
 
 void DrawTile(TileInfo *ti)
@@ -305,7 +326,7 @@ void DrawTile(TileInfo *ti)
 
 void GetTileDesc(uint tile, TileDesc *td)
 {
-	_tile_type_procs[GET_TILETYPE(tile)]->get_tile_desc_proc(tile, td);
+	_tile_type_procs[TileType(tile)]->get_tile_desc_proc(tile, td);
 }
 
 /* Clear a piece of landscape
@@ -319,7 +340,7 @@ int32 CmdLandscapeClear(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
 	tile = TILE_FROM_XY(x,y);
-	return _tile_type_procs[GET_TILETYPE(tile)]->clear_tile_proc(tile, flags);
+	return _tile_type_procs[TileType(tile)]->clear_tile_proc(tile, flags);
 }
 
 // p1 = end tile
@@ -330,9 +351,11 @@ int32 CmdClearArea(int ex, int ey, uint32 flags, uint32 p1, uint32 p2)
 	int x,y;
 	bool success = false;
 
+	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
+
 	// make sure sx,sy are smaller than ex,ey
-	sx = GET_TILE_X(p1)*16;
-	sy = GET_TILE_Y(p1)*16;
+	sx = TileX(p1) * 16;
+	sy = TileY(p1) * 16;
 	if (ex < sx) intswap(ex, sx);
 	if (ey < sy) intswap(ey, sy);
 
@@ -351,7 +374,7 @@ int32 CmdClearArea(int ex, int ey, uint32 flags, uint32 p1, uint32 p2)
 					_additional_cash_required = ret;
 					return cost - ret;
 				}
-				DoCommandByTile(TILE_FROM_XY(x,y), 0, 0, flags, CMD_LANDSCAPE_CLEAR);	
+				DoCommandByTile(TILE_FROM_XY(x,y), 0, 0, flags, CMD_LANDSCAPE_CLEAR);
 
 				// draw explosion animation...
 				if ((x==sx || x==ex) && (y==sy || y==ey)) {
@@ -377,7 +400,7 @@ void CDECL ModifyTile(uint tile, uint flags, ...)
 	va_start(va, flags);
 
 	if ((i = (flags >> 8) & 0xF) != 0) {
-		_map_type_and_height[tile] = (_map_type_and_height[tile]&~0xF0)|((i-1) << 4);
+		SetTileType(tile, i - 1);
 	}
 
 	if (flags & (MP_MAP2_CLEAR | MP_MAP2)) {
@@ -422,15 +445,16 @@ void SetMapExtraBits(uint tile, byte bits)
 
 uint GetMapExtraBits(uint tile)
 {
-	return (_map_extra_bits[tile >> 2] >> (tile&3)*2)&3;
+	assert(tile < MapSize());
+	return (_map_extra_bits[tile >> 2] >> (tile & 3) * 2) & 3;
 }
 
 #define TILELOOP_BITS 4
 #define TILELOOP_SIZE (1 << TILELOOP_BITS)
-#define TILELOOP_ASSERTMASK ((TILELOOP_SIZE-1) + ((TILELOOP_SIZE-1) << TILE_X_BITS))
-#define TILELOOP_CHKMASK (((1 << (TILE_X_BITS - TILELOOP_BITS))-1) << TILELOOP_BITS)
+#define TILELOOP_ASSERTMASK ((TILELOOP_SIZE-1) + ((TILELOOP_SIZE-1) << MapLogX()))
+#define TILELOOP_CHKMASK (((1 << (MapLogX() - TILELOOP_BITS))-1) << TILELOOP_BITS)
 
-void RunTileLoop()
+void RunTileLoop(void)
 {
 	uint tile;
 	uint count;
@@ -438,56 +462,59 @@ void RunTileLoop()
 	tile = _cur_tileloop_tile;
 
 	assert( (tile & ~TILELOOP_ASSERTMASK) == 0);
-	count = (TILES_X/TILELOOP_SIZE) * (TILES_Y/TILELOOP_SIZE);
+	count = (MapSizeX() / TILELOOP_SIZE) * (MapSizeY() / TILELOOP_SIZE);
 	do {
-		_tile_type_procs[GET_TILETYPE(tile)]->tile_loop_proc(tile);
+		_tile_type_procs[TileType(tile)]->tile_loop_proc(tile);
 
-		if ( GET_TILE_X(tile) < TILES_X - TILELOOP_SIZE) {
+		if (TileX(tile) < MapSizeX() - TILELOOP_SIZE) {
 			tile += TILELOOP_SIZE; /* no overflow */
 		} else {
-			tile = TILE_MASK(tile - TILELOOP_SIZE * (TILES_X/TILELOOP_SIZE-1) + TILE_XY(0, TILELOOP_SIZE)); /* x would overflow, also increase y */
+			tile = TILE_MASK(tile - TILELOOP_SIZE * (MapSizeX() / TILELOOP_SIZE-1) + TILE_XY(0, TILELOOP_SIZE)); /* x would overflow, also increase y */
 		}
 	} while (--count);
 	assert( (tile & ~TILELOOP_ASSERTMASK) == 0);
 
 	tile += 9;
 	if (tile & TILELOOP_CHKMASK)
-		tile = (tile + TILES_X) & TILELOOP_ASSERTMASK;
+		tile = (tile + MapSizeX()) & TILELOOP_ASSERTMASK;
 	_cur_tileloop_tile = tile;
 }
 
-void InitializeLandscape()
+void InitializeLandscape(void)
 {
-	int i;
+	uint map_size = MapSize();
+	uint i;
 
-	memset(_map_owner, OWNER_NONE, sizeof(_map_owner));
-	memset(_map2, 0, sizeof(_map2));
-	memset(_map3_lo, 0, sizeof(_map3_lo));
-	memset(_map3_hi, 0, sizeof(_map3_hi));
-	memset(_map_extra_bits, 0, sizeof(_map_extra_bits));
-	memset(_map_type_and_height, MP_WATER << 4, sizeof(_map_type_and_height));
+	memset(_map_owner, OWNER_NONE, map_size);
+	memset(_map2, 0, map_size * sizeof(uint16));
+	memset(_map3_lo, 0, map_size);
+	memset(_map3_hi, 0, map_size);
+	memset(_map_extra_bits, 0, map_size / 4);
+	memset(_map_type_and_height, MP_CLEAR << 4, map_size);
 
-	for(i=0; i!=TILES_Y-1; i++)
-		memset(_map_type_and_height + i*TILES_X, 0, TILES_X-1);
+	// create void tiles on the border
+	for (i = 0; i != MapMaxY(); i++)
+		_map_type_and_height[ i * MapSizeX() + MapMaxY() ] = MP_VOID << 4;
+	memset(_map_type_and_height + MapMaxY() * MapSizeX(), MP_VOID << 4, MapSizeX());
 
-	memset(_map5, 3, sizeof(_map5));
+	memset(_map5, 3, map_size);
 }
 
-void ConvertGroundTilesIntoWaterTiles()
+void ConvertGroundTilesIntoWaterTiles(void)
 {
 	uint tile = 0;
 	int h;
 
 	while(true) {
-		if (IS_TILETYPE(tile, MP_CLEAR) && GetTileSlope(tile, &h) == 0 && h == 0) {
-			_map_type_and_height[tile] = MP_WATER << 4;
+		if (IsTileType(tile, MP_CLEAR) && GetTileSlope(tile, &h) == 0 && h == 0) {
+			SetTileType(tile, MP_WATER);
 			_map5[tile] = 0;
 			_map_owner[tile] = OWNER_WATER;
 		}
 		tile++;
-		if (GET_TILE_X(tile) == TILE_X_MAX) {
-			tile += TILE_XY(-TILE_X_MAX, 1);
-			if (GET_TILE_Y(tile) == TILE_Y_MAX)
+		if (TileX(tile) == MapMaxX()) {
+			tile += TILE_XY(-(int)MapMaxX(), 1);
+			if (TileY(tile) == MapMaxY())
 				break;
 		}
 	}
@@ -499,7 +526,7 @@ static const byte _genterrain_tbl_2[5] = { 0, 0, 0, 0, 33 };
 static void GenerateTerrain(int type, int flag)
 {
 	uint32 r;
-	int x,y;
+	uint x,y;
 	int w,h;
 	byte *p,*tile;
 	byte direction;
@@ -507,8 +534,8 @@ static void GenerateTerrain(int type, int flag)
 	r = Random();
 	p = GetSpritePtr((((r >> 24) * _genterrain_tbl_1[type]) >> 8) + _genterrain_tbl_2[type] + 4845);
 
-	x = r & TILE_X_MAX;
-	y = (r >> TILE_X_BITS) & TILE_Y_MAX;
+	x = r & MapMaxX();
+	y = (r >> MapLogX()) & MapMaxY();
 
 
 	if (x < 2 || y < 2)
@@ -518,7 +545,7 @@ static void GenerateTerrain(int type, int flag)
 	w = p[2];
 	h = p[1];
 	if (direction & 1) { w = p[1]; h = p[2]; }
-	p += 8;	
+	p += 8;
 
 	if (flag & 4) {
 		if (!(flag & 2)) {
@@ -540,10 +567,10 @@ static void GenerateTerrain(int type, int flag)
 		}
 	}
 
-	if (x + w >= TILE_X_MAX-1)
+	if (x + w >= MapMaxX() - 1)
 		return;
 
-	if (y + h >= TILE_Y_MAX-1)
+	if (y + h >= MapMaxY() - 1)
 		return;
 
 	tile = &_map_type_and_height[TILE_XY(x,y)];
@@ -600,51 +627,47 @@ static void GenerateTerrain(int type, int flag)
 
 #include "table/genland.h"
 
-static void CreateDesertOrRainForest()
+static void CreateDesertOrRainForest(void)
 {
 	uint tile;
-	const TileIndexDiff *data;
-	byte mt;
+	const TileIndexDiffC *data;
 	int i;
 
-	tile = 0;
-	do {
-		data = _make_desert_or_rainforest_data;
-		do {
-			if ((i = *data++) == MDORD_LAST) {
-				SetMapExtraBits(tile, 1);
-				break;
-			}
-			mt = _map_type_and_height[tile + i];
-		} while ((mt & 0xC) == 0 && (mt >> 4) != MP_WATER);
-	} while (++tile != TILES_X*TILES_Y);
-	
+	for (tile = 0; tile != MapSize(); ++tile) {
+		for (data = _make_desert_or_rainforest_data;
+				data != endof(_make_desert_or_rainforest_data); ++data) {
+			TileIndex t = tile + ToTileIndexDiff(*data);
+			if (TileHeight(t) >= 4 || IsTileType(t, MP_WATER)) break;
+		}
+		if (data == endof(_make_desert_or_rainforest_data))
+			SetMapExtraBits(tile, 1);
+	}
+
 	for(i=0; i!=256; i++)
 		RunTileLoop();
 
-	tile = 0;
-	do {
-		data = _make_desert_or_rainforest_data;
-		do {
-			if ((i = *data++) == MDORD_LAST) {
-				SetMapExtraBits(tile, 2);
-				break;
-			}
-		} while ( !IS_TILETYPE(tile+i, MP_CLEAR) || (_map5[tile + i]&0x1C) != 0x14);
-	} while (++tile != TILES_X*TILES_Y);
+	for (tile = 0; tile != MapSize(); ++tile) {
+		for (data = _make_desert_or_rainforest_data;
+				data != endof(_make_desert_or_rainforest_data); ++data) {
+			TileIndex t = TILE_MASK(tile + ToTileIndexDiff(*data));
+			if (IsTileType(t, MP_CLEAR) && (_map5[t] & 0x1c) == 0x14) break;
+		}
+		if (data == endof(_make_desert_or_rainforest_data))
+			SetMapExtraBits(tile, 2);
+	}
 }
 
-void GenerateLandscape()
+void GenerateLandscape(void)
 {
 	int i,flag;
 	uint32 r;
-	
+
 	if (_opt.landscape == LT_HILLY) {
 		i = ((Random() & 0x7F) + 950) * LANDSCAPE_SIZE_FACTOR;
 		do {
 			GenerateTerrain(2, 0);
 		} while (--i);
-		
+
 		r = Random();
 		flag = (r & 3) | 4;
 		i = (((r >> 16) & 0x7F) + 450) * LANDSCAPE_SIZE_FACTOR;
@@ -656,7 +679,7 @@ void GenerateLandscape()
 		do {
 			GenerateTerrain(0, 0);
 		} while (--i);
-		
+
 		r = Random();
 		flag = (r & 3) | 4;
 		i = (((r >> 16) & 0xFF) + 1700) * LANDSCAPE_SIZE_FACTOR;
@@ -668,7 +691,7 @@ void GenerateLandscape()
 
 		i = ((Random() & 0x7F) + 410) * LANDSCAPE_SIZE_FACTOR;
 		do {
-			GenerateTerrain(3, flag);	
+			GenerateTerrain(3, flag);
 		} while (--i);
 	} else {
 		i = ((Random() & 0x7F) + (3 - _opt.diff.quantity_sea_lakes)*256 + 100) * LANDSCAPE_SIZE_FACTOR;
@@ -683,15 +706,15 @@ void GenerateLandscape()
 		CreateDesertOrRainForest();
 }
 
-void OnTick_Town();
-void OnTick_Trees();
-void OnTick_Station();
-void OnTick_Industry();
+void OnTick_Town(void);
+void OnTick_Trees(void);
+void OnTick_Station(void);
+void OnTick_Industry(void);
 
-void OnTick_Players();
-void OnTick_Train();
+void OnTick_Players(void);
+void OnTick_Train(void);
 
-void CallLandscapeTick()
+void CallLandscapeTick(void)
 {
 	OnTick_Town();
 	OnTick_Trees();
@@ -708,20 +731,31 @@ TileIndex AdjustTileCoordRandomly(TileIndex a, byte rng)
 	uint32 r = Random();
 
 	return TILE_XY(
-		GET_TILE_X(a) + ((byte)r * rn * 2 >> 8) - rn,
-		GET_TILE_Y(a) + ((byte)(r>>8) * rn * 2 >> 8) - rn
+		TileX(a) + ((byte)r * rn * 2 >> 8) - rn,
+		TileY(a) + ((byte)(r >> 8) * rn * 2 >> 8) - rn
 	);
 }
 
-uint TileAddWrap(TileIndex tile, int add)
+// This function checks if we add addx/addy to tile, if we
+//  do wrap around the edges. For example, tile = (10,2) and
+//  addx = +3 and addy = -4. This function will now return
+//  TILE_WRAPPED, because the y is wrapped. This is needed in
+//  for example, farmland. When the tile is not wrapped,
+//  the result will be tile + TILE_XY(addx, addy)
+uint TileAddWrap(TileIndex tile, int addx, int addy)
 {
-	uint t = tile + add;
-	if (t < TILES_X * TILE_Y_MAX && GET_TILE_X(t) != TILE_X_MAX)
-		return t;
+	uint x, y;
+	x = TileX(tile) + addx;
+	y = TileY(tile) + addy;
+
+	// Are we about to wrap?
+	if (x < MapMaxX() && y < MapMaxY())
+		return tile + TILE_XY(addx, addy);
+
 	return TILE_WRAPPED;
 }
 
 bool IsValidTile(uint tile)
 {
-	return (tile < TILES_X * TILE_Y_MAX && GET_TILE_X(tile) != TILE_X_MAX);
+	return (tile < MapSizeX() * MapMaxY() && TileX(tile) != MapMaxX());
 }
