@@ -1,49 +1,101 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
+#include "map.h"
 #include "vehicle.h"
 #include "gfx.h"
 #include "assert.h"
 #include "saveload.h"
+#include "network.h"
+#include "network_data.h"
+#include "network_server.h"
+#include "engine.h"
 
-extern void StartupEconomy();
-extern void InitNewsItemStructs();
-
-static uint32 _random_seed_3, _random_seed_4;
+extern void StartupEconomy(void);
+extern void InitNewsItemStructs(void);
 
 byte _name_array[512][32];
 
-static INLINE uint32 ROR(uint32 x, int n)
+static inline uint32 ROR(uint32 x, int n)
 {
 	return (x >> n) + (x << ((sizeof(x)*8)-n));
 }
 
+/* XXX - Player-seeds don't seem to be used anymore.. which is a good thing
+     so I just disabled them for now. If there are no problems, we can remove
+     it completely! -- TrueLight */
+#undef PLAYER_SEED_RANDOM
 
-uint32 Random()
+#ifdef RANDOM_DEBUG
+#include "network_data.h"
+
+uint32 DoRandom(int line, const char *file)
+#else
+uint32 Random(void)
+#endif
 {
-	uint32 t = _random_seed_2;
-	uint32 s = _random_seed_1;
-	_random_seed_1 = s + ROR(t ^ 0x1234567F, 7);
-	return _random_seed_2 = ROR(s, 3);
+
+uint32 s;
+uint32 t;
+
+#ifdef RANDOM_DEBUG
+	if (_networking && (DEREF_CLIENT(0)->status != STATUS_INACTIVE || !_network_server))
+		printf("Random [%d/%d] %s:%d\n",_frame_counter, _current_player, file, line);
+#endif
+
+#ifdef PLAYER_SEED_RANDOM
+	if (_current_player>=MAX_PLAYERS || !_networking) {
+		s = _random_seeds[0][0];
+		t = _random_seeds[0][1];
+		_random_seeds[0][0] = s + ROR(t ^ 0x1234567F, 7);
+		return _random_seeds[0][1] = ROR(s, 3);
+	} else {
+		uint32 s = _player_seeds[_current_player][0];
+		uint32 t = _player_seeds[_current_player][1];
+		_player_seeds[_current_player][0] = s + ROR(t ^ 0x1234567F, 7);
+		DEBUG(net, 1)("[NET-Seeds] Player seed called!");
+		return _player_seeds[_current_player][1] = ROR(s, 3);
+	}
+#else
+	s = _random_seeds[0][0];
+	t = _random_seeds[0][1];
+	_random_seeds[0][0] = s + ROR(t ^ 0x1234567F, 7);
+	return _random_seeds[0][1] = ROR(s, 3);
+#endif
 }
 
+#ifdef RANDOM_DEBUG
+uint DoRandomRange(uint max, int line, const char *file)
+{
+	return (uint16)DoRandom(line, file) * max >> 16;
+}
+#else
 uint RandomRange(uint max)
 {
 	return (uint16)Random() * max >> 16;
 }
+#endif
 
-uint32 InteractiveRandom()
+uint32 InteractiveRandom(void)
 {
-	uint32 t = _random_seed_4;
-	uint32 s = _random_seed_3;
-	_random_seed_3 = s + ROR(t ^ 0x1234567F, 7);
-	return _random_seed_4 = ROR(s, 3);
+	uint32 t = _random_seeds[1][1];
+	uint32 s = _random_seeds[1][0];
+	_random_seeds[1][0] = s + ROR(t ^ 0x1234567F, 7);
+	return _random_seeds[1][1] = ROR(s, 3);
 }
 
-void memswap(void *a, void *b, size_t size) {
-	void *c = alloca(size);
-	memcpy(c, a, size);
-	memcpy(a, b, size);
-	memcpy(b, c, size);
+uint InteractiveRandomRange(uint max)
+{
+	return (uint16)InteractiveRandom() * max >> 16;
+}
+
+void InitPlayerRandoms(void)
+{
+	int i;
+	for (i=0; i<MAX_PLAYERS; i++) {
+		_player_seeds[i][0]=InteractiveRandom();
+		_player_seeds[i][1]=InteractiveRandom();
+	}
 }
 
 void SetDate(uint date)
@@ -52,41 +104,98 @@ void SetDate(uint date)
 	ConvertDayToYMD(&ymd, _date = date);
 	_cur_year = ymd.year;
 	_cur_month = ymd.month;
+#ifdef ENABLE_NETWORK
+	_network_last_advertise_date = 0;
+#endif /* ENABLE_NETWORK */
 }
 
-void InitializeClearLand();
-void InitializeRail();
-void InitializeRailGui();
-void InitializeRoad();
-void InitializeRoadGui();
-void InitializeAirportGui();
-void InitializeDock();
-void InitializeDockGui();
-void InitializeIndustries();
-void InitializeLandscape();
-void InitializeTowns();
-void InitializeTrees();
-void InitializeStations();
-void InitializeNameMgr();
-void InitializePlayers();
-void InitializeCheats();
 
-void GenerateLandscape();
-void GenerateClearTile();
+// multi os compatible sleep function
 
-void GenerateIndustries();
-void GenerateUnmovables();
-void GenerateTowns();
+#ifdef __AMIGA__
+// usleep() implementation
+#	include <devices/timer.h>
+#	include <dos/dos.h>
 
-void StartupPlayers();
-void StartupEngines();
-void StartupDisasters();
-void GenerateTrees();
+	extern struct Device      *TimerBase    = NULL;
+	extern struct MsgPort     *TimerPort    = NULL;
+	extern struct timerequest *TimerRequest = NULL;
+#endif // __AMIGA__
 
-void ConvertGroundTilesIntoWaterTiles();
-
-void InitializeGame()
+void CSleep(int milliseconds)
 {
+	#if defined(WIN32)
+		Sleep(milliseconds);
+	#endif
+	#if defined(UNIX)
+		#if !defined(__BEOS__) && !defined(__AMIGA__)
+			usleep(milliseconds * 1000);
+		#endif
+		#ifdef __BEOS__
+			snooze(milliseconds * 1000);
+		#endif
+		#if defined(__AMIGA__)
+		{
+			ULONG signals;
+			ULONG TimerSigBit = 1 << TimerPort->mp_SigBit;
+
+			// send IORequest
+			TimerRequest->tr_node.io_Command = TR_ADDREQUEST;
+			TimerRequest->tr_time.tv_secs    = (milliseconds * 1000) / 1000000;
+			TimerRequest->tr_time.tv_micro   = (milliseconds * 1000) % 1000000;
+			SendIO((struct IORequest *)TimerRequest);
+
+			if (!((signals = Wait(TimerSigBit | SIGBREAKF_CTRL_C)) & TimerSigBit) ) {
+				AbortIO((struct IORequest *)TimerRequest);
+			}
+			WaitIO((struct IORequest *)TimerRequest);
+		}
+		#endif // __AMIGA__
+	#endif
+}
+
+void InitializeVehicles(void);
+void InitializeOrders(void);
+void InitializeClearLand(void);
+void InitializeRail(void);
+void InitializeRailGui(void);
+void InitializeRoad(void);
+void InitializeRoadGui(void);
+void InitializeAirportGui(void);
+void InitializeDock(void);
+void InitializeDockGui(void);
+void InitializeIndustries(void);
+void InitializeLandscape(void);
+void InitializeTowns(void);
+void InitializeTrees(void);
+void InitializeSigns(void);
+void InitializeStations(void);
+static void InitializeNameMgr(void);
+void InitializePlayers(void);
+static void InitializeCheats(void);
+
+void GenerateLandscape(void);
+void GenerateClearTile(void);
+
+void GenerateIndustries(void);
+void GenerateUnmovables(void);
+void GenerateTowns(void);
+
+void StartupPlayers(void);
+void StartupDisasters(void);
+void GenerateTrees(void);
+
+void ConvertGroundTilesIntoWaterTiles(void);
+
+void InitializeGame(void)
+{
+	// Initialize the autoreplace array. Needs to be cleared between each game
+	uint i;
+	for (i = 0; i < lengthof(_autoreplace_array); i++)
+		_autoreplace_array[i] = i;
+
+	AddTypeToEngines(); // make sure all engines have a type
+
 	SetObjectToPlace(1, 0, 0, 0);
 
 	_pause = 0;
@@ -95,7 +204,7 @@ void InitializeGame()
 	_date_fract = 0;
 	_cur_tileloop_tile = 0;
 	_vehicle_id_ctr_day = 0;
-	
+
 	{
 		uint starting = ConvertIntDate(_patches.starting_date);
 		if ( starting == (uint)-1) starting = 10958;
@@ -103,7 +212,7 @@ void InitializeGame()
 	}
 
 	InitializeVehicles();
-	_backup_orders_tile = 0;
+	InitializeOrders();
 
 	InitNewsItemStructs();
 	InitializeLandscape();
@@ -117,16 +226,19 @@ void InitializeGame()
 	InitializeDockGui();
 	InitializeTowns();
 	InitializeTrees();
+	InitializeSigns();
 	InitializeStations();
 	InitializeIndustries();
-	
+
 	InitializeNameMgr();
+	InitializeVehiclesGuiList();
 	InitializeTrains();
 
 	InitializePlayers();
 	InitializeCheats();
 
 	InitTextEffects();
+	InitTextMessage();
 	InitializeAnimatedTiles();
 
 	InitializeLandscapeVariables(false);
@@ -138,6 +250,9 @@ void GenerateWorld(int mode)
 {
 	int i;
 
+	// Make sure everything is done via OWNER_NONE
+	_current_player = OWNER_NONE;
+
 	_generating_world = true;
 	InitializeGame();
 	SetObjectToPlace(1, 0, 0, 0);
@@ -152,7 +267,7 @@ void GenerateWorld(int mode)
 	} else {
 		GenerateLandscape();
 		GenerateClearTile();
-		
+
 		// only generate towns, tree and industries in newgame mode.
 		if (mode == 0) {
 			GenerateTowns();
@@ -162,7 +277,7 @@ void GenerateWorld(int mode)
 		}
 	}
 
-	// These are probably pointless when inside the scenario editor.	
+	// These are probably pointless when inside the scenario editor.
 	StartupPlayers();
 	StartupEngines();
 	StartupDisasters();
@@ -208,18 +323,18 @@ byte *GetName(int id, byte *buff)
 }
 
 
-void InitializeCheats()
+static void InitializeCheats(void)
 {
-	memset(&_cheats, 0, sizeof(Cheats)); 
+	memset(&_cheats, 0, sizeof(Cheats));
 }
 
 
-void InitializeNameMgr()
+static void InitializeNameMgr(void)
 {
 	memset(_name_array, 0, sizeof(_name_array));
 }
 
-StringID AllocateName(const byte *name, byte skip)
+StringID RealAllocateName(const byte *name, byte skip, bool check_double)
 {
 	int free_item = -1;
 	const byte *names;
@@ -233,7 +348,7 @@ StringID AllocateName(const byte *name, byte skip)
 			if (free_item == -1)
 				free_item = i;
 		} else {
-			if (str_eq(names, name)) {
+			if (check_double && str_eq(names, name)) {
 				_error_message = STR_0132_CHOSEN_NAME_IN_USE_ALREADY;
 				return 0;
 			}
@@ -249,17 +364,10 @@ StringID AllocateName(const byte *name, byte skip)
 
 	for(i=0; (dst[i] = name[i]) != 0 && ++i != 32; ) {}
 	dst[31] = 0;
-		
+
 	return free_item | 0x7800 | (skip << 8);
 }
 
-const TileIndexDiff _tileoffs_by_dir[4] = {
-	TILE_XY(-1, 0),
-	TILE_XY(0, 1),
-	TILE_XY(1, 0),
-	TILE_XY(0, -1),
-};
-
 
 #define M(a,b) ((a<<5)|b)
 static const uint16 _month_date_from_year_day[] = {
@@ -318,7 +426,7 @@ void ConvertDayToYMD(YearMonthDay *ymd, uint16 date)
 	}
 
 	ymd->year = yr;
-		
+
 	x = _month_date_from_year_day[rem];
 	ymd->month = x >> 5;
 	ymd->day = x & 0x1F;
@@ -330,7 +438,7 @@ void ConvertDayToYMD(YearMonthDay *ymd, uint16 date)
 uint ConvertYMDToDay(uint year, uint month, uint day)
 {
 	uint rem;
-	
+
 	// day in the year
 	rem = _accum_days_for_month[month] + day - 1;
 
@@ -342,7 +450,7 @@ uint ConvertYMDToDay(uint year, uint month, uint day)
 }
 
 // convert a date on the form
-// 1920 - 2090
+// 1920 - 2090 (MAX_YEAR_END_REAL)
 // 192001 - 209012
 // 19200101 - 20901231
 // or if > 2090 and below 65536, treat it as a daycount
@@ -351,14 +459,14 @@ uint ConvertIntDate(uint date)
 {
 	uint year, month = 0, day = 1;
 
-	if (IS_INT_INSIDE(date, 1920, 2090 + 1)) {
+	if (IS_INT_INSIDE(date, 1920, MAX_YEAR_END_REAL + 1)) {
 		year = date - 1920;
 	} else if (IS_INT_INSIDE(date, 192001, 209012+1)) {
-		month = date % 100 - 1;	
+		month = date % 100 - 1;
 		year = date / 100 - 1920;
 	} else if (IS_INT_INSIDE(date, 19200101, 20901231+1)) {
 		day = date % 100; date /= 100;
-		month = date % 100 - 1;	
+		month = date % 100 - 1;
 		year = date / 100 - 1920;
 	} else if (IS_INT_INSIDE(date, 2091, 65536))
 		return date;
@@ -404,7 +512,7 @@ void InitializeLandscapeVariables(bool only_constants)
 
 	for(i=0; i!=NUM_CARGO; i++) {
 		_cargoc.sprites[i] = lpd->sprites[i];
-		
+
 		str = lpd->names[i];
 		_cargoc.names_s[i] = str;
 		_cargoc.names_p[i] = (str += 0x20);
@@ -412,7 +520,7 @@ void InitializeLandscapeVariables(bool only_constants)
 		_cargoc.names_long_p[i] = (str += 0x20);
 		_cargoc.names_short[i] = (str += 0x20);
 		_cargoc.weights[i] = lpd->weights[i];
-		
+
 		if (!only_constants) {
 			_cargo_payment_rates[i] = lpd->initial_cargo_payment[i];
 			_cargo_payment_rates_frac[i] = 0;
@@ -426,21 +534,21 @@ void InitializeLandscapeVariables(bool only_constants)
 // distance in Manhattan metric
 uint GetTileDist(TileIndex xy1, TileIndex xy2)
 {
-	return myabs(GET_TILE_X(xy1) - GET_TILE_X(xy2)) + 
-				myabs(GET_TILE_Y(xy1) - GET_TILE_Y(xy2));
+	return myabs(TileX(xy1) - TileX(xy2)) +
+				myabs(TileY(xy1) - TileY(xy2));
 }
 
 // maximum distance in x _or_ y
 uint GetTileDist1D(TileIndex xy1, TileIndex xy2)
 {
-	return max(myabs(GET_TILE_X(xy1) - GET_TILE_X(xy2)),
-						 myabs(GET_TILE_Y(xy1) - GET_TILE_Y(xy2)));
+	return max(myabs(TileX(xy1) - TileX(xy2)),
+						 myabs(TileY(xy1) - TileY(xy2)));
 }
 
 uint GetTileDist1Db(TileIndex xy1, TileIndex xy2)
 {
-	int a = myabs(GET_TILE_X(xy1) - GET_TILE_X(xy2));
-	int b = myabs(GET_TILE_Y(xy1) - GET_TILE_Y(xy2));
+	int a = myabs(TileX(xy1) - TileX(xy2));
+	int b = myabs(TileY(xy1) - TileY(xy2));
 
 	if (a > b)
 		return a*2+b;
@@ -450,22 +558,22 @@ uint GetTileDist1Db(TileIndex xy1, TileIndex xy2)
 
 uint GetTileDistAdv(TileIndex xy1, TileIndex xy2)
 {
-	uint a = myabs(GET_TILE_X(xy1) - GET_TILE_X(xy2));
-	uint b = myabs(GET_TILE_Y(xy1) - GET_TILE_Y(xy2));
+	uint a = myabs(TileX(xy1) - TileX(xy2));
+	uint b = myabs(TileY(xy1) - TileY(xy2));
 	return a*a+b*b;
 }
 
 bool CheckDistanceFromEdge(TileIndex tile, uint distance)
 {
-	return IS_INT_INSIDE(GET_TILE_X(tile), distance, TILE_X_MAX + 1 - distance) &&
-			IS_INT_INSIDE(GET_TILE_Y(tile), distance, TILE_Y_MAX + 1 - distance);
+	return IS_INT_INSIDE(TileX(tile), distance, MapSizeX() - distance) &&
+			IS_INT_INSIDE(TileY(tile), distance, MapSizeY() - distance);
 }
 
 void OnNewDay_Train(Vehicle *v);
 void OnNewDay_RoadVeh(Vehicle *v);
 void OnNewDay_Aircraft(Vehicle *v);
 void OnNewDay_Ship(Vehicle *v);
-void OnNewDay_EffectVehicle(Vehicle *v) { /* empty */ }
+static void OnNewDay_EffectVehicle(Vehicle *v) { /* empty */ }
 void OnNewDay_DisasterVehicle(Vehicle *v);
 
 typedef void OnNewVehicleDayProc(Vehicle *v);
@@ -479,21 +587,21 @@ static OnNewVehicleDayProc * _on_new_vehicle_day_proc[] = {
 	OnNewDay_DisasterVehicle,
 };
 
-void EnginesDailyLoop();
-void DisasterDailyLoop();
-void PlayersMonthlyLoop();
-void EnginesMonthlyLoop();
-void CitiesMonthlyLoop();
-void IndustryMonthlyLoop();
-void StationMonthlyLoop();
+void EnginesDailyLoop(void);
+void DisasterDailyLoop(void);
+void PlayersMonthlyLoop(void);
+void EnginesMonthlyLoop(void);
+void TownsMonthlyLoop(void);
+void IndustryMonthlyLoop(void);
+void StationMonthlyLoop(void);
 
-void PlayersYearlyLoop();
-void TrainsYearlyLoop();
-void RoadVehiclesYearlyLoop();
-void AircraftYearlyLoop();
-void ShipsYearlyLoop();
+void PlayersYearlyLoop(void);
+void TrainsYearlyLoop(void);
+void RoadVehiclesYearlyLoop(void);
+void AircraftYearlyLoop(void);
+void ShipsYearlyLoop(void);
 
-void CheckpointsDailyLoop();
+void WaypointsDailyLoop(void);
 
 
 static const uint16 _autosave_months[] = {
@@ -504,9 +612,12 @@ static const uint16 _autosave_months[] = {
 	0x001, // every 12 months
 };
 
-void IncreaseDate()
+void IncreaseDate(void)
 {
-	int i,ctr,t;
+	const int vehicles_per_day = (1 << (sizeof(_date_fract) * 8)) / 885;
+	uint i;
+	VehicleID ctr;
+	int t;
 	YearMonthDay ymd;
 
 	if (_game_mode == GM_MENU) {
@@ -520,23 +631,24 @@ void IncreaseDate()
 		65536 / 885 = 74; 74x12 = 888. So max 888. Any vehicles above that were not _on_new_vehicle_day_proc'd
 		eg. aged.
 		So new code updates it for max vehicles.
-		(NUM_VEHICLES / maximum number of times ctr is incremented before reset ) + 1 (to get last vehicles too)
+		(_vehicles_size / maximum number of times ctr is incremented before reset ) + 1 (to get last vehicles too)
 		max size of _date_fract / 885 (added each tick) is number of times before ctr is reset.
 		Calculation might look complicated, but compiler just replaces it with 35, so that's ok
 	*/
 
 	ctr = _vehicle_id_ctr_day;
-	for(i=0; i!=(NUM_VEHICLES / ((1<<sizeof(_date_fract)*8) / 885)) + 1 && ctr != lengthof(_vehicles); i++) {
-		Vehicle *v = &_vehicles[ctr++];
-		if ((t=v->type) != 0)
+	for (i = 0; i != (_vehicles_size / vehicles_per_day) + 1 && ctr != _vehicles_size; i++) {
+		Vehicle *v = GetVehicle(ctr++);
+		if ((t = v->type) != 0)
 			_on_new_vehicle_day_proc[t - 0x10](v);
 	}
 	_vehicle_id_ctr_day = ctr;
 
 	/* increase day, and check if a new day is there? */
 	_tick_counter++;
-	
-	if ( (_date_fract += 885) >= 885)
+
+	_date_fract += 885;
+	if (_date_fract >= 885)
 		return;
 
 	/* yeah, increse day counter and call various daily loops */
@@ -544,8 +656,10 @@ void IncreaseDate()
 
 	_vehicle_id_ctr_day = 0;
 
+	TextMessageDailyLoop();
+
 	DisasterDailyLoop();
-	CheckpointsDailyLoop();
+	WaypointsDailyLoop();
 
 	if (_game_mode != GM_MENU) {
 		InvalidateWindowWidget(WC_STATUS_BAR, 0, 0);
@@ -558,8 +672,6 @@ void IncreaseDate()
 		return;
 	_cur_month = ymd.month;
 
-//	printf("Month %d, %X\n", ymd.month, _random_seed_1);
-
 	/* yes, call various monthly loops */
 	if (_game_mode != GM_MENU) {
 		if (HASBIT(_autosave_months[_opt.autosave], _cur_month)) {
@@ -569,9 +681,13 @@ void IncreaseDate()
 
 		PlayersMonthlyLoop();
 		EnginesMonthlyLoop();
-		CitiesMonthlyLoop();
+		TownsMonthlyLoop();
 		IndustryMonthlyLoop();
 		StationMonthlyLoop();
+#ifdef ENABLE_NETWORK
+		if (_network_server)
+			NetworkServerMonthlyLoop();
+#endif /* ENABLE_NETWORK */
 	}
 
 	/* check if we entered a new year? */
@@ -579,18 +695,29 @@ void IncreaseDate()
 		return;
 	_cur_year = ymd.year;
 
-	/* yes, call various yearly loops */ 
+	/* yes, call various yearly loops */
 
 	PlayersYearlyLoop();
 	TrainsYearlyLoop();
 	RoadVehiclesYearlyLoop();
 	AircraftYearlyLoop();
 	ShipsYearlyLoop();
+#ifdef ENABLE_NETWORK
+	if (_network_server)
+		NetworkServerYearlyLoop();
+#endif /* ENABLE_NETWORK */
 
-	/* check if we reached 2090, that's the maximum year. */
-	if (_cur_year == 171) {
-		_cur_year = 170;
+	/* check if we reached end of the game (31 dec 2050) */
+	if (_cur_year == _patches.ending_date - MAX_YEAR_BEGIN_REAL) {
+			ShowEndGameChart();
+	/* check if we reached 2090 (MAX_YEAR_END_REAL), that's the maximum year. */
+	} else if (_cur_year == (MAX_YEAR_END + 1)) {
+		Vehicle *v;
+		_cur_year = MAX_YEAR_END;
 		_date = 62093;
+		FOR_ALL_VEHICLES(v) {
+			v->date_of_last_service -= 365; // 1 year is 365 days long
+		}
 	}
 
 	if (_patches.auto_euro)
@@ -599,37 +726,23 @@ void IncreaseDate()
 	/* XXX: check if year 2050 was reached */
 }
 
-int FindFirstBit(uint32 x)
+int FindFirstBit(uint32 value)
 {
-	int i = 0;
-	assert(x != 0);
-	for(;!(x&1);i++,x>>=1);
+	// This is much faster than the one that was before here.
+	//  Created by Darkvater.. blame him if it is wrong ;)
+	// Btw, the macro FINDFIRSTBIT is better to use when your value is
+	//  not more than 128.
+	byte i = 0;
+	if (value & 0xffff0000) { value >>= 16; i += 16; }
+	if (value & 0x0000ff00) { value >>= 8;  i += 8; }
+	if (value & 0x000000f0) { value >>= 4;  i += 4; }
+	if (value & 0x0000000c) { value >>= 2;  i += 2; }
+	if (value & 0x00000002) { i += 1; }
 	return i;
 }
 
 
-extern uint SafeTileAdd(uint tile, int add, const char *exp, const char *file, int line)
-{
-	int x = GET_TILE_X(tile) + (signed char)(add & 0xFF);
-	int y = GET_TILE_Y(tile) + ((((0x8080 + add)>>8) & 0xFF) - 0x80);
-
-	if (x < 0 || y < 0 || x >= TILES_X || y >= TILES_Y) {
-		char buf[512];
-
-		sprintf(buf, "TILE_ADD(%s) when adding 0x%.4X and %d failed", exp, tile, add);
-#if !defined(_DEBUG) || !defined(_MSC_VER)
-		printf("%s\n", buf);
-#else
-		_assert(buf, (char*)file, line);
-#endif
-	}
-
-	assert(TILE_XY(x,y) == TILE_MASK(tile + add));
-
-	return TILE_XY(x,y);
-}
-
-static void Save_NAME()
+static void Save_NAME(void)
 {
 	int i;
 	byte *b = _name_array[0];
@@ -638,14 +751,14 @@ static void Save_NAME()
 		if (*b) {
 			SlSetArrayIndex(i);
 			SlArray(b, strlen(b), SLE_UINT8);
-		}	
+		}
 	}
 }
 
-static void Load_NAME()
+static void Load_NAME(void)
 {
 	int index;
-	
+
 	while ((index = SlIterateArray()) != -1) {
 		SlArray(_name_array[index],SlGetFieldLength(),SLE_UINT8);
 	}
@@ -658,7 +771,7 @@ static const byte _game_opt_desc[] = {
 	SLE_VAR(GameOptions,diff_level,			SLE_UINT8),
 	SLE_VAR(GameOptions,currency,				SLE_UINT8),
 	SLE_VAR(GameOptions,kilometers,			SLE_UINT8),
-	SLE_VAR(GameOptions,city_name,			SLE_UINT8),
+	SLE_VAR(GameOptions,town_name,			SLE_UINT8),
 	SLE_VAR(GameOptions,landscape,			SLE_UINT8),
 	SLE_VAR(GameOptions,snow_line,			SLE_UINT8),
 	SLE_VAR(GameOptions,autosave,				SLE_UINT8),
@@ -667,9 +780,9 @@ static const byte _game_opt_desc[] = {
 };
 
 // Save load game options
-static void SaveLoad_OPTS()
+static void SaveLoad_OPTS(void)
 {
-	SlObject(&_opt, _game_opt_desc);	
+	SlObject(&_opt, _game_opt_desc);
 }
 
 
@@ -683,66 +796,78 @@ static const SaveLoadGlobVarList _date_desc[] = {
 	{&_cur_tileloop_tile, 			SLE_UINT16, 0, 255},
 	{&_disaster_delay, 					SLE_UINT16, 0, 255},
 	{&_station_tick_ctr, 				SLE_UINT16, 0, 255},
-	{&_random_seed_1, 					SLE_UINT32, 0, 255},
-	{&_random_seed_2, 					SLE_UINT32, 0, 255},
+	{&_random_seeds[0][0], 					SLE_UINT32, 0, 255},
+	{&_random_seeds[0][1], 					SLE_UINT32, 0, 255},
 	{&_cur_town_ctr, 						SLE_UINT8,	0, 255},
 	{&_cur_player_tick_index, 	SLE_FILE_U8 | SLE_VAR_UINT, 0, 255},
 	{&_next_competitor_start, 	SLE_FILE_U16 | SLE_VAR_UINT, 0, 255},
 	{&_trees_tick_ctr, 					SLE_UINT8,	0, 255},
 	{&_pause, 									SLE_UINT8,	4, 255},
-	{NULL,0}
+	{NULL,											0,					0,   0}
 };
 
 // Save load date related variables as well as persistent tick counters
 // XXX: currently some unrelated stuff is just put here
-static void SaveLoad_DATE()
+static void SaveLoad_DATE(void)
 {
 	SlGlobList(_date_desc);
 }
 
 
 static const SaveLoadGlobVarList _view_desc[] = {
-	{&_saved_scrollpos_x, SLE_FILE_I16 | SLE_VAR_INT, 0, 255},
-	{&_saved_scrollpos_y, SLE_FILE_I16 | SLE_VAR_INT, 0, 255},
-	{&_saved_scrollpos_zoom, SLE_UINT8, 0, 255},
-	{NULL,0}
+	{&_saved_scrollpos_x,			SLE_FILE_I16 | SLE_VAR_INT, 0, 255},
+	{&_saved_scrollpos_y,			SLE_FILE_I16 | SLE_VAR_INT, 0, 255},
+	{&_saved_scrollpos_zoom,	SLE_UINT8,	0, 255},
+	{NULL,										0,					0,   0}
 };
 
-static void SaveLoad_VIEW()
+static void SaveLoad_VIEW(void)
 {
 	SlGlobList(_view_desc);
 }
 
-static void SaveLoad_MAPT() {
-  SlArray(_map_type_and_height, lengthof(_map_type_and_height), SLE_UINT8);
+static void SaveLoad_MAPT(void)
+{
+  SlArray(_map_type_and_height, MapSize(), SLE_UINT8);
 }
 
-static void SaveLoad_MAP2() {
-  SlArray(_map2, lengthof(_map2), SLE_UINT8);
+static void SaveLoad_MAP2(void)
+{
+	if (_sl.version < 5) {
+		/* In those versions the _map2 was 8 bits */
+		SlArray(_map2, MapSize(), SLE_FILE_U8 | SLE_VAR_U16);
+	} else {
+		SlArray(_map2, MapSize(), SLE_UINT16);
+	}
 }
 
-static void SaveLoad_M3LO() {
-  SlArray(_map3_lo, lengthof(_map3_lo), SLE_UINT8);
+static void SaveLoad_M3LO(void)
+{
+  SlArray(_map3_lo, MapSize(), SLE_UINT8);
 }
 
-static void SaveLoad_M3HI() {
-  SlArray(_map3_hi, lengthof(_map3_hi), SLE_UINT8);
+static void SaveLoad_M3HI(void)
+{
+  SlArray(_map3_hi, MapSize(), SLE_UINT8);
 }
 
-static void SaveLoad_MAPO() {
-  SlArray(_map_owner, lengthof(_map_owner), SLE_UINT8);
+static void SaveLoad_MAPO(void)
+{
+  SlArray(_map_owner, MapSize(), SLE_UINT8);
 }
 
-static void SaveLoad_MAP5() {
-  SlArray(_map5, lengthof(_map5), SLE_UINT8);
+static void SaveLoad_MAP5(void)
+{
+  SlArray(_map5, MapSize(), SLE_UINT8);
 }
 
-static void SaveLoad_MAPE() {
-  SlArray(_map_extra_bits, lengthof(_map_extra_bits), SLE_UINT8);
+static void SaveLoad_MAPE(void)
+{
+  SlArray(_map_extra_bits, MapSize() / 4, SLE_UINT8);
 }
 
 
-static void Save_CHTS()
+static void Save_CHTS(void)
 {
 	byte count = sizeof(_cheats)/sizeof(Cheat);
 	Cheat* cht = (Cheat*) &_cheats;
@@ -755,7 +880,7 @@ static void Save_CHTS()
 	}
 }
 
-static void Load_CHTS()
+static void Load_CHTS(void)
 {
 	Cheat* cht = (Cheat*) &_cheats;
 
