@@ -1,18 +1,23 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
+#include "map.h"
 #include "player.h"
-#include "city.h"
+#include "town.h"
 #include "vehicle.h"
 #include "station.h"
 #include "gfx.h"
 #include "news.h"
 #include "saveload.h"
 #include "command.h"
+#include "ai.h"
+#include "sound.h"
+#include "network.h"
 
-extern void StartupEconomy();
+extern void StartupEconomy(void);
 
 static const SpriteID cheeks_table[4] = {
-	0x325, 0x326, 
+	0x325, 0x326,
 	0x390, 0x3B0,
 };
 
@@ -59,10 +64,10 @@ void DrawPlayerFace(uint32 face, int color, int x, int y)
 
 		if (!(flag & 2)) {
 			if (!(flag & 1)) {
-				DrawSprite(high+((val1 * 12 >> 4) + 0x832B), x, y); 			
+				DrawSprite(high+((val1 * 12 >> 4) + 0x832B), x, y);
 			} else {
 				DrawSprite(high+(val1 + 0x8337), x, y);
-			}	
+			}
 		} else {
 			if (!(flag & 1)) {
 				DrawSprite(high+((val1 * 11 >> 4) + 0x839A), x, y);
@@ -79,11 +84,11 @@ void DrawPlayerFace(uint32 face, int color, int x, int y)
 
 		if (!(flag&1)) {
 			val2 = ((val&0xF) * 15 >> 4);
-			
+
 			if (val2 < 3) {
 				DrawSprite((flag&2 ? 0x397 : 0x367) + val2, x, y);
 				/* skip the rest */
-				goto skip_mouth;				
+				goto skip_mouth;
 			}
 
 			val2 -= 3;
@@ -147,7 +152,7 @@ void DrawPlayerFace(uint32 face, int color, int x, int y)
 		} else {
 			DrawSprite(0x378 + ((val&3)*3>>2), x, y);
 			DrawSprite(0x37B + ((val>>2)&3), x, y);
-			
+
 			val >>= 4;
 			if (val < 3) {
 				DrawSprite((flag&2 ? 0x3D1 : 0x37F) + val, x, y);
@@ -185,7 +190,7 @@ bool CheckPlayerHasMoney(int32 cost)
 	if (cost > 0) {
 		uint pid = _current_player;
 		if (pid < MAX_PLAYERS && cost > DEREF_PLAYER(pid)->player_money) {
-			SET_DPARAM32(0, cost);
+			SetDParam(0, cost);
 			_error_message = STR_0003_NOT_ENOUGH_CASH_REQUIRES;
 			return false;
 		}
@@ -197,14 +202,14 @@ static void SubtractMoneyFromAnyPlayer(Player *p, int32 cost)
 {
 	p->money64 -= cost;
 	UpdatePlayerMoney32(p);
-	
+
 	p->yearly_expenses[0][_yearly_expenses_type] += cost;
 
 	if ( ( 1 << _yearly_expenses_type ) & (1<<7|1<<8|1<<9|1<<10))
 		p->cur_economy.income -= cost;
 	else if (( 1 << _yearly_expenses_type ) & (1<<2|1<<3|1<<4|1<<5|1<<6|1<<11))
 		p->cur_economy.expenses -= cost;
-		
+
 	InvalidatePlayerWindows(p);
 }
 
@@ -235,25 +240,25 @@ void UpdatePlayerMoney32(Player *p)
 	else if (p->money64 > 2000000000)
 		p->player_money = 2000000000;
 	else
-		p->player_money = (int32)p->money64; 
+		p->player_money = (int32)p->money64;
 }
 
 void GetNameOfOwner(byte owner, uint tile)
 {
-	SET_DPARAM8(2, owner);
-		
+	SetDParam(2, owner);
+
 	if (owner != OWNER_TOWN) {
 		if (owner >= 8)
-			SET_DPARAM16(0, STR_0150_SOMEONE);
+			SetDParam(0, STR_0150_SOMEONE);
 		else {
 			Player *p = DEREF_PLAYER(owner);
-			SET_DPARAM16(0, p->name_1);
-			SET_DPARAM32(1, p->name_2);
+			SetDParam(0, p->name_1);
+			SetDParam(1, p->name_2);
 		}
 	} else {
-		City *city = ClosestCityFromTile(tile, (uint)-1);
-		SET_DPARAM16(0, city->citynametype);
-		SET_DPARAM32(1, city->citynameparts);
+		Town *t = ClosestTownFromTile(tile, (uint)-1);
+		SetDParam(0, t->townnametype);
+		SetDParam(1, t->townnameparts);
 	}
 }
 
@@ -273,10 +278,10 @@ bool CheckTileOwnership(uint tile)
 {
 	byte owner = _map_owner[tile];
 	assert(owner <= OWNER_WATER);
-	if (owner == _current_player) 
+	if (owner == _current_player)
 		return true;
 	_error_message = STR_013B_OWNED_BY;
-	
+
 	// no need to get the name of the owner unless we're the local player (saves some time)
 	if (_current_player == _local_player)
 		GetNameOfOwner(owner, tile);
@@ -286,7 +291,7 @@ bool CheckTileOwnership(uint tile)
 static void GenerateCompanyName(Player *p)
 {
 	uint tile;
-	City *c;
+	Town *t;
 	StringID str;
 	Player *pp;
 	uint32 strp;
@@ -299,22 +304,22 @@ static void GenerateCompanyName(Player *p)
 	if (tile == 0)
 		return;
 
-	c = ClosestCityFromTile(tile, (uint)-1);
+	t = ClosestTownFromTile(tile, (uint)-1);
 
-	if (IS_INT_INSIDE(c->citynametype, SPECSTR_TOWNNAME_START, SPECSTR_TOWNNAME_LAST+1)) {
-		str = c->citynametype - SPECSTR_TOWNNAME_START + SPECSTR_PLAYERNAME_START;
-		strp = c->citynameparts;
+	if (IS_INT_INSIDE(t->townnametype, SPECSTR_TOWNNAME_START, SPECSTR_TOWNNAME_LAST+1)) {
+		str = t->townnametype - SPECSTR_TOWNNAME_START + SPECSTR_PLAYERNAME_START;
+		strp = t->townnameparts;
 
 verify_name:;
 		// No player must have this name already
 		FOR_ALL_PLAYERS(pp) {
 			if (pp->name_1 == str && pp->name_2 == strp)
-				goto bad_city_name;
+				goto bad_town_name;
 		}
 
 		GetString(buffer, str);
 		if (strlen(buffer) >= 32 || GetStringWidth(buffer) >= 150)
-			goto bad_city_name;
+			goto bad_town_name;
 
 set_name:;
 		p->name_1 = str;
@@ -323,17 +328,17 @@ set_name:;
 		MarkWholeScreenDirty();
 
 		if (!IS_HUMAN_PLAYER(p->index)) {
-			SET_DPARAM16(0, c->index);
+			SetDParam(0, t->index);
 			AddNewsItem(p->index + (4 << 4), NEWS_FLAGS(NM_CALLBACK, NF_TILE, NT_COMPANY_INFO, DNC_BANKRUPCY), p->last_build_coordinate, 0);
 		}
 		return;
 	}
-bad_city_name:;
-	
+bad_town_name:;
+
 	if (p->president_name_1 == SPECSTR_PRESIDENT_NAME) {
 		str = SPECSTR_ANDCO_NAME;
 		strp = p->president_name_2;
-		goto set_name;	
+		goto set_name;
 	} else {
 		str = SPECSTR_ANDCO_NAME;
 		strp = Random();
@@ -347,7 +352,7 @@ static const byte _color_sort[16] = {2, 2, 3, 2, 3, 2, 3, 2, 3, 2, 2, 2, 3, 1, 1
 static const byte _color_similar_1[16] = {8, 6, 255, 12,  255, 0, 1, 1, 0, 13,  11,  10, 3,   9,  15, 14};
 static const byte _color_similar_2[16] = {5, 7, 255, 255, 255, 8, 7, 6, 5, 12, 255, 255, 9, 255, 255, 255};
 
-static byte GeneratePlayerColor()
+static byte GeneratePlayerColor(void)
 {
 	byte colors[16], pcolor, t2;
 	int i,j,n;
@@ -370,7 +375,7 @@ static byte GeneratePlayerColor()
 	do {
 		for(j=0; j!=15; j++) {
 			if (_color_sort[colors[j]] < _color_sort[colors[j+1]]) {
-				COLOR_SWAP(j,j+1);	
+				COLOR_SWAP(j,j+1);
 			}
 		}
 	} while (--i);
@@ -381,7 +386,7 @@ static byte GeneratePlayerColor()
 		for(i=0; i!=16; i++) if (colors[i] == pcolor) {
 			colors[i] = 0xFF;
 
-			t2 = _color_similar_1[pcolor]; 
+			t2 = _color_similar_1[pcolor];
 			if (t2 == 0xFF) break;
 			for(i=0; i!=15; i++) {
 				if (colors[i] == t2) {
@@ -390,7 +395,7 @@ static byte GeneratePlayerColor()
 				}
 			}
 
-			t2 = _color_similar_2[pcolor]; 
+			t2 = _color_similar_2[pcolor];
 			if (t2 == 0xFF) break;
 			for(i=0; i!=15; i++) {
 				if (colors[i] == t2) {
@@ -421,15 +426,15 @@ restart:;
 
 		p->president_name_2 = Random();
 		p->president_name_1 = SPECSTR_PRESIDENT_NAME;
-		
-		SET_DPARAM32(0, p->president_name_2);
+
+		SetDParam(0, p->president_name_2);
 		GetString(buffer, p->president_name_1);
 		if (strlen(buffer) >= 32 || GetStringWidth(buffer) >= 94)
 			continue;
 
 		FOR_ALL_PLAYERS(pp) {
 			if (pp->is_active && p != pp) {
-				SET_DPARAM32(0, pp->president_name_2);
+				SetDParam(0, pp->president_name_2);
 				GetString(buffer2, pp->president_name_1);
 				if (str_eq(buffer2, buffer))
 					goto restart;
@@ -441,7 +446,7 @@ restart:;
 
 extern int GetPlayerMaxRailtype(int p);
 
-static Player *AllocatePlayer()
+static Player *AllocatePlayer(void)
 {
 	Player *p;
 	// Find a free slot
@@ -463,7 +468,7 @@ Player *DoStartupNewPlayer(bool is_ai)
 
 	p = AllocatePlayer();
 	if (p == NULL) return NULL;
-		
+
 	index = p->index;
 
 	// Make a color
@@ -471,7 +476,7 @@ Player *DoStartupNewPlayer(bool is_ai)
 	_player_colors[index] = p->player_color;
 	p->name_1 = STR_SV_UNNAMED;
 	p->is_active = true;
-	
+
 	p->money64 = p->player_money = p->current_loan = 100000;
 
 	p->is_ai = is_ai;
@@ -486,23 +491,22 @@ Player *DoStartupNewPlayer(bool is_ai)
 
 	InvalidateWindow(WC_GRAPH_LEGEND, 0);
 	InvalidateWindow(WC_TOOLBAR_MENU, 0);
+	InvalidateWindow(WC_CLIENT_LIST, 0);
 
 	return p;
 }
 
-void StartupPlayers()
+void StartupPlayers(void)
 {
-	uint32 r;
-	
-	r = Random();
-	_next_competitor_start = (byte)r + _opt.diff.competitor_start_time * 6882 + 1;
+	// The AI starts like in the setting with +2 month max
+	_next_competitor_start = _opt.diff.competitor_start_time * 90 * DAY_TICKS + RandomRange(60 * DAY_TICKS) + 1;
 }
 
-static void MaybeStartNewPlayer()
+static void MaybeStartNewPlayer(void)
 {
 	uint n;
 	Player *p;
-	
+
 	// count number of competitors
 	n = 0;
 	for(p=_players; p!=endof(_players); p++)
@@ -510,13 +514,14 @@ static void MaybeStartNewPlayer()
 			n++;
 
 	// when there's a lot of computers in game, the probability that a new one starts is lower
-	if (n < (uint)_opt.diff.max_no_competitors && n < RandomRange(_opt.diff.max_no_competitors))
+	if (n < (uint)_opt.diff.max_no_competitors && n < RandomRange(_opt.diff.max_no_competitors + 2))
 		DoStartupNewPlayer(true);
-	
-	_next_competitor_start = RandomRange(19980) + 11100;
+
+	// The next AI starts like the difficulty setting said, with +2 month max
+	_next_competitor_start = _opt.diff.competitor_start_time * 90 * DAY_TICKS + RandomRange(60 * DAY_TICKS) + 1;
 }
 
-void InitializePlayers()
+void InitializePlayers(void)
 {
 	int i;
 	memset(_players, 0, sizeof(_players));
@@ -525,50 +530,55 @@ void InitializePlayers()
 	_cur_player_tick_index = 0;
 }
 
-void OnTick_Players()
+void OnTick_Players(void)
 {
 	Player *p;
 
 	if (_game_mode == GM_EDITOR)
 		return;
-	
+
 	p = DEREF_PLAYER(_cur_player_tick_index);
 	_cur_player_tick_index = (_cur_player_tick_index + 1) % MAX_PLAYERS;
 	if (p->name_1 != 0) GenerateCompanyName(p);
 
-	if (_game_mode != GM_MENU && !--_next_competitor_start) {
-		MaybeStartNewPlayer();		
+	if (!_networking && _game_mode != GM_MENU && !--_next_competitor_start) {
+		MaybeStartNewPlayer();
 	}
 }
 
-void RunOtherPlayersLoop()
+void RunOtherPlayersLoop(void)
 {
 	Player *p;
 
 	_is_ai_player = true;
 
 	FOR_ALL_PLAYERS(p) {
-		if (p->is_active) {
+		if (p->is_active && p->is_ai) {
 			_current_player = p->index;
-			AiDoGameLoop(p);
+			if (_patches.ainew_active)
+				AiNewDoGameLoop(p);
+			else
+				AiDoGameLoop(p);
 		}
 	}
 
 	_is_ai_player = false;
-// XXX: is this needed?
-	_current_player = 0;
+	_current_player = OWNER_NONE;
 }
 
-StringID GetPlayerNameString(byte player)
+// index is the next parameter in _decode_parameters to set up
+StringID GetPlayerNameString(byte player, byte index)
 {
-	if (IS_HUMAN_PLAYER(player) && player < 2) // temporarily fixes the names in the list.
-		return STR_7002_PLAYER_1+player;
+	if (IS_HUMAN_PLAYER(player) && player < MAX_PLAYERS) {
+		SetDParam(index, player+1);
+		return STR_7002_PLAYER;
+	}
 	return STR_EMPTY;
 }
 
 extern void ShowPlayerFinances(int player);
 
-void PlayersYearlyLoop()
+void PlayersYearlyLoop(void)
 {
 	Player *p;
 
@@ -581,13 +591,13 @@ void PlayersYearlyLoop()
 		}
 	}
 
-	if (_patches.show_finances && _local_player != 0xff) {
+	if (_patches.show_finances && _local_player != OWNER_SPECTATOR) {
 		ShowPlayerFinances(_local_player);
 		p = DEREF_PLAYER(_local_player);
 		if (p->num_valid_stat_ent > 5 && p->old_economy[0].performance_history < p->old_economy[4].performance_history) {
-			SndPlayFx(0x28);
+			SndPlayFx(SND_01_BAD_YEAR);
 		} else {
-			SndPlayFx(0x27);
+			SndPlayFx(SND_00_GOOD_YEAR);
 		}
 	}
 }
@@ -619,7 +629,7 @@ static void DeletePlayerStuff(int pi)
 // functionality.
 // 0 - make new player
 // 1 - make new AI player
-// 2 - delete player (p1 >> 8) & 0xFF
+// 2 - delete player (p2)
 // 3 - join player (p1 >> 8) & 0xFF with (p1 >> 16) & 0xFF
 int32 CmdPlayerCtrl(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
@@ -628,22 +638,76 @@ int32 CmdPlayerCtrl(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 	if (!(flags & DC_EXEC))
 		return 0;
-	
+
+	_current_player = OWNER_NONE;
+
 	switch(p1 & 0xff) {
 	case 0: // make new player
 		p = DoStartupNewPlayer(false);
-		if (_local_player == 0xff && p != NULL) {
-			_local_player = p->index;
-			MarkWholeScreenDirty();
+
+#ifdef ENABLE_NETWORK
+		if (_networking && !_network_server && _local_player == OWNER_SPECTATOR)
+			/* In case we are a client joining a server... */
+			DeleteWindowById(WC_NETWORK_STATUS_WINDOW, 0);
+#endif /* ENABLE_NETWORK */
+
+		if (p != NULL) {
+			if (_local_player == OWNER_SPECTATOR) {
+				/* Check if we do not want to be a spectator in network */
+				if (!_networking || _network_server || _network_playas != OWNER_SPECTATOR) {
+					_local_player = p->index;
+					MarkWholeScreenDirty();
+				}
+			}
+#ifdef ENABLE_NETWORK
+			if (_network_server) {
+				NetworkClientInfo *ci;
+				// UGLY! p2 is mis-used to fetch the client-id
+				ci = &_network_client_info[p2];
+				ci->client_playas = p->index + 1;
+				NetworkUpdateClientInfo(ci->client_index);
+
+				if (ci->client_playas != 0 && ci->client_playas <= MAX_PLAYERS) {
+					_network_player_info[p->index].months_empty = 0;
+
+					memcpy(_decode_parameters, ci->client_name, 32);
+					/* XXX - What are the consequents of this? It is needed, but is it bad? */
+					_docommand_recursive = 0;
+					DoCommandP(0, ci->client_playas-1, 0, NULL, CMD_CHANGE_PRESIDENT_NAME | CMD_MSG(STR_700D_CAN_T_CHANGE_PRESIDENT));
+				}
+			}
+		} else {
+			if (_network_server) {
+				NetworkClientInfo *ci;
+				// UGLY! p2 is mis-used to fetch the client-id
+				ci = &_network_client_info[p2];
+				ci->client_playas = OWNER_SPECTATOR;
+				NetworkUpdateClientInfo(ci->client_index);
+			}
+#endif /* ENABLE_NETWORK */
 		}
 		break;
 	case 1: // make new ai player
 		DoStartupNewPlayer(true);
 		break;
 	case 2: // delete player
-		pi = (byte)(p1 >> 8);
-		ChangeOwnershipOfPlayerItems(pi, 255);
-		DeletePlayerStuff(pi);
+		p = DEREF_PLAYER(p2);
+
+		/* Only allow removal of HUMAN companies */
+		if (IS_HUMAN_PLAYER(p2)) {
+			/* Delete any open window of the company */
+			DeletePlayerWindows(p2);
+
+			/* Show the bankrupt news */
+			SetDParam(0, p->name_1);
+			SetDParam(1, p->name_2);
+			AddNewsItem( (StringID)(p2 + 16*3), NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
+
+			/* Remove the company */
+			ChangeOwnershipOfPlayerItems(p2, 255);
+			p->money64 = p->player_money = 100000000;
+			p->is_active = false;
+		}
 		break;
 
 	case 3: // join player
@@ -657,6 +721,186 @@ int32 CmdPlayerCtrl(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	return 0;
 }
 
+static const StringID _endgame_performance_titles[16] = {
+	STR_0213_BUSINESSMAN,
+	STR_0213_BUSINESSMAN,
+	STR_0213_BUSINESSMAN,
+	STR_0213_BUSINESSMAN,
+	STR_0213_BUSINESSMAN,
+	STR_0214_ENTREPRENEUR,
+	STR_0214_ENTREPRENEUR,
+	STR_0215_INDUSTRIALIST,
+	STR_0215_INDUSTRIALIST,
+	STR_0216_CAPITALIST,
+	STR_0216_CAPITALIST,
+	STR_0217_MAGNATE,
+	STR_0217_MAGNATE,
+	STR_0218_MOGUL,
+	STR_0218_MOGUL,
+	STR_0219_TYCOON_OF_THE_CENTURY,
+};
+
+StringID EndGameGetPerformanceTitleFromValue(uint value)
+{
+	return _endgame_performance_titles[minu(value, 1000) >> 6];
+}
+
+/* Return true if any cheat has been used, false otherwise */
+static bool CheatHasBeenUsed(void)
+{
+	const Cheat* cht = (Cheat*) &_cheats;
+	const Cheat* cht_last = &cht[sizeof(_cheats) / sizeof(Cheat)];
+
+	for (; cht != cht_last; cht++) {
+		if (cht->been_used)
+			return true;
+	}
+
+	return false;
+}
+
+/* Save the highscore for the player */
+int8 SaveHighScoreValue(const Player *p)
+{
+	HighScore *hs = _highscore_table[_opt.diff_level];
+	uint i;
+	uint16 score = p->old_economy[0].performance_history;
+
+	/* Exclude cheaters from the honour of being in the highscore table */
+	if (CheatHasBeenUsed())
+		return -1;
+
+	for (i = 0; i < lengthof(_highscore_table[0]); i++) {
+		/* You are in the TOP5. Move all values one down and save us there */
+		if (hs[i].score <= score) {
+			byte buf[sizeof(hs[i].company)];
+
+			// move all elements one down starting from the replaced one
+			memmove(&hs[i + 1], &hs[i], sizeof(HighScore) * (lengthof(_highscore_table[0]) - i - 1));
+			SetDParam(0, p->president_name_1);
+			SetDParam(1, p->president_name_2);
+			SetDParam(2, p->name_1);
+			SetDParam(3, p->name_2);
+			GetString(buf, STR_HIGHSCORE_NAME); // get manager/company name string
+			ttd_strlcpy(hs[i].company, buf, sizeof(buf));
+			hs[i].score = score;
+			hs[i].title = EndGameGetPerformanceTitleFromValue(score);
+			return i;
+		}
+	}
+
+	return -1; // too bad; we did not make it into the top5
+}
+
+/* Sort all players given their performance */
+static int CDECL HighScoreSorter(const void *a, const void *b)
+{
+	const Player *pa = *(const Player* const*)a;
+	const Player *pb = *(const Player* const*)b;
+
+	return pb->old_economy[0].performance_history - pa->old_economy[0].performance_history;
+}
+
+/* Save the highscores in a network game when it has ended */
+#define LAST_HS_ITEM lengthof(_highscore_table) - 1
+int8 SaveHighScoreValueNetwork(void)
+{
+	Player *p, *player_sort[MAX_PLAYERS];
+	size_t count = 0;
+	int8 player = -1;
+
+	/* Sort all active players with the highest score first */
+	FOR_ALL_PLAYERS(p) {
+		if (p->is_active)
+			player_sort[count++] = p;
+	}
+	qsort(player_sort, count, sizeof(player_sort[0]), HighScoreSorter);
+
+	{
+		HighScore *hs;
+		byte buf[sizeof(_highscore_table[0]->company)];
+		Player* const *p_cur = &player_sort[0];
+		uint8 i;
+
+		memset(_highscore_table[LAST_HS_ITEM], 0, sizeof(_highscore_table[0]));
+
+		/* Copy over Top5 companies */
+		for (i = 0; i < lengthof(_highscore_table[LAST_HS_ITEM]) && i < (uint8)count; i++) {
+			hs = &_highscore_table[LAST_HS_ITEM][i];
+			SetDParam(0, (*p_cur)->president_name_1);
+			SetDParam(1, (*p_cur)->president_name_2);
+			SetDParam(2, (*p_cur)->name_1);
+			SetDParam(3, (*p_cur)->name_2);
+			GetString(buf, STR_HIGHSCORE_NAME); // get manager/company name string
+
+			ttd_strlcpy(hs->company, buf, sizeof(buf));
+			hs->score = (*p_cur)->old_economy[0].performance_history;
+			hs->title = EndGameGetPerformanceTitleFromValue(hs->score);
+
+			// get the ranking of the local player
+			if ((*p_cur)->index == (int8)_local_player)
+				player = i;
+
+			p_cur++;
+		}
+	}
+
+	/* Add top5 players to highscore table */
+	return player;
+}
+
+/* Save HighScore table to file */
+void SaveToHighScore(void)
+{
+	FILE *fp = fopen(_highscore_file, "w");
+
+	if (fp != NULL) {
+		uint i;
+		HighScore *hs;
+
+		for (i = 0; i < LAST_HS_ITEM; i++) { // don't save network highscores
+			for (hs = _highscore_table[i]; hs != endof(_highscore_table[i]); hs++) {
+				/* First character is a command character, so strlen will fail on that */
+				byte length = min(sizeof(hs->company), (hs->company[0] == '\0') ? 0 : strlen(&hs->company[1]) + 1);
+
+				fwrite(&length, sizeof(length), 1, fp); // write away string length
+				fwrite(hs->company, length, 1, fp);
+				fwrite(&hs->score, sizeof(hs->score), 1, fp);
+				fwrite(&hs->title, sizeof(hs->title), 1, fp);
+			}
+		}
+		fclose(fp);
+	}
+}
+
+/* Initialize the highscore table to 0 and if any file exists, load in values */
+void LoadFromHighScore(void)
+{
+	FILE *fp = fopen(_highscore_file, "r");
+
+	memset(_highscore_table, 0, sizeof(_highscore_table));
+
+	if (fp != NULL) {
+		uint i;
+		HighScore *hs;
+
+		for (i = 0; i < LAST_HS_ITEM; i++) { // don't load network highscores
+			for (hs = _highscore_table[i]; hs != endof(_highscore_table[i]); hs++) {
+				byte length;
+				fread(&length, sizeof(length), 1, fp);
+
+				fread(hs->company, 1, length, fp);
+				fread(&hs->score, sizeof(hs->score), 1, fp);
+				fread(&hs->title, sizeof(hs->title), 1, fp);
+			}
+		}
+		fclose(fp);
+	}
+
+	/* Initialize end of game variable (when to show highscore chart) */
+	 _patches.ending_date = 2051;
+}
+
 // Save/load of players
 static const byte _player_desc[] = {
 	SLE_VAR(Player,name_2,					SLE_UINT32),
@@ -666,7 +910,7 @@ static const byte _player_desc[] = {
 	SLE_VAR(Player,president_name_2,SLE_UINT32),
 
 	SLE_VAR(Player,face,						SLE_UINT32),
-	
+
 	// money was changed to a 64 bit field in savegame version 1.
 	SLE_CONDVAR(Player,money64,			SLE_VAR_I64 | SLE_FILE_I32, 0, 0),
 	SLE_CONDVAR(Player,money64,			SLE_INT64, 1, 255),
@@ -716,7 +960,7 @@ static const byte _player_economy_desc[] = {
 
 	SLE_VAR(PlayerEconomyEntry,delivered_cargo,			SLE_INT32),
 	SLE_VAR(PlayerEconomyEntry,performance_history,	SLE_INT32),
-	
+
 	SLE_END()
 };
 
@@ -796,7 +1040,7 @@ static void SaveLoad_PLYR(Player *p) {
 	}
 }
 
-static void Save_PLYR()
+static void Save_PLYR(void)
 {
 	Player *p;
 	FOR_ALL_PLAYERS(p) {
@@ -807,11 +1051,11 @@ static void Save_PLYR()
 	}
 }
 
-static void Load_PLYR()
+static void Load_PLYR(void)
 {
 	int index;
 	while ((index = SlIterateArray()) != -1) {
-		Player *p = DEREF_PLAYER(index);		
+		Player *p = DEREF_PLAYER(index);
 		p->is_ai = (index != 0);
 		SaveLoad_PLYR(p);
 		_player_colors[index] = p->player_color;
