@@ -1,12 +1,20 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
+#include "map.h"
 #include "vehicle.h"
 #include "viewport.h"
 #include "command.h"
 #include "player.h"
-#include "city.h"
+#include "town.h"
 #include "gfx.h"
+#include "table/directions.h"
+#include "sound.h"
 
+/* When true, GetTrackStatus for roads will treat roads under reconstruction
+ * as normal roads instead of impassable. This is used when detecting whether
+ * a road can be removed. This is of course ugly, but I don't know a better
+ * solution just like that... */
 static bool _road_special_gettrackstatus;
 
 void RoadVehEnterDepot(Vehicle *v);
@@ -17,10 +25,10 @@ static bool HasTileRoadAt(uint tile, int i)
 	int mask;
 	byte b;
 
-	switch(GET_TILETYPE(tile)) {
+	switch (TileType(tile)) {
 	case MP_STREET:
 		b = _map5[tile];
-		
+
 		if ((b & 0xF0) == 0) {
 		} else if ((b & 0xF0) == 0x10) {
 			b = (b&8)?5:10;
@@ -29,7 +37,7 @@ static bool HasTileRoadAt(uint tile, int i)
 		} else
 			return false;
 		break;
-						
+
 	case MP_STATION:
 		b = _map5[tile];
 		if (!IS_BYTE_INSIDE(b, 0x43, 0x43+8))
@@ -49,11 +57,12 @@ static bool HasTileRoadAt(uint tile, int i)
 	return HASBIT(b, i);
 }
 
-static bool CheckAllowRemoveRoad(uint tile, uint br)
+static bool CheckAllowRemoveRoad(uint tile, uint br, bool *edge_road)
 {
 	int blocks;
 	byte owner;
 	uint n;
+	*edge_road = true;
 
 	if (_game_mode == GM_EDITOR)
 		return true;
@@ -67,7 +76,7 @@ static bool CheckAllowRemoveRoad(uint tile, uint br)
 		return true;
 
 	// A railway crossing has the road owner in the map3_lo byte.
-	if (IS_TILETYPE(tile, MP_STREET) && (_map5[tile] & 0xF0) == 0x10) {
+	if (IsTileType(tile, MP_STREET) && (_map5[tile] & 0xF0) == 0x10) {
 		owner = _map3_lo[tile];
 	} else {
 		owner = _map_owner[tile];
@@ -78,8 +87,6 @@ static bool CheckAllowRemoveRoad(uint tile, uint br)
 		return owner == OWNER_NONE || CheckOwnership(owner);
 	}
 
-	if (_patches.extra_dynamite)
-		return true;
 	if (_cheats.magic_bulldozer.value)
 		return true;
 
@@ -89,12 +96,18 @@ static bool CheckAllowRemoveRoad(uint tile, uint br)
 	if (blocks&0x2A && HasTileRoadAt(TILE_ADDXY(tile, 0, 1), 0)) n |= 4;
 	if (blocks&0x19 && HasTileRoadAt(TILE_ADDXY(tile, 1, 0), 3)) n |= 2;
 	if (blocks&0x16 && HasTileRoadAt(TILE_ADDXY(tile, 0,-1), 2)) n |= 1;
-	
+
 	// If 0 or 1 bits are set in n, or if no bits that match the bits to remove,
 	// then allow it
 	if ((n & (n-1)) != 0 && (n & br) != 0) {
-		City *c = ClosestCityFromTile(tile, (uint)-1);
-		SET_DPARAM16(0, c->index);
+		Town *t;
+		*edge_road = false;
+		// you can remove all kind of roads with extra dynamite
+		if (_patches.extra_dynamite)
+			return true;
+
+		t = ClosestTownFromTile(tile, (uint)-1);
+		SetDParam(0, t->index);
 		_error_message = STR_2009_LOCAL_AUTHORITY_REFUSES;
 		return false;
 	}
@@ -104,16 +117,19 @@ static bool CheckAllowRemoveRoad(uint tile, uint br)
 
 bool IsRoadDepotTile(TileIndex tile)
 {
-	return IS_TILETYPE(tile, MP_STREET) &&
+	return IsTileType(tile, MP_STREET) &&
 					(_map5[tile] & 0xF0) == 0x20;
 }
 
 uint GetRoadBitsByTile(TileIndex tile)
 {
-	uint32 r = GetTileTrackStatus(tile, 2);
+	uint32 r = GetTileTrackStatus(tile, TRANSPORT_ROAD);
 	return (byte)(r | (r >> 8));
 }
 
+// cost for removing inner/edge -roads
+static const uint16 _road_remove_cost[2] = {50, 18};
+
 /* Delete a piece of road
  * p1 = piece type
  */
@@ -122,14 +138,18 @@ int32 CmdRemoveRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	TileInfo ti;
 	int32 cost;
 	uint tile;
-	City *c;
-	
+	Town *t;
+	/*	true if the roadpiece was always removeable,
+			false if it was a center piece. Affects town ratings drop
+	*/
+	bool edge_road;
+
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
 	FindLandscapeHeight(&ti, x, y);
 	tile = ti.tile;
-	c = ClosestCityFromTile(tile, (uint)-1); // needed for city rating penalty
-	
+	t = ClosestTownFromTile(tile, (uint)-1); // needed for town rating penalty
+
 	// allow deleting road under bridge
 	if (ti.type != MP_TUNNELBRIDGE && !EnsureNoVehicle(tile))
 		return CMD_ERROR;
@@ -137,14 +157,14 @@ int32 CmdRemoveRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	{
 		bool b;
 		_road_special_gettrackstatus = true;
-		b = CheckAllowRemoveRoad(tile, p1);
+		b = CheckAllowRemoveRoad(tile, p1, &edge_road);
 		_road_special_gettrackstatus = false;
 		if (!b)
 			return CMD_ERROR;
 	}
 
 	if (ti.type == MP_TUNNELBRIDGE) {
-		if (!EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)))
+		if (!EnsureNoVehicleZ(tile, TilePixelHeight(tile)))
 			return CMD_ERROR;
 
 		if ((ti.map5 & 0xE9) == 0xE8) {
@@ -164,64 +184,62 @@ int32 CmdRemoveRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		return cost;
 	} else if (ti.type == MP_STREET) {
 		if (!(ti.map5 & 0xF0)) {
-			uint t = p1, t2;
+			uint c = p1, t2;
 
 			if (ti.tileh != 0  && (ti.map5 == 5 || ti.map5 == 10)) {
-				t |= (t & 0xC) >> 2;
-				t |= (t & 0x3) << 2;
+				c |= (c & 0xC) >> 2;
+				c |= (c & 0x3) << 2;
 			}
 
 			// limit the bits to delete to the existing bits.
-			if ((t &= ti.map5) == 0) goto return_error;
-			
+			if ((c &= ti.map5) == 0) goto return_error;
+
 			// calculate the cost
-			t2 = t;
+			t2 = c;
 			cost = 0;
 			do {
-				if (t2&1) cost += _price.remove_road;			
+				if (t2&1) cost += _price.remove_road;
 			} while(t2>>=1);
-			
-			// check if you're allowed to remove the street owned by a town
-			// no removal if rating is lower than "Poor" (15) <- this should depend on difficulty setting
-			if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR) {
-				if (c->ratings[_current_player] < 15 && !(flags & DC_NO_CITY_RATING) && !_cheats.magic_bulldozer.value) {
-					SET_DPARAM16(0, c->index);
-					return_cmd_error(STR_2009_LOCAL_AUTHORITY_REFUSES);
-				}
-			}
-			
-			if (flags & DC_EXEC) {
-				// checks if the owner is town than decrease city rating by 50 until you have 
-				// a "Poor" city rating
-				if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR)
-					ChangeCityRating(c, -50, -100); 
 
-				_map5[tile] ^= t;
+			// check if you're allowed to remove the street owned by a town
+			// removal allowance depends on difficulty setting
+			if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR) {
+				if (!CheckforTownRating(tile, flags, t, ROAD_REMOVE))
+					return CMD_ERROR;
+			}
+
+			if (flags & DC_EXEC) {
+				// checks if the owner is town than decrease town rating by 50 until you have
+				// a "Poor" town rating
+				if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR)
+					ChangeTownRating(t, -_road_remove_cost[(byte)edge_road], RATING_ROAD_MINIMUM);
+
+				_map5[tile] ^= c;
 				if ((_map5[tile]&0xF) == 0)
 					DoClearSquare(tile);
 				else
 					MarkTileDirtyByTile(tile);
 
 			}
-			return cost;			
+			return cost;
 		} else if (!(ti.map5 & 0xE0)) {
-			byte t;
+			byte c;
 
 			if (!(ti.map5 & 8)) {
-				t = 2;	
+				c = 2;
 				if (p1 & 5) goto return_error;
 			} else {
-				t = 1;
+				c = 1;
 				if (p1 & 10) goto return_error;
 			}
 
 			cost = _price.remove_road * 2;
 			if (flags & DC_EXEC) {
-				ModifyTile(tile, 
+				ModifyTile(tile,
 					MP_SETTYPE(MP_RAILWAY) |
 					MP_MAP2_CLEAR | MP_MAP3LO | MP_MAP3HI_CLEAR | MP_MAP5,
 					_map3_hi[tile] & 0xF, /* map3_lo */
-					t											/* map5 */
+					c											/* map5 */
 				);
 			}
 			return cost;
@@ -242,8 +260,8 @@ enum {
 	ROAD_NE = 8, // NE road track
 	ROAD_ALL = (ROAD_NW | ROAD_SW | ROAD_SE | ROAD_NE)
 };
- 
-static const byte _valid_tileh_slopes_road[2][15] = {
+
+static const byte _valid_tileh_slopes_road[3][15] = {
 	// set of normal ones
 	{
 		ROAD_ALL, 0, 0,
@@ -251,9 +269,9 @@ static const byte _valid_tileh_slopes_road[2][15] = {
 		ROAD_NW | ROAD_SE, 0, 0,
 		ROAD_NW | ROAD_SE, 0, 0,  // 9, 10, 11
 		ROAD_SW | ROAD_NE, 0, 0
-	},  
+	},
 	// allowed road for an evenly raised platform
-	{ 
+	{
 		0,
 		ROAD_SW | ROAD_NW,
 		ROAD_SW | ROAD_SE,
@@ -272,11 +290,19 @@ static const byte _valid_tileh_slopes_road[2][15] = {
 		ROAD_NW | ROAD_SE | ROAD_NE, // 12
 		ROAD_ALL,
 		ROAD_ALL
+	},
+	// valid railway crossings on slopes
+	{
+		1, 0, 0, // 0, 1, 2
+		0, 0, 1, // 3, 4, 5
+		0, 1, 0, // 6, 7, 8
+		0, 1, 1, // 9, 10, 11
+		0, 1, 1, // 12, 13, 14
 	}
 };
 
 
-static uint32 CheckRoadSlope(int tileh, byte *pieces, byte existing)	
+static uint32 CheckRoadSlope(int tileh, byte *pieces, byte existing)
 {
 	if (!(tileh & 0x10)) {
 		byte road_bits = *pieces | existing;
@@ -287,10 +313,10 @@ static uint32 CheckRoadSlope(int tileh, byte *pieces, byte existing)
 			if (tileh != 0) *pieces |= _valid_tileh_slopes_road[0][tileh];
 			return 0; // no extra cost
 		}
-		
+
 		// foundation is used. Whole tile is leveled up
 		if ((~_valid_tileh_slopes_road[1][tileh] & road_bits) == 0) {
-			return existing ? 0 : _price.terraform;	
+			return existing ? 0 : _price.terraform;
 		}
 
 		// partly leveled up tile, only if there's no road on that tile
@@ -298,7 +324,7 @@ static uint32 CheckRoadSlope(int tileh, byte *pieces, byte existing)
 			// force full pieces.
 			*pieces |= (*pieces & 0xC) >> 2;
 			*pieces |= (*pieces & 0x3) << 2;
-			return existing ? 0 : _price.terraform;	
+			return (*pieces == (ROAD_NE|ROAD_SW) || *pieces == (ROAD_SE|ROAD_NW)) ? _price.terraform : CMD_ERROR;
 		}
 	}
 	return CMD_ERROR;
@@ -314,7 +340,7 @@ int32 CmdBuildRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	int32 cost;
 	byte pieces = (byte)p1, existing = 0;
 	uint tile;
-	
+
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
 	FindLandscapeHeight(&ti, x, y);
@@ -337,8 +363,12 @@ int32 CmdBuildRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	} else if (ti.type == MP_RAILWAY) {
 		byte m5;
 
-		if (ti.tileh != 0) goto do_clear;
-		
+		if (ti.tileh & 0x10) // very steep tile
+				return_cmd_error(STR_1000_LAND_SLOPED_IN_WRONG_DIRECTION);
+
+		if(!_valid_tileh_slopes_road[2][ti.tileh]) // prevent certain slopes
+				return_cmd_error(STR_1000_LAND_SLOPED_IN_WRONG_DIRECTION);
+
 		if (ti.map5 == 2) {
 			if (pieces & 5) goto do_clear;
 			m5 = 0x10;
@@ -359,33 +389,33 @@ int32 CmdBuildRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		}
 		return _price.build_road * 2;
 	} else if (ti.type == MP_TUNNELBRIDGE) {
-		byte t;
 
-		if (!EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)))
-			return CMD_ERROR;
-		
-		t = ti.map5 & 0xF9;
+		/* check for flat land */
+		if (ti.tileh & 0x10) // very steep tile
+				return_cmd_error(STR_1000_LAND_SLOPED_IN_WRONG_DIRECTION);
 
-		if (ti.tileh & 0x10) goto do_clear;
+		/* is this middle part of a bridge? */
+		if ((ti.map5 & 0xC0) != 0xC0)
+				goto do_clear;
 
-		if (t == 0xE8) {
-			if (!(pieces & 10))
+		/* only allow roads pertendicular to bridge */
+		if ((pieces & 5U) == (ti.map5 & 0x01U))
+				goto do_clear;
+
+		/* check if clear land under bridge */
+		if ((ti.map5 & 0xF8) == 0xE8) 			/* road under bridge */
 				return_cmd_error(STR_1007_ALREADY_BUILT);
-		} else if (t == 0xE9) {
-			if (!(pieces & 5))
-				return_cmd_error(STR_1007_ALREADY_BUILT);
-		} else if (t == 0xC0) {
-			if (pieces & 10) goto do_clear;
-		} else if (t == 0xC1) {
-			if (pieces & 5) goto do_clear;
-		} else
-			goto do_clear;
+		else if ((ti.map5 & 0xE0) == 0xE0) 	/* other transport route under bridge */
+				return_cmd_error(STR_1008_MUST_REMOVE_RAILROAD_TRACK);
+		else if ((ti.map5 & 0xF8) == 0xC8) 	/* water under bridge */
+				return_cmd_error(STR_3807_CAN_T_BUILD_ON_WATER);
 
+		/* all checked, can build road now! */
 		cost = _price.build_road * 2;
 		if (flags & DC_EXEC) {
 			ModifyTile(tile,
 				MP_MAPOWNER_CURRENT | MP_MAP5,
-				(ti.map5 & 0xC7) | 0x28 /*map5 */
+				(ti.map5 & 0xC7) | 0x28 // map5
 			);
 		}
 		return cost;
@@ -398,8 +428,7 @@ do_clear:;
 	cost = CheckRoadSlope(ti.tileh, &pieces, existing);
 	if (cost == CMD_ERROR) return_cmd_error(STR_1800_LAND_SLOPED_IN_WRONG_DIRECTION);
 
-	// the AI is not allowed to used foundationed tiles.
-	if (cost && (_is_ai_player || !_patches.build_on_slopes))
+	if (cost && (!_patches.build_on_slopes || (!_patches.ainew_active && _is_ai_player)))
 		return CMD_ERROR;
 
 	if (!(ti.type == MP_STREET && (ti.map5 & 0xF0) == 0)) {
@@ -419,8 +448,7 @@ do_clear:;
 
 	if (flags & DC_EXEC) {
 		if (ti.type != MP_STREET) {
-			_map_type_and_height[tile] &= 0xF;
-			_map_type_and_height[tile] |= MP_STREET << 4;
+			SetTileType(tile, MP_STREET);
 			_map5[tile] = 0;
 			_map_owner[tile] = _current_player;
 		}
@@ -465,6 +493,8 @@ int32 CmdBuildLongRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	int mode;
 	int32 cost,ret;
 
+	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
+
 	start_tile = p1;
 	end_tile = TILE_FROM_XY(x, y);
 
@@ -516,6 +546,8 @@ int32 CmdRemoveLongRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	uint start_tile, end_tile, tile;
 	int32 cost,ret;
 
+	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
+
 	start_tile = p1;
 	end_tile = TILE_FROM_XY(x, y);
 
@@ -531,7 +563,7 @@ int32 CmdRemoveLongRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		uint bits = (p2 & 4) ? ROAD_SE | ROAD_NW : ROAD_SW | ROAD_NE;
 		if (tile == end_tile && !(p2&2)) bits &= ROAD_NW | ROAD_NE;
 		if (tile == start_tile && (p2&1)) bits &= ROAD_SE | ROAD_SW;
-		
+
 		// try to remove the halves.
 		if (bits) {
 			ret = DoCommandByTile(tile, bits, 0, flags, CMD_REMOVE_ROAD);
@@ -552,7 +584,7 @@ int32 CmdRemoveLongRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	return cost;
 }
 
-/* Build a road depot 
+/* Build a road depot
  * p1 - direction (0-3)
  * p2 - unused
  */
@@ -574,7 +606,7 @@ int32 CmdBuildRoadDepot(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		return CMD_ERROR;
 
 	if (ti.tileh != 0) {
-		if (_is_ai_player || !_patches.build_on_slopes || (ti.tileh & 0x10 || !((0x4C >> p1) & ti.tileh) ))
+		if (!_patches.build_on_slopes || (ti.tileh & 0x10 || !((0x4C >> p1) & ti.tileh) ))
 			return_cmd_error(STR_0007_FLAT_LAND_REQUIRED);
 	}
 
@@ -586,12 +618,12 @@ int32 CmdBuildRoadDepot(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (dep == NULL)
 		return CMD_ERROR;
 
-	if (flags & DC_EXEC) {	
+	if (flags & DC_EXEC) {
 		if (_current_player == _local_player)
 			_last_built_road_depot_tile = (TileIndex)tile;
 
 		dep->xy = tile;
-		dep->city_index = ClosestCityFromTile(tile, (uint)-1)->index;
+		dep->town_index = ClosestTownFromTile(tile, (uint)-1)->index;
 
 		ModifyTile(tile,
 			MP_SETTYPE(MP_STREET) |
@@ -605,7 +637,7 @@ int32 CmdBuildRoadDepot(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 static int32 RemoveRoadDepot(uint tile, uint32 flags)
 {
-	if (!CheckTileOwnership(tile))
+	if (!CheckTileOwnership(tile) && _current_player != OWNER_WATER)
 		return CMD_ERROR;
 
 	if (!EnsureNoVehicle(tile))
@@ -639,10 +671,10 @@ static int32 ClearTile_Road(uint tile, byte flags) {
 		ret = DoCommandByTile(tile, (m5&8)?5:10, 0, flags, CMD_REMOVE_ROAD);
 		if (ret == CMD_ERROR)
 			return CMD_ERROR;
-		
+
 		if (flags & DC_EXEC) {
 			DoCommandByTile(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
-		}	
+		}
 
 		return ret;
 	} else {
@@ -670,17 +702,21 @@ typedef struct DrawRoadSeqStruct {
 #include "table/road_land.h"
 
 
-static uint GetRoadFoundation(uint tileh, uint bits) {
+uint GetRoadFoundation(uint tileh, uint bits) {
 	int i;
 	// normal level sloped building
 	if ((~_valid_tileh_slopes_road[1][tileh] & bits) == 0)
 		return tileh;
 
 	// inclined sloped building
-	if ( ((i=0, tileh == 1) || (i+=2, tileh == 2) || (i+=2, tileh == 4) || (i+=2, tileh == 8)) && 
+	if ( ((i=0, tileh == 1) || (i+=2, tileh == 2) || (i+=2, tileh == 4) || (i+=2, tileh == 8)) &&
 		((bits == (ROAD_SW | ROAD_NE)) || (i++, bits == (ROAD_NW | ROAD_SE))))
 		return i + 15;
 
+	// rail crossing
+	if ((bits & 0x10) && _valid_tileh_slopes_road[2][tileh])
+		return tileh;
+
 	return 0;
 }
 
@@ -693,8 +729,8 @@ const byte _road_sloped_sprites[14] = {
 
 static void DrawTile_Road(TileInfo *ti)
 {
-	uint32 image;	
-	byte m2;
+	uint32 image;
+	uint16 m2;
 	const byte *s;
 
 	if ( (ti->map5 & 0xF0) == 0) { // if it is a road the upper 4 bits are 0
@@ -703,7 +739,7 @@ static void DrawTile_Road(TileInfo *ti)
 		if (ti->tileh != 0) {
 			int f = GetRoadFoundation(ti->tileh, ti->map5 & 0xF);
 			if (f) DrawFoundation(ti, f);
-			
+
 			// default sloped sprites..
 			if (ti->tileh != 0) {
 				image = _road_sloped_sprites[ti->tileh - 1] + 0x53F;
@@ -717,7 +753,7 @@ static void DrawTile_Road(TileInfo *ti)
 		m2 = _map2[ti->tile] & 7;
 
 		if (m2 == 0) image |= 0x3178000;
-		
+
 		if (_map3_hi[ti->tile] & 0x80) {
 			image += 19;
 		} else if (m2 > 1 && m2 != 6) {
@@ -736,7 +772,7 @@ static void DrawTile_Road(TileInfo *ti)
 		}
 
 		drts = (const DrawRoadTileStruct*)_road_display_table[m2][ti->map5 & 0xF];
-		
+
 		while ((image = drts->image) != 0) {
 			int x = ti->x | drts->subcoord_x;
 			int y = ti->y | drts->subcoord_y;
@@ -745,7 +781,10 @@ static void DrawTile_Road(TileInfo *ti)
 			AddSortableSpriteToDraw(image, x, y, 2, 2, 0x10, z);
 			drts++;
 		}
-	} else if ( (ti->map5 & 0xE0) == 0) {
+	} else if ( (ti->map5 & 0xE0) == 0) { // railroad crossing
+		int f = GetRoadFoundation(ti->tileh, ti->map5 & 0xF);
+		if (f) DrawFoundation(ti, f);
+
 		image = 0x55B;
 
 		if ( (ti->map5 & 8) != 0)
@@ -767,7 +806,7 @@ static void DrawTile_Road(TileInfo *ti)
 		uint32 ormod;
 		int player;
 		const DrawRoadSeqStruct *drss;
-		
+
 		if (ti->tileh != 0) { DrawFoundation(ti, ti->tileh); }
 
 		ormod = 0x315;
@@ -776,16 +815,18 @@ static void DrawTile_Road(TileInfo *ti)
 			ormod = PLAYER_SPRITE_COLOR(player);
 
 		s = _road_display_datas[ti->map5 & 0xF];
-		
-		DrawGroundSprite(*(uint32*)s);
+
+		DrawGroundSprite(*(const uint32*)s);
 		s += sizeof(uint32);
-		drss = (DrawRoadSeqStruct*)s;
+		drss = (const DrawRoadSeqStruct*)s;
 
 		while ((image=drss->image) != 0) {
 			if (image & 0x8000)
 				image |= ormod;
+			if (_display_opt & DO_TRANS_BUILDINGS) // show transparent depots
+				image = (image & 0x3FFF) | 0x3224000;
 
-			AddSortableSpriteToDraw(image, ti->x | drss->subcoord_x, 
+			AddSortableSpriteToDraw(image, ti->x | drss->subcoord_x,
 				ti->y | drss->subcoord_y, drss->width, drss->height, 0x14, ti->z);
 			drss++;
 		}
@@ -805,12 +846,12 @@ void DrawRoadDepotSprite(int x, int y, int image)
 	x+=33;
 	y+=17;
 
-	DrawSprite(*(uint32*)t, x, y);
+	DrawSprite(*(const uint32*)t, x, y);
 	t += sizeof(uint32);
 
-	for(dtss = (DrawRoadSeqStruct *)t; dtss->image != 0; dtss++) {
+	for(dtss = (const DrawRoadSeqStruct *)t; dtss->image != 0; dtss++) {
 		Point pt = RemapCoords(dtss->subcoord_x, dtss->subcoord_y, 0);
-		
+
 		image = dtss->image;
 		if (image & 0x8000)
 			image |= ormod;
@@ -819,14 +860,14 @@ void DrawRoadDepotSprite(int x, int y, int image)
 	}
 }
 
-uint GetSlopeZ_Road(TileInfo *ti)
+static uint GetSlopeZ_Road(TileInfo *ti)
 {
 	uint z = ti->z;
 	int th = ti->tileh;
 
 	// check if it's a foundation
 	if (ti->tileh != 0) {
-		if ((ti->map5 & 0xF0) == 0) {
+		if ((ti->map5 & 0xE0) == 0) { /* road or crossing */
 			uint f = GetRoadFoundation(ti->tileh, ti->map5 & 0x3F);
 			if (f != 0) {
 				if (f < 15) {
@@ -842,10 +883,32 @@ uint GetSlopeZ_Road(TileInfo *ti)
 		}
 		return GetPartialZ(ti->x&0xF, ti->y&0xF, th) + z;
 	}
-	return z;
+	return z; // normal Z if no slope
 }
 
-static void GetAcceptedCargo_Road(uint tile, AcceptedCargo *ac)
+static uint GetSlopeTileh_Road(TileInfo *ti)
+{
+	// check if it's a foundation
+	if (ti->tileh != 0) {
+		if ((ti->map5 & 0xE0) == 0) { /* road or crossing */
+			uint f = GetRoadFoundation(ti->tileh, ti->map5 & 0x3F);
+			if (f != 0) {
+				if (f < 15) {
+					// leveled foundation
+					return 0;
+				}
+				// inclined foundation
+				return _inclined_tileh[f - 15];
+			}
+		} else if ((ti->map5 & 0xF0) == 0x20) {
+			// depot
+			return 0;
+		}
+	}
+	return ti->tileh;
+}
+
+static void GetAcceptedCargo_Road(uint tile, AcceptedCargo ac)
 {
 	/* not used */
 }
@@ -857,7 +920,7 @@ static void AnimateTile_Road(uint tile)
 	}
 }
 
-static const byte _city_road_types[5][2] = {
+static const byte _town_road_types[5][2] = {
 	{1,1},
 	{2,2},
 	{2,2},
@@ -865,7 +928,7 @@ static const byte _city_road_types[5][2] = {
 	{3,2},
 };
 
-static const byte _city_road_types_2[5][2] = {
+static const byte _town_road_types_2[5][2] = {
 	{1,1},
 	{2,2},
 	{3,2},
@@ -876,9 +939,9 @@ static const byte _city_road_types_2[5][2] = {
 
 static void TileLoop_Road(uint tile)
 {
-	City *c;
+	Town *t;
 	int grp;
-	
+
 	if (_opt.landscape == LT_HILLY) {
 		// Fix snow style if the road is above the snowline
 		if ((_map3_hi[tile] & 0x80) != ((GetTileZ(tile) > _opt.snow_line) ? 0x80 : 0x00)) {
@@ -897,27 +960,27 @@ static void TileLoop_Road(uint tile)
 		return;
 
 	if ((_map2[tile] & 7) < 6) {
-		c = ClosestCityFromTile(tile, (uint)-1);
+		t = ClosestTownFromTile(tile, (uint)-1);
 		grp = 0;
-		if (c != NULL) {
+		if (t != NULL) {
 			// If in the scenario editor, set the owner to a town.
 			if (_game_mode == GM_EDITOR) {
 				_map_owner[tile] = OWNER_TOWN;
 			}
 
-			grp = GetCityRadiusGroup(c, tile);
-			
+			grp = GetTownRadiusGroup(t, tile);
+
 			// Show an animation to indicate road work
-			if (c->road_build_months != 0 && 
-					!(GetTileDist(c->xy, tile) >= 8 && grp==0) &&
+			if (t->road_build_months != 0 &&
+					!(GetTileDist(t->xy, tile) >= 8 && grp==0) &&
 					(_map5[tile]==5 || _map5[tile]==10)) {
 				if (GetTileSlope(tile, NULL) == 0 && EnsureNoVehicle(tile) && CHANCE16(1,20)) {
 					_map2[tile] = ((_map2[tile]&7) <= 1) ? 6 : 7;
-					
-					SndPlayTileFx(0x1F,tile);
+
+					SndPlayTileFx(SND_21_JACKHAMMER, tile);
 					CreateEffectVehicleAbove(
-						GET_TILE_X(tile) * 16 + 7,
-						GET_TILE_Y(tile) * 16 + 7,
+						TileX(tile) * 16 + 7,
+						TileY(tile) * 16 + 7,
 						0,
 						EV_ROADWORK);
 					MarkTileDirtyByTile(tile);
@@ -927,12 +990,12 @@ static void TileLoop_Road(uint tile)
 		}
 
 		{
-			const byte *p = (_opt.landscape == LT_CANDY) ? _city_road_types_2[grp] : _city_road_types[grp];
+			const byte *p = (_opt.landscape == LT_CANDY) ? _town_road_types_2[grp] : _town_road_types[grp];
 			byte b = _map2[tile] & 7;
 
 			if (b == p[0])
 				return;
-			
+
 			if (b == p[1]) {
 				b = p[0];
 			} else if (b == 0) {
@@ -945,8 +1008,8 @@ static void TileLoop_Road(uint tile)
 		}
 	} else {
 		// Handle road work
-		
-		byte b = _map2[tile];
+
+		uint16 b = _map2[tile];
 		if (b < 0x80) {
 			_map2[tile] = b + 8;
 			return;
@@ -969,24 +1032,26 @@ static const byte _road_trackbits[16] = {
 	0x0, 0x0, 0x0, 0x10, 0x0, 0x2, 0x8, 0x1A, 0x0, 0x4, 0x1, 0x15, 0x20, 0x26, 0x29, 0x3F,
 };
 
-static uint32 GetTileTrackStatus_Road(uint tile, int mode)	{
-	if (mode == 0) {
+static uint32 GetTileTrackStatus_Road(uint tile, TransportType mode)	{
+	if (mode == TRANSPORT_RAIL) {
 		if ((_map5[tile] & 0xF0) != 0x10)
 			return 0;
 		return _map5[tile] & 8 ? 0x101 : 0x202;
-	} else if  (mode == 2) {
+	} else if  (mode == TRANSPORT_ROAD) {
 		byte b = _map5[tile];
 		if ((b & 0xF0) == 0) {
+			/* Ordinary road */
 			if (!_road_special_gettrackstatus && (_map2[tile]&7) >= 6)
 				return 0;
 			return _road_trackbits[b&0xF] * 0x101;
-		} else if ((b&0xE0) == 0) {
+		} else if ((b&0xF0) == 0x10) {
+			/* Crossing */
 			uint32 r = 0x101;
 			if (b&8) r <<= 1;
-				
+
 			if (b&4) {
 				r *= 0x10001;
-			}			
+			}
 			return r;
 		}
 	}
@@ -1025,7 +1090,7 @@ static uint32 VehicleEnter_Road(Vehicle *v, uint tile, int x, int y)
 	if ((_map5[tile] & 0xF0) == 0x10)	{
 		if (v->type == VEH_Train && (_map5[tile] & 4) == 0) {
 			/* train crossing a road */
-			SndPlayVehicleFx(12, v);
+			SndPlayVehicleFx(SND_0E_LEVEL_CROSSING, v);
 			_map5[tile] |= 4;
 			MarkTileDirtyByTile(tile);
 		}
@@ -1077,7 +1142,7 @@ static void ChangeTileOwner_Road(uint tile, byte old_player, byte new_player)
 	}
 }
 
-void InitializeRoad()
+void InitializeRoad(void)
 {
 	_last_built_road_depot_tile = 0;
 }
@@ -1096,4 +1161,5 @@ const TileTypeProcs _tile_type_road_procs = {
 	NULL,											/* get_produced_cargo_proc */
 	VehicleEnter_Road,				/* vehicle_enter_tile_proc */
 	VehicleLeave_Road,				/* vehicle_leave_tile_proc */
+	GetSlopeTileh_Road,				/* get_slope_tileh_proc */
 };
