@@ -1,7 +1,9 @@
 #ifndef SAVELOAD_H
 #define SAVELOAD_H
 
-typedef void ChunkSaveLoadProc();
+#include <setjmp.h>
+
+typedef void ChunkSaveLoadProc(void);
 typedef void AutolengthProc(void *arg);
 
 typedef struct SaveLoadGlobVarList {
@@ -23,11 +25,56 @@ typedef struct {
 	byte null;
 } NullStruct;
 
+typedef void WriterProc(uint len);
+typedef uint ReaderProc(void);
+
+typedef uint ReferenceToIntProc(void *v, uint t);
+typedef void *IntToReferenceProc(uint r, uint t);
+
+typedef struct {
+	bool save;
+	byte need_length;
+	byte block_mode;
+	bool error;
+	byte version;
+	uint16 full_version;
+
+	int obj_len;
+	int array_index, last_array_index;
+
+	uint32 offs_base;
+
+	WriterProc *write_bytes;
+	ReaderProc *read_bytes;
+
+	ReferenceToIntProc *ref_to_int_proc;
+	IntToReferenceProc *int_to_ref_proc;
+
+	const ChunkHandler * const * chs;
+	const byte * const *includes;
+
+	byte *bufp, *bufe;
+
+	int tmp;
+
+	// these 3 may be used by compressor/decompressors.
+	byte *buf; // pointer and size to read/write, initialized by init
+	uint bufsize;
+	FILE *fh;
+
+	void (*excpt_uninit)(void);
+	const char *excpt_msg;
+	jmp_buf excpt; // used to jump to "exception handler"
+} SaverLoader;
+
+extern SaverLoader _sl;
+
 enum {
-	REF_SCHEDULE = 0,
-	REF_VEHICLE = 1,
-	REF_STATION = 2,
-	REF_CITY = 3,
+	REF_ORDER       = 0,
+	REF_VEHICLE     = 1,
+	REF_STATION     = 2,
+	REF_TOWN        = 3,
+	REF_VEHICLE_OLD = 4
 };
 
 
@@ -112,17 +159,16 @@ enum {
 
 
 void SlSetArrayIndex(uint index);
-int SlIterateArray();
+int SlIterateArray(void);
 void SlArray(void *array, uint length, uint conv);
 void SlObject(void *object, const void *desc);
 void SlAutolength(AutolengthProc *proc, void *arg);
-uint SlGetFieldLength();
-int SlReadByte();
+uint SlGetFieldLength(void);
+int SlReadByte(void);
 void SlSetLength(uint length);
 void SlWriteByte(byte v);
 void SlGlobList(const SaveLoadGlobVarList *desc);
 
-
-
+//int GetSavegameType(char *file);
 
 #endif /* SAVELOAD_H */
