@@ -1,5 +1,7 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
+#include "map.h"
 #include "gui.h"
 #include "window.h"
 #include "gfx.h"
@@ -7,67 +9,31 @@
 #include "player.h"
 #include "vehicle.h"
 #include "town.h"
+#include "sound.h"
 
-static const Widget _smallmap_megabig_widgets[] = {
-{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    13,    11,   592,     0,    13, STR_00B0_MAP,STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    13,   593,   606,     0,    13, 0x2AA,				STR_01CC_TOGGLE_LARGE_SMALL_MAP},
-{     WWT_IMGBTN,    13,     0,   606,    14,   407, 0x0,					STR_NULL},
-{          WWT_6,    13,     2,   604,    16,   405, 0x0,					STR_NULL},
-{     WWT_IMGBTN,    13,   541,   562,   408,   429, 0x2E2,				STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
-{     WWT_IMGBTN,    13,   563,   584,   408,   429, 0x2E3,				STR_0192_SHOW_VEHICLES_ON_MAP},
-{     WWT_IMGBTN,    13,   585,   606,   408,   429, 0x2E5,				STR_0193_SHOW_INDUSTRIES_ON_MAP},
-{     WWT_IMGBTN,    13,   519,   540,   430,   451, 0x2E4,				STR_0194_SHOW_TRANSPORT_ROUTES_ON},
-{     WWT_IMGBTN,    13,   541,   562,   430,   451, 0x2E6,				STR_0195_SHOW_VEGETATION_ON_MAP},
-{     WWT_IMGBTN,    13,   563,   584,   430,   451, 0x2E7,				STR_0196_SHOW_LAND_OWNERS_ON_MAP},
-{     WWT_IMGBTN,    13,   585,   606,   430,   451, 0xFED,				STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,     0,   518,   408,   451, 0x0,					STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,   519,   540,   408,   429, 0x0,					STR_NULL},
-{   WIDGETS_END},
-};
-
-static const Widget _smallmap_big_widgets[] = {
-{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    13,    11,   431,     0,    13, STR_00B0_MAP,STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    13,   432,   445,     0,    13, 0x2AA,				STR_01CC_TOGGLE_LARGE_SMALL_MAP},
-{     WWT_IMGBTN,    13,     0,   445,    14,   257, 0x0,					STR_NULL},
-{          WWT_6,    13,     2,   443,    16,   255, 0x0,					STR_NULL},
-{     WWT_IMGBTN,    13,   380,   401,   258,   279, 0x2E2,				STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
-{     WWT_IMGBTN,    13,   402,   423,   258,   279, 0x2E3,				STR_0192_SHOW_VEHICLES_ON_MAP},
-{     WWT_IMGBTN,    13,   424,   445,   258,   279, 0x2E5,				STR_0193_SHOW_INDUSTRIES_ON_MAP},
-{     WWT_IMGBTN,    13,   380,   401,   280,   301, 0x2E4,				STR_0194_SHOW_TRANSPORT_ROUTES_ON},
-{     WWT_IMGBTN,    13,   402,   423,   280,   301, 0x2E6,				STR_0195_SHOW_VEGETATION_ON_MAP},
-{     WWT_IMGBTN,    13,   424,   445,   280,   301, 0x2E7,				STR_0196_SHOW_LAND_OWNERS_ON_MAP},
-{     WWT_IMGBTN,    13,   358,   379,   280,   301, 0xFED,				STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,     0,   357,   258,   301, 0x0,					STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,   358,   379,   258,   279, 0x0,					STR_NULL},
-{   WIDGETS_END},
-};
-
-
-static const Widget _smallmap_small_widgets[] = {
-{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    13,    11,   233,     0,    13, STR_00B0_MAP,STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    13,   234,   247,     0,    13, 0x2AA,				STR_01CC_TOGGLE_LARGE_SMALL_MAP},
-{     WWT_IMGBTN,    13,     0,   225,    14,   167, 0x0,					STR_NULL},
-{          WWT_6,    13,     2,   223,    16,   165, 0x0,					STR_NULL},
-{     WWT_IMGBTN,    13,   226,   247,    14,    35, 0x2E2,				STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,    36,    57, 0x2E3,				STR_0192_SHOW_VEHICLES_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,    58,    79, 0x2E5,				STR_0193_SHOW_INDUSTRIES_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,    80,   101, 0x2E4,				STR_0194_SHOW_TRANSPORT_ROUTES_ON},
-{     WWT_IMGBTN,    13,   226,   247,   102,   123, 0x2E6,				STR_0195_SHOW_VEGETATION_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,   124,   145, 0x2E7,				STR_0196_SHOW_LAND_OWNERS_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,   146,   167, 0xFED,				STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,     0,   247,   168,   211, 0x0,					STR_NULL},
+static const Widget _smallmap_widgets[] = {
+{    WWT_TEXTBTN,   RESIZE_NONE,    13,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,  RESIZE_RIGHT,    13,    11,   433,     0,    13, STR_00B0_MAP,STR_018C_WINDOW_TITLE_DRAG_THIS},
+{  WWT_STICKYBOX,     RESIZE_LR,    13,   434,   445,     0,    13, 0x0,         STR_STICKY_BUTTON},
+{     WWT_IMGBTN,     RESIZE_RB,    13,     0,   445,    14,   257, 0x0,					STR_NULL},
+{          WWT_6,     RESIZE_RB,    13,     2,   443,    16,   255, 0x0,					STR_NULL},
+{     WWT_IMGBTN,   RESIZE_LRTB,    13,   380,   401,   258,   279, 0x2E2,				STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
+{     WWT_IMGBTN,   RESIZE_LRTB,    13,   402,   423,   258,   279, 0x2E3,				STR_0192_SHOW_VEHICLES_ON_MAP},
+{     WWT_IMGBTN,   RESIZE_LRTB,    13,   424,   445,   258,   279, 0x2E5,				STR_0193_SHOW_INDUSTRIES_ON_MAP},
+{     WWT_IMGBTN,   RESIZE_LRTB,    13,   380,   401,   280,   301, 0x2E4,				STR_0194_SHOW_TRANSPORT_ROUTES_ON},
+{     WWT_IMGBTN,   RESIZE_LRTB,    13,   402,   423,   280,   301, 0x2E6,				STR_0195_SHOW_VEGETATION_ON_MAP},
+{     WWT_IMGBTN,   RESIZE_LRTB,    13,   424,   445,   280,   301, 0x2E7,				STR_0196_SHOW_LAND_OWNERS_ON_MAP},
+{     WWT_IMGBTN,   RESIZE_LRTB,    13,   358,   379,   258,   279, 0x0,					STR_NULL},
+{     WWT_IMGBTN,   RESIZE_LRTB,    13,   358,   379,   280,   301, 0xFED,				STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
+{     WWT_IMGBTN,    RESIZE_RTB,    13,     0,   357,   258,   301, 0x0,					STR_NULL},
+{      WWT_PANEL,    RESIZE_RTB,    13,     0,   433,   302,   313, 0x0,					STR_NULL},
+{  WWT_RESIZEBOX,   RESIZE_LRTB,    13,   434,   445,   302,   313, 0x0,					STR_RESIZE_BUTTON},
 {   WIDGETS_END},
 };
 
 static int _smallmap_type;
-static byte _smallmap_size;
 static bool _smallmap_show_towns = true;
 
-static void DoShowSmallMap(int big);
-
 #define MK(a,b) a,b
 #define MKEND() 0xffff
 #define MS(a,b) (a|0x100),b
@@ -220,7 +186,7 @@ static const uint16 * const _legend_table[] = {
 };
 
 #if defined(TTD_ALIGNMENT_4)
-	static INLINE void WRITE_PIXELS(void *dst, uint32 val)
+	static inline void WRITE_PIXELS(void *dst, uint32 val)
 	{
 		byte *d = (byte*)dst;
 #	if defined(TTD_BIG_ENDIAN)
@@ -237,7 +203,7 @@ static const uint16 * const _legend_table[] = {
 	}
 
 /* need to use OR, otherwise we will overwrite the wrong pixels at the edges :( */
-	static INLINE void WRITE_PIXELS_OR(void *dst, uint32 val)
+	static inline void WRITE_PIXELS_OR(void *dst, uint32 val)
 	{
 		byte *d = (byte*)dst;
 #	if defined(TTD_BIG_ENDIAN)
@@ -327,11 +293,11 @@ static const uint32 _smallmap_vegetation_andor[12][2] = {
 	{MKCOLOR(0x00D7D700),MKCOLOR(0xFF0000FF)},
 };
 
-static uint32 INLINE GetSmallMapCountoursPixels(uint tile)
+static inline uint32 GetSmallMapCountoursPixels(uint tile)
 {
 	uint t;
 
-	t = GET_TILETYPE(tile);
+	t = TileType(tile);
 	if (t == MP_TUNNELBRIDGE) {
 		t = _map5[tile];
 		if ((t & 0x80) == 0) t>>=1;
@@ -344,24 +310,24 @@ static uint32 INLINE GetSmallMapCountoursPixels(uint tile)
 		}
 	}
 
-	return (_map_height_bits[_map_type_and_height[tile] & 0xF] & _smallmap_contours_andor[t][1]) | _smallmap_contours_andor[t][0];
+	return (_map_height_bits[TileHeight(tile)] & _smallmap_contours_andor[t][1]) | _smallmap_contours_andor[t][0];
 }
 
 static void DrawSmallMapContours(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
 {
 	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+		if (xc < MapMaxX() && yc < MapMaxY())
 		    if (dst > _screen.dst_ptr && dst < (_screen.dst_ptr + _screen.width * _screen.height - _screen.width) )
 		        WRITE_PIXELS_OR( dst, GetSmallMapCountoursPixels(TILE_XY(xc,yc)) & mask );
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
 
-static uint32 INLINE GetSmallMapVehiclesPixels(uint tile)
+static inline uint32 GetSmallMapVehiclesPixels(uint tile)
 {
 	uint t;
 
-	t = GET_TILETYPE(tile);
+	t = TileType(tile);
 	if (t == MP_TUNNELBRIDGE) {
 		t = _map5[tile];
 		if ((t & 0x80) == 0) t>>=1;
@@ -380,7 +346,7 @@ static uint32 INLINE GetSmallMapVehiclesPixels(uint tile)
 static void DrawSmallMapVehicles(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
 {
 	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+		if (xc < MapMaxX() && yc < MapMaxY())
 			WRITE_PIXELS_OR( dst, GetSmallMapVehiclesPixels(TILE_XY(xc,yc)) & mask );
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
@@ -410,11 +376,11 @@ static const byte _industry_smallmap_colors[175] = {
 	 15, 15, 15, 15, 15, 15, 15,
 };
 
-static uint32 INLINE GetSmallMapIndustriesPixels(uint tile)
+static inline uint32 GetSmallMapIndustriesPixels(uint tile)
 {
 	int t;
 
-	t = GET_TILETYPE(tile);
+	t = TileType(tile);
 	if (t == MP_INDUSTRY) {
 		byte color = _industry_smallmap_colors[_map5[tile]];
 		return color + (color << 8) + (color << 16) + (color << 24);
@@ -437,17 +403,17 @@ static uint32 INLINE GetSmallMapIndustriesPixels(uint tile)
 static void DrawSmallMapIndustries(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
 {
 	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+		if (xc < MapMaxX() && yc < MapMaxY())
 			WRITE_PIXELS_OR(dst, GetSmallMapIndustriesPixels(TILE_XY(xc,yc)) & mask);
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
-static uint32 INLINE GetSmallMapRoutesPixels(uint tile)
+static inline uint32 GetSmallMapRoutesPixels(uint tile)
 {
 	int t;
 	uint32 bits;
 
-	t = GET_TILETYPE(tile);
+	t = TileType(tile);
 	if (t == MP_STATION) {
 		byte m5 = _map5[tile];
 		(bits = MKCOLOR(0x56565656), m5 < 8) ||			//   8 - railroad station (green)
@@ -478,7 +444,7 @@ static uint32 INLINE GetSmallMapRoutesPixels(uint tile)
 static void DrawSmallMapRoutes(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
 {
 	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+		if (xc < MapMaxX() && yc < MapMaxY())
 			WRITE_PIXELS_OR(dst, GetSmallMapRoutesPixels(TILE_XY(xc,yc)) & mask);
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
@@ -498,12 +464,12 @@ static const uint32 _vegetation_clear_bits[4 + 7] = {
 	MKCOLOR(0x54545454),
 };
 
-static uint32 INLINE GetSmallMapVegetationPixels(uint tile)
+static inline uint32 GetSmallMapVegetationPixels(uint tile)
 {
 	int i,t;
 	uint32 bits;
 
-	t = GET_TILETYPE(tile);
+	t = TileType(tile);
 	if (t == MP_CLEAR) {
 		i = (_map5[tile] & 0x1F) - 4;
 		if (i >= 0) i = (i >> 2);
@@ -536,19 +502,19 @@ static uint32 INLINE GetSmallMapVegetationPixels(uint tile)
 static void DrawSmallMapVegetation(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
 {
 	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+		if (xc < MapMaxX() && yc < MapMaxY())
 			WRITE_PIXELS_OR(dst, GetSmallMapVegetationPixels(TILE_XY(xc,yc)) & mask);
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
 
-static uint32 *_owner_colors;
+static uint32 _owner_colors[256];
 
-static uint32 INLINE GetSmallMapOwnerPixels(uint tile)
+static inline uint32 GetSmallMapOwnerPixels(uint tile)
 {
 	int t;
 
-	t = GET_TILETYPE(tile);
+	t = TileType(tile);
 	if (t == MP_HOUSE || _map_owner[tile] == OWNER_TOWN) {
 		t = 0x80;
 	} else if (t == MP_INDUSTRY) {
@@ -564,7 +530,7 @@ static uint32 INLINE GetSmallMapOwnerPixels(uint tile)
 static void DrawSmallMapOwners(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
 {
 	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+		if (xc < MapMaxX() && yc < MapMaxY())
 			WRITE_PIXELS_OR(dst, GetSmallMapOwnerPixels(TILE_XY(xc,yc)) & mask);
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
@@ -600,7 +566,7 @@ static const byte _vehicle_type_colors[6] = {
 	184, 191, 152, 15, 215, 184
 };
 
-static INLINE uint32 dup_byte32(byte b) {
+static inline uint32 dup_byte32(byte b) {
 	return b + (b << 8) + (b << 16) + (b << 24);
 }
 
@@ -637,24 +603,20 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 	/* setup owner table */
 	if (type == 5) {
 		Player *p;
-		uint32 *tbl;
-
-		/* setup owners ptr */
-		_owner_colors = tbl = alloca(256 * sizeof(*_owner_colors));
 
 		/* clear the town colors */
-		memset(tbl + 128, 0xB4, 128 * sizeof(*_owner_colors));
+		memset(&_owner_colors[128], 0xB4, 128 * sizeof(*_owner_colors));
 
 		/* fill with some special colors */
-		tbl[0x10] = MKCOLOR(0x54545454);
-		tbl[0x11] = MKCOLOR(0xCACACACA);
-		tbl[0xff] = MKCOLOR(0x20202020); /* industry */
+		_owner_colors[0x10] = MKCOLOR(0x54545454);
+		_owner_colors[0x11] = MKCOLOR(0xCACACACA);
+		_owner_colors[0xff] = MKCOLOR(0x20202020); /* industry */
 
 		/* now fill with the player colors */
 		FOR_ALL_PLAYERS(p) {
-			if (p->is_active) {
-				tbl[p->index] = dup_byte32(GetSpritePtr(0x307 + p->player_color)[0xCB]);
-			}
+			if (p->is_active)
+				_owner_colors[p->index] =
+					dup_byte32(GetSpritePtr(0x307 + p->player_color)[0xCB]);
 		}
 	}
 
@@ -784,8 +746,8 @@ skip_column:
 			if (t->xy != 0) {
 				// Remap the town coordinate
 				Point pt = RemapCoords(
-					(int)(GET_TILE_X(t->xy)*16 - WP(w,smallmap_d).scroll_x) >> 4,
-					(int)(GET_TILE_Y(t->xy)*16 - WP(w,smallmap_d).scroll_y) >> 4,
+					(int)(TileX(t->xy) * 16 - WP(w,smallmap_d).scroll_x) >> 4,
+					(int)(TileY(t->xy) * 16 - WP(w,smallmap_d).scroll_y) >> 4,
 					0);
 				x = pt.x - WP(w,smallmap_d).subscroll + 3 - (t->sign.width_2 >> 1);
 				y = pt.y;
@@ -796,7 +758,7 @@ skip_column:
 						y + 6 > dpi->top &&
 						y < dpi->top + dpi->height) {
 					// And draw it.
-					SET_DPARAM16(0, t->index);
+					SetDParam(0, t->index);
 					DrawString(x, y, STR_2056, 12);
 				}
 			}
@@ -842,17 +804,16 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 		int x,y,y_org;
 		DrawPixelInfo new_dpi;
 
-
 		/* draw the window */
-		SET_DPARAM16(0, STR_00E5_CONTOURS + _smallmap_type);
+		SetDParam(0, STR_00E5_CONTOURS + _smallmap_type);
 		DrawWindowWidgets(w);
 
 		/* draw the legend */
 		tbl = _legend_table[(_smallmap_type != 2) ? _smallmap_type : (_opt.landscape + IND_OFFS)];
 		x = 4;
-		y_org = w->height - 43;
+		y_org = w->height - 43 - 11;
 		y = y_org;
-		while(1) {
+		while (true) {
 			GfxFillRect(x, y+1, x+8, y + 5, 0);
 			GfxFillRect(x+1, y+2, x+7, y + 4, (byte)tbl[0]);
 			DrawString(x+11, y, tbl[1], 0);
@@ -868,7 +829,7 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 			}
 		}
 
-		if (!FillDrawPixelInfo(&new_dpi, NULL, 3, 17, w->width - 28 + (w->widget == _smallmap_small_widgets ? 0 : 22), w->height - 64))
+		if (!FillDrawPixelInfo(&new_dpi, NULL, 3, 17, w->width - 28 + 22, w->height - 64 - 11))
 			return;
 
 		DrawSmallMap(&new_dpi, w, _smallmap_type, _smallmap_show_towns);
@@ -876,17 +837,6 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 
 	case WE_CLICK:
 		switch(e->click.widget) {
-		case 2: {/* big/small size */
-			// const Widget *wi = w->widget;
-			DeleteWindow(w);
-			SndPlayFx(0x13);
-			{
-				int i = _smallmap_size + 1;
-				if (i == 3) i = 0;
-				DoShowSmallMap(i);
-			}
-		} break;
-
 		case 4: {/* main wnd */
 			Window *w2;
 			Point pt;
@@ -911,14 +861,14 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 			_smallmap_type = e->click.widget - 5;
 
 			SetWindowDirty(w);
-			SndPlayFx(0x13);
+			SndPlayFx(SND_15_BEEP);
 			break;
 
-		case 11: /* toggle town names */
-			w->click_state ^= (1 << 11);
-			_smallmap_show_towns = (w->click_state >> 11) & 1;
+		case 12: /* toggle town names */
+			w->click_state ^= (1 << 12);
+			_smallmap_show_towns = (w->click_state >> 12) & 1;
 			SetWindowDirty(w);
-			SndPlayFx(0x13);
+			SndPlayFx(SND_15_BEEP);
 			break;
 		}
 		break;
@@ -941,72 +891,50 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 	}
 }
 
-static const WindowDesc _smallmap_small_desc = {
-	-1,-1, 248, 212,
+static const WindowDesc _smallmap_desc = {
+	-1,-1, 446, 314,
 	WC_SMALLMAP,0,
-	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
-	_smallmap_small_widgets,
+	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_STICKY_BUTTON | WDF_RESIZABLE,
+	_smallmap_widgets,
 	SmallMapWindowProc
 };
 
-static const WindowDesc _smallmap_big_desc = {
-	-1,-1, 446, 302,
-	WC_SMALLMAP,0,
-	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
-	_smallmap_big_widgets,
-	SmallMapWindowProc
-};
-
-static const WindowDesc _smallmap_megabig_desc = {
-	-1,-1, 607, 452,
-	WC_SMALLMAP,0,
-	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
-	_smallmap_megabig_widgets,
-	SmallMapWindowProc
-};
-
-static const WindowDesc * const _smallmap_descs[3] = {
-	&_smallmap_small_desc,
-	&_smallmap_big_desc,
-	&_smallmap_megabig_desc
-};
-
-static void DoShowSmallMap(int big)
+void ShowSmallMap(void)
 {
 	Window *w;
 	ViewPort *vp;
 	int x,y;
 
-	_smallmap_size = big;
-
-	w = AllocateWindowDescFront(_smallmap_descs[big], 0);
+	w = AllocateWindowDescFront(&_smallmap_desc, 0);
 	if (w) {
-		w->click_state = (1<<5) << _smallmap_type | _smallmap_show_towns << 11;
+		w->click_state = ((1<<5) << _smallmap_type) | (_smallmap_show_towns << 12);
+		w->resize.width = 350;
+		w->resize.height = 250;
+
 		vp = FindWindowById(WC_MAIN_WINDOW, 0)->viewport;
 
-		x = (((vp->virtual_width - (big?220*32:110*32)) >> 1) + vp->virtual_left) >> 2;
-		y = ((((vp->virtual_height- (big?120*32:75*32)) >> 1) + vp->virtual_top) >> 1) - 32;
+		x =  (((vp->virtual_width - (220*32)) / 2) + vp->virtual_left) / 4;
+		y = ((((vp->virtual_height- (120*32)) / 2) + vp->virtual_top ) / 2) - 32;
 		WP(w,smallmap_d).scroll_x = (y-x) & ~0xF;
 		WP(w,smallmap_d).scroll_y = (x+y) & ~0xF;
 		WP(w,smallmap_d).subscroll = 0;
 	}
 }
 
-void ShowSmallMap()
-{
-	DoShowSmallMap(_smallmap_size);
-}
-
 /* Extra ViewPort Window Stuff */
-static Widget _extra_view_port_widgets[] = {
-{   WWT_CLOSEBOX,    14,     0,    10,     0,    13, STR_00C5,	STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   299,     0,    13, STR_EXTRA_VIEW_PORT_TITLE,				STR_018C_WINDOW_TITLE_DRAG_THIS},
-{      WWT_PANEL,    14,     0,   299,    14,   233, 0x0,				STR_NULL},
-{          WWT_6,    14,     2,   297,    16,   231, 0x0,				STR_NULL},
-{      WWT_PANEL,    14,     0,    21,   234,   255, 0x2DF,			STR_017F_ZOOM_THE_VIEW_IN},
-{      WWT_PANEL,    14,    22,    43,   234,   255, 0x2E0,			STR_0180_ZOOM_THE_VIEW_OUT},
-{ WWT_PUSHTXTBTN,    14,    44,   171,   234,   255, STR_EXTRA_VIEW_MOVE_MAIN_TO_VIEW,STR_EXTRA_VIEW_MOVE_MAIN_TO_VIEW_TT},
-{ WWT_PUSHTXTBTN,    14,   172,   299,   234,   255, STR_EXTRA_VIEW_MOVE_VIEW_TO_MAIN,STR_EXTRA_VIEW_MOVE_VIEW_TO_MAIN_TT},
+static const Widget _extra_view_port_widgets[] = {
+{   WWT_CLOSEBOX,   RESIZE_NONE,    14,     0,    10,     0,    13, STR_00C5,	STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,  RESIZE_RIGHT,    14,    11,   287,     0,    13, STR_EXTRA_VIEW_PORT_TITLE,				STR_018C_WINDOW_TITLE_DRAG_THIS},
+{  WWT_STICKYBOX,     RESIZE_LR,    14,   288,   299,     0,    13, 0x0,       STR_STICKY_BUTTON},
+{      WWT_PANEL,     RESIZE_RB,    14,     0,   299,    14,   233, 0x0,				STR_NULL},
+{          WWT_6,     RESIZE_RB,    14,     2,   297,    16,   231, 0x0,				STR_NULL},
+{      WWT_PANEL,     RESIZE_TB,    14,     0,    21,   234,   255, 0x2DF,			STR_017F_ZOOM_THE_VIEW_IN},
+{      WWT_PANEL,     RESIZE_TB,    14,    22,    43,   234,   255, 0x2E0,			STR_0180_ZOOM_THE_VIEW_OUT},
+{ WWT_PUSHTXTBTN,     RESIZE_TB,    14,    44,   171,   234,   255, STR_EXTRA_VIEW_MOVE_MAIN_TO_VIEW,STR_EXTRA_VIEW_MOVE_MAIN_TO_VIEW_TT},
+{ WWT_PUSHTXTBTN,     RESIZE_TB,    14,   172,   298,   234,   255, STR_EXTRA_VIEW_MOVE_VIEW_TO_MAIN,STR_EXTRA_VIEW_MOVE_VIEW_TO_MAIN_TT},
+{      WWT_PANEL,    RESIZE_RTB,    14,   299,   299,   234,   255, 0x0,				STR_NULL},
+{      WWT_PANEL,    RESIZE_RTB,    14,     0,   287,   256,   267, 0x0,				STR_NULL},
+{  WWT_RESIZEBOX,   RESIZE_LRTB,    14,   288,   299,   256,   267, 0x0,				STR_RESIZE_BUTTON},
 {   WIDGETS_END},
 };
 
@@ -1017,66 +945,73 @@ static void ExtraViewPortWndProc(Window *w, WindowEvent *e)
 	switch(e->event) {
 	case WE_PAINT: {
 		// set the number in the title bar
-		SET_DPARAM16(0, (w->window_number+1));
+		SetDParam(0, (w->window_number+1));
 
 		DrawWindowWidgets(w);
 		DrawWindowViewport(w);
 	}	break;
 	case WE_CLICK: {
 		switch(e->click.widget) {
-		case 4: { /* zoom in */
-			DoZoomInOutWindow(ZOOM_IN,w);
+		case 5: { /* zoom in */
+			DoZoomInOutWindow(ZOOM_IN, w);
 		} break;
 
-		case 5: { /* zoom out */
-			DoZoomInOutWindow(ZOOM_OUT,w);
+		case 6: { /* zoom out */
+			DoZoomInOutWindow(ZOOM_OUT, w);
 		} break;
 
-		case 6: { /* location button (move main view to same spot as this view) */
+		case 7: { /* location button (move main view to same spot as this view) */
 			Window * w2 = FindWindowById(WC_MAIN_WINDOW, 0);
 			int x = WP(w,vp_d).scrollpos_x; // Where is the main looking at
 			int y = WP(w,vp_d).scrollpos_y;
 
 			// set this view to same location. Based on the center, adjusting for zoom
-			WP(w2,vp_d).scrollpos_x =  x - (w2->viewport->virtual_width - (294 <<vp->zoom) )/2;
-			WP(w2,vp_d).scrollpos_y =  y - (w2->viewport->virtual_height - (214 << vp->zoom) )/2;
+			WP(w2,vp_d).scrollpos_x =  x - (w2->viewport->virtual_width -  (w->viewport->virtual_width  << vp->zoom)) / 2;
+			WP(w2,vp_d).scrollpos_y =  y - (w2->viewport->virtual_height - (w->viewport->virtual_height << vp->zoom)) / 2;
 		} break;
-		case 7: { /* inverse location button (move this view to same spot as main view) */
+		case 8: { /* inverse location button (move this view to same spot as main view) */
 			Window * w2 = FindWindowById(WC_MAIN_WINDOW, 0);
 			int x = WP(w2,vp_d).scrollpos_x;
 			int y = WP(w2,vp_d).scrollpos_y;
 
-			WP(w,vp_d).scrollpos_x =  x + (w2->viewport->virtual_width - (294 <<vp->zoom) )/2;
-			WP(w,vp_d).scrollpos_y =  y + (w2->viewport->virtual_height - (214 << vp->zoom) )/2;
+			WP(w,vp_d).scrollpos_x =  x + (w2->viewport->virtual_width -  (w->viewport->virtual_width  << vp->zoom)) / 2;
+			WP(w,vp_d).scrollpos_y =  y + (w2->viewport->virtual_height - (w->viewport->virtual_height << vp->zoom)) / 2;
 		} break;
 		}
 	} break;
+	case WE_RESIZE: {
+		w->viewport->width  += e->sizing.diff.x;
+		w->viewport->height += e->sizing.diff.y;
+
+		w->viewport->virtual_width  += e->sizing.diff.x;
+		w->viewport->virtual_height += e->sizing.diff.y;
+	} break;
 	}
 }
 
 static const WindowDesc _extra_view_port_desc = {
-	-1,-1, 300, 256,
+	-1,-1, 300, 268,
 	WC_EXTRA_VIEW_PORT,0,
-	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
+	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS | WDF_STICKY_BUTTON | WDF_RESIZABLE,
 	_extra_view_port_widgets,
 	ExtraViewPortWndProc
 };
 
-void ShowExtraViewPortWindow()
+void ShowExtraViewPortWindow(void)
 {
 	Window *w, *v;
 	int i = 0;
 
 	// find next free window number for extra viewport
-	while (FindWindowById(WC_EXTRA_VIEW_PORT,i) ) {
+	while (FindWindowById(WC_EXTRA_VIEW_PORT, i) ) {
 		i++;
 	}
 
-	w = AllocateWindowDescFront(&_extra_view_port_desc,i);
+	w = AllocateWindowDescFront(&_extra_view_port_desc, i);
 	if (w) {
-		int x,y;
+		int x, y;
 		// disable zoom in button
-		w->disabled_state = 1 << 4;
+		w->disabled_state = (1 << 4);
 		// the main window with the main view
 		v = FindWindowById(WC_MAIN_WINDOW, 0);
 		// New viewport start ats (zero,zero)
@@ -1085,7 +1020,7 @@ void ShowExtraViewPortWindow()
 		// center on same place as main window (zoom is maximum, no adjustment needed)
 		x = WP(v,vp_d).scrollpos_x;
 		y = WP(v,vp_d).scrollpos_y;
-		WP(w,vp_d).scrollpos_x =  x + (v->viewport->virtual_width - (294) )/2;
-		WP(w,vp_d).scrollpos_y =  y + (v->viewport->virtual_height - (214)  )/2;
+		WP(w,vp_d).scrollpos_x = x + (v->viewport->virtual_width  - (294)) / 2;
+		WP(w,vp_d).scrollpos_y = y + (v->viewport->virtual_height - (214)) / 2;
 	}
 }
