@@ -2,6 +2,8 @@
 #include "ttd.h"
 #include "gfx.h"
 #include "fileio.h"
+#include "newgrf.h"
+#include "md5.h"
 #include <ctype.h>
 
 #define SPRITECACHE_ID 0xF00F0006
@@ -13,12 +15,19 @@
 //#define WANT_LOCKED
 
 
+/* These are used in newgrf.c: */
+
 int _skip_sprites = 0;
+int _replace_sprites_count[16];
+int _replace_sprites_offset[16];
 
 static const char *_cur_grffile;
+static int _loading_stage;
 static int _skip_specials;
+uint16 _custom_sprites_base;
 static SpriteHdr _cur_sprite;
 
+
 static byte *_sprite_ptr[NUM_SPRITES];
 static uint16 _sprite_size[NUM_SPRITES];
 static uint32 _sprite_file_pos[NUM_SPRITES];
@@ -47,22 +56,17 @@ static byte *_spritecache_ptr;
 static uint32 _spritecache_size;
 static int _compact_cache_counter;
 
+typedef struct MD5File {
+	const char * const filename;     // filename
+	const md5_byte_t hash[16]; // md5 sum of the file
+} MD5File;
 
-static const char * const _filename_list[] = {
-	"TRG1R.GRF",
-	"TRGIR.GRF",
-	"signalsw.grf", //0x1320 - 0x1405 inclusive
-//	"openttd.grf",	//0x1406 - 
-	NULL
-};
-
-static const char * const _landscape_filenames[] = {
-	"TRGCR.GRF",
-	"TRGHR.GRF",
-	"TRGTR.GRF"
-};
-
+typedef struct FileList {
+	const MD5File basic[4];     // grf files that always have to be loaded
+	const MD5File landscape[3]; // landscape specific grf files
+} FileList;
 
+#include "table/files.h"
 #include "table/landscape_sprite.h"
 
 static const uint16 * const _landscape_spriteindexes[] = {
@@ -71,9 +75,14 @@ static const uint16 * const _landscape_spriteindexes[] = {
 	_landscape_spriteindexes_3,
 };
 
-static void CompactSpriteCache();
+static const uint16 * const _slopes_spriteindexes[] = {
+	_slopes_spriteindexes_0,
+	_slopes_spriteindexes_1,
+	_slopes_spriteindexes_2,
+	_slopes_spriteindexes_3,
+};
 
-void DecodeSpecialSprite(const char *filename, int num, int load_index);
+static void CompactSpriteCache(void);
 
 static void ReadSpriteHeaderSkipData(int num, int load_index)
 {
@@ -86,8 +95,9 @@ static void ReadSpriteHeaderSkipData(int num, int load_index)
 			_skip_sprites--;
 		deaf = 1;
 	}
-	
+
 	type = FioReadByte();
+	_cur_sprite.info = type;
 	if (type == 0xFF) {
 		/* We need to really skip only special sprites in the deaf
 		 * mode.  It won't hurt to proceed regular sprites as usual
@@ -97,12 +107,11 @@ static void ReadSpriteHeaderSkipData(int num, int load_index)
 		if (_skip_specials || deaf) {
 			FioSkipBytes(num);
 		} else {
-			DecodeSpecialSprite(_cur_grffile, num, load_index);
+			DecodeSpecialSprite(_cur_grffile, num, load_index, _loading_stage);
 		}
 		return;
 	}
-	
-	_cur_sprite.info = type;
+
 #ifdef WANT_SPRITESIZES
 	_cur_sprite.height = FioReadByte();
 	_cur_sprite.width = FioReadWord();
@@ -139,7 +148,7 @@ static void ReadSprite(int num, byte *dest)
 	byte *rel;
 	int8 i;
 	int j, dist;
-	
+
 	type = FioReadByte();
 	/* We've decoded special sprites when reading headers. */
 	if (type != 0xFF) {
@@ -166,7 +175,7 @@ static void ReadSprite(int num, byte *dest)
 			dist = -(((i&7)<<8)|FioReadByte());
 			i = -(i >> 3);
 			num -= i;
-			
+
 			rel = &dest[dist];
 			while (i--)
 				*dest++ = *rel++;
@@ -178,15 +187,46 @@ static void ReadSprite(int num, byte *dest)
 static bool LoadNextSprite(int load_index, byte file_index)
 {
 	uint16 size;
+	uint32 file_pos;
 
-	if ( (size = FioReadWord()) == 0)
+	if ((size = FioReadWord()) == 0)
 		return false;
 
-	_sprite_size[load_index] = size;
-	_sprite_file_pos[load_index] = FioGetPos() | (file_index << 24);
+	file_pos = FioGetPos() | (file_index << 24);
 
 	ReadSpriteHeaderSkipData(size, load_index);
 
+	if ((_replace_sprites_count[0] > 0) && (_cur_sprite.info != 0xFF)) {
+		int count = _replace_sprites_count[0];
+		int offset = _replace_sprites_offset[0];
+
+		_replace_sprites_offset[0]++;
+		_replace_sprites_count[0]--;
+
+		if ((offset + count) <= NUM_SPRITES) {
+			load_index = offset;
+		} else {
+			DEBUG(spritecache, 1) ("Sprites to be replaced are out of range: %x+%x",
+					count, offset);
+			_replace_sprites_offset[0] = 0;
+			_replace_sprites_count[0] = 0;
+		}
+
+		if (_replace_sprites_count[0] == 0) {
+			int i;
+
+			for (i = 0; i < 15; i++) {
+				_replace_sprites_count[i] = _replace_sprites_count[i + 1];
+				_replace_sprites_offset[i] = _replace_sprites_offset[i + 1];
+			}
+			_replace_sprites_count[i] = 0;
+			_replace_sprites_offset[i] = 0;
+		}
+	}
+
+	_sprite_size[load_index] = size;
+	_sprite_file_pos[load_index] = file_pos;
+
 #ifdef WANT_SPRITESIZES
 	_sprite_xsize[load_index] = _cur_sprite.width;
 	_sprite_ysize[load_index] = _cur_sprite.height;
@@ -202,7 +242,7 @@ static bool LoadNextSprite(int load_index, byte file_index)
 
 #if defined(WANT_NEW_LRU)
 	_sprite_lru_new[load_index] = 0;
-#else	
+#else
 	_sprite_lru[load_index] = 0xFFFF;
 	_sprite_lru_cur[load_index] = 0;
 #endif
@@ -210,14 +250,17 @@ static bool LoadNextSprite(int load_index, byte file_index)
 	return true;
 }
 
-// Checks, if trg1r.grf is the Windows version
-static bool CheckGrfFile()
+static void SkipSprites(int count)
 {
-	byte check;
-	FioSeekToFile(38); // Byte 38 has the value 0x21 in Windows version, 0x07 in DOS
-	check = FioReadWord();
-	FioSeekToFile(0);
-	return (check==0x21);
+	while(count>0)
+	{
+		uint16 size;
+		if ( (size = FioReadWord()) == 0)
+			return;
+
+		ReadSpriteHeaderSkipData(size, NUM_SPRITES-1);
+		count--;
+	}
 }
 
 static int LoadGrfFile(const char *filename, int load_index, int file_index)
@@ -225,12 +268,13 @@ static int LoadGrfFile(const char *filename, int load_index, int file_index)
 	int load_index_org = load_index;
 
 	FioOpenFile(file_index, filename);
-	_cur_grffile = filename;
-	_skip_specials = !strncmp(filename, "TRG", 3);
 
- 	if(file_index==0 && !_ignore_wrong_grf)
- 		if(!CheckGrfFile())
- 			error("Wrong version of grf files!\nThe Windows 95 edition of Transport Tycoon Deluxe is required to play OTTD!\n(you can disable this message by starting with the \"-i\" switch.");
+	/* Thou shalt use LoadNewGrfFile() if thou loadeth a GRF file that
+	 * might contain some special sprites. */
+	_skip_specials = 1;
+	_skip_sprites = 0;
+
+	DEBUG(spritecache, 2) ("Reading grf-file ``%s''", filename);
 
 	while (LoadNextSprite(load_index, file_index)) {
 		load_index++;
@@ -239,25 +283,74 @@ static int LoadGrfFile(const char *filename, int load_index, int file_index)
 		}
 	}
 
-	_skip_sprites = 0; // clean up
-
 	return load_index - load_index_org;
 }
 
+static int LoadNewGrfFile(const char *filename, int load_index, int file_index)
+{
+	int i;
+
+	FioOpenFile(file_index, filename);
+	_cur_grffile = filename;
+	_skip_specials = 0;
+	_skip_sprites = 0;
+
+	DEBUG(spritecache, 2) ("Reading newgrf-file ``%s'' [offset: %u]",
+			filename, load_index);
+
+	/* Skip the first sprite; we don't care about how many sprites this
+	 * does contain; newest TTDPatches and George's longvehicles don't
+	 * neither, apparently. */
+	{
+		int length;
+		byte type;
+
+		length = FioReadWord();
+		type = FioReadByte();
+
+		if ((length == 4) && (type == 0xFF)) {
+			FioReadDword();
+		} else {
+			error("Custom .grf has invalid format.");
+		}
+	}
+
+	for (i = 0; LoadNextSprite(load_index + i, file_index); i++) {
+		if (load_index + i >= NUM_SPRITES)
+			error("Too many sprites (%x). Recompile with higher NUM_SPRITES value or remove some custom GRF files.",
+			      load_index + i);
+	}
+
+	/* Clean up. */
+	_skip_sprites = 0;
+	memset(_replace_sprites_count, 0, 16 * sizeof(*_replace_sprites_count));
+	memset(_replace_sprites_offset, 0, 16 * sizeof(*_replace_sprites_offset));
+
+	return i;
+}
+
 static void LoadGrfIndexed(const char *filename, const uint16 *index_tbl, int file_index)
 {
-	int start, end;
+	int start;
 
 	FioOpenFile(file_index, filename);
+	_skip_specials = 1;
+	_skip_sprites = 0;
+
+	DEBUG(spritecache, 2) ("Reading indexed grf-file ``%s''", filename);
 
 	for(;(start=*index_tbl++) != 0xffff;) {
-		end = *index_tbl++;
-		do {
-			bool b = LoadNextSprite(start, file_index);
-			assert(b);
-		} while (++start <= end);
+		int end = *index_tbl++;
+		if(start==0xfffe) { // skip sprites (amount in second var)
+			SkipSprites(end);
+		} else { // load sprites and use indexes from start to end
+			do {
+				bool b = LoadNextSprite(start, file_index);
+				assert(b);
+			} while (++start <= end);
+		}
 	}
-}	
+}
 
 typedef size_t CDECL fread_t(void*,size_t,size_t,FILE*);
 
@@ -316,7 +409,7 @@ static bool HandleCachedSpriteHeaders(const char *filename, bool read)
 #define S_FREE_MASK 1
 #define S_HDRSIZE sizeof(uint32)
 
-static uint32 GetSpriteCacheUsage()
+static uint32 GetSpriteCacheUsage(void)
 {
 	byte *s = _spritecache_ptr;
 	size_t cur_size, tot_size = 0;
@@ -332,7 +425,7 @@ static uint32 GetSpriteCacheUsage()
 }
 
 
-void IncreaseSpriteLRU()
+void IncreaseSpriteLRU(void)
 {
 	int i;
 
@@ -368,25 +461,25 @@ void IncreaseSpriteLRU()
 
 // Called when holes in the sprite cache should be removed.
 // That is accomplished by moving the cached data.
-static void CompactSpriteCache()
+static void CompactSpriteCache(void)
 {
 	byte *s, *t;
 	size_t size, sizeb, cur_size;
 	int i;
-	
+
 	DEBUG(spritecache, 2) ("compacting sprite cache, inuse=%d", GetSpriteCacheUsage());
 
 	s = _spritecache_ptr;
-	
+
 	while (true) {
 		size = S_DATA(s);
-		
+
 		// Only look for free blocks.
 		if (size & S_FREE_MASK) {
 			size -= S_FREE_MASK;
 			// Since free blocks are automatically coalesced, this should hold true.
 			assert(!(S_DATA(s+size) & S_FREE_MASK));
-			
+
 			// If the next block is the sentinel block, we can safely return
 			if ( (sizeb=S_DATA(s + size)) == 0)
 				break;
@@ -402,13 +495,13 @@ static void CompactSpriteCache()
 				// Offset the sprite pointer by the size of the free block
 				_sprite_ptr[i] -= size;
 
-				// Move the memory 
-				memcpy_overlapping(s+S_HDRSIZE, s+S_HDRSIZE+size, sizeb - S_HDRSIZE );
+				// Move the memory
+				memmove(s + S_HDRSIZE, s + S_HDRSIZE + size, sizeb - S_HDRSIZE);
 
 				// What we just did had the effect of swapping the allocated block with the free block, so we need to update
 				//  the block pointers. First update the allocated one. It is in use.
 				S_DATA(s) = sizeb;
-							
+
 				// Then coalesce the free ones that follow.
 				s += sizeb;
 				while ((cur_size = S_DATA(s+size)) & S_FREE_MASK)
@@ -426,7 +519,7 @@ static void CompactSpriteCache()
 	}
 }
 
-static void DeleteEntryFromSpriteCache()
+static void DeleteEntryFromSpriteCache(void)
 {
 	int i;
 	int best = -1;
@@ -460,7 +553,7 @@ static void DeleteEntryFromSpriteCache()
 #endif
 				_sprite_lru[i] < cur_lru)
 					continue;
-		
+
 		// Found a sprite with a higher LRU value, then remember it.
 		if (_sprite_lru[i] != cur_lru) {
 			cur_lru = _sprite_lru[i];
@@ -526,7 +619,7 @@ restart:
 			// Now s points at a free block.
 			// The block is exactly the size we need?
 			if (cur_size != mem_req) {
-				
+
 				// No.. is it too small?
 				if (cur_size < mem_req + S_HDRSIZE)
 					break;
@@ -542,7 +635,7 @@ restart:
 
 			FioSeekToFile(_sprite_file_pos[sprite]);
 			ReadSprite(_sprite_size[sprite], s);
-			
+
 			// Patch the height to compensate for a TTD bug?
 			if (sprite == 142) { s[1] = 10; }
 
@@ -555,7 +648,7 @@ restart:
 			DeleteEntryFromSpriteCache();
 			goto restart;
 		}
-		
+
 		s += cur_size;
 	}
 }
@@ -566,7 +659,7 @@ restart:
 #define MAP(from,to,map) else if (s >= from && s <= to) { s = map[s - from] + from; }
 
 
-uint RotateSprite(uint s)
+static uint RotateSprite(uint s)
 {
 	static const byte _rotate_tile_sprite[19] = { 0,2,4,6,8,10,12,14,1,3,5,7,9,11,13,17,18,16,15 };
 	static const byte _coast_map[9] = {0, 4, 3, 1, 2, 6, 8, 5, 7};
@@ -603,7 +696,7 @@ uint RotateSprite(uint s)
 }
 #endif
 
-byte *GetSpritePtr(uint sprite) 
+byte *GetSpritePtr(uint sprite)
 {
 	byte *p;
 
@@ -625,7 +718,7 @@ byte *GetSpritePtr(uint sprite)
 	p = _sprite_ptr[sprite];
 	if (p == NULL)
 		p = LoadSpriteToMem(sprite);  // No, need to load it.
-	return p;	
+	return p;
 }
 
 byte _sprite_page_to_load = 0xFF;
@@ -637,61 +730,213 @@ static const char * const _cached_filenames[4] = {
 	"cached_sprites.xx3",
 };
 
+#define OPENTTD_SPRITES_COUNT 98
 static const uint16 _openttd_grf_indexes[] = {
 	SPR_OPENTTD_BASE+0,	SPR_OPENTTD_BASE+7, // icons etc
-	98,98,     // euro symbol medium size
-	546,546,   // euro symbol large size
-	SPR_OPENTTD_BASE+10, SPR_OPENTTD_BASE+57, // more icons
+	134, 134,  // euro symbol medium size
+	582, 582,  // euro symbol large size
+	358, 358,  // euro symbol tiny
+	SPR_OPENTTD_BASE+11, SPR_OPENTTD_BASE+57, // more icons
 	648, 648, // nordic char: 
 	616, 616, // nordic char: 
 	666, 666, // nordic char: 
 	634, 634, // nordic char: 
-	SPR_OPENTTD_BASE+62, SPR_OPENTTD_BASE+67, // more icons
+	SPR_OPENTTD_BASE+62, SPR_OPENTTD_BASE + OPENTTD_SPRITES_COUNT, // more icons
 	0xffff,
 };
 
-static void LoadSpriteTables()
+/* FUNCTIONS FOR CHECKING MD5 SUMS OF GRF FILES */
+
+/* Check that the supplied MD5 hash matches that stored for the supplied filename */
+static bool CheckMD5Digest(const MD5File file, md5_byte_t *digest, bool warn)
+{
+	int i, matching_bytes=0;
+
+	/* Loop through each byte of the file MD5 and the stored MD5... */
+	for (i = 0; i < 16; i++)
+	{
+		if (file.hash[i] == digest[i])
+			matching_bytes++;
+	};
+
+		/* If all bytes of the MD5's match (i.e. the MD5's match)... */
+	if (matching_bytes == 16) {
+		return true;
+	} else {
+		if (warn) printf("MD5 of %s is ****INCORRECT**** - File Corrupt.\n", file.filename);
+		return false;
+	};
+}
+
+/* Calculate and check the MD5 hash of the supplied filename.
+ * returns true if the checksum is correct */
+static bool FileMD5(const MD5File file, bool warn)
+{
+	FILE *f;
+	char buf[MAX_PATH];
+
+	md5_state_t filemd5state;
+	int len=0;
+	md5_byte_t buffer[1024], digest[16];
+
+	// open file
+	sprintf(buf, "%s%s", _path.data_dir, file.filename);
+	f = fopen(buf, "rb");
+
+#if !defined(WIN32)
+	if (f == NULL) {
+		byte *s;
+	// make lower case and check again
+		for (s = buf + strlen(_path.data_dir) - 1; *s != 0; s++)
+			*s = tolower(*s);
+		f = fopen(buf, "rb");
+	}
+#endif
+
+	if (f != NULL) {
+		md5_init(&filemd5state);
+		while ( (len = fread (buffer, 1, 1024, f)) )
+			md5_append(&filemd5state, buffer, len);
+
+		if (ferror(f))
+			if (warn) printf ("Error Reading from %s \n", buf);
+		fclose(f);
+
+		md5_finish(&filemd5state, digest);
+	  return CheckMD5Digest(file, digest, warn);
+	} else { // file not found
+		return false;
+	}
+}
+
+/* Checks, if either the Windows files exist (TRG1R.GRF) or the DOS files (TRG1.GRF)
+ * by comparing the MD5 checksums of the files. _use_dos_palette is set accordingly.
+ * If neither are found, Windows palette is assumed.
+ *
+ * (Note: Also checks sample.cat for corruption) */
+void CheckExternalFiles(void)
+{
+	int i;
+	int dos=0, win=0; // count of files from this version
+
+	for (i=0; i<2; i++)
+	  if ( FileMD5(files_dos.basic[i], true) )
+			dos++;
+	for (i=0; i<3; i++)
+	  if ( FileMD5(files_dos.landscape[i], true) )
+			dos++;
+
+	for (i=0; i<2; i++)
+	  if ( FileMD5(files_win.basic[i], true) )
+			win++;
+	for (i=0; i<3; i++)
+	  if ( FileMD5(files_win.landscape[i], true) )
+			win++;
+
+	if ( !FileMD5(sample_cat_win, false) && !FileMD5(sample_cat_dos, false) )
+		printf("Your sample.cat file is corrupted or missing!");
+
+	if (win == 5) {       // always use the Windows palette if all Windows files are present
+		_use_dos_palette = false;
+	} else if (dos == 5) { // else use the DOS palette if all DOS files are present
+		_use_dos_palette = true;
+	} else {							// some files are missing, regardless of palette. Use Windows
+		_use_dos_palette = false;
+	}
+}
+
+static void LoadSpriteTables(void)
 {
 	int i,j;
+	FileList *files; // list of grf files to be loaded. Either Windows files or DOS files
 
-	/* load initial sprites */
+	_loading_stage = 1;
+
+	/*
+	 * Note for developers:
+	 *   Keep in mind that when you add a LoadGrfIndexed in the 'if'-section below
+	 *   that you should also add the corresponding FioOpenFile to the 'else'-section
+	 *   below.
+	 *
+	 * TODO:
+	 *   I think we can live entirely without Indexed GRFs, but I have to
+	 *   invest that further. --octo
+	 */
+
+	files = _use_dos_palette?(&files_dos):(&files_win);
+
+	// Try to load the sprites from cache
 	if (!HandleCachedSpriteHeaders(_cached_filenames[_opt.landscape], true)) {
-		
+		// We do not have the sprites in cache yet, or cache is disabled
+		// So just load all files from disk..
+
 		int load_index = 0;
-		for(i=0; _filename_list[i] != NULL; i++) {
-			load_index += LoadGrfFile(_filename_list[i], load_index, (byte)i);
+
+		for(i=0; files->basic[i].filename != NULL; i++) {
+			load_index += LoadGrfFile(files->basic[i].filename, load_index, (byte)i);
 		}
 
 		LoadGrfIndexed("openttd.grf", _openttd_grf_indexes, i++);
 
-		{
-			int l;
-			if ((l=_sprite_page_to_load) != 0)
-				LoadGrfIndexed(_landscape_filenames[l-1], _landscape_spriteindexes[l-1], i++); 
-		}
+		if (_sprite_page_to_load != 0)
+			LoadGrfIndexed(files->landscape[_sprite_page_to_load-1].filename, _landscape_spriteindexes[_sprite_page_to_load-1], i++);
+
+		LoadGrfIndexed("trkfoundw.grf", _slopes_spriteindexes[_opt.landscape], i++);
+
+		load_index = SPR_AUTORAIL_BASE;
+		load_index += LoadGrfFile("autorail.grf", load_index, i++);
 
 		load_index = SPR_CANALS_BASE;
 		load_index += LoadGrfFile("canalsw.grf", load_index, i++);
-		/* XXX: Only for debugging. Will be more generic. */
-		
-		for(j=0; j!=lengthof(_newgrf_files) && _newgrf_files[j]; j++)
-			load_index += LoadGrfFile(_newgrf_files[j], load_index, i++);
 
-//		load_index += LoadGrfFile("arcticseto.grf", load_index, i++);
-//		load_index += LoadGrfFile("tempsetpo.grf", load_index, i++);
-//		load_index += LoadGrfFile("newshipso.grf", load_index, i++);
-		//load_index += LoadGrfFile("brseto.grf", load_index, i++);
+		load_index = SPR_OPENTTD_BASE + OPENTTD_SPRITES_COUNT + 1;
 
+
+		/* Load newgrf sprites */
+
+		_custom_sprites_base = load_index;
+
+		_loading_stage = 0;
+		for (j = 0; j != lengthof(_newgrf_files) && _newgrf_files[j]; j++) {
+			InitNewGRFFile(_newgrf_files[j], load_index);
+			load_index += LoadNewGrfFile(_newgrf_files[j], load_index, i++);
+		}
+
+		_loading_stage = 1;
+		load_index = _custom_sprites_base;
+		for (j = 0; j != lengthof(_newgrf_files) && _newgrf_files[j]; j++)
+			load_index += LoadNewGrfFile(_newgrf_files[j], load_index, i++);
+
+
+		// If needed, save the cache to file
 		HandleCachedSpriteHeaders(_cached_filenames[_opt.landscape], false);
 	} else {
-		for(i=0; _filename_list[i] != NULL; i++)
-			FioOpenFile(i,_filename_list[i]); 
-		
-		FioOpenFile(i, "openttd.grf");
-		FioOpenFile(i+1, "canalsw.grf");
-		
+		// We have sprites cached. We just loaded the cached files
+		//  now we only have to open a file-pointer to all the original grf-files
+		// This is very important. Not all sprites are in the cache. So sometimes
+		//  the game needs to load the sprite from disk. When the file is not
+		//  open it can not read. So all files that are in the 'if'-section
+		//  above should also be in this 'else'-section.
+		//
+		// NOTE: the order of the files must be identical as in the section above!!
+
+		for(i = 0; files->basic[i].filename != NULL; i++)
+			FioOpenFile(i,files->basic[i].filename);
+
+		FioOpenFile(i++, "openttd.grf");
+
 		if (_sprite_page_to_load != 0)
-			FioOpenFile(i+2, _landscape_filenames[_sprite_page_to_load-1]);
+			FioOpenFile(i++, files->landscape[_sprite_page_to_load-1].filename);
+
+		FioOpenFile(i++, "trkfoundw.grf");
+		FioOpenFile(i++, "canalsw.grf");
+
+		// FIXME: if a user changes his newgrf's, the cached-sprites gets
+		//  invalid. We should have some kind of check for this.
+		// The best solution for this is to delete the cached-sprites.. but how
+		//  do we detect it?
+		for(j=0; j!=lengthof(_newgrf_files) && _newgrf_files[j]; j++)
+			FioOpenFile(i++, _newgrf_files[j]);
 	}
 
 	_compact_cache_counter = 0;
@@ -712,9 +957,10 @@ void GfxInitSpriteMem(byte *ptr, uint32 size)
 }
 
 
-void GfxLoadSprites() {
+void GfxLoadSprites(void)
+{
 	static byte *_sprite_mem;
-	
+
 	// Need to reload the sprites only if the landscape changed
 	if (_sprite_page_to_load != _opt.landscape) {
 		_sprite_page_to_load = _opt.landscape;
@@ -763,4 +1009,4 @@ const SpriteDimension *GetSpriteDimension(uint sprite)
 */
 	return sd;
 }
- 
+
