@@ -1,4 +1,6 @@
 #include "stdafx.h"
+#include "table/strings.h"
+#include "map.h"
 
 #define VARDEF
 #include "ttd.h"
@@ -10,7 +12,7 @@
 #include "window.h"
 #include "player.h"
 #include "command.h"
-#include "city.h"
+#include "town.h"
 #include "industry.h"
 #include "news.h"
 #include "engine.h"
@@ -19,36 +21,39 @@
 #include "fileio.h"
 #include "hal.h"
 #include "airport.h"
+#include "saveload.h"
+#include "ai.h"
+#include "console.h"
+#include "screenshot.h"
+#include "network.h"
+#include "signs.h"
 
 #include <stdarg.h>
 
-void GameLoop();
-
-void IncreaseSpriteLRU();
-void InitializeGame();
+void IncreaseSpriteLRU(void);
 void GenerateWorld(int mode);
-void CallLandscapeTick();
-void IncreaseDate();
-void RunOtherPlayersLoop();
-void DoPaletteAnimations();
-void MusicLoop();
-void ResetMusic();
-void InitializeStations();
-void DeleteAllPlayerStations();
+void CallLandscapeTick(void);
+void IncreaseDate(void);
+void RunOtherPlayersLoop(void);
+void DoPaletteAnimations(void);
+void MusicLoop(void);
+void ResetMusic(void);
+void InitializeStations(void);
+void DeleteAllPlayerStations(void);
 
-extern void SetDifficultyLevel(int mode);
+extern void SetDifficultyLevel(int mode, GameOptions *gm_opt);
 extern void DoStartupNewPlayer(bool is_ai);
-extern void UpdateAllSignVirtCoords();
 extern void ShowOSErrorBox(const char *buf);
 
 void redsq_debug(int tile);
 bool LoadSavegame(const char *filename);
 
-extern void HalGameLoop();
+extern void HalGameLoop(void);
 
 uint32 _pixels_redrawn;
 bool _dbg_screen_rect;
-bool disable_computer;
+bool disable_computer; // We should get ride of this thing.. is only used for a debug-cheat
+static byte _os_version = 0;
 
 void CDECL error(const char *s, ...) {
 	va_list va;
@@ -56,7 +61,7 @@ void CDECL error(const char *s, ...) {
 	va_start(va, s);
 	vsprintf(buf, s, va);
 	va_end(va);
-	
+
 	ShowOSErrorBox(buf);
 	if (_video_driver)
 		_video_driver->stop();
@@ -72,8 +77,8 @@ void CDECL debug(const char *s, ...)
 	va_start(va, s);
 	vsprintf(buf, s, va);
 	va_end(va);
-
 	fprintf(stderr, "dbg: %s\n", buf);
+	IConsoleDebug(buf);
 }
 
 void CDECL ShowInfoF(const char *str, ...)
@@ -105,10 +110,10 @@ char * CDECL str_fmt(const char *str, ...)
 
 // NULL midi driver
 static char *NullMidiStart(char **parm) { return NULL; }
-static void NullMidiStop() {}
+static void NullMidiStop(void) {}
 static void NullMidiPlaySong(const char *filename) {}
-static void NullMidiStopSong() {}
-static bool NullMidiIsSongPlaying() { return true; }
+static void NullMidiStopSong(void) {}
+static bool NullMidiIsSongPlaying(void) { return true; }
 static void NullMidiSetVolume(byte vol) {}
 
 const HalMusicDriver _null_music_driver = {
@@ -128,9 +133,10 @@ static const char *NullVideoStart(char **parm) {
 	_null_video_mem = malloc(_cur_resolution[0]*_cur_resolution[1]);
 	return NULL;
 }
-static void NullVideoStop() { free(_null_video_mem); }
+static void NullVideoStop(void) { free(_null_video_mem); }
 static void NullVideoMakeDirty(int left, int top, int width, int height) {}
-static int NullVideoMainLoop() {
+static int NullVideoMainLoop(void)
+{
 	int i = 1000;
 	do {
 		GameLoop();
@@ -142,7 +148,7 @@ static int NullVideoMainLoop() {
 
 static bool NullVideoChangeRes(int w, int h) { return false; }
 
-	
+
 const HalVideoDriver _null_video_driver = {
 	NullVideoStart,
 	NullVideoStop,
@@ -153,7 +159,7 @@ const HalVideoDriver _null_video_driver = {
 
 // NULL sound driver
 static char *NullSoundStart(char **parm) { return NULL; }
-static void NullSoundStop() {}
+static void NullSoundStop(void) {}
 const HalSoundDriver _null_sound_driver = {
 	NullSoundStart,
 	NullSoundStop,
@@ -189,7 +195,7 @@ static const DriverDesc *ChooseDefaultDriver(const DriverDesc *dd)
 	const DriverDesc *best = NULL;
 	int best_pri = -1;
 	do {
-		if ((int)(dd->flags&DF_PRIORITY_MASK) > best_pri) {
+		if ((int)(dd->flags&DF_PRIORITY_MASK) > best_pri && _os_version >= (byte)dd->flags) {
 			best_pri = dd->flags&DF_PRIORITY_MASK;
 			best = dd;
 		}
@@ -197,15 +203,23 @@ static const DriverDesc *ChooseDefaultDriver(const DriverDesc *dd)
 	return best;
 }
 
-void ttd_strlcpy(char *dst, const char *src, size_t len)
+void ttd_strlcpy(char *dst, const char *src, size_t size)
 {
-	assert(len > 0);
-	while (--len && *src)
-		*dst++=*src++;
-	*dst = 0;
+	assert(size > 0);
+	while (--size > 0 && *src != '\0') *dst++ = *src++;
+	*dst = '\0';
 }
 
-char *strecpy(char *dst, const char *src)
+void ttd_strlcat(char *dst, const char *src, size_t size)
+{
+	assert(size > 0);
+	for (; size > 0 && *dst != '\0'; --size, ++dst) {}
+	assert(size > 0);
+	while (--size > 0 && *src != '\0') *dst++ = *src++;
+	*dst = '\0';
+}
+
+static char *strecpy(char *dst, const char *src)
 {
 	while ( (*dst++ = *src++) != 0) {}
 	return dst - 1;
@@ -214,7 +228,7 @@ char *strecpy(char *dst, const char *src)
 byte *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize)
 {
 	FILE *in;
-	void *mem;
+	byte *mem;
 	size_t len;
 
 	in = fopen(filename, "rb");
@@ -224,11 +238,11 @@ byte *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize)
 	fseek(in, 0, SEEK_END);
 	len = ftell(in);
 	fseek(in, 0, SEEK_SET);
-	if (len > maxsize || (mem=(byte*)malloc(len + 1)) == NULL) {
+	if (len > maxsize || (mem = malloc(len + 1)) == NULL) {
 		fclose(in);
 		return NULL;
 	}
-	((byte*)mem)[len] = 0;
+	mem[len] = 0;
 	if (fread(mem, len, 1, in) != 1) {
 		fclose(in);
 		free(mem);
@@ -260,7 +274,7 @@ void LoadDriver(int driver, const char *name)
 		ttd_strlcpy(buffer, name, sizeof(buffer));
 		parm = strchr(buffer, ':');
 		if (parm) {
-			int np = 0;
+			uint np = 0;
 			// Tokenize the parm.
 			do {
 				*parm++ = 0;
@@ -276,41 +290,44 @@ void LoadDriver(int driver, const char *name)
 			error("No such %s driver: %s\n", dc->name, buffer);
 	}
 	var = dc->var;
-	if (*var != NULL) ((HalCommonDriver*)*var)->stop();
+	if (*var != NULL) ((const HalCommonDriver*)*var)->stop();
 	*var = NULL;
 	drv = dd->drv;
-	if ((err=((HalCommonDriver*)drv)->start(parms)) != NULL)
+	if ((err=((const HalCommonDriver*)drv)->start(parms)) != NULL)
 		error("Unable to load driver %s(%s). The error was: %s\n", dd->name, dd->longname, err);
 	*var = drv;
 }
 
-void PrintDriverList()
-{
-}
-
-static void showhelp()
+static void showhelp(void)
 {
 	char buf[4096], *p;
 	const DriverClass *dc = _driver_classes;
 	const DriverDesc *dd;
 	int i;
 
-	p = strecpy(buf, 
+	p = strecpy(buf,
 		"Command line options:\n"
-		"  -v drv = Set video driver (see below)\n"
-		"  -s drv = Set sound driver (see below)\n"
-		"  -m drv = Set music driver (see below)\n"
-		"  -r res = Set resolution (for instance 800x600)\n"
-		"  -h     = Display this help text\n"
-		"  -t date= Set starting date\n"
-		"  -d dbg = Debug mode\n"
-		"  -l lng = Select Language\n"
-		"  -e     = Start Editor\n"
-		"  -g     = Start new game immediately (can optionally take a game to load)\n"
-		"  -G seed= Set random seed\n"
+		"  -v drv              = Set video driver (see below)\n"
+		"  -s drv              = Set sound driver (see below)\n"
+		"  -m drv              = Set music driver (see below)\n"
+		"  -r res              = Set resolution (for instance 800x600)\n"
+		"  -h                  = Display this help text\n"
+		"  -t date             = Set starting date\n"
+		"  -d [dbg]            = Debug mode\n"
+		"  -l lng              = Select Language\n"
+		"  -e                  = Start Editor\n"
+		"  -g [savegame]       = Start new/save game immediately\n"
+		"  -G seed             = Set random seed\n"
+		"  -n [ip#player:port] = Start networkgame\n"
+		"  -D                  = Start dedicated server\n"
+		#if !defined(__MORPHOS__) && !defined(__AMIGA__)
+		"  -f                  = Fork into the background (dedicated only)\n"
+		#endif
+		"  -i                  = Force to use the DOS palette (use this if you see a lot of pink)\n"
+		"  -p #player          = Player as #player (deprecated) (network only)\n"
 	);
 
-	for(i=0; i!=lengthof(_driver_classes); i++,dc++) { 
+	for(i=0; i!=lengthof(_driver_classes); i++,dc++) {
 		p += sprintf(p, "List of %s drivers:\n", dc->name);
 		dd = dc->descs;
 		do {
@@ -424,9 +441,12 @@ void SetDebugString(const char *s)
 	if (*s >= '0' && *s <= '9') {
 		v = strtoul(s, &end, 0);
 		s = end;
-		
+
 		_debug_spritecache_level = v;
 		_debug_misc_level = v;
+		_debug_grf_level = v;
+		_debug_ai_level = v;
+		_debug_net_level = v;
 	}
 
 	// individual levels
@@ -442,6 +462,9 @@ void SetDebugString(const char *s)
 		// check debugging levels
 		if IS_LVL("misc") p = &_debug_misc_level;
 		else if IS_LVL("spritecache") p = &_debug_spritecache_level;
+		else if IS_LVL("grf") p = &_debug_grf_level;
+		else if IS_LVL("ai") p = &_debug_ai_level;
+		else if IS_LVL("net") p = &_debug_net_level;
 		else {
 			ShowInfoF("Unknown debug level '%.*s'", s-t, t);
 			return;
@@ -451,10 +474,10 @@ void SetDebugString(const char *s)
 		v = strtoul(s, &end, 0);
 		s = end;
 		if (p) *p = v;
-	}		
+	}
 }
 
-void ParseResolution(int res[2], char *s)
+static void ParseResolution(int res[2], char *s)
 {
 	char *t = strchr(s, 'x');
 	if (t == NULL) {
@@ -462,33 +485,132 @@ void ParseResolution(int res[2], char *s)
 		return;
 	}
 
-	res[0] = (strtoul(s, NULL, 0) + 7) & ~7;
-	res[1] = (strtoul(t+1, NULL, 0) + 7) & ~7;
-} 
+	res[0] = strtoul(s, NULL, 0);
+	res[1] = strtoul(t + 1, NULL, 0);
+}
+
+static void InitializeDynamicVariables(void)
+{
+	/* Dynamic stuff needs to be initialized somewhere... */
+	_stations_size = lengthof(_stations);
+	_station_sort = NULL;
+
+	_vehicles_size = lengthof(_vehicles);
+	_vehicle_sort = NULL;
+
+	_towns_size = lengthof(_towns);
+	_town_sort = NULL;
+
+	_industries_size = lengthof(_industries);
+	_industry_sort = NULL;
+
+	_sign_size = lengthof(_sign_list);
+	_orders_size = lengthof(_orders);
+}
+
+static void UnInitializeDynamicVariables(void)
+{
+	/* Dynamic stuff needs to be free'd somewhere... */
+	free(_station_sort);
+
+	free(_vehicle_sort);
+
+	free(_town_sort);
+
+	free(_industry_sort);
+}
+
+
+static void LoadIntroGame(void)
+{
+	char filename[256];
+	_game_mode = GM_MENU;
+	_display_opt &= ~DO_TRANS_BUILDINGS; // don't make buildings transparent in intro
+
+	_opt_mod_ptr = &_new_opt;
+	GfxLoadSprites();
+	LoadStringWidthTable();
+
+	// Setup main window
+	InitWindowSystem();
+	SetupColorsAndInitialWindow();
+
+	// Generate a world.
+	sprintf(filename, "%sopntitle.dat",  _path.data_dir);
+	if (SaveOrLoad(filename, SL_LOAD) != SL_OK) {
+#if defined SECOND_DATA_DIR
+		sprintf(filename, "%sopntitle.dat",  _path.second_data_dir);
+		if (SaveOrLoad(filename, SL_LOAD) != SL_OK)
+#endif
+			GenerateWorld(1); // if failed loading, make empty world.
+	}
+	_opt.currency = _new_opt.currency;
+
+	_pause = 0;
+	_local_player = 0;
+	MarkWholeScreenDirty();
+
+	// Play main theme
+	if (_music_driver->is_song_playing()) ResetMusic();
+}
+
+extern void DedicatedFork(void);
+extern void CheckExternalFiles(void);
 
 int ttd_main(int argc, char* argv[])
 {
 	MyGetOptData mgo;
 	int i;
-	int network = 0;
+	bool network = false;
 	char *network_conn = NULL;
 	char *language = NULL;
-	char musicdriver[32], sounddriver[32], videodriver[32];
+	const char *optformat;
+	char musicdriver[16], sounddriver[16], videodriver[16];
 	int resolution[2] = {0,0};
 	uint startdate = -1;
-	_ignore_wrong_grf = false;
 	musicdriver[0] = sounddriver[0] = videodriver[0] = 0;
 
 	_game_mode = GM_MENU;
 	_switch_mode = SM_MENU;
+	_switch_mode_errorstr = INVALID_STRING_ID;
+	_dedicated_forks = false;
+	_dedicated_enabled = false;
 
-	MyGetOptInit(&mgo, argc-1, argv+1, "m:s:v:hn::l:eit:d::r:g::G:cp:");
+	_redirect_console_to_client = 0;
+
+	// The last param of the following function means this:
+	//   a letter means: it accepts that param (e.g.: -h)
+	//   a ':' behind it means: it need a param (e.g.: -m<driver>)
+	//   a '::' behind it means: it can optional have a param (e.g.: -d<debug>)
+	#if !defined(__MORPHOS__) && !defined(__AMIGA__)
+		optformat = "m:s:v:hDfn::l:eit:d::r:g::G:p:";
+	#else
+		optformat = "m:s:v:hDn::l:eit:d::r:g::G:p:"; // no fork option
+	#endif
+
+	MyGetOptInit(&mgo, argc-1, argv+1, optformat);
 	while ((i = MyGetOpt(&mgo)) != -1) {
 		switch(i) {
 		case 'm': ttd_strlcpy(musicdriver, mgo.opt, sizeof(musicdriver)); break;
 		case 's': ttd_strlcpy(sounddriver, mgo.opt, sizeof(sounddriver)); break;
 		case 'v': ttd_strlcpy(videodriver, mgo.opt, sizeof(videodriver)); break;
-		case 'n': network=1; if (mgo.opt) {network_conn = mgo.opt; network++;} break;
+		case 'D': {
+				sprintf(musicdriver,"null");
+				sprintf(sounddriver,"null");
+				sprintf(videodriver,"dedicated");
+				_dedicated_enabled = true;
+			} break;
+		case 'f': {
+				_dedicated_forks = true;
+			}; break;
+		case 'n': {
+				network = true;
+				if (mgo.opt)
+					// Optional, you can give an IP
+					network_conn = mgo.opt;
+				else
+					network_conn = NULL;
+			} break;
 		case 'r': ParseResolution(resolution, mgo.opt); break;
 		case 'l': {
 				language = mgo.opt;
@@ -504,20 +626,21 @@ int ttd_main(int argc, char* argv[])
 					SetDebugString(mgo.opt);
 			} break;
 		case 'e': _switch_mode = SM_EDITOR; break;
-		case 'i': _ignore_wrong_grf = true; break;
-		case 'g': 
+		case 'i': _use_dos_palette = true; break;
+		case 'g':
 			if (mgo.opt) {
-				strcpy(_file_to_saveload, mgo.opt);
+				strcpy(_file_to_saveload.name, mgo.opt);
 				_switch_mode = SM_LOAD;
 			} else
 				_switch_mode = SM_NEWGAME;
 			break;
 		case 'G':
-			_random_seed_1 = atoi(mgo.opt);
+			_random_seeds[0][0] = atoi(mgo.opt);
 			break;
 		case 'p': {
 			int i = atoi(mgo.opt);
-			if (IS_INT_INSIDE(i, 0, 8))	_network_playas = i + 1;
+			// Play as an other player in network games
+			if (IS_INT_INSIDE(i, 1, MAX_PLAYERS)) _network_playas = i;
 			break;
 		}
 		case -2:
@@ -528,14 +651,26 @@ int ttd_main(int argc, char* argv[])
 	}
 
 	DeterminePaths();
+	CheckExternalFiles();
+
+#ifdef UNIX
+	// We must fork here, or we'll end up without some resources we need (like sockets)
+	if (_dedicated_forks)
+		DedicatedFork();
+#endif
+
 	LoadFromConfig();
+	LoadFromHighScore();
 
 	// override config?
-	if (musicdriver[0]) strcpy(_ini_musicdriver, musicdriver);
-	if (sounddriver[0]) strcpy(_ini_sounddriver, sounddriver);
-	if (videodriver[0]) strcpy(_ini_videodriver, videodriver);
+	if (musicdriver[0]) ttd_strlcpy(_ini_musicdriver, musicdriver, sizeof(_ini_musicdriver));
+	if (sounddriver[0]) ttd_strlcpy(_ini_sounddriver, sounddriver, sizeof(_ini_sounddriver));
+	if (videodriver[0]) ttd_strlcpy(_ini_videodriver, videodriver, sizeof(_ini_videodriver));
 	if (resolution[0]) { _cur_resolution[0] = resolution[0]; _cur_resolution[1] = resolution[1]; }
-	if (startdate != -1) _patches.starting_date = startdate;
+	if (startdate != (uint)-1) _patches.starting_date = startdate;
+
+	if (_dedicated_forks && !_dedicated_enabled)
+		_dedicated_forks = false;
 
 	// enumerate language files
 	InitializeLanguagePacks();
@@ -543,12 +678,16 @@ int ttd_main(int argc, char* argv[])
 	// initialize screenshot formats
 	InitializeScreenshotFormats();
 
-  // initialize airport state machines
-  InitializeAirports();
-	
+	// initialize airport state machines
+	InitializeAirports();
+
+	/* initialize all variables that are allocated dynamically */
+	InitializeDynamicVariables();
+
 	// Sample catalogue
 	DEBUG(misc, 1) ("Loading sound effects...");
-	MxInitialize(11025, "sample.cat"); 
+	_os_version = GetOSVersion();
+	MxInitialize(11025, "sample.cat");
 
 	// This must be done early, since functions use the InvalidateWindow* calls
 	InitWindowSystem();
@@ -561,42 +700,75 @@ int ttd_main(int argc, char* argv[])
 	LoadDriver(MUSIC_DRIVER, _ini_musicdriver);
 	LoadDriver(VIDEO_DRIVER, _ini_videodriver); // load video last, to prevent an empty window while sound and music loads
 	MusicLoop();
+	_savegame_sort_order = 1; // default sorting of savegames is by date, newest first
+
+#ifdef ENABLE_NETWORK
+	// initialize network-core
+	NetworkStartUp();
+#endif /* ENABLE_NETWORK */
 
 	// Default difficulty level
 	_opt_mod_ptr = &_new_opt;
-	
+
 	// ugly hack, if diff_level is 9, it means we got no setting from the config file, so we load the default settings.
 	if (_opt_mod_ptr->diff_level == 9)
-		SetDifficultyLevel(0);
+		SetDifficultyLevel(0, _opt_mod_ptr);
 
-	if (network) {
-		_networking = true;
-		
-		NetworkInitialize();
-		if (network==1) {
-			DEBUG(misc, 1) ("Listening on port %d\n", _network_port);
-			NetworkListen(_network_port);
-			_networking_server = true;
-		} else {
-			DEBUG(misc, 1) ("Connecting to %s %d\n", network_conn, _network_port);
-			NetworkConnect(network_conn, _network_port);
+	// initialize the ingame console
+	IConsoleInit();
+	IConsoleCmdExec("exec scripts/autoexec.scr 0");
+
+	InitPlayerRandoms();
+
+#ifdef ENABLE_NETWORK
+	if ((network) && (_network_available)) {
+		if (network_conn != NULL) {
+			const byte *port = NULL;
+			const byte *player = NULL;
+			uint16 rport;
+
+			rport = NETWORK_DEFAULT_PORT;
+
+			ParseConnectionString(&player, &port, network_conn);
+
+			if (player != NULL) _network_playas = atoi(player);
+			if (port != NULL) rport = atoi(port);
+
+			LoadIntroGame();
+			_switch_mode = SM_NONE;
+			NetworkClientConnectGame(network_conn, rport);
 		}
 	}
+#endif /* ENABLE_NETWORK */
 
 	while (_video_driver->main_loop() == ML_SWITCHDRIVER) {}
 
-	if (network) {
-		NetworkShutdown();
+	IConsoleFree();
+
+#ifdef ENABLE_NETWORK
+	if (_network_available) {
+		// Shut down the network and close any open connections
+		NetworkDisconnect();
+		NetworkUDPClose();
+		NetworkShutDown();
 	}
+#endif /* ENABLE_NETWORK */
 
 	_video_driver->stop();
 	_music_driver->stop();
 	_sound_driver->stop();
 
 	SaveToConfig();
+	SaveToHighScore();
 
-  // uninitialize airport state machines
-  UnInitializeAirports();
+	// uninitialize airport state machines
+	UnInitializeAirports();
+
+	/* uninitialize variables that are allocated dynamic */
+	UnInitializeDynamicVariables();
+
+	/* Close all and any open filehandles */
+	FioCloseAll();
 
 	return 0;
 }
@@ -604,7 +776,7 @@ int ttd_main(int argc, char* argv[])
 static void ShowScreenshotResult(bool b)
 {
 	if (b) {
-		SET_DPARAM16(0, STR_SPEC_SCREENSHOT_NAME);
+		SetDParam(0, STR_SPEC_SCREENSHOT_NAME);
 		ShowErrorMessage(INVALID_STRING_ID, STR_031B_SCREENSHOT_SUCCESSFULLY, 0, 0);
 	} else {
 		ShowErrorMessage(INVALID_STRING_ID, STR_031C_SCREENSHOT_FAILED, 0, 0);
@@ -612,7 +784,7 @@ static void ShowScreenshotResult(bool b)
 
 }
 
-void MakeNewGame()
+static void MakeNewGame(void)
 {
 	_game_mode = GM_NORMAL;
 
@@ -631,22 +803,27 @@ void MakeNewGame()
 	// Randomize world
 	GenerateWorld(0);
 
-	// Create a single player
-	DoStartupNewPlayer(false);
-	
-	_local_player = 0;
+	// In a dedicated server, the server does not play
+	if (_network_dedicated) {
+		_local_player = OWNER_SPECTATOR;
+	}	else {
+		// Create a single player
+		DoStartupNewPlayer(false);
+
+		_local_player = 0;
+	}
 
 	MarkWholeScreenDirty();
 }
 
-void MakeNewEditorWorld()
+static void MakeNewEditorWorld(void)
 {
 	_game_mode = GM_EDITOR;
 
 	// Copy in game options
 	_opt_mod_ptr = &_opt;
 	memcpy(&_opt, &_new_opt, sizeof(_opt));
-	
+
 	GfxLoadSprites();
 
 	// Re-init the windowing system
@@ -662,13 +839,25 @@ void MakeNewEditorWorld()
 	MarkWholeScreenDirty();
 }
 
-void StartScenario()
+void StartupPlayers(void);
+void StartupDisasters(void);
+
+static void StartScenario(void)
 {
 	_game_mode = GM_NORMAL;
 
+	// invalid type
+	if (_file_to_saveload.mode == SL_INVALID) {
+		printf("Savegame is obsolete or invalid format: %s\n", _file_to_saveload.name);
+		ShowErrorMessage(_error_message, STR_4009_GAME_LOAD_FAILED, 0, 0);
+		_game_mode = GM_MENU;
+		return;
+	}
+
 	// Copy in game options
-	_opt_mod_ptr = &_opt;
-	memcpy(&_opt, &_new_opt, sizeof(_opt));
+	// Removed copying of game options when using "new game". --dominik
+	// _opt_mod_ptr = &_opt;
+	// memcpy(&_opt, &_new_opt, sizeof(_opt));
 
 	GfxLoadSprites();
 
@@ -679,46 +868,26 @@ void StartScenario()
 	SetupColorsAndInitialWindow();
 
 	// Load game
-	if (SaveOrLoad(_file_to_saveload, 0) != SL_OK)
-		GenerateWorld(0);
+	if (SaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode) != SL_OK) {
+		LoadIntroGame();
+		ShowErrorMessage(_error_message, STR_4009_GAME_LOAD_FAILED, 0, 0);
+	}
+
+	// Inititalize data
+	StartupPlayers();
+	StartupEngines();
+	StartupDisasters();
+
+	// When starting a scenario, is it really a load..
+	//  and in AfterLoad a player is started when it is
+	//  a scenario.. so we do not need it here.
+//	DoStartupNewPlayer(false);
 
-	// Create a single player
-	DoStartupNewPlayer(false);
-	
 	_local_player = 0;
 
 	MarkWholeScreenDirty();
 }
 
-
-void LoadIntroGame()
-{
-	char filename[256];
-	_game_mode = GM_MENU;
-	_display_opt |= DO_TRANS_BUILDINGS; // don't make buildings transparent in intro
-
-	_opt_mod_ptr = &_new_opt;
-	GfxLoadSprites();
-	LoadStringWidthTable();
-	// Setup main window
-	InitWindowSystem();
-	SetupColorsAndInitialWindow();
-
-	// Generate a world.
-	sprintf(filename, "%sopntitle.dat",  _path.data_dir);
-	if (SaveOrLoad(filename, 0) != SL_OK)
-		GenerateWorld(1); // if failed loading, make empty world.
-	
-	_opt.currency = _new_opt.currency;
-
-	_pause = 0;
-	_local_player = 0;
-	MarkWholeScreenDirty();
-
-	// Play main theme
-	if (_music_driver->is_song_playing()) ResetMusic();
-}
-
 bool SafeSaveOrLoad(const char *filename, int mode, int newgm)
 {
 	byte ogm = _game_mode;
@@ -736,50 +905,84 @@ bool SafeSaveOrLoad(const char *filename, int mode, int newgm)
 		return false;
 	} else if (r != SL_OK) {
 		_game_mode = ogm;
-		return false;		
+		return false;
 	} else
 		return true;
 }
 
-static void SwitchMode(int new_mode)
+void SwitchMode(int new_mode)
 {
 	_in_state_game_loop = true;
-	
+
+#ifdef ENABLE_NETWORK
+	// If we are saving something, the network stays in his current state
+	if (new_mode != SM_SAVE) {
+		// If the network is active, make it not-active
+		if (_networking) {
+			if (_network_server && (new_mode == SM_LOAD || new_mode == SM_NEWGAME)) {
+				NetworkReboot();
+				NetworkUDPClose();
+			} else {
+				NetworkDisconnect();
+				NetworkUDPClose();
+			}
+		}
+
+		// If we are a server, we restart the server
+		if (_is_network_server) {
+			// But not if we are going to the menu
+			if (new_mode != SM_MENU) {
+				NetworkServerStart();
+			} else {
+				// This client no longer wants to be a network-server
+				_is_network_server = false;
+			}
+		}
+	}
+#endif /* ENABLE_NETWORK */
+
 	switch(new_mode) {
 	case SM_EDITOR: // Switch to scenario editor
 		MakeNewEditorWorld();
 		break;
 
 	case SM_NEWGAME:
-		if (_networking) { NetworkStartSync(); } // UGLY HACK HACK HACK
+#ifdef ENABLE_NETWORK
+		if (_network_server)
+			snprintf(_network_game_info.map_name, 40, "Random");
+#endif /* ENABLE_NETWORK */
 		MakeNewGame();
 		break;
 
+	case SM_START_SCENARIO:
+		StartScenario();
+		break;
+
 normal_load:
 	case SM_LOAD: { // Load game
-		char *s;
 
-		if (_networking) { NetworkStartSync(); } // UGLY HACK HACK HACK
-
-		s = strrchr(_file_to_saveload, '.');
-		if (!SafeSaveOrLoad(_file_to_saveload, s && strcmp(s, ".sv1") == 0 ? 2 : 0, GM_NORMAL)) {
-			ShowErrorMessage(INVALID_STRING_ID, STR_4009_GAME_LOAD_FAILED, 0, 0);
+		_error_message = INVALID_STRING_ID;
+		if (!SafeSaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_NORMAL)) {
+			LoadIntroGame();
+			ShowErrorMessage(_error_message, STR_4009_GAME_LOAD_FAILED, 0, 0);
 		} else {
 			_opt_mod_ptr = &_opt;
 			_local_player = 0;
 			DoCommandP(0, 0, 0, NULL, CMD_PAUSE); // decrease pause counter (was increased from opening load dialog)
+#ifdef ENABLE_NETWORK
+			if (_network_server)
+				snprintf(_network_game_info.map_name, 40, "Loaded game");
+#endif /* ENABLE_NETWORK */
 		}
 		break;
 	}
 
 	case SM_LOAD_SCENARIO: {
-		char *s;
 		int i;
 
 		if (_game_mode == GM_MENU) goto normal_load;
 
-		s = strrchr(_file_to_saveload, '.');
-		if (SafeSaveOrLoad(_file_to_saveload, s && strcmp(s, ".sv1") == 0 ? 2 : 0, GM_EDITOR)) {
+		if (SafeSaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_EDITOR)) {
 			_opt_mod_ptr = &_opt;
 
 			_local_player = OWNER_NONE;
@@ -792,6 +995,11 @@ normal_load:
 			_generating_world = false;
 			// delete all stations owned by a player
 			DeleteAllPlayerStations();
+
+#ifdef ENABLE_NETWORK
+			if (_network_server)
+				snprintf(_network_game_info.map_name, 40, "Loaded scenario");
+#endif /* ENABLE_NETWORK */
 		} else
 			ShowErrorMessage(INVALID_STRING_ID, STR_4009_GAME_LOAD_FAILED, 0, 0);
 
@@ -804,7 +1012,7 @@ normal_load:
 		break;
 
 	case SM_SAVE: // Save game
-		if (SaveOrLoad(_file_to_saveload, 1) != SL_OK)
+		if (SaveOrLoad(_file_to_saveload.name, SL_SAVE) != SL_OK)
 			ShowErrorMessage(INVALID_STRING_ID, STR_4007_GAME_SAVE_FAILED, 0, 0);
 		else
 			DeleteWindowById(WC_SAVELOAD, 0);
@@ -818,6 +1026,9 @@ normal_load:
 		break;
 	}
 
+	if (_switch_mode_errorstr!=INVALID_STRING_ID)
+		ShowErrorMessage(INVALID_STRING_ID,_switch_mode_errorstr,0,0);
+
 	_in_state_game_loop = false;
 }
 
@@ -826,19 +1037,22 @@ normal_load:
 // The state must not be changed from anywhere
 // but here.
 // That check is enforced in DoCommand.
-void StateGameLoop()
+void StateGameLoop(void)
 {
+	// dont execute the state loop during pause
+	if (_pause) return;
+
 	_in_state_game_loop = true;
-	_frame_counter++;
+	// _frame_counter is increased somewhere else when in network-mode
+	//  Sidenote: _frame_counter is ONLY used for _savedump in non-MP-games
+	//    Should that not be deleted? If so, the next 2 lines can also be deleted
+	if (!_networking)
+		_frame_counter++;
 
-	// store the random seed to be able to detect out of sync errors
-	_sync_seed_1 = _random_seed_1;
-	_sync_seed_2 = _random_seed_2;
-
-	if (_savedump_path[0] && (uint)_frame_counter >= _savedump_first && (uint)(_frame_counter -_savedump_first) % _savedump_freq == 0 ) { 
-		char buf[100]; 
-		sprintf(buf, "%s%.5d.sav", _savedump_path, _frame_counter); 
-		SaveOrLoad(buf, 1); 
+	if (_savedump_path[0] && (uint)_frame_counter >= _savedump_first && (uint)(_frame_counter -_savedump_first) % _savedump_freq == 0 ) {
+		char buf[100];
+		sprintf(buf, "%s%.5d.sav", _savedump_path, _frame_counter);
+		SaveOrLoad(buf, SL_SAVE);
 		if ((uint)_frame_counter >= _savedump_last) exit(1);
 	}
 
@@ -849,46 +1063,56 @@ void StateGameLoop()
 		CallWindowTickEvent();
 		NewsLoop();
 	} else {
+		// All these actions has to be done from OWNER_NONE
+		//  for multiplayer compatibility
+		uint p = _current_player;
+		_current_player = OWNER_NONE;
+
 		AnimateAnimatedTiles();
 		IncreaseDate();
 		RunTileLoop();
 		CallVehicleTicks();
 		CallLandscapeTick();
 
-		if (!disable_computer)
+		// To bad the AI does not work in multiplayer, because states are not saved
+		//  perfectly
+		if (!disable_computer && !_networking)
 			RunOtherPlayersLoop();
 
 		CallWindowTickEvent();
 		NewsLoop();
+		_current_player = p;
 	}
+
 	_in_state_game_loop = false;
 }
 
-static void DoAutosave()
+static void DoAutosave(void)
 {
 	char buf[200];
-	
-	if (_patches.keep_all_autosave && _local_player != 255) {
+
+	if (_patches.keep_all_autosave && _local_player != OWNER_SPECTATOR) {
 		Player *p;
 		char *s;
-		strcpy(buf, _path.autosave_dir);
+		sprintf(buf, "%s%s", _path.autosave_dir, PATHSEP);
 		p = DEREF_PLAYER(_local_player);
-		SET_DPARAM16(0, p->name_1);
-		SET_DPARAM32(1, p->name_2);
-		SET_DPARAM16(2, _date);
-		s= (char*)GetString(buf + strlen(_path.autosave_dir) - 1, STR_4004);
+		SetDParam(0, p->name_1);
+		SetDParam(1, p->name_2);
+		SetDParam(2, _date);
+		s= (char*)GetString(buf + strlen(_path.autosave_dir) + strlen(PATHSEP), STR_4004);
 		strcpy(s, ".sav");
 	} else {
 		int n = _autosave_ctr;
 		_autosave_ctr = (_autosave_ctr + 1) & 15;
-		sprintf(buf, "%sautosave%d.sav", _path.autosave_dir, n);
+		sprintf(buf, "%s%sautosave%d.sav", _path.autosave_dir, PATHSEP, n);
 	}
 
-	if (SaveOrLoad(buf, 1) != SL_OK)
+	DEBUG(misc, 2) ("Autosaving to %s", buf);
+	if (SaveOrLoad(buf, SL_SAVE) != SL_OK)
 		ShowErrorMessage(INVALID_STRING_ID, STR_AUTOSAVE_FAILED, 0, 0);
 }
 
-void GameLoop()
+void GameLoop(void)
 {
 	int m;
 
@@ -908,7 +1132,7 @@ void GameLoop()
 			ShowScreenshotResult(MakeScreenshot());
 			break;
 		case 2: // make large screenshot
-			ShowScreenshotResult(MakeWorldScreenshot(-(TILE_X_MAX)*32, 0, TILE_X_MAX*32 + (TILE_X_MAX)*32, TILES_Y * 32, 0));
+			ShowScreenshotResult(MakeWorldScreenshot(-(int)MapMaxX() * 32, 0, MapMaxX() * 64, MapSizeY() * 32, 0));
 			break;
 		}
 	}
@@ -931,66 +1155,61 @@ void GameLoop()
 	_timer_counter+=8;
 	CursorTick();
 
-	if (_networking) {
-		NetworkReceive();
-		NetworkProcessCommands(); // to check if we got any new commands belonging to the current frame before we increase it.
-	}
+#ifdef ENABLE_NETWORK
+	// Check for UDP stuff
+	NetworkUDPGameLoop();
 
-	if (_networking_sync) {
-		// make sure client's time is synched to the server by running frames quickly up to where the server is.
-		if (!_networking_server) {
-			while (_frame_counter < _frame_counter_srv) {
-				StateGameLoop();
-				NetworkProcessCommands(); // need to process queue to make sure that packets get executed.
-			}
-		}
-		// don't exceed the max count told by the server
-		if (_frame_counter < _frame_counter_max) {
-			StateGameLoop();
-			NetworkProcessCommands();
-		}
+	if (_networking) {
+		// Multiplayer
+		NetworkGameLoop();
 	} else {
-		if (!_pause)
-			StateGameLoop();
+		if (_network_reconnect > 0 && --_network_reconnect == 0) {
+			// This means that we want to reconnect to the last host
+			// We do this here, because it means that the network is really closed
+			NetworkClientConnectGame(_network_last_host, _network_last_port);
+		}
+		// Singleplayer
+		StateGameLoop();
 	}
+#else
+	StateGameLoop();
+#endif /* ENABLE_NETWORK */
 
 	if (!_pause && _display_opt&DO_FULL_ANIMATION)
 		DoPaletteAnimations();
 
-	if (!_pause || _patches.build_in_pause)
+	if (!_pause || _cheats.build_in_pause.value)
 		MoveAllTextEffects();
 
 	MouseLoop();
 
-	// send outgoing packets.
-	if (_networking)
-		NetworkSend();
-
 	if (_game_mode != GM_MENU)
 		MusicLoop();
 }
 
-void BeforeSaveGame()
+void BeforeSaveGame(void)
 {
 	Window *w = FindWindowById(WC_MAIN_WINDOW, 0);
 
-	_saved_scrollpos_x = WP(w,vp_d).scrollpos_x;
-	_saved_scrollpos_y = WP(w,vp_d).scrollpos_y;
-	_saved_scrollpos_zoom = w->viewport->zoom;
+	if (w != NULL) {
+		_saved_scrollpos_x = WP(w,vp_d).scrollpos_x;
+		_saved_scrollpos_y = WP(w,vp_d).scrollpos_y;
+		_saved_scrollpos_zoom = w->viewport->zoom;
+	}
 }
 
-void ConvertCityOwner()
+static void ConvertTownOwner(void)
 {
 	uint tile;
 
-	for(tile=0; tile!=TILES_X * TILES_Y; tile++) {
-		if (IS_TILETYPE(tile, MP_STREET)) {
+	for (tile = 0; tile != MapSize(); tile++) {
+		if (IsTileType(tile, MP_STREET)) {
 			if ((_map5[tile] & 0xF0) == 0x10 && _map3_lo[tile] & 0x80)
 				_map3_lo[tile] = OWNER_TOWN;
 
 			if (_map_owner[tile] & 0x80)
 				_map_owner[tile] = OWNER_TOWN;
-		} else if (IS_TILETYPE(tile, MP_TUNNELBRIDGE)) {
+		} else if (IsTileType(tile, MP_TUNNELBRIDGE)) {
 			if (_map_owner[tile] & 0x80)
 				_map_owner[tile] = OWNER_TOWN;
 		}
@@ -998,7 +1217,7 @@ void ConvertCityOwner()
 }
 
 // before savegame version 4, the name of the company determined if it existed
-void CheckIsPlayerActive()
+static void CheckIsPlayerActive(void)
 {
 	Player *p;
 	FOR_ALL_PLAYERS(p) {
@@ -1008,16 +1227,84 @@ void CheckIsPlayerActive()
 	}
 }
 
-extern void UpdateOldAircraft();
+// since savegame version 4.1, exclusive transport rights are stored at towns
+static void UpdateExclusiveRights(void)
+{
+	Town *t;
+	FOR_ALL_TOWNS(t) if (t->xy != 0) {
+		t->exclusivity=(byte)-1;
+	}
+
+	/* FIXME old exclusive rights status is not being imported (stored in s->blocked_months_obsolete)
+			could be implemented this way:
+			1.) Go through all stations
+					Build an array town_blocked[ town_id ][ player_id ]
+				 that stores if at least one station in that town is blocked for a player
+			2.) Go through that array, if you find a town that is not blocked for
+				 	one player, but for all others, then give him exclusivity.
+	*/
+}
+
+byte convert_currency[] = {
+	 0,  1, 12,  8,  3,
+	10, 14, 19,  4,  5,
+	 9, 11, 13,  6, 17,
+	16, 22, 21,  7, 15,
+	18,  2, 20, };
+
+// since savegame version 4.2 the currencies are arranged differently
+static void UpdateCurrencies(void)
+{
+	_opt.currency = convert_currency[_opt.currency];
+}
+
+// up to revision 1413, the invisible tiles at the southern border have not been MP_VOID
+// even though they should have. This is fixed by this function
+static void UpdateVoidTiles(void)
+{
+	uint i;
+	// create void tiles on the border
+	for (i = 0; i != MapMaxY(); i++)
+		_map_type_and_height[ i * MapSizeX() + MapMaxY() ] = MP_VOID << 4;
+	memset(_map_type_and_height + MapMaxY() * MapSizeX(), MP_VOID << 4, MapSizeX());
+}
+
+extern void UpdateOldAircraft( void );
+extern void UpdateOilRig( void );
 
 bool AfterLoadGame(uint version)
 {
 	Window *w;
 	ViewPort *vp;
 
-	// in version 2.1 of the savegame, city owner was unified.
+	// in version 2.1 of the savegame, town owner was unified.
 	if (version <= 0x200) {
-		ConvertCityOwner();
+		ConvertTownOwner();
+	}
+
+	// from version 4.1 of the savegame, exclusive rights are stored at towns
+	if (version <= 0x400) {
+		UpdateExclusiveRights();
+	}
+
+	// from version 4.2 of the savegame, currencies are in a different order
+	if (version <= 0x401) {
+		UpdateCurrencies();
+	}
+
+	/* In old version there seems to be a problem that water is owned by
+	    OWNER_NONE, not OWNER_WATER.. I can't replicate it for the current
+	    (0x402) version, so I just check when versions are older, and then
+	    walk through the whole map.. */
+	if (version <= 0x402) {
+		TileIndex tile = TILE_XY(0,0);
+		uint w = MapSizeX();
+		uint h = MapSizeY();
+
+		BEGIN_TILE_LOOP(tile_cur, w, h, tile)
+			if (IsTileType(tile_cur, MP_WATER) && _map_owner[tile_cur] >= MAX_PLAYERS)
+				_map_owner[tile_cur] = OWNER_WATER;
+		END_TILE_LOOP(tile_cur, w, h, tile)
 	}
 
 	// convert road side to my format.
@@ -1031,7 +1318,7 @@ bool AfterLoadGame(uint version)
 
 	// reinit the landscape variables (landscape might have changed)
 	InitializeLandscapeVariables(true);
-	
+
 	// Update all vehicles
 	AfterLoadVehicles();
 	// in version 2.2 of the savegame, we have new airports
@@ -1041,13 +1328,16 @@ bool AfterLoadGame(uint version)
 
 	UpdateAllStationVirtCoord();
 
-	// Setup city coords
-	AfterLoadCity();
+	// Setup town coords
+	AfterLoadTown();
 	UpdateAllSignVirtCoords();
 
-	// make sure there is a city in the game
-	if (_game_mode == GM_NORMAL && !ClosestCityFromTile(0, (uint)-1))
+	// make sure there is a town in the game
+	if (_game_mode == GM_NORMAL && !ClosestTownFromTile(0, (uint)-1))
+	{
+		_error_message = STR_NO_TOWN_IN_SCENARIO;
 		return false;
+	}
 
 	// Initialize windows
 	InitWindowSystem();
@@ -1057,7 +1347,7 @@ bool AfterLoadGame(uint version)
 
 	WP(w,vp_d).scrollpos_x = _saved_scrollpos_x;
 	WP(w,vp_d).scrollpos_y = _saved_scrollpos_y;
-	
+
 	vp = w->viewport;
 	vp->zoom = _saved_scrollpos_zoom;
 	vp->virtual_width = vp->width << vp->zoom;
@@ -1070,17 +1360,28 @@ bool AfterLoadGame(uint version)
 		CheckIsPlayerActive();
 	}
 
-	// in case the player doesn't exist, create one (for scenario)
-	if (!_players[0].is_active)
+	// the void tiles on the southern border used to belong to a wrong class.
+	if (version <= 0x402)
+		UpdateVoidTiles();
+
+	// If Load Scenario / New (Scenario) Game is used,
+	//  a player does not exist yet. So create one here.
+	// 1 exeption: network-games. Those can have 0 players
+	//   But this exeption is not true for network_servers!
+	if (!_players[0].is_active && (!_networking || (_networking && _network_server)))
 		DoStartupNewPlayer(false);
 
-	DoZoomInOut(ZOOM_NONE); // update button status
+	DoZoomInOutWindow(ZOOM_NONE, w); // update button status
 	MarkWholeScreenDirty();
 
+	//In 5.1, Oilrigs have been moved (again)
+	if (version <= 0x500) {
+		UpdateOilRig();
+	}
+
 	return true;
 }
 
-
 void DebugProc(int i)
 {
 	switch(i) {
@@ -1088,7 +1389,11 @@ void DebugProc(int i)
 		*(byte*)0 = 0;
 		break;
 	case 1:
-		DoCommandP(0, -10000000, 0, NULL, CMD_MONEY_CHEAT);
+		/* Server can not cheat in advertise mode either! */
+#ifdef ENABLE_NETWORK
+		if (!_networking || !_network_server || !_network_advertise)
+#endif /* ENABLE_NETWORK */
+			DoCommandP(0, -10000000, 0, NULL, CMD_MONEY_CHEAT);
 		break;
 	case 2:
 		UpdateAllStationVirtCoord();
