@@ -1,10 +1,13 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
+#include "map.h"
 #include "vehicle.h"
 #include "viewport.h"
 #include "command.h"
 #include "player.h"
 #include "town.h"
+#include "sound.h"
 
 extern void DrawCanalWater(uint tile);
 
@@ -131,10 +134,10 @@ static uint32 CheckBridgeSlope(uint direction, uint tileh, bool is_start_tile)
 uint32 GetBridgeLength(TileIndex begin, TileIndex end)
 {
 	int x1, y1, x2, y2;	// coordinates of starting and end tiles
-	x1 = GET_TILE_X(begin);
-	y1 = GET_TILE_Y(begin);
-	x2 = GET_TILE_X(end);
-	y2 = GET_TILE_Y(end);
+	x1 = TileX(begin);
+	y1 = TileY(begin);
+	x2 = TileX(end);
+	y2 = TileY(end);
 
 	return abs((x2 + y2 - x1 - y1)) - 1;
 }
@@ -187,8 +190,8 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		rail_or_road = 0;
 	}
 
-	sx = GET_TILE_X(p1) * 16;
-	sy = GET_TILE_Y(p1) * 16;
+	sx = TileX(p1) * 16;
+	sy = TileY(p1) * 16;
 
 	direction = 0;
 
@@ -307,7 +310,7 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		}
 
 		if (ti.type == MP_WATER) {
-			if (ti.map5 != 0) goto not_valid_below;
+			if (ti.map5 > 1) goto not_valid_below;
 			m5 = 0xC8;
 		} else if (ti.type == MP_RAILWAY) {
 			if (direction == 0) {
@@ -335,8 +338,7 @@ not_valid_below:;
 		/* do middle part of bridge */
 		if (flags & DC_EXEC) {
 			_map5[ti.tile] = (byte)(m5 | direction | rail_or_road);
-			_map_type_and_height[ti.tile] &= ~0xF0;
-			_map_type_and_height[ti.tile] |= MP_TUNNELBRIDGE << 4;
+			SetTileType(ti.tile, MP_TUNNELBRIDGE);
 
 			//bridges pieces sequence (middle parts)
 			// bridge len 1: 0
@@ -394,7 +396,7 @@ static bool DoCheckTunnelInWay(uint tile, uint z, uint dir)
 	TileInfo ti;
 	int delta;
 
-	delta = _tileoffs_by_dir[dir];
+	delta = TileOffsByDir(dir);
 
 	do {
 		tile -= delta;
@@ -428,7 +430,7 @@ static int32 DoBuildTunnel(int x, int y, int x2, int y2, uint32 flags, uint exc_
 	TileInfo ti;
 	uint z;
 
-	if ( (uint) x > TILE_X_MAX * 16 - 1 ||  (uint) y > TILE_X_MAX * 16 - 1)
+	if ((uint)x > MapMaxX() * 16 - 1 || (uint)y > MapMaxY() * 16 - 1)
 		return CMD_ERROR;
 
 	/* check if valid, and make sure that (x,y) is smaller than (x2,y2) */
@@ -585,11 +587,20 @@ uint CheckTunnelBusy(uint tile, int *length)
 {
 	int z = GetTileZ(tile);
 	byte m5 = _map5[tile];
-	int delta = _tileoffs_by_dir[m5 & 3], len = 0;
+	int delta = TileOffsByDir(m5 & 3);
+	int len = 0;
 	uint starttile = tile;
 	Vehicle *v;
 
-	do { tile += delta; len++; } while (!IS_TILETYPE(tile, MP_TUNNELBRIDGE) || _map5[tile]&0xF0 || (byte)(_map5[tile] ^ 2) != m5 || GetTileZ(tile) != z);
+	do {
+		tile += delta;
+		len++;
+	} while (
+		!IsTileType(tile, MP_TUNNELBRIDGE) ||
+		(_map5[tile] & 0xF0) != 0 ||
+		(byte)(_map5[tile] ^ 2) != m5 ||
+		GetTileZ(tile) != z
+	);
 
 	if ((v=FindVehicleBetween(starttile, tile, z)) != NULL) {
 		_error_message = v->type == VEH_Train ? STR_5000_TRAIN_IN_TUNNEL : STR_5001_ROAD_VEHICLE_IN_TUNNEL;
@@ -624,7 +635,7 @@ static int32 DoClearTunnel(uint tile, uint32 flags)
 	// removal allowal depends on difficulty settings
 	if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR ) {
 		if (!CheckforTownRating(tile, flags, t, TUNNELBRIDGE_REMOVE)) {
-			SET_DPARAM16(0, t->index);
+			SetDParam(0, t->index);
 			return_cmd_error(STR_2009_LOCAL_AUTHORITY_REFUSES);
 		}
 	}
@@ -639,7 +650,7 @@ static int32 DoClearTunnel(uint tile, uint32 flags)
 		UpdateSignalsOnSegment(tile, _updsignals_tunnel_dir[tile_dir]);
 		UpdateSignalsOnSegment(endtile, _updsignals_tunnel_dir[endtile_dir]);
 		if (_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR)
-			ChangeTownRating(t, -250, 0);
+			ChangeTownRating(t, RATING_TUNNEL_BRIDGE_DOWN_STEP, RATING_TUNNEL_BRIDGE_MINIMUM);
 	}
 	return  _price.clear_tunnel * (length + 1);
 }
@@ -651,7 +662,7 @@ static uint FindEdgesOfBridge(uint tile, uint *endtile)
 
 	// find start of bridge
 	for(;;) {
-		if (IS_TILETYPE(tile, MP_TUNNELBRIDGE) && (_map5[tile] & 0xE0) == 0x80)
+		if (IsTileType(tile, MP_TUNNELBRIDGE) && (_map5[tile] & 0xE0) == 0x80)
 			break;
 		tile += direction ? TILE_XY(0,-1) : TILE_XY(-1,0);
 	}
@@ -660,7 +671,7 @@ static uint FindEdgesOfBridge(uint tile, uint *endtile)
 
 	// find end of bridge
 	for(;;) {
-		if (IS_TILETYPE(tile, MP_TUNNELBRIDGE) && (_map5[tile] & 0xE0) == 0xA0)
+		if (IsTileType(tile, MP_TUNNELBRIDGE) && (_map5[tile] & 0xE0) == 0xA0)
 			break;
 		tile += direction ? TILE_XY(0,1) : TILE_XY(1,0);
 	}
@@ -686,7 +697,7 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 		int32 cost;
 
 		// check if we own the tile below the bridge..
-		if (_current_player != OWNER_WATER && (!CheckTileOwnership(tile) || !EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)) ))
+		if (_current_player != OWNER_WATER && (!CheckTileOwnership(tile) || !EnsureNoVehicleZ(tile, TilePixelHeight(tile))))
 			return CMD_ERROR;
 
 		cost = (_map5[tile] & 8) ? _price.remove_road * 2 : _price.remove_rail;
@@ -699,11 +710,11 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 		return cost;
 
 	/* delete canal under bridge */
-	} else if(_map5[tile]==0xC8 && GET_TILEHEIGHT(tile)!=0) {
+	} else if(_map5[tile] == 0xC8 && TilePixelHeight(tile) != 0) {
 		int32 cost;
 
 		// check for vehicles under bridge
-		if ( !EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)) )
+		if (!EnsureNoVehicleZ(tile, TilePixelHeight(tile)))
 			return CMD_ERROR;
 		cost = _price.clear_water;
 		if (flags & DC_EXEC) {
@@ -731,7 +742,8 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 	*/
 	tile		+= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
 	endtile	-= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
-	if ((v=FindVehicleBetween(tile, endtile, GET_TILEHEIGHT(tile) + 8)) != NULL) {
+	/* Bridges on slopes might have their Z-value offset..correct this */
+	if ((v = FindVehicleBetween(tile, endtile, TilePixelHeight(tile) + 8 + GetCorrectTileHeight(tile))) != NULL) {
 		VehicleInTheWayErrMsg(v);
 		return CMD_ERROR;
 	}
@@ -754,10 +766,10 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 		uint c = tile;
 		uint16 new_data;
 
-		//checks if the owner is town then decrease town rating by 250 until
+		//checks if the owner is town then decrease town rating by RATING_TUNNEL_BRIDGE_DOWN_STEP until
 		// you have a "Poor" (0) town rating
 		if (_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR)
-			ChangeTownRating(t, -250, 0);
+			ChangeTownRating(t, RATING_TUNNEL_BRIDGE_DOWN_STEP, RATING_TUNNEL_BRIDGE_MINIMUM);
 
 		do {
 			m5 = _map5[c];
@@ -768,11 +780,10 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 					new_data = _new_data_table[((m5 & 0x18) >> 2) | (m5&1)];
 				}	else {
 					if (!(m5 & 0x18)) goto clear_it;
-					new_data = 0x6000;
+					new_data = (GetTileSlope(c, NULL) == 0) ? 0x6000 : 0x6001;
 				}
 
-				_map_type_and_height[c] &= 0x0F;
-				_map_type_and_height[c] |= new_data >> 8;
+				SetTileType(c, new_data >> 12);
 				_map5[c] = (byte)new_data;
 				_map2[c] = 0;
 
@@ -794,25 +805,21 @@ clear_it:;
 }
 
 static int32 ClearTile_TunnelBridge(uint tile, byte flags) {
-	int32 ret;
 	byte m5 = _map5[tile];
 
 	if ((m5 & 0xF0) == 0) {
 		if (flags & DC_AUTO)
 			return_cmd_error(STR_5006_MUST_DEMOLISH_TUNNEL_FIRST);
+
 		return DoClearTunnel(tile, flags);
 	} else if (m5 & 0x80) {
 		if (flags & DC_AUTO)
 			return_cmd_error(STR_5007_MUST_DEMOLISH_BRIDGE_FIRST);
 
-		ret = DoClearBridge(tile, flags);
-		if (ret == CMD_ERROR)
-			return CMD_ERROR;
-
-		return ret;
-	} else {
-		return CMD_ERROR;
+		return DoClearBridge(tile, flags);
 	}
+
+	return CMD_ERROR;
 }
 
 int32 DoConvertTunnelBridgeRail(uint tile, uint totype, bool exec)
@@ -840,7 +847,7 @@ int32 DoConvertTunnelBridgeRail(uint tile, uint totype, bool exec)
 	} else if ((_map5[tile] & 0xF8) == 0xE0) {
 		// bridge middle part with rail below
 		// only check for train under bridge
-		if (!CheckTileOwnership(tile) || !EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)))
+		if (!CheckTileOwnership(tile) || !EnsureNoVehicleZ(tile, TilePixelHeight(tile)))
 			return CMD_ERROR;
 
 		// tile is already of requested type?
@@ -854,17 +861,25 @@ int32 DoConvertTunnelBridgeRail(uint tile, uint totype, bool exec)
 	} else if ((_map5[tile]&0xC6) == 0x80) {
 		uint starttile;
 		int32 cost;
+		uint z = TilePixelHeight(tile);
+
+		z += 8;
 
 		if (!CheckTileOwnership(tile)) return CMD_ERROR;
 
 		// railway bridge
 		starttile = tile = FindEdgesOfBridge(tile, &endtile);
 		// Make sure there's no vehicle on the bridge
-		if ((v=FindVehicleBetween(tile, endtile, 0xff)) != NULL) {
+		if ((v=FindVehicleBetween(tile, endtile, z)) != NULL) {
 			VehicleInTheWayErrMsg(v);
 			return CMD_ERROR;
 		}
 
+		if (!EnsureNoVehicle(starttile) || !EnsureNoVehicle(endtile)) {
+			_error_message = STR_8803_TRAIN_IN_THE_WAY;
+			return CMD_ERROR;
+		}
+
 		if ( (uint)(_map3_lo[tile] & 0xF) == totype) return CMD_ERROR;
 		cost = 0;
 		do {
@@ -887,7 +902,7 @@ int32 DoConvertTunnelBridgeRail(uint tile, uint totype, bool exec)
 
 
 // fast routine for getting the height of a middle bridge tile. 'tile' MUST be a middle bridge tile.
-uint GetBridgeHeight(const TileInfo *ti)
+static uint GetBridgeHeight(const TileInfo *ti)
 {
 	uint delta;
 	TileInfo ti_end;
@@ -908,7 +923,7 @@ uint GetBridgeHeight(const TileInfo *ti)
 		z_correction += 8;
 
 	// return the height there (the height of the NORTH CORNER)
-	return GET_TILEHEIGHT(tile) + z_correction;
+	return TilePixelHeight(tile) + z_correction;
 }
 
 static const byte _bridge_foundations[2][16] = {
@@ -920,7 +935,9 @@ static const byte _bridge_foundations[2][16] = {
 extern const byte _track_sloped_sprites[14];
 extern const byte _road_sloped_sprites[14];
 
+#include "table/bridge_land.h"
 #include "table/tunnel_land.h"
+#include "table/water_land.h"
 
 static void DrawBridgePillars(TileInfo *ti, int x, int y, int z)
 {
@@ -934,11 +951,12 @@ static void DrawBridgePillars(TileInfo *ti, int x, int y, int z)
 	// (necessary for cantilever bridges)
 	image = b[12 + (ti->map5&0x01)];
 	piece = _map2[ti->tile]&0xF;
-	if( image!=0 && piece!=0 )
-		DrawGroundSpriteAt(image, x,y, z);
+	if (image != 0 && piece != 0) {
+		if (_display_opt & DO_TRANS_BUILDINGS) image = (image & 0x3FFF) | 0x03224000;
+		DrawGroundSpriteAt(image, x, y, z);
+	}
 
 	image = b[(ti->map5&0x01)*6 + piece];
-	if (!(_display_opt & DO_TRANS_BUILDINGS)) image = (image & 0x3FFF) | 0x03224000;
 
 	if (image != 0) {
 		int back_height, front_height, i=z;
@@ -951,6 +969,8 @@ static void DrawBridgePillars(TileInfo *ti, int x, int y, int z)
 			{2,4,8,1,  11,16,9,0},
 		};
 
+		if (_display_opt & DO_TRANS_BUILDINGS) image = (image & 0x3FFF) | 0x03224000;
+
 		p = _tileh_bits[(image & 1) * 2 + (ti->map5&0x01)];
 		front_height = ti->z + ((ti->tileh & p[0])?8:0);
 		back_height = ti->z + ((ti->tileh & p[1])?8:0);
@@ -1043,7 +1063,7 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 			}
 
 			// draw ramp
-			if (!(_display_opt & DO_TRANS_BUILDINGS)) image = (image & 0x3FFF) | 0x03224000;
+			if (_display_opt & DO_TRANS_BUILDINGS) image = (image & 0x3FFF) | 0x03224000;
 			AddSortableSpriteToDraw(image, ti->x, ti->y, 16, 16, 7, ti->z);
 		} else {
 			// bridge middle part.
@@ -1056,7 +1076,11 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 			if (!(ti->map5 & 0x20)) {
 				// draw land under bridge
 				if (ice) image += 2;					// ice too?
-				DrawGroundSprite(_bridge_land_below[image] + _tileh_to_sprite[ti->tileh]);
+
+				if (image != 1 || ti->tileh == 0)
+					DrawGroundSprite(_bridge_land_below[image] + _tileh_to_sprite[ti->tileh]);
+				else
+					DrawGroundSprite(_water_shore_sprites[ti->tileh]);
 
 				// draw canal water?
 				if (ti->map5 & 8 && ti->z != 0) DrawCanalWater(ti->tile);
@@ -1090,13 +1114,13 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 
 			// draw rail
 			image = b[0];
-			if (!(_display_opt & DO_TRANS_BUILDINGS)) image = (image & 0x3FFF) | 0x03224000;
+			if (_display_opt & DO_TRANS_BUILDINGS) image = (image & 0x3FFF) | 0x03224000;
 			AddSortableSpriteToDraw(image, ti->x, ti->y, (ti->map5&1)?11:16, (ti->map5&1)?16:11, 1, z);
 
 			x = ti->x;
 			y = ti->y;
 			image = b[1];
-			if (!(_display_opt & DO_TRANS_BUILDINGS)) image = (image & 0x3FFF) | 0x03224000;
+			if (_display_opt & DO_TRANS_BUILDINGS) image = (image & 0x3FFF) | 0x03224000;
 
 			// draw roof
 			if (ti->map5&1) {
@@ -1111,7 +1135,7 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 				// draw poles below for small bridges
 				image = b[2];
 				if (image) {
-					if (!(_display_opt & DO_TRANS_BUILDINGS)) image = (image & 0x3FFF) | 0x03224000;
+					if (_display_opt & DO_TRANS_BUILDINGS) image = (image & 0x3FFF) | 0x03224000;
 					DrawGroundSpriteAt(image, x, y, z);
 				}
 			} else if (_patches.bridge_pillars) {
@@ -1204,7 +1228,7 @@ static uint GetSlopeTileh_TunnelBridge(TileInfo *ti) {
 }
 
 
-static void GetAcceptedCargo_TunnelBridge(uint tile, AcceptedCargo *ac)
+static void GetAcceptedCargo_TunnelBridge(uint tile, AcceptedCargo ac)
 {
 	/* not used */
 }
@@ -1303,14 +1327,14 @@ static uint32 GetTileTrackStatus_TunnelBridge(uint tile, TransportType mode)
 
 	if ((m5 & 0xF0) == 0) {
 		/* This is a tunnel */
-		if (((m5 & 0xC) >> 2) == mode) {
+		if (((m5 & 0xCU) >> 2) == mode) {
 			/* Tranport in the tunnel is compatible */
 			return m5&1 ? 0x202 : 0x101;
 		}
 	} else if (m5 & 0x80) {
 		/* This is a bridge */
 		result = 0;
-		if (((m5 & 0x6) >> 1) == mode) {
+		if (((m5 & 0x6U) >> 1) == mode) {
 			/* Transport over the bridge is compatible */
 			result = m5&1 ? 0x202 : 0x101;
 		}
@@ -1326,7 +1350,7 @@ static uint32 GetTileTrackStatus_TunnelBridge(uint tile, TransportType mode)
 						return result;
 			} else {
 				/* Transport underneath */
-				if ((m5 & 0x18) >> 3 != mode)
+				if ((m5 & 0x18U) >> 3 != mode)
 					/* Incompatible transport underneath */
 					return result;
 			}
@@ -1399,9 +1423,9 @@ static uint32 VehicleEnter_TunnelBridge(Vehicle *v, uint tile, int x, int y)
 			vdir = v->direction >> 1;
 
 			if (v->u.rail.track != 0x40 && dir == vdir) {
-				if (v->subtype == 0 && fc == _tunnel_fractcoord_1[dir]) {
+				if (v->subtype == TS_Front_Engine && fc == _tunnel_fractcoord_1[dir]) {
 					if (v->spritenum < 4)
-						SndPlayVehicleFx(3, v);
+						SndPlayVehicleFx(SND_05_TRAIN_THROUGH_TUNNEL, v);
 					return 0;
 				}
 				if (fc == _tunnel_fractcoord_2[dir]) {
@@ -1451,8 +1475,11 @@ static uint32 VehicleEnter_TunnelBridge(Vehicle *v, uint tile, int x, int y)
 			}
 		}
 	} else if (_map5[tile] & 0x80) {
-		if (v->type == VEH_Road || (v->type == VEH_Train && v->subtype == 0)) {
-			if (GetTileSlope(tile, &h) != 0 || myabs(h - v->z_pos) > 2) {
+		if (v->type == VEH_Road || (v->type == VEH_Train && v->subtype == TS_Front_Engine)) {
+			if (GetTileSlope(tile, &h) != 0)
+				h += 8; // Compensate for possible foundation
+			if (!(_map5[tile] & 0x40) || // start/end tile of bridge
+					myabs(h - v->z_pos) > 2) { // high above the ground -> on the bridge
 				/* modify speed of vehicle */
 				uint16 spd = _bridge_speeds[_map2[tile] >> 4];
 				if (v->type == VEH_Road) spd<<=1;
