@@ -1,5 +1,6 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
 #include "window.h"
 #include "gfx.h"
 #include "viewport.h"
@@ -45,23 +46,39 @@ void ScrollbarClickHandler(Window *w, const Widget *wi, int x, int y)
 	int mi, ma, pos;
 	Scrollbar *sb;
 
-	if (wi->type == WWT_SCROLLBAR) {
-		// vertical scroller
-		w->flags4 &= ~WF_HSCROLL;
-		mi = wi->top;
-		ma = wi->bottom;
-		pos = y;
-		sb = &w->vscroll;
-	} else {
-		// horizontal scroller
-		assert(wi->type == WWT_HSCROLLBAR);
-		w->flags4 |= WF_HSCROLL;
-		mi = wi->left;
-		ma = wi->right;
-		pos = x;
-		sb = &w->hscroll;
+	switch (wi->type) {
+		case WWT_SCROLLBAR: {
+			// vertical scroller
+			w->flags4 &= ~WF_HSCROLL;
+			w->flags4 &= ~WF_SCROLL2;
+			mi = wi->top;
+			ma = wi->bottom;
+			pos = y;
+			sb = &w->vscroll;
+			break;
+		}
+		case WWT_SCROLL2BAR: {
+			// 2nd vertical scroller
+			w->flags4 &= ~WF_HSCROLL;
+			w->flags4 |= WF_SCROLL2;
+			mi = wi->top;
+			ma = wi->bottom;
+			pos = y;
+			sb = &w->vscroll2;
+			break;
+		}
+		case  WWT_HSCROLLBAR: {
+			// horizontal scroller
+			w->flags4 &= ~WF_SCROLL2;
+			w->flags4 |= WF_HSCROLL;
+			mi = wi->left;
+			ma = wi->right;
+			pos = x;
+			sb = &w->hscroll;
+			break;
+		}
+		default: return; //this should never happen
 	}
-
 	if (pos <= mi+9) {
 		// Pressing the upper button?
 		if (!_demo_mode) {
@@ -145,7 +162,7 @@ void DrawWindowWidgets(Window *w)
 	const Widget *wi;
 	DrawPixelInfo *dpi = _cur_dpi;
 	Rect r;
-	uint32 dparam_backup[10];
+	uint32 dparam_backup[20];
 	uint32 cur_click, cur_disabled, cur_hidden;
 
 	if (w->desc_flags & WDF_RESTORE_DPARAM)	COPY_OUT_DPARAM(dparam_backup, 0, lengthof(dparam_backup));
@@ -256,9 +273,11 @@ void DrawWindowWidgets(Window *w)
 			Point pt;
 			int c1,c2;
 
+			assert(r.right - r.left == 11); // XXX - to ensure the same sizes are used everywhere!
+
 			// draw up/down buttons
-			DrawFrameRect(r.left, r.top, r.right, r.top+9, wi->color, (w->flags4 & (WF_SCROLL_UP | WF_HSCROLL)) == WF_SCROLL_UP ? 0x20 : 0);
-			DrawFrameRect(r.left, r.bottom-9, r.right, r.bottom, wi->color, (w->flags4 & (WF_SCROLL_DOWN | WF_HSCROLL)) == WF_SCROLL_DOWN ? 0x20 : 0);
+			DrawFrameRect(r.left, r.top, r.right, r.top+9, wi->color, (w->flags4 & (WF_SCROLL_UP | WF_HSCROLL | WF_SCROLL2)) == WF_SCROLL_UP ? 0x20 : 0);
+			DrawFrameRect(r.left, r.bottom-9, r.right, r.bottom, wi->color, (w->flags4 & (WF_SCROLL_DOWN | WF_HSCROLL | WF_SCROLL2)) == WF_SCROLL_DOWN ? 0x20 : 0);
 
 			// draw icons in up/down buttons
 			DoDrawString("\xA0", r.left+2, r.top, 0x10);
@@ -278,7 +297,38 @@ void DrawWindowWidgets(Window *w)
 			GfxFillRect(r.left+8, r.top+10, r.left+8, r.bottom-10, c2);
 
 			pt = HandleScrollbarHittest(&w->vscroll, r.top, r.bottom);
-			DrawFrameRect(r.left, pt.x, r.right, pt.y, wi->color, (w->flags4 & (WF_SCROLL_MIDDLE | WF_HSCROLL)) == WF_SCROLL_MIDDLE ? 0x20 : 0);
+			DrawFrameRect(r.left, pt.x, r.right, pt.y, wi->color, (w->flags4 & (WF_SCROLL_MIDDLE | WF_HSCROLL | WF_SCROLL2)) == WF_SCROLL_MIDDLE ? 0x20 : 0);
+			break;
+		}
+		case WWT_SCROLL2BAR: {
+			Point pt;
+			int c1,c2;
+
+			assert(r.right - r.left == 11); // XXX - to ensure the same sizes are used everywhere!
+
+			// draw up/down buttons
+			DrawFrameRect(r.left, r.top, r.right, r.top+9, wi->color, (w->flags4 & (WF_SCROLL_UP | WF_HSCROLL | WF_SCROLL2)) == (WF_SCROLL_UP | WF_SCROLL2) ? 0x20 : 0);
+			DrawFrameRect(r.left, r.bottom-9, r.right, r.bottom, wi->color, (w->flags4 & (WF_SCROLL_DOWN | WF_HSCROLL | WF_SCROLL2)) == (WF_SCROLL_DOWN | WF_SCROLL2) ? 0x20 : 0);
+
+			// draw icons in up/down buttons
+			DoDrawString("\xA0", r.left+2, r.top, 0x10);
+			DoDrawString("\xAA", r.left+2, r.bottom-9, 0x10);
+
+			c1 = _color_list[wi->color&0xF].window_color_1a;
+			c2 = _color_list[wi->color&0xF].window_color_2;
+
+			// draw "shaded" background
+			GfxFillRect(r.left, r.top+10, r.right, r.bottom-10, c2);
+			GfxFillRect(r.left, r.top+10, r.right, r.bottom-10, c1 | 0x8000);
+
+			// draw shaded lines
+			GfxFillRect(r.left+2, r.top+10, r.left+2, r.bottom-10, c1);
+			GfxFillRect(r.left+3, r.top+10, r.left+3, r.bottom-10, c2);
+			GfxFillRect(r.left+7, r.top+10, r.left+7, r.bottom-10, c1);
+			GfxFillRect(r.left+8, r.top+10, r.left+8, r.bottom-10, c2);
+
+			pt = HandleScrollbarHittest(&w->vscroll2, r.top, r.bottom);
+			DrawFrameRect(r.left, pt.x, r.right, pt.y, wi->color, (w->flags4 & (WF_SCROLL_MIDDLE | WF_HSCROLL | WF_SCROLL2)) == (WF_SCROLL_MIDDLE | WF_SCROLL2) ? 0x20 : 0);
 			break;
 		}
 
@@ -287,12 +337,14 @@ void DrawWindowWidgets(Window *w)
 			Point pt;
 			int c1,c2;
 
+			assert(r.bottom - r.top == 11); // XXX - to ensure the same sizes are used everywhere!
+
 			DrawFrameRect(r.left, r.top, r.left + 9, r.bottom, wi->color, (w->flags4 & (WF_SCROLL_UP | WF_HSCROLL)) == (WF_SCROLL_UP | WF_HSCROLL) ? 0x20 : 0);
 			DrawFrameRect(r.right-9, r.top, r.right, r.bottom, wi->color, (w->flags4 & (WF_SCROLL_DOWN | WF_HSCROLL)) == (WF_SCROLL_DOWN | WF_HSCROLL) ? 0x20 : 0);
 
 			// draw icons in up/down buttons
-			DoDrawString("\xA0", r.left+1, r.top+1, 0x10);
-			DoDrawString("\xAA", r.right-8, r.top+1, 0x10);
+			DrawSprite(SPR_ARROW_LEFT, r.left + 3, r.top + 2);
+			DrawSprite(SPR_ARROW_RIGHT, r.right - 6, r.top + 2);
 
 			c1 = _color_list[wi->color&0xF].window_color_1a;
 			c2 = _color_list[wi->color&0xF].window_color_2;
@@ -316,7 +368,7 @@ void DrawWindowWidgets(Window *w)
 
 		case WWT_FRAME: {
 			int c1,c2;
-			int x2 = 0;
+			int x2 = r.left; // by default the left side is the left side of the widget
 
 			if (wi->unkA != 0) {
 				x2 = DrawString(r.left+6, r.top, wi->unkA, 0);
@@ -347,7 +399,23 @@ void DrawWindowWidgets(Window *w)
 			goto restore_dparam;
 		}
 
+		case WWT_STICKYBOX: {
+			assert(r.right - r.left == 11); // XXX - to ensure the same sizes are used everywhere!
+			DrawFrameRect(r.left, r.top, r.right, r.bottom, wi->color, (cur_click & 1) ? 0x20 : 0);
+			DrawSprite((cur_click & 1) ? SPR_PIN_UP : SPR_PIN_DOWN, r.left + 2, r.top + 3);
+			break;
+		}
+
+		case WWT_RESIZEBOX: {
+			assert(r.right - r.left == 11); // XXX - to ensure the same sizes are used everywhere!
+
+			DrawFrameRect(r.left, r.top, r.right, r.bottom, wi->color, 0);
+			DrawSprite(SPR_WINDOW_RESIZE, r.left + 3, r.top + 3);
+			break;
+		}
+
 		case WWT_CAPTION: {
+			assert(r.bottom - r.top == 13); // XXX - to ensure the same sizes are used everywhere!
 			DrawFrameRect(r.left, r.top, r.right, r.bottom, wi->color, 0x10);
 			DrawFrameRect(r.left+1, r.top+1, r.right-1, r.bottom-1, wi->color, (w->caption_color == 0xFF) ? 0x60 : 0x70);
 
@@ -376,6 +444,7 @@ draw_default:;
 
 static uint _dropdown_item_count;
 static uint32 _dropdown_disabled;
+static bool _dropdown_hide_disabled;
 static const StringID *_dropdown_items;
 static int _dropdown_selindex;
 static byte _dropdown_button;
@@ -384,8 +453,8 @@ static WindowNumber _dropdown_windownum;
 static byte _dropdown_var1;
 static byte _dropdown_var2;
 
-static Widget _dropdown_menu_widgets[] = {
-{     WWT_IMGBTN,     0,     0, 0,     0, 0, 0x0, STR_NULL},
+static const Widget _dropdown_menu_widgets[] = {
+{     WWT_IMGBTN,   RESIZE_NONE,     0,     0, 0,     0, 0, 0x0, STR_NULL},
 {   WIDGETS_END},
 };
 
@@ -409,109 +478,115 @@ static int GetDropdownItem(Window *w)
 	return item;
 }
 
-void DropdownMenuWndProc(Window *w, WindowEvent *e)
+static void DropdownMenuWndProc(Window *w, WindowEvent *e)
 {
 	int item;
 
 	switch(e->event) {
-	case WE_PAINT: {
-		int x,y,i,sel;
-		uint32 dis;
+		case WE_PAINT: {
+			int x,y,i,sel;
+			uint32 dis;
+			bool hidden;
 
-		DrawWindowWidgets(w);
+			DrawWindowWidgets(w);
 
-		x = 1;
-		y = 2;
-		sel = _dropdown_selindex;
-		dis = _dropdown_disabled;
+			x = 1;
+			y = 2;
+			sel    = _dropdown_selindex;
+			dis    = _dropdown_disabled;
+			hidden = _dropdown_hide_disabled;
 
-		for(i=0; _dropdown_items[i] != INVALID_STRING_ID; i++) {
-			if (_dropdown_items[i] != 0) {
-				if (sel == 0) {
-					GfxFillRect(x+1, y, x+w->width-4, y + 9, 0);
+
+			for(i=0; _dropdown_items[i] != INVALID_STRING_ID; i++) {
+				if (!(hidden && (dis & 1))) {
+					if (_dropdown_items[i] != 0) {
+						if (sel == 0) {
+							GfxFillRect(x+1, y, x+w->width-4, y + 9, 0);
+						}
+						DrawString(x+2, y, _dropdown_items[i], sel==0 ? 12 : 16);
+
+						if (dis & 1) {
+							GfxFillRect(x, y, x+w->width-3, y + 9, 0x8000 +
+							_color_list[w->widget[0].color].window_color_bga);
+						}
+					} else {
+						int color_1 = _color_list[w->widget[0].color].window_color_1a;
+						int color_2 = _color_list[w->widget[0].color].window_color_2;
+						GfxFillRect(x+1, y+3, x+w->width-5, y+3, color_1);
+						GfxFillRect(x+1, y+4, x+w->width-5, y+4, color_2);
+					}
+					y += 10;
+					sel--;
 				}
-				DrawString(x+2, y, _dropdown_items[i], sel==0 ? 12 : 16);
-
-				if (dis & 1) {
-					GfxFillRect(x, y, x+w->width-3, y + 9, 0x8000 +
-						_color_list[_dropdown_menu_widgets[0].color].window_color_bga);
-				}
-			} else {
-				int color_1 = _color_list[_dropdown_menu_widgets[0].color].window_color_1a;
-				int color_2 = _color_list[_dropdown_menu_widgets[0].color].window_color_2;
-				GfxFillRect(x+1, y+3, x+w->width-5, y+3, color_1);
-				GfxFillRect(x+1, y+4, x+w->width-5, y+4, color_2);
+				dis>>=1;
 			}
-			y += 10;
-			sel--;
-			dis>>=1;
-		}
-	} break;
+		} break;
 
-	case WE_CLICK: {
-		item = GetDropdownItem(w);
-		if (item >= 0) {
-			_dropdown_var1 = 4;
-			_dropdown_selindex = item;
-			SetWindowDirty(w);
-		}
-	} break;
-
-	case WE_MOUSELOOP: {
-		Window *w2 = FindWindowById(_dropdown_windowclass, _dropdown_windownum);
-		if (w2 == NULL) {
-			DeleteWindow(w);
-			return;
-		}
-
-		if (_dropdown_var1 != 0 && --_dropdown_var1 == 0) {
-			WindowEvent e;
-			e.event = WE_DROPDOWN_SELECT;
-			e.dropdown.button = _dropdown_button;
-			e.dropdown.index = _dropdown_selindex;
-			w2->wndproc(w2, &e);
-			DeleteWindow(w);
-			return;
-		}
-
-		if (_dropdown_var2 != 0) {
+		case WE_CLICK: {
 			item = GetDropdownItem(w);
+			if (item >= 0) {
+				_dropdown_var1 = 4;
+				_dropdown_selindex = item;
+				SetWindowDirty(w);
+			}
+		} break;
 
-			if (!_left_button_clicked) {
-				_dropdown_var2 = 0;
-				if (item < 0)
-					return;
-				_dropdown_var1 = 2;
-			} else {
-				if (item < 0)
-					return;
+		case WE_MOUSELOOP: {
+			Window *w2 = FindWindowById(_dropdown_windowclass, _dropdown_windownum);
+			if (w2 == NULL) {
+				DeleteWindow(w);
+				return;
 			}
 
-			_dropdown_selindex = item;
-			SetWindowDirty(w);
-		}
-	} break;
+			if (_dropdown_var1 != 0 && --_dropdown_var1 == 0) {
+				WindowEvent e;
+				e.event = WE_DROPDOWN_SELECT;
+				e.dropdown.button = _dropdown_button;
+				e.dropdown.index = _dropdown_selindex;
+				w2->wndproc(w2, &e);
+				DeleteWindow(w);
+				return;
+			}
 
-	case WE_DESTROY: {
-		Window *w2 = FindWindowById(_dropdown_windowclass, _dropdown_windownum);
-		if (w2 != NULL) {
-			CLRBIT(w2->click_state, _dropdown_button);
-			InvalidateWidget(w2, _dropdown_button);
-		}
-	} break;
+			if (_dropdown_var2 != 0) {
+				item = GetDropdownItem(w);
+
+				if (!_left_button_clicked) {
+					_dropdown_var2 = 0;
+					if (item < 0)
+						return;
+					_dropdown_var1 = 2;
+				} else {
+					if (item < 0)
+						return;
+				}
+
+				_dropdown_selindex = item;
+				SetWindowDirty(w);
+			}
+		} break;
+
+		case WE_DESTROY: {
+			Window *w2 = FindWindowById(_dropdown_windowclass, _dropdown_windownum);
+			if (w2 != NULL) {
+				CLRBIT(w2->click_state, _dropdown_button);
+				InvalidateWidget(w2, _dropdown_button);
+			}
+		} break;
 	}
 }
 
-void ShowDropDownMenu(Window *w, const StringID *strings, int selected, int button, uint32 disabled_mask)
+void ShowDropDownMenu(Window *w, const StringID *strings, int selected, int button, uint32 disabled_mask, bool remove_filtered_strings)
 {
 	WindowNumber num;
 	WindowClass cls;
-	int i,t1,t2;
+	int i;
 	const Widget *wi;
 	Window *w2;
 	uint32 old_click_state = w->click_state;
 
 	_dropdown_disabled = disabled_mask;
+	_dropdown_hide_disabled = remove_filtered_strings;
 
 	cls = w->window_class;
 	num = w->window_number;
@@ -542,17 +617,28 @@ void ShowDropDownMenu(Window *w, const StringID *strings, int selected, int butt
 
 	wi = &w->widget[button];
 
-	_dropdown_menu_widgets[0].color = wi->color;
+	if ( remove_filtered_strings ) {
+		int j;
+		for(j=0; _dropdown_items[j] != INVALID_STRING_ID; j++) {
+			if ( disabled_mask & ( 1 << j )) {
+				_dropdown_item_count--;
+				i--;
+			}
+		}
+	}
 
 	w2 = AllocateWindow(
 		w->left + wi[-1].left + 1,
 		w->top + wi->bottom + 2,
-		(_dropdown_menu_widgets[0].right=t1=wi->right - wi[-1].left, t1 + 1),
-		(_dropdown_menu_widgets[0].bottom=t2=i*10+3, t2+1),
+		wi->right - wi[-1].left + 1,
+		i * 10 + 4,
 		DropdownMenuWndProc,
 		0x3F,
 		_dropdown_menu_widgets);
 
+	w2->widget[0].color = wi->color;
+	w2->widget[0].right = wi->right - wi[-1].left;
+	w2->widget[0].bottom = i * 10 + 3;
 
 	w2->flags4 &= ~WF_WHITE_BORDER_MASK;
 }
