@@ -1,8 +1,13 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "map.h"
 #include "window.h"
 #include "gfx.h"
 #include "viewport.h"
+#include "console.h"
+
+// delta between mouse cursor and upper left corner of dragged window
+static Point _drag_delta;
 
 void HandleButtonClick(Window *w, byte widget)
 {
@@ -20,45 +25,55 @@ void DispatchLeftClickEvent(Window *w, int x, int y) {
 	e.event = WE_CLICK;
 
 	if (w->desc_flags & WDF_DEF_WIDGET) {
-		e.click.widget = GetWidgetFromPos(w, x, y); 
+		e.click.widget = GetWidgetFromPos(w, x, y);
 		if (e.click.widget < 0) return; /* exit if clicked outside of widgets */
-		
+
 		wi = &w->widget[e.click.widget];
 
+		/* don't allow any interaction if the button has been disabled */
+		if (HASBIT(w->disabled_state, e.click.widget))
+			return;
+
 		if (wi->type & 0xE0) {
-			/* special widget handling */
+			/* special widget handling for buttons*/
 			switch(wi->type) {
-			case WWT_IMGBTN | WWB_PUSHBUTTON:
-			case WWT_TEXTBTN | WWB_PUSHBUTTON:
-				if (HASBIT(w->disabled_state, e.click.widget))
-					return; /* don't allow click if disabled */
+			case WWT_IMGBTN  | WWB_PUSHBUTTON: /* WWT_PUSHIMGBTN */
+			case WWT_TEXTBTN | WWB_PUSHBUTTON: /* WWT_PUSHTXTBTN */
 				HandleButtonClick(w, e.click.widget);
 				break;
 			case WWT_NODISTXTBTN:
-				if (HASBIT(w->disabled_state, e.click.widget))
-					return; /* don't allow click if disabled */
-				break;				
+				break;
 			}
-		} else if (wi->type == WWT_SCROLLBAR || wi->type == WWT_HSCROLLBAR) {
+		} else if (wi->type == WWT_SCROLLBAR || wi->type == WWT_SCROLL2BAR || wi->type == WWT_HSCROLLBAR) {
 			ScrollbarClickHandler(w, wi, e.click.pt.x, e.click.pt.y);
 		}
 
-		w->wndproc(w, &e);
-		
 		if (w->desc_flags & WDF_STD_BTN) {
-			if (e.click.widget == 0) DeleteWindow(w);
-			else { 
-				if (e.click.widget == 1) {
-					if (_ctrl_pressed) 
-						StartWindowSizing(w); 
-					else 
-						StartWindowDrag(w);
-				}
+			if (e.click.widget == 0) { /* 'X' */
+				DeleteWindow(w);
+				return;
+			}
+
+			if (e.click.widget == 1) { /* 'Title bar' */
+				StartWindowDrag(w); // if not return then w = StartWindowDrag(w); to get correct pointer
+				return;
 			}
 		}
-	} else {
-		w->wndproc(w, &e);
+
+		if (w->desc_flags & WDF_RESIZABLE && wi->type == WWT_RESIZEBOX) {
+			StartWindowSizing(w); // if not return then w = StartWindowSizing(w); to get correct pointer
+			return;
+		}
+
+		if (w->desc_flags & WDF_STICKY_BUTTON && wi->type == WWT_STICKYBOX) {
+			TOGGLEBIT(w->click_state, e.click.widget);
+			w->flags4 ^= WF_STICKY;
+			InvalidateWidget(w, e.click.widget);
+			return;
+		}
 	}
+
+	w->wndproc(w, &e);
 }
 
 void DispatchRightClickEvent(Window *w, int x, int y) {
@@ -79,18 +94,29 @@ void DispatchRightClickEvent(Window *w, int x, int y) {
 	e.event = WE_RCLICK;
 	e.click.pt.x = x;
 	e.click.pt.y = y;
-	w->wndproc(w, &e);	
+	w->wndproc(w, &e);
 }
 
 
-void DispatchMouseWheelEvent(Window *w, int wheel)
+void DispatchMouseWheelEvent(Window *w, uint widget, int wheel)
 {
-	
-	if (w->vscroll.count) {
-		int pos = clamp(w->vscroll.pos + wheel, 0, w->vscroll.count - w->vscroll.cap);
-		if (pos != w->vscroll.pos) {
-			w->vscroll.pos = pos;
-			SetWindowDirty(w);
+	const Widget *wi1 = &w->widget[widget];
+	const Widget *wi2 = &w->widget[widget + 1];
+	Scrollbar *sb;
+
+	/* The listbox can only scroll if scrolling was done on the scrollbar itself,
+	 * or on the listbox (and the next item is (must be) the scrollbar)
+	 * XXX - should be rewritten as a widget-dependent scroller but that's
+	 * not happening until someone rewrites the whole widget-code */
+	if ((sb = &w->vscroll,  wi1->type == WWT_SCROLLBAR)  || (sb = &w->vscroll2, wi1->type == WWT_SCROLL2BAR)  ||
+			(sb = &w->vscroll2, wi2->type == WWT_SCROLL2BAR) || (sb = &w->vscroll, wi2->type == WWT_SCROLLBAR) ) {
+
+		if (sb->count > sb->cap) {
+			int pos = clamp(sb->pos + wheel, 0, sb->count - sb->cap);
+			if (pos != sb->pos) {
+				sb->pos = pos;
+				SetWindowDirty(w);
+			}
 		}
 	}
 }
@@ -103,12 +129,12 @@ void DrawOverlappedWindowForAll(int left, int top, int right, int bottom)
 	_cur_dpi = &bk;
 
 	for(w=_windows; w!=_last_window; w++) {
-		if (right > w->left && 
+		if (right > w->left &&
 				bottom > w->top &&
 				left < w->left + w->width &&
 				top < w->top + w->height) {
-				DrawOverlappedWindow(w, left, top, right, bottom);	
-			}		
+				DrawOverlappedWindow(w, left, top, right, bottom);
+			}
 	}
 }
 
@@ -122,7 +148,7 @@ void DrawOverlappedWindow(Window *w, int left, int top, int right, int bottom)
 		 		bottom > v->top &&
 				left < v->left + v->width &&
 				top < v->top + v->height) {
-		
+
 			if (left < (x=v->left)) {
 				DrawOverlappedWindow(w, left, top, x, bottom);
 				DrawOverlappedWindow(w, x, top, right, bottom);
@@ -152,7 +178,7 @@ void DrawOverlappedWindow(Window *w, int left, int top, int right, int bottom)
 	}
 
 	{
-		DrawPixelInfo *dp = _cur_dpi;	
+		DrawPixelInfo *dp = _cur_dpi;
 		dp->width = right - left;
 		dp->height = bottom - top;
 		dp->left = left - w->left;
@@ -167,6 +193,7 @@ void DrawOverlappedWindow(Window *w, int left, int top, int right, int bottom)
 void CallWindowEventNP(Window *w, int event)
 {
 	WindowEvent e;
+
 	e.event = event;
 	w->wndproc(w, &e);
 }
@@ -175,7 +202,7 @@ void SetWindowDirty(Window *w)
 {
 	if (w == NULL)
 		return;
-	
+
 	SetDirtyBlocks(w->left, w->top, w->left + w->width, w->top + w->height);
 }
 
@@ -210,9 +237,11 @@ void DeleteWindow(Window *w)
 
 	SetWindowDirty(w);
 
+	free(w->widget);
+
 	v = --_last_window;
 	count = (byte*)v - (byte*)w;
-	memcpy(w, w + 1, count);
+	memmove(w, w + 1, count);
 }
 
 Window *FindWindowById(WindowClass cls, WindowNumber number)
@@ -220,11 +249,11 @@ Window *FindWindowById(WindowClass cls, WindowNumber number)
 	Window *w;
 
 	for(w=_windows; w!=_last_window; w++) {
-		if (w->window_class == cls && 
+		if (w->window_class == cls &&
 			  w->window_number == number) {
 			return w;
 		}
-	}		
+	}
 
 	return NULL;
 }
@@ -232,7 +261,19 @@ Window *FindWindowById(WindowClass cls, WindowNumber number)
 void DeleteWindowById(WindowClass cls, WindowNumber number)
 {
 	DeleteWindow(FindWindowById(cls, number));
-}	
+}
+
+void DeleteWindowByClass(WindowClass cls)
+{
+	Window *w;
+	for (w = _windows; w != _last_window;) {
+		if (w->window_class == cls) {
+			DeleteWindow(w);
+			w = _windows;
+		} else
+			w++;
+	}
+}
 
 Window *BringWindowToFrontById(WindowClass cls, WindowNumber number)
 {
@@ -241,7 +282,7 @@ Window *BringWindowToFrontById(WindowClass cls, WindowNumber number)
 	if (w != NULL) {
 		w->flags4 |= WF_WHITE_BORDER_MASK;
 		SetWindowDirty(w);
-		BringWindowToFront(w);
+		w = BringWindowToFront(w);
 	}
 
 	return w;
@@ -250,6 +291,7 @@ Window *BringWindowToFrontById(WindowClass cls, WindowNumber number)
 Window *BringWindowToFront(Window *w)
 {
 	Window *v;
+	Window temp;
 
 	v = _last_window;
 	do {
@@ -262,14 +304,66 @@ Window *BringWindowToFront(Window *w)
 
 	assert(w < v);
 
-	do {
-		memswap(w, w+1, sizeof(Window));
-		w++;
-	} while (v != w);
+	temp = *w;
+	memmove(w, w + 1, (v - w) * sizeof(Window));
+	*v = temp;
 
-	SetWindowDirty(w);
+	SetWindowDirty(v);
 
-	return w;
+	return v;
+}
+
+/* We have run out of windows, so find a suitable candidate for replacement.
+ * Keep all important windows intact */
+static Window *FindDeletableWindow(void)
+{
+	Window *w;
+	for (w = _windows; w < endof(_windows); w++) {
+		if (w->window_class != WC_MAIN_WINDOW && w->window_class != WC_MAIN_TOOLBAR &&
+			  w->window_class != WC_STATUS_BAR && w->window_class != WC_NEWS_WINDOW &&
+				!(w->flags4 & WF_STICKY) )
+				return w;
+	}
+	return NULL;
+}
+
+/* A window must be freed, and all are marked as important windows. Ease the
+ * restriction a bit by allowing to delete sticky windows */
+static Window *ForceFindDeletableWindow(void)
+{
+	Window *w;
+	for (w = _windows;; w++) {
+		assert(w < _last_window);
+
+		if (w->window_class != WC_MAIN_WINDOW && w->window_class != WC_MAIN_TOOLBAR &&
+			  w->window_class != WC_STATUS_BAR && w->window_class != WC_NEWS_WINDOW)
+				return w;
+	}
+}
+
+bool IsWindowOfPrototype(Window *w, const Widget *widget)
+{
+	return (w->original_widget == widget);
+}
+
+/* Copies 'widget' to 'w->widget' */
+void AssignWidgetToWindow(Window *w, const Widget *widget)
+{
+	w->original_widget = widget;
+
+	if (widget != NULL) {
+		const Widget *wi = widget;
+		uint index = 1;
+		while (wi->type != WWT_LAST) {
+			wi++;
+			index++;
+		}
+
+		w->widget = malloc(sizeof(Widget) * index);
+		memcpy(w->widget, widget, sizeof(Widget) * index);
+	} else {
+		w->widget = NULL;
+	}
 }
 
 Window *AllocateWindow(
@@ -277,26 +371,22 @@ Window *AllocateWindow(
 							int y,
 							int width,
 							int height,
-							WindowProc *proc, 
+							WindowProc *proc,
 							WindowClass cls,
 							const Widget *widget)
 {
 	Window *w;
 
-restart:;
 	w = _last_window;
 
 	if (w >= endof(_windows)) {
-		for(w=_windows; ;w++) {
-			assert(w < _last_window);
+		w = FindDeletableWindow();
 
-			if (w->window_class != WC_MAIN_WINDOW && w->window_class != WC_MAIN_TOOLBAR &&
-			    w->window_class != WC_STATUS_BAR && w->window_class != WC_NEWS_WINDOW) {
-			
-					DeleteWindow(w);
-					goto restart;
-			}
-		}
+		if (w == NULL) // no window found, force it!
+			w = ForceFindDeletableWindow();
+
+		DeleteWindow(w);
+		w = _last_window;
 	}
 
 	if (w != _windows && cls != WC_NEWS_WINDOW) {
@@ -336,18 +426,24 @@ restart:;
 	w->vscroll.count = 0;
 	w->hscroll.pos = 0;
 	w->hscroll.count = 0;
-	w->widget = widget;
-
-	((uint32*)w->custom)[0] = 0;
-	((uint32*)w->custom)[1] = 0;
-	((uint32*)w->custom)[2] = 0;
-	((uint32*)w->custom)[3] = 0;
+	AssignWidgetToWindow(w, widget);
+	w->resize.width = width;
+	w->resize.height = height;
+	w->resize.step_width = 1;
+	w->resize.step_height = 1;
 
+	{
+		uint i;
+		for (i=0;i<lengthof(w->custom);i++)
+			w->custom[i] = 0;
+	}
 
 	_last_window++;
 
 	SetWindowDirty(w);
 
+	CallWindowEventNP(w, WE_CREATE);
+
 	return w;
 }
 
@@ -356,17 +452,17 @@ Window *AllocateWindowAutoPlace2(
 	WindowNumber exist_num,
 	int width,
 	int height,
-	WindowProc *proc, 
+	WindowProc *proc,
 	WindowClass cls,
 	const Widget *widget)
 {
 	Window *w;
 	int x;
-	
+
 	w = FindWindowById(exist_class, exist_num);
 	if (w == NULL || w->left >= (_screen.width-20) || w->left <= -60 || w->top >= (_screen.height-20)) {
 		return AllocateWindowAutoPlace(width,height,proc,cls,widget);
-	}	
+	}
 
 	x = w->left;
 	if (x > _screen.width - width)
@@ -396,17 +492,17 @@ static bool IsGoodAutoPlace1(int left, int top)
 	if (left < 0 || top < 22 || right > _screen.width || bottom > _screen.height)
 		return false;
 
-	// Make sure it is not obscured by any window.	
+	// Make sure it is not obscured by any window.
 	for(w=_windows; w!=_last_window; w++) {
 		if (w->window_class == WC_MAIN_WINDOW)
 			continue;
 
-		if (right > w->left && 
+		if (right > w->left &&
 		    w->left + w->width > left &&
 				bottom > w->top &&
 				w->top + w->height > top)
 					return false;
-	}	
+	}
 
 	return true;
 }
@@ -426,22 +522,23 @@ static bool IsGoodAutoPlace2(int left, int top)
 	if (top < 22 || top > _screen.height - (height>>2))
 		return false;
 
-	// Make sure it is not obscured by any window.	
+	// Make sure it is not obscured by any window.
 	for(w=_windows; w!=_last_window; w++) {
 		if (w->window_class == WC_MAIN_WINDOW)
 			continue;
 
-		if (left + width > w->left && 
+		if (left + width > w->left &&
 		    w->left + w->width > left &&
 				top + height > w->top &&
 				w->top + w->height > top)
 					return false;
-	}	
+	}
 
 	return true;
 }
 
-Point GetAutoPlacePosition(int width, int height) {
+static Point GetAutoPlacePosition(int width, int height)
+{
 	Window *w;
 	Point pt;
 
@@ -462,8 +559,8 @@ Point GetAutoPlacePosition(int width, int height) {
 		if (IsGoodAutoPlace1(w->left-   width-2,w->top+w->height-height)) goto ok_pos;
 		if (IsGoodAutoPlace1(w->left+w->width-width,w->top+w->height+2)) goto ok_pos;
 		if (IsGoodAutoPlace1(w->left+w->width-width,w->top-   height-2)) goto ok_pos;
-	}	
-	
+	}
+
 	for(w=_windows; w!=_last_window; w++) {
 		if (w->window_class == WC_MAIN_WINDOW)
 			continue;
@@ -476,7 +573,7 @@ Point GetAutoPlacePosition(int width, int height) {
 
 	{
 		int left=0,top=24;
-		
+
 restart:;
 		for(w=_windows; w!=_last_window; w++) {
 			if (w->left == left && w->top == top) {
@@ -485,12 +582,12 @@ restart:;
 				goto restart;
 			}
 		}
-		
+
 		pt.x = left;
 		pt.y = top;
 		return pt;
 	}
-	
+
 ok_pos:;
 	pt.x = _awap_r.left;
 	pt.y = _awap_r.top;
@@ -500,7 +597,7 @@ ok_pos:;
 Window *AllocateWindowAutoPlace(
 	int width,
 	int height,
-	WindowProc *proc, 
+	WindowProc *proc,
 	WindowClass cls,
 	const Widget *widget) {
 
@@ -524,13 +621,22 @@ Window *AllocateWindowDesc(const WindowDesc *desc)
 	Point pt;
 	Window *w;
 
-	if (desc->parent_cls != WC_MAIN_WINDOW && 
+	if (desc->parent_cls != WC_MAIN_WINDOW &&
 			(w = FindWindowById(desc->parent_cls, _alloc_wnd_parent_num), _alloc_wnd_parent_num=0, w) != NULL &&
 			w->left < _screen.width-20 && w->left > -60 && w->top < _screen.height-20) {
 		pt.x = w->left + 10;
 		if (pt.x > _screen.width + 10 - desc->width)
 			pt.x = (_screen.width + 10 - desc->width) - 20;
 		pt.y = w->top + 10;
+	} else if (desc->cls == WC_BUILD_TOOLBAR) {	// open Build Toolbars aligned
+		/* Override the position if a toolbar is opened according to the place of the maintoolbar
+		 * The main toolbar (WC_MAIN_TOOLBAR) is 640px in width */
+		switch (_patches.toolbar_pos) {
+		case 1:		pt.x = ((_screen.width + 640) >> 1) - desc->width; break;
+		case 2:		pt.x = _screen.width - desc->width; break;
+		default:	pt.x = 640 - desc->width;
+		}
+		pt.y = desc->top;
 	} else {
 		pt.x = desc->left;
 		pt.y = desc->top;
@@ -539,6 +645,7 @@ Window *AllocateWindowDesc(const WindowDesc *desc)
 		} else {
 			if (pt.x == WDP_CENTER) pt.x = (_screen.width - desc->width) >> 1;
 			if (pt.y == WDP_CENTER) pt.y = (_screen.height - desc->height) >> 1;
+			else if(pt.y < 0) pt.y = _screen.height + pt.y; // if y is negative, it's from the bottom of the screen
 		}
 	}
 
@@ -562,15 +669,16 @@ Window *FindWindowFromPt(int x, int y)
 }
 
 
-void InitWindowSystem()
+void InitWindowSystem(void)
 {
+	IConsoleClose();
 	memset(&_windows, 0, sizeof(_windows));
 	_last_window = _windows;
 	memset(_viewports, 0, sizeof(_viewports));
 	_active_viewports = 0;
 }
 
-void DecreaseWindowCounters()
+static void DecreaseWindowCounters(void)
 {
 	Window *w;
 
@@ -587,7 +695,7 @@ void DecreaseWindowCounters()
 
 	for(w=_last_window; w != _windows;) {
 		--w;
-		
+
 		if (w->flags4&WF_TIMEOUT_MASK && !(--w->flags4&WF_TIMEOUT_MASK)) {
 			CallWindowEventNP(w, WE_TIMEOUT);
 			if (w->desc_flags & WDF_UNCLICK_BUTTONS)
@@ -596,12 +704,12 @@ void DecreaseWindowCounters()
 	}
 }
 
-Window *GetCallbackWnd()
+Window *GetCallbackWnd(void)
 {
 	return FindWindowById(_thd.window_class, _thd.window_number);
 }
 
-void HandlePlacePresize()
+static void HandlePlacePresize(void)
 {
 	Window *w;
 	WindowEvent e;
@@ -622,7 +730,7 @@ void HandlePlacePresize()
 	w->wndproc(w, &e);
 }
 
-bool HandleDragDrop()
+static bool HandleDragDrop(void)
 {
 	Window *w;
 	WindowEvent e;
@@ -632,7 +740,7 @@ bool HandleDragDrop()
 
 	if (_left_button_down)
 		return false;
-	
+
 	w = GetCallbackWnd();
 
 	ResetObjectToPlace();
@@ -648,7 +756,7 @@ bool HandleDragDrop()
 	return false;
 }
 
-bool HandlePopupMenu()
+static bool HandlePopupMenu(void)
 {
 	Window *w;
 	WindowEvent e;
@@ -665,64 +773,272 @@ bool HandlePopupMenu()
 	if (_left_button_down) {
 		e.event = WE_POPUPMENU_OVER;
 		e.popupmenu.pt = _cursor.pos;
-		w->wndproc(w, &e);
 	} else {
 		_popup_menu_active = false;
 		e.event = WE_POPUPMENU_SELECT;
 		e.popupmenu.pt = _cursor.pos;
-		w->wndproc(w, &e);
 	}
 
+	w->wndproc(w, &e);
+
 	return false;
 }
 
-bool HandleWindowDragging() 
+static bool HandleMouseOver(void)
 {
 	Window *w;
-	int x, y, t;
+	WindowEvent e;
+	static Window *last_w = NULL;
 
+	w = FindWindowFromPt(_cursor.pos.x, _cursor.pos.y);
+
+	// We changed window, put a MOUSEOVER event to the last window
+	if (last_w && last_w != w) {
+		e.event = WE_MOUSEOVER;
+		e.mouseover.pt.x = -1;
+		e.mouseover.pt.y = -1;
+		if (last_w->wndproc)
+			last_w->wndproc(last_w, &e);
+	}
+	last_w = w;
+
+	if (w) {
+		// send an event in client coordinates.
+		e.event = WE_MOUSEOVER;
+		e.mouseover.pt.x = _cursor.pos.x - w->left;
+		e.mouseover.pt.y = _cursor.pos.y - w->top;
+		if (w->widget != NULL) {
+			e.mouseover.widget = GetWidgetFromPos(w, e.mouseover.pt.x, e.mouseover.pt.y);
+		}
+		w->wndproc(w, &e);
+	}
+
+	// Mouseover never stops execution
+	return true;
+}
+
+static bool HandleWindowDragging(void)
+{
+	Window *w;
 	// Get out immediately if no window is being dragged at all.
 	if (!_dragging_window)
 		return true;
 
 	// Otherwise find the window...
-	for(w=_windows; w != _last_window; w++) {
-		if (w->flags4&(WF_DRAGGING|WF_SIZING)) {
-			
+	for (w = _windows; w != _last_window; w++) {
+		if (w->flags4 & WF_DRAGGING) {
+			const Widget *t = &w->widget[1]; // the title bar ... ugh
+			const Window *v;
+			int x;
+			int y;
+			int nx;
+			int ny;
+
 			// Stop the dragging if the left mouse button was released
 			if (!_left_button_down) {
-				w->flags4 &= ~(WF_DRAGGING | WF_SIZING);
+				w->flags4 &= ~WF_DRAGGING;
 				break;
 			}
 
-			// Otherwise calculate the delta position.
-			x = _cursor.pos.x;
-			y = clamp2(_cursor.pos.y, 23, _screen.height-12);
-			t = _cursorpos_drag_start.x; _cursorpos_drag_start.x = x; x-=t;
-			t = _cursorpos_drag_start.y; _cursorpos_drag_start.y = y; y-=t;
+			SetWindowDirty(w);
 
-			// No movement?
+			x = _cursor.pos.x + _drag_delta.x;
+			y = _cursor.pos.y + _drag_delta.y;
+			nx = x;
+			ny = y;
+
+			if (_patches.window_snap_radius != 0) {
+				int hsnap = _patches.window_snap_radius;
+				int vsnap = _patches.window_snap_radius;
+				int delta;
+
+				for (v = _windows; v != _last_window; ++v) {
+					if (v == w) continue; // Don't snap at yourself
+
+					if (y + w->height > v->top && y < v->top + v->height) {
+						// Your left border <-> other right border
+						delta = abs(v->left + v->width - x);
+						if (delta <= hsnap) {
+							nx = v->left + v->width;
+							hsnap = delta;
+						}
+
+						// Your right border <-> other left border
+						delta = abs(v->left - x - w->width);
+						if (delta <= hsnap) {
+							nx = v->left - w->width;
+							hsnap = delta;
+						}
+					}
+
+					if (w->top + w->height >= v->top && w->top <= v->top + v->height) {
+						// Your left border <-> other left border
+						delta = abs(v->left - x);
+						if (delta <= hsnap) {
+							nx = v->left;
+							hsnap = delta;
+						}
+
+						// Your right border <-> other right border
+						delta = abs(v->left + v->width - x - w->width);
+						if (delta <= hsnap) {
+							nx = v->left + v->width - w->width;
+							hsnap = delta;
+						}
+					}
+
+					if (x + w->width > v->left && x < v->left + v->width) {
+						// Your top border <-> other bottom border
+						delta = abs(v->top + v->height - y);
+						if (delta <= vsnap) {
+							ny = v->top + v->height;
+							vsnap = delta;
+						}
+
+						// Your bottom border <-> other top border
+						delta = abs(v->top - y - w->height);
+						if (delta <= vsnap) {
+							ny = v->top - w->height;
+							vsnap = delta;
+						}
+					}
+
+					if (w->left + w->width >= v->left && w->left <= v->left + v->width) {
+						// Your top border <-> other top border
+						delta = abs(v->top - y);
+						if (delta <= vsnap) {
+							ny = v->top;
+							vsnap = delta;
+						}
+
+						// Your bottom border <-> other bottom border
+						delta = abs(v->top + v->height - y - w->height);
+						if (delta <= vsnap) {
+							ny = v->top + v->height - w->height;
+							vsnap = delta;
+						}
+					}
+				}
+			}
+
+			// Make sure the window doesn't leave the screen
+			// 13 is the height of the title bar
+			nx = clamp(nx, 13 - t->right, _screen.width - 13 - t->left);
+			ny = clamp(ny, 0, _screen.height - 13);
+
+			// Make sure the title bar isn't hidden by behind the main tool bar
+			v = FindWindowById(WC_MAIN_TOOLBAR, 0);
+			if (v != NULL) {
+				int v_bottom = v->top + v->height;
+				int v_right = v->left + v->width;
+				if (ny + t->top >= v->top && ny + t->top < v_bottom) {
+					if ((v->left < 13 && nx + t->left < v->left) ||
+							(v_right > _screen.width - 13 && nx + t->right > v_right)) {
+						ny = v_bottom;
+					} else {
+						if (nx + t->left > v->left - 13 &&
+								nx + t->right < v_right + 13) {
+							if (w->top >= v_bottom)
+								ny = v_bottom;
+							else if (w->left < nx)
+								nx = v->left - 13 - t->left;
+							else
+								nx = v_right + 13 - t->right;
+						}
+					}
+				}
+			}
+
+			if (w->viewport != NULL) {
+				w->viewport->left += nx - w->left;
+				w->viewport->top  += ny - w->top;
+			}
+			w->left = nx;
+			w->top  = ny;
+
+			SetWindowDirty(w);
+			return false;
+		} else if (w->flags4 & WF_SIZING) {
+			WindowEvent e;
+			int x, y;
+
+			/* Stop the sizing if the left mouse button was released */
+			if (!_left_button_down) {
+				w->flags4 &= ~WF_SIZING;
+				break;
+			}
+
+			x = _cursor.pos.x - _drag_delta.x;
+			y = _cursor.pos.y - _drag_delta.y;
+
+			/* X and Y has to go by step.. calculate it */
+			if (w->resize.step_width > 1)
+				x = x - (x % (int)w->resize.step_width);
+
+			if (w->resize.step_height > 1)
+				y = y - (y % (int)w->resize.step_height);
+
+			/* Check if we don't go below the minimum set size */
+			if ((int)w->width + x < (int)w->resize.width)
+				x = w->resize.width - w->width;
+			if ((int)w->height + y < (int)w->resize.height)
+				y = w->resize.height - w->height;
+
+			/* Window already on size */
 			if (x == 0 && y == 0)
 				return false;
 
-			// Mark the old position dirty
+			/* Now find the new cursor pos.. this is NOT _cursor, because
+			    we move in steps. */
+			_drag_delta.x += x;
+			_drag_delta.y += y;
+
 			SetWindowDirty(w);
 
-			if (w->flags4 & WF_SIZING) {
-				// Size the window
-				w->width += x;
-				w->height += y;
-			} else {
-				// Move the window and viewport
-				w->left += x;
-				w->top += y;
-				if (w->viewport) {
-					w->viewport->left += x;
-					w->viewport->top += y;
+			/* Scroll through all the windows and update the widgets if needed */
+			{
+				Widget *wi = w->widget;
+				bool resize_height = false;
+				bool resize_width = false;
+
+				while (wi->type != WWT_LAST) {
+					if (wi->resize_flag != RESIZE_NONE) {
+						/* Resize this widget */
+						if (wi->resize_flag & RESIZE_LEFT) {
+							wi->left += x;
+							resize_width = true;
+						}
+						if (wi->resize_flag & RESIZE_RIGHT) {
+							wi->right += x;
+							resize_width = true;
+						}
+
+						if (wi->resize_flag & RESIZE_TOP) {
+							wi->top += y;
+							resize_height = true;
+						}
+						if (wi->resize_flag & RESIZE_BOTTOM) {
+							wi->bottom += y;
+							resize_height = true;
+						}
+					}
+					wi++;
 				}
+
+				/* We resized at least 1 widget, so let's rezise the window totally */
+				if (resize_width)
+					w->width  = x + w->width;
+				if (resize_height)
+					w->height = y + w->height;
 			}
-				
-			// And also mark the new position dirty.
+
+			e.event = WE_RESIZE;
+			e.sizing.size.x = x + w->width;
+			e.sizing.size.y = y + w->height;
+			e.sizing.diff.x = x;
+			e.sizing.diff.y = y;
+			w->wndproc(w, &e);
+
 			SetWindowDirty(w);
 			return false;
 		}
@@ -736,7 +1052,10 @@ Window *StartWindowDrag(Window *w)
 {
 	w->flags4 |= WF_DRAGGING;
 	_dragging_window = true;
-	_cursorpos_drag_start = _cursor.pos;
+
+	_drag_delta.x = w->left - _cursor.pos.x;
+	_drag_delta.y = w->top  - _cursor.pos.y;
+
 	w = BringWindowToFront(w);
 	DeleteWindowById(WC_DROPDOWN_MENU, 0);
 	return w;
@@ -746,14 +1065,17 @@ Window *StartWindowSizing(Window *w)
 {
 	w->flags4 |= WF_SIZING;
 	_dragging_window = true;
-	_cursorpos_drag_start = _cursor.pos;
+
+	_drag_delta.x = _cursor.pos.x;
+	_drag_delta.y = _cursor.pos.y;
+
 	w = BringWindowToFront(w);
 	DeleteWindowById(WC_DROPDOWN_MENU, 0);
 	return w;
 }
 
 
-bool HandleScrollbarScrolling()
+static bool HandleScrollbarScrolling(void)
 {
 	Window *w;
 	int i;
@@ -772,11 +1094,14 @@ bool HandleScrollbarScrolling()
 				w->flags4 &= ~WF_SCROLL_MIDDLE;
 				SetWindowDirty(w);
 				break;
-			}	
+			}
 
 			if (w->flags4 & WF_HSCROLL) {
 				sb = &w->hscroll;
 				i = _cursor.pos.x - _cursorpos_drag_start.x;
+			} else if (w->flags4 & WF_SCROLL2){
+				sb = &w->vscroll2;
+				i = _cursor.pos.y - _cursorpos_drag_start.y;
 			} else {
 				sb = &w->vscroll;
 				i = _cursor.pos.y - _cursorpos_drag_start.y;
@@ -789,14 +1114,14 @@ bool HandleScrollbarScrolling()
 				SetWindowDirty(w);
 			}
 			return false;
-		}		
+		}
 	}
-	
+
 	_scrolling_scrollbar = false;
 	return false;
 }
 
-bool HandleViewportScroll() 
+static bool HandleViewportScroll(void)
 {
 	Window *w;
 	ViewPort *vp;
@@ -811,7 +1136,7 @@ stop_capt:;
 		_scrolling_viewport = false;
 		return true;
 	}
-	
+
 	w = FindWindowFromPt(_cursor.pos.x, _cursor.pos.y);
 	if (w == NULL) goto stop_capt;
 
@@ -826,9 +1151,13 @@ stop_capt:;
 		return false;
 	} else {
 		// scroll the smallmap ?
-		
+		int hx;
+		int hy;
+		int hvx;
+		int hvy;
+
 		_cursor.fix_at = true;
-		
+
 		dx = _cursor.delta.x;
 		dy = _cursor.delta.y;
 
@@ -854,10 +1183,14 @@ stop_capt:;
 			}
 		}
 
-		if (x < 16) { x = 16; sub = 0; }
-		if (x > (TILES_X-2)*16) { x = (TILES_X-2)*16; sub = 0; }
-		if (y < -1120) { y = -1120; sub = 0; }
-		if (y > (TILE_X_MAX-40) * 16) { y = (TILE_X_MAX-40) * 16; sub = 0; }
+		hx = (w->widget[4].right  - w->widget[4].left) / 2;
+		hy = (w->widget[4].bottom - w->widget[4].top ) / 2;
+		hvx = hx * -4 + hy * 8;
+		hvy = hx *  4 + hy * 8;
+		if (x < -hvx) { x = -hvx; sub = 0; }
+		if (x > (int)MapMaxX() * 16 - hvx) { x = MapMaxX() * 16 - hvx; sub = 0; }
+		if (y < -hvy) { y = -hvy; sub = 0; }
+		if (y > (int)MapMaxY() * 16 - hvy) { y = MapMaxY() * 16 - hvy; sub = 0; }
 
 		WP(w,smallmap_d).scroll_x = x;
 		WP(w,smallmap_d).scroll_y = y;
@@ -892,8 +1225,8 @@ static Window *MaybeBringWindowToFront(Window *w)
 				w->top  + w->height <= u->top ||
 				u->top + u->height <= w->top)
 					continue;
-		
-		return BringWindowToFront(w);	
+
+		return BringWindowToFront(w);
 	}
 
 	return w;
@@ -903,26 +1236,37 @@ static void HandleKeypress(uint32 key)
 {
 	Window *w;
 	WindowEvent we;
-	
+ /* Stores if a window with a textfield for typing is open
+  * If this is the case, keypress events are only passed to windows with text fields and
+	* to thein this main toolbar. */
+	bool query_open = false;
+
 	// Setup event
 	we.keypress.event = WE_KEYPRESS;
 	we.keypress.ascii = key & 0xFF;
 	we.keypress.keycode = key >> 16;
 	we.keypress.cont = true;
 
+	// check if we have a query string window open before allowing hotkeys
+	if(FindWindowById(WC_QUERY_STRING, 0)!=NULL || FindWindowById(WC_SEND_NETWORK_MSG, 0)!=NULL)
+		query_open = true;
+
 	// Call the event, start with the uppermost window.
 	for(w=_last_window; w != _windows;) {
 		--w;
+		// if a query window is open, only call the event for certain window types
+		if(query_open && w->window_class!=WC_QUERY_STRING && w->window_class!=WC_SEND_NETWORK_MSG && w->window_class!=WC_MAIN_TOOLBAR)
+			continue;
 		w->wndproc(w, &we);
 		if (!we.keypress.cont)
 			break;
 	}
 }
 
-extern void UpdateTileSelection();
-extern bool VpHandlePlaceSizingDrag();
+extern void UpdateTileSelection(void);
+extern bool VpHandlePlaceSizingDrag(void);
 
-void MouseLoop()
+void MouseLoop(void)
 {
 	int x,y;
 	Window *w;
@@ -953,7 +1297,7 @@ void MouseLoop()
 		mousewheel = _cursor.wheel;
 		_cursor.wheel = 0;
 	}
-	
+
 	DecreaseWindowCounters();
 	HandlePlacePresize();
 	UpdateTileSelection();
@@ -974,7 +1318,10 @@ void MouseLoop()
 
 	if (!HandleViewportScroll())
 		return;
-	
+
+	if (!HandleMouseOver())
+		return;
+
 	x = _cursor.pos.x;
 	y = _cursor.pos.y;
 
@@ -993,7 +1340,7 @@ void MouseLoop()
 				else if (15-(vp->width-x) > 0) { WP(w,vp_d).scrollpos_x += (15-(vp->width-x))*scrollspeed << vp->zoom; }
 				if (y-15<0) { WP(w,vp_d).scrollpos_y += (y-15)*scrollspeed << vp->zoom; }
 				else if (15-(vp->height-y) > 0) { WP(w,vp_d).scrollpos_y += (15-(vp->height-y))*scrollspeed << vp->zoom; }
-#undef scrollspeed		
+#undef scrollspeed
 			}
 		}
 		return;
@@ -1008,17 +1355,21 @@ void MouseLoop()
 		if (_game_mode == GM_MENU)
 			return;
 
-		if (mousewheel && !(w->flags4 & WF_DISABLE_VP_SCROLL)) {
-			ZoomInOrOutToCursor(mousewheel < 0);
+		// only allow zooming in-out in main window, or in viewports
+		if ( mousewheel && !(w->flags4 & WF_DISABLE_VP_SCROLL) &&
+			   (w->window_class == WC_MAIN_WINDOW || w->window_class == WC_EXTRA_VIEW_PORT) ) {
+			ZoomInOrOutToCursorWindow(mousewheel < 0,w);
 		}
 
 		if (click == 1) {
+			DEBUG(misc, 2) ("cursor: 0x%X (%d)", _cursor.sprite, _cursor.sprite);
 			if (_thd.place_mode != 0 &&
-					_cursor.sprite != 0x2CF &&
+					// query button and place sign button work in pause mode
+					!(_cursor.sprite == 0x2CF || _cursor.sprite == 0x2D2) &&
 					_pause != 0 &&
-					!_patches.build_in_pause)
+					!_cheats.build_in_pause.value)
 						return;
-			
+
 			if (_thd.place_mode == 0) {
 				HandleViewportClicked(vp, x, y);
 			} else {
@@ -1032,7 +1383,7 @@ void MouseLoop()
 		}
 	} else {
 		if (mousewheel)
-			DispatchMouseWheelEvent(w, mousewheel);
+			DispatchMouseWheelEvent(w, GetWidgetFromPos(w, x - w->left, y - w->top), mousewheel);
 
 		if (click == 1)
 			DispatchLeftClickEvent(w, x - w->left, y - w->top);
@@ -1045,7 +1396,7 @@ static int _we4_timer;
 
 extern uint32 _pixels_redrawn;
 
-void UpdateWindows()
+void UpdateWindows(void)
 {
 	Window *w;
 	int t;
@@ -1076,9 +1427,10 @@ void UpdateWindows()
 		if (w->viewport != NULL)
 			UpdateViewportPosition(w);
 	}
+	DrawTextMessage();
 	// Redraw mouse cursor in case it was hidden
 	DrawMouseCursor();
-}	
+}
 
 
 int GetMenuItemIndex(Window *w, int x, int y)
@@ -1107,11 +1459,11 @@ void InvalidateWidget(Window *w, byte widget_index)
 	const Widget *wi = &w->widget[widget_index];
 //	if (wi->left != -2) {
 		SetDirtyBlocks(
-			w->left + wi->left, 
+			w->left + wi->left,
 			w->top + wi->top,
 			w->left + wi->right + 1,
 			w->top + wi->bottom + 1);
-//	}		
+//	}
 }
 
 void InvalidateWindowWidget(byte cls, WindowNumber number, byte widget_index)
@@ -1135,7 +1487,7 @@ void InvalidateWindowClasses(byte cls)
 }
 
 
-void CallWindowTickEvent()
+void CallWindowTickEvent(void)
 {
 	Window *w;
 	for(w=_last_window; w != _windows;) {
@@ -1144,7 +1496,7 @@ void CallWindowTickEvent()
 	}
 }
 
-void DeleteNonVitalWindows()
+void DeleteNonVitalWindows(void)
 {
 	Window *w;
 	for(w=_windows; w!=_last_window;) {
@@ -1153,7 +1505,8 @@ void DeleteNonVitalWindows()
 				w->window_class != WC_MAIN_TOOLBAR &&
 				w->window_class != WC_STATUS_BAR &&
 				w->window_class != WC_TOOLBAR_MENU &&
-				w->window_class != WC_TOOLTIPS) {
+				w->window_class != WC_TOOLTIPS &&
+				(w->flags4 & WF_STICKY) == 0) { // do not delete windows which are 'pinned'
 			DeleteWindow(w);
 			w = _windows;
 		} else {
@@ -1162,13 +1515,56 @@ void DeleteNonVitalWindows()
 	}
 }
 
+/* It is possible that a stickied window gets to a position where the
+ * 'close' button is outside the gaming area. You cannot close it then; except
+ * with this function. It closes all windows calling the standard function,
+ * then, does a little hacked loop of closing all stickied windows. Note
+ * that standard windows (status bar, etc.) are not stickied, so these aren't affected */
+void DeleteAllNonVitalWindows(void)
+{
+	Window *w;
+	// Delete every window except for stickied ones
+	DeleteNonVitalWindows();
+	// Delete all sticked windows
+	for (w = _windows; w != _last_window;) {
+		if (w->flags4 & WF_STICKY) {
+			DeleteWindow(w);
+			w = _windows;
+		} else
+			w++;
+	}
+}
+
+/* Delete all always on-top windows to get an empty screen */
+void HideVitalWindows(void)
+{
+	DeleteWindowById(WC_MAIN_TOOLBAR, 0);
+	DeleteWindowById(WC_STATUS_BAR, 0);
+}
+
+int PositionMainToolbar(Window *w)
+{
+	DEBUG(misc, 1) ("Repositioning Main Toolbar...");
+
+	if (w == NULL || w->window_class != WC_MAIN_TOOLBAR)
+		w = FindWindowById(WC_MAIN_TOOLBAR, 0);
+
+	switch (_patches.toolbar_pos) {
+		case 1:		w->left = (_screen.width - w->width) >> 1; break;
+		case 2:		w->left = _screen.width - w->width; break;
+		default:	w->left = 0;
+	}
+	SetDirtyBlocks(0, 0, _screen.width, w->height); // invalidate the whole top part
+	return w->left;
+}
+
 void RelocateAllWindows(int neww, int newh)
 {
 	Window *w;
 
 	for(w=_windows; w!= _last_window ;w++) {
 		int left, top;
-	
+
 		if (w->window_class == WC_MAIN_WINDOW) {
 			ViewPort *vp = w->viewport;
 			vp->width = w->width = neww;
@@ -1178,10 +1574,12 @@ void RelocateAllWindows(int neww, int newh)
 			continue; // don't modify top,left
 		}
 
+		IConsoleResize();
+
 		if (w->window_class == WC_MAIN_TOOLBAR) {
 			top = w->top;
-			left = (neww - w->width) >> 1;
-		} else if (w->window_class == WC_SELECT_GAME || w->window_class == WC_GAME_OPTIONS){	
+			left = PositionMainToolbar(w); // changes toolbar orientation
+		} else if (w->window_class == WC_SELECT_GAME || w->window_class == WC_GAME_OPTIONS || w->window_class == WC_NETWORK_WINDOW){
 			top = (newh - w->height) >> 1;
 			left = (neww - w->width) >> 1;
 		} else if (w->window_class == WC_NEWS_WINDOW) {
@@ -1189,7 +1587,10 @@ void RelocateAllWindows(int neww, int newh)
 			left = (neww - w->width) >> 1;
 		} else if (w->window_class == WC_STATUS_BAR) {
 			top = newh - w->height;
-			left = (neww - w->width) >> 1;	
+			left = (neww - w->width) >> 1;
+		} else if (w->window_class == WC_SEND_NETWORK_MSG) {
+			top = (newh - 26); // 26 = height of status bar + height of chat bar
+			left = (neww - w->width) >> 1;
 		} else {
 			left = w->left;
 			if (left + (w->width>>1) >= neww) left = neww - w->width;
@@ -1206,4 +1607,3 @@ void RelocateAllWindows(int neww, int newh)
 		w->top = top;
 	}
 }
-
