@@ -1,14 +1,60 @@
 #ifndef WINDOW_H
 #define WINDOW_H
 
+#include "vehicle_gui.h"
+
 typedef union WindowEvent WindowEvent;
 
 //typedef void WindowProc(Window *w, int event, int wparam, long lparam);
 
 typedef void WindowProc(Window *w, WindowEvent *e);
 
+/* How the resize system works:
+    First, you need to add a WWT_RESIZEBOX to the widgets, and you need
+     to add the flag WDF_RESIZABLE to the window. Now the window is ready
+     to resize itself.
+    As you may have noticed, all widgets have a RESIZE_XXX in their line.
+     This lines controls how the widgets behave on resize. RESIZE_NONE means
+     it doesn't do anything. Any other option let's one of the borders
+     move with the changed width/height. So if a widget has
+     RESIZE_RIGHT, and the window is made 5 pixels wider by the user,
+     the right of the window will also be made 5 pixels wider.
+    Now, what if you want to clamp a widget to the bottom? Give it the flag
+     RESIZE_TB. This is RESIZE_TOP + RESIZE_BOTTOM. Now if the window gets
+     5 pixels bigger, both the top and bottom gets 5 bigger, so the whole
+     widgets moves downwards without resizing, and appears to be clamped
+     to the bottom. Nice aint it?
+   You should know one more thing about this system. Most windows can't
+    handle an increase of 1 pixel. So there is a step function, which
+    let the windowsize only be changed by X pixels. You configure this
+    after making the window, like this:
+      w->resize.step_height = 10;
+    Now the window will only change in height in steps of 10.
+   You can also give a minimum width and height. The default value is
+    the default height/width of the window itself. You can change this
+    AFTER window-creation, with:
+     w->resize.width or w->resize.height.
+   That was all.. good luck, and enjoy :) -- TrueLight */
+
+enum {
+	RESIZE_NONE   = 0,
+
+	RESIZE_LEFT   = 1,
+	RESIZE_RIGHT  = 2,
+	RESIZE_TOP    = 4,
+	RESIZE_BOTTOM = 8,
+
+	RESIZE_LR     = RESIZE_LEFT  | RESIZE_RIGHT,
+	RESIZE_RB     = RESIZE_RIGHT | RESIZE_BOTTOM,
+	RESIZE_TB     = RESIZE_TOP   | RESIZE_BOTTOM,
+	RESIZE_LRB    = RESIZE_LEFT  | RESIZE_RIGHT  | RESIZE_BOTTOM,
+	RESIZE_LRTB   = RESIZE_LEFT  | RESIZE_RIGHT  | RESIZE_TOP | RESIZE_BOTTOM,
+	RESIZE_RTB    = RESIZE_RIGHT | RESIZE_TOP    | RESIZE_BOTTOM,
+};
+
 typedef struct Widget {
 	byte type;
+	byte resize_flag;
 	byte color;
 	uint16 left, right, top, bottom;
 	uint16 unkA;
@@ -37,6 +83,12 @@ union WindowEvent {
 		int widget;
 	} dragdrop;
 
+	struct {
+		byte event;
+		Point size;
+		Point diff;
+	} sizing;
+
 	struct {
 		byte event;
 		byte *str;
@@ -53,6 +105,12 @@ union WindowEvent {
 		int index;
 	} dropdown;
 
+	struct {
+		byte event;
+		Point pt;
+		int widget;
+	} mouseover;
+
 	struct {
 		byte event;
 		bool cont;   // continue the search? (default true)
@@ -128,6 +186,7 @@ enum WindowKeyCodes {
 	// we only store this key here, no matter what character is really mapped to it
 	// on a particular keyboard. (US keyboard: ` and ~ ; German keyboard: ^ and )
 	WKC_BACKQUOTE = 45,
+	WKC_PAUSE     = 46,
 
 	// 0-9 are mapped to 48-57
 	// A-Z are mapped to 65-90
@@ -148,11 +207,13 @@ typedef struct WindowDesc {
 } WindowDesc;
 
 enum {
-	WDF_STD_TOOLTIPS = 1,		/* use standard routine when displaying tooltips */
-	WDF_DEF_WIDGET = 2,			/* default widget control for some widgets in the on click event */
-	WDF_STD_BTN = 4,				/* default handling for close and drag widgets (widget no 0 and 1) */
+	WDF_STD_TOOLTIPS   = 1, /* use standard routine when displaying tooltips */
+	WDF_DEF_WIDGET     = 2,	/* default widget control for some widgets in the on click event */
+	WDF_STD_BTN        = 4,	/* default handling for close and drag widgets (widget no 0 and 1) */
 	WDF_RESTORE_DPARAM = 8, /* when drawing widgets, restore the dparam so all widgets recieve the same set of them */
 	WDF_UNCLICK_BUTTONS=16, /* Unclick buttons when the window event times out */
+	WDF_STICKY_BUTTON  =32, /* Set window to sticky mode; they are not closed unless closed with 'X' (widget 2) */
+	WDF_RESIZABLE      =64, /* A window can be resized */
 };
 
 /* can be used as x or y coordinates to cause a specific placement */
@@ -173,7 +234,7 @@ typedef struct {
 #define WP(ptr,str) (*(str*)(ptr)->custom)
 // querystr_d is the bigest struct that comes in w->custom
 //  because 64-bit systems use 64-bit pointers, it is bigger on a 64-bit system
-//  then on a 32-bit system. Therefor the size is calculated from querystr_d
+//  than on a 32-bit system. Therefore, the size is calculated from querystr_d
 //  instead of a hardcoded number.
 // if any struct becomes bigger the querystr_d, it should be replaced.
 #define WINDOW_CUSTOM_SIZE sizeof(querystr_d)
@@ -182,22 +243,32 @@ typedef struct {
 	uint16 count, cap, pos;
 } Scrollbar;
 
+typedef struct {
+	uint width; /* Minimum width and height */
+	uint height;
+
+	uint step_width; /* In how big steps the width and height go */
+	uint step_height;
+} ResizeInfo;
+
 struct Window {
 	uint16 flags4;
 	WindowClass window_class;
 	WindowNumber window_number;
 
-	int left,top;
-	int width,height;
+	int left, top;
+	int width, height;
 
-	Scrollbar hscroll, vscroll;
+	Scrollbar hscroll, vscroll, vscroll2;
+	ResizeInfo resize;
 
 	byte caption_color;
 
 	uint32 click_state, disabled_state, hidden_state;
 	WindowProc *wndproc;
 	ViewPort *viewport;
-	const Widget *widget;
+	const Widget *original_widget;
+ 	Widget *widget;
 	//const WindowDesc *desc;
 	uint32 desc_flags;
 
@@ -212,6 +283,7 @@ typedef struct {
 	StringID string_id; /* unk30 */
 	uint16 checked_items; /* unk32 */
 } menu_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(menu_d));
 
 typedef struct {
 	int16 data_1, data_2, data_3;
@@ -219,23 +291,28 @@ typedef struct {
 	bool close; /* scrollpos_y */
 	byte byte_1;
 } def_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(def_d));
 
 typedef struct {
 	void *data;
 } void_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(void_d));
 
 typedef struct {
 	uint16 base; /* follow_vehicle */
 	uint16 count;/* scrollpos_x */
 } tree_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(tree_d));
 
 typedef struct {
 	byte refresh_counter; /* follow_vehicle */
 } plstations_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(plstations_d));
 
 typedef struct {
 	StringID string_id;
 } tooltips_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(tooltips_d));
 
 typedef struct {
 	byte railtype;
@@ -243,43 +320,103 @@ typedef struct {
 	int16 sel_engine;
 	int16 rename_engine;
 } buildtrain_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(buildtrain_d));
+
+typedef struct {
+	byte railtype;
+	byte vehicletype;
+	byte sel_index[2];
+	int16 sel_engine[2];
+	uint16 count[2];
+} replaceveh_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(replaceveh_d));
 
 typedef struct {
 	VehicleID sel;
 } traindepot_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(traindepot_d));
 
 typedef struct {
 	int sel;
 } order_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(order_d));
 
 typedef struct {
 	byte tab;
 } traindetails_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(traindetails_d));
 
 typedef struct {
-	int16 scroll_x, scroll_y, subscroll;
+	int32 scroll_x;
+	int32 scroll_y;
+	int32 subscroll;
 } smallmap_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(traindetails_d));
 
 typedef struct {
 	uint32 face;
 	byte gender;
 } facesel_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(facesel_d));
 
 typedef struct {
 	int sel;
 	byte cargo;
 } refit_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(refit_d));
 
 typedef struct {
 	uint16 follow_vehicle;
-	int16 scrollpos_x, scrollpos_y;
+	int32 scrollpos_x;
+	int32 scrollpos_y;
 } vp_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(vp_d) + 3 * sizeof(byte)); // + 3 * byte is a hack from Miham
+
+// vp2_d is the same as vp_d, except for the data_# values..
+typedef struct {
+	uint16 follow_vehicle;
+	int32 scrollpos_x;
+	int32 scrollpos_y;
+	byte data_1;
+	byte data_2;
+	byte data_3;
+} vp2_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(vp2_d));
 
 typedef struct {
 	uint16 follow_vehicle;
-	int16 scrollpos_x, scrollpos_y;
+	int32 scrollpos_x;
+	int32 scrollpos_y;
 	NewsItem *ni;
 } news_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(news_d));
+
+typedef struct {
+	uint32 background_img;
+	int8 rank;
+} highscore_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(highscore_d));
+
+typedef struct {
+	int height;
+	uint16 counter;
+} scroller_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(scroller_d));
+
+typedef enum VehicleListFlags {
+	VL_DESC    = 0x01,
+	VL_RESORT  = 0x02,
+	VL_REBUILD = 0x04
+} VehicleListFlags;
+
+typedef struct vehiclelist_d {
+	SortStruct *sort_list;
+	uint16 list_length;
+	byte sort_type;
+	VehicleListFlags flags;
+	uint16 resort_timer;
+} vehiclelist_d;
+assert_compile(WINDOW_CUSTOM_SIZE >= sizeof(vehiclelist_d));
 
 enum WindowEvents {
 	WE_CLICK = 0,
@@ -302,6 +439,9 @@ enum WindowEvents {
 	WE_RCLICK = 17,
 	WE_KEYPRESS = 18,
 	WE_CREATE = 19,
+	WE_MOUSEOVER = 20,
+	WE_ON_EDIT_TEXT_CANCEL = 21,
+	WE_RESIZE = 22,
 };
 
 
@@ -330,7 +470,10 @@ enum WindowWidgetTypes {
 	WWT_CAPTION = 10,
 
 	WWT_HSCROLLBAR = 11,
-	WWT_LAST = 12,						/* Last Item. use WIDGETS_END to fill up padding!! */
+	WWT_STICKYBOX = 12,
+	WWT_SCROLL2BAR = 13,				/* 2nd vertical scrollbar*/
+	WWT_RESIZEBOX = 14,
+	WWT_LAST = 15,						/* Last Item. use WIDGETS_END to fill up padding!! */
 
 	WWT_MASK = 31,
 
@@ -339,7 +482,7 @@ enum WindowWidgetTypes {
 	WWT_NODISTXTBTN = WWT_TEXTBTN	| WWB_NODISBUTTON,
 };
 
-#define WIDGETS_END WWT_LAST,     0,     0,     0,     0,     0, 0, STR_NULL
+#define WIDGETS_END WWT_LAST,   RESIZE_NONE,     0,     0,     0,     0,     0, 0, STR_NULL
 
 enum WindowFlags {
 	WF_TIMEOUT_SHL = 0,
@@ -350,24 +493,24 @@ enum WindowFlags {
 	WF_SCROLL_MIDDLE = 1 << 6,
 	WF_HSCROLL = 1 << 7,
 	WF_SIZING = 1 << 8,
+	WF_STICKY = 1 << 9,
 
 	WF_DISABLE_VP_SCROLL = 1 << 10,
 
 	WF_WHITE_BORDER_ONE = 1 << 11,
 	WF_WHITE_BORDER_MASK = 3 << 11,
+	WF_SCROLL2 = 1 << 13,
 };
 
 
 void DispatchLeftClickEvent(Window *w, int x, int y);
 void DispatchRightClickEvent(Window *w, int x, int y);
-void DispatchMouseWheelEvent(Window *w, int wheel);
+void DispatchMouseWheelEvent(Window *w, uint widget, int wheel);
 
 /* window.c */
 void DrawOverlappedWindow(Window *w, int left, int top, int right, int bottom);
 void CallWindowEventNP(Window *w, int event);
-void CallWindowTickEvent();
-void DrawDirtyBlocks();
-void SetDirtyBlocks(int left, int top, int right, int bottom);
+void CallWindowTickEvent(void);
 void SetWindowDirty(Window *w);
 
 Window *FindWindowById(WindowClass cls, WindowNumber number);
@@ -378,6 +521,10 @@ Window *StartWindowDrag(Window *w);
 Window *StartWindowSizing(Window *w);
 Window *FindWindowFromPt(int x, int y);
 
+bool IsWindowOfPrototype(Window *w, const Widget *widget);
+void AssignWidgetToWindow(Window *w, const Widget *widget);
+/* Use this function to save the current widget to be the global default */
+void MakeWindowWidgetDefault(Window *w);
 Window *AllocateWindow(
 							int x,
 							int y,
@@ -408,10 +555,10 @@ Window *AllocateWindowAutoPlace2(
 
 void DrawWindowViewport(Window *w);
 
-void InitWindowSystem();
+void InitWindowSystem(void);
 int GetMenuItemIndex(Window *w, int x, int y);
-void MouseLoop();
-void UpdateWindows();
+void MouseLoop(void);
+void UpdateWindows(void);
 void InvalidateWidget(Window *w, byte widget_index);
 
 void GuiShowTooltips(uint16 string_id);
@@ -419,20 +566,23 @@ void GuiShowTooltips(uint16 string_id);
 void UnclickWindowButtons(Window *w);
 void UnclickSomeWindowButtons(Window *w, uint32 mask);
 void RelocateAllWindows(int neww, int newh);
-int32 PositionMainToolbar(Window *w);
+int PositionMainToolbar(Window *w);
 
 /* widget.c */
 int GetWidgetFromPos(Window *w, int x, int y);
 void DrawWindowWidgets(Window *w);
-void ShowDropDownMenu(Window *w, const StringID *strings, int selected, int button, uint32 disabled_mask);
+void ShowDropDownMenu(Window *w, const StringID *strings, int selected, int button, uint32 disabled_mask, bool remove_filtered_strings);
 
 void HandleButtonClick(Window *w, byte widget);
 
-Window *GetCallbackWnd();
-void DeleteNonVitalWindows();
+Window *GetCallbackWnd(void);
+void DeleteNonVitalWindows(void);
+void DeleteAllNonVitalWindows(void);
+void HideVitalWindows(void);
+void ShowVitalWindows(void);
 
 /* window.c */
-VARDEF Window _windows[20];
+VARDEF Window _windows[25];
 VARDEF Window *_last_window;
 
 VARDEF Point _cursorpos_drag_start;
